//
// Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
// DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
//
// This code is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License version 2 only, as
// published by the Free Software Foundation.
//
// This code is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// version 2 for more details (a copy is included in the LICENSE file that
// accompanied this code).
//
// You should have received a copy of the GNU General Public License version
// 2 along with this work; if not, write to the Free Software Foundation,
// Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
//
// Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
// or visit www.oracle.com if you need additional information or have any
// questions.
//
//

// Machine Generated File.  Do Not Edit!

#include "precompiled.hpp"
#include "adfiles/adGlobals_x86_64.hpp"
#include "adfiles/ad_x86_64.hpp"
#include "memory/allocation.inline.hpp"
#include "asm/assembler.hpp"
#include "code/compiledIC.hpp"
#include "code/vmreg.hpp"
#include "gc_interface/collectedHeap.inline.hpp"
#include "oops/compiledICHolderOop.hpp"
#include "oops/markOop.hpp"
#include "oops/methodOop.hpp"
#include "oops/oop.inline.hpp"
#include "oops/oop.inline2.hpp"
#include "opto/cfgnode.hpp"
#include "opto/locknode.hpp"
#include "opto/opcodes.hpp"
#include "opto/regalloc.hpp"
#include "opto/regmask.hpp"
#include "opto/runtime.hpp"
#include "runtime/biasedLocking.hpp"
#include "runtime/sharedRuntime.hpp"
#include "runtime/stubRoutines.hpp"
#include "utilities/growableArray.hpp"
#include "assembler_x86.inline.hpp"
#include "nativeInst_x86.hpp"
#include "vmreg_x86.inline.hpp"

//SourceForm

#line 393 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

#define   RELOC_IMM64    Assembler::imm_operand
#define   RELOC_DISP32   Assembler::disp32_operand

#define __ _masm.

static int preserve_SP_size() {
  return 3;  // rex.w, op, rm(reg/reg)
}
static int clear_avx_size() {
  return (Compile::current()->max_vector_size() > 16) ? 3 : 0;  // vzeroupper
}

// !!!!! Special hack to get all types of calls to specify the byte offset
//       from the start of the call to the point where the return address
//       will point.
int MachCallStaticJavaNode::ret_addr_offset()
{
  int offset = 5; // 5 bytes from start of call to where return address points
  offset += clear_avx_size();
  if (_method_handle_invoke)
    offset += preserve_SP_size();
  return offset;
}

int MachCallDynamicJavaNode::ret_addr_offset()
{
  int offset = 15; // 15 bytes from start of call to where return address points
  offset += clear_avx_size();
  return offset;
}

int MachCallRuntimeNode::ret_addr_offset() {
  int offset = 13; // movq r10,#addr; callq (r10)
  offset += clear_avx_size();
  return offset;
}

// Indicate if the safepoint node needs the polling page as an input,
// it does if the polling page is more than disp32 away.
bool SafePointNode::needs_polling_address_input()
{
  return Assembler::is_polling_page_far();
}

//
// Compute padding required for nodes which need alignment
//

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += clear_avx_size(); // skip vzeroupper
  current_offset += 1; // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallStaticJavaHandleNode::compute_padding(int current_offset) const
{
  current_offset += preserve_SP_size();   // skip mov rbp, rsp
  current_offset += clear_avx_size(); // skip vzeroupper
  current_offset += 1; // skip call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// The address of the call instruction needs to be 4-byte aligned to
// ensure that it does not span a cache line so that it can be patched.
int CallDynamicJavaDirectNode::compute_padding(int current_offset) const
{
  current_offset += clear_avx_size(); // skip vzeroupper
  current_offset += 11; // skip movq instruction + call opcode byte
  return round_to(current_offset, alignment_required()) - current_offset;
}

// EMIT_RM()
void emit_rm(CodeBuffer &cbuf, int f1, int f2, int f3) {
  unsigned char c = (unsigned char) ((f1 << 6) | (f2 << 3) | f3);
  cbuf.insts()->emit_int8(c);
}

// EMIT_CC()
void emit_cc(CodeBuffer &cbuf, int f1, int f2) {
  unsigned char c = (unsigned char) (f1 | f2);
  cbuf.insts()->emit_int8(c);
}

// EMIT_OPCODE()
void emit_opcode(CodeBuffer &cbuf, int code) {
  cbuf.insts()->emit_int8((unsigned char) code);
}

// EMIT_OPCODE() w/ relocation information
void emit_opcode(CodeBuffer &cbuf,
                 int code, relocInfo::relocType reloc, int offset, int format)
{
  cbuf.relocate(cbuf.insts_mark() + offset, reloc, format);
  emit_opcode(cbuf, code);
}

// EMIT_D8()
void emit_d8(CodeBuffer &cbuf, int d8) {
  cbuf.insts()->emit_int8((unsigned char) d8);
}

// EMIT_D16()
void emit_d16(CodeBuffer &cbuf, int d16) {
  cbuf.insts()->emit_int16(d16);
}

// EMIT_D32()
void emit_d32(CodeBuffer &cbuf, int d32) {
  cbuf.insts()->emit_int32(d32);
}

// EMIT_D64()
void emit_d64(CodeBuffer &cbuf, int64_t d64) {
  cbuf.insts()->emit_int64(d64);
}

// emit 32 bit value and construct relocation entry from relocInfo::relocType
void emit_d32_reloc(CodeBuffer& cbuf,
                    int d32,
                    relocInfo::relocType reloc,
                    int format)
{
  assert(reloc != relocInfo::external_word_type, "use 2-arg emit_d32_reloc");
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int32(d32);
}

// emit 32 bit value and construct relocation entry from RelocationHolder
void emit_d32_reloc(CodeBuffer& cbuf, int d32, RelocationHolder const& rspec, int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type &&
      d32 != 0 && d32 != (intptr_t) Universe::non_oop_word()) {
    assert(oop((intptr_t)d32)->is_oop() && (ScavengeRootsInCode || !oop((intptr_t)d32)->is_scavengable()), "cannot embed scavengable oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int32(d32);
}

void emit_d32_reloc(CodeBuffer& cbuf, address addr) {
  address next_ip = cbuf.insts_end() + 4;
  emit_d32_reloc(cbuf, (int) (addr - next_ip),
                 external_word_Relocation::spec(addr),
                 RELOC_DISP32);
}


// emit 64 bit value and construct relocation entry from relocInfo::relocType
void emit_d64_reloc(CodeBuffer& cbuf, int64_t d64, relocInfo::relocType reloc, int format) {
  cbuf.relocate(cbuf.insts_mark(), reloc, format);
  cbuf.insts()->emit_int64(d64);
}

// emit 64 bit value and construct relocation entry from RelocationHolder
void emit_d64_reloc(CodeBuffer& cbuf, int64_t d64, RelocationHolder const& rspec, int format) {
#ifdef ASSERT
  if (rspec.reloc()->type() == relocInfo::oop_type &&
      d64 != 0 && d64 != (int64_t) Universe::non_oop_word()) {
    assert(oop(d64)->is_oop() && (ScavengeRootsInCode || !oop(d64)->is_scavengable()),
           "cannot embed scavengable oops in code");
  }
#endif
  cbuf.relocate(cbuf.insts_mark(), rspec, format);
  cbuf.insts()->emit_int64(d64);
}

// Access stack slot for load or store
void store_to_stackslot(CodeBuffer &cbuf, int opcode, int rm_field, int disp)
{
  emit_opcode(cbuf, opcode);                  // (e.g., FILD   [RSP+src])
  if (-0x80 <= disp && disp < 0x80) {
    emit_rm(cbuf, 0x01, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d8(cbuf, disp);     // Displacement  // R/M byte
  } else {
    emit_rm(cbuf, 0x02, rm_field, RSP_enc);   // R/M byte
    emit_rm(cbuf, 0x00, RSP_enc, RSP_enc);    // SIB byte
    emit_d32(cbuf, disp);     // Displacement // R/M byte
  }
}

   // rRegI ereg, memory mem) %{    // emit_reg_mem
void encode_RegMem(CodeBuffer &cbuf,
                   int reg,
                   int base, int index, int scale, int disp, bool disp_is_oop)
{
  assert(!disp_is_oop, "cannot have disp");
  int regenc = reg & 7;
  int baseenc = base & 7;
  int indexenc = index & 7;

  // There is no index & no scale, use form without SIB byte
  if (index == 0x4 && scale == 0 && base != RSP_enc && base != R12_enc) {
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) {
      emit_rm(cbuf, 0x0, regenc, baseenc); // *
    } else if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
      // If 8-bit displacement, mode 0x1
      emit_rm(cbuf, 0x1, regenc, baseenc); // *
      emit_d8(cbuf, disp);
    } else {
      // If 32-bit displacement
      if (base == -1) { // Special flag for absolute address
        emit_rm(cbuf, 0x0, regenc, 0x5); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      } else {
        // Normal base + offset
        emit_rm(cbuf, 0x2, regenc, baseenc); // *
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  } else {
    // Else, encode with the SIB byte
    // If no displacement, mode is 0x0; unless base is [RBP] or [R13]
    if (disp == 0 && base != RBP_enc && base != R13_enc) {
      // If no displacement
      emit_rm(cbuf, 0x0, regenc, 0x4); // *
      emit_rm(cbuf, scale, indexenc, baseenc);
    } else {
      if (-0x80 <= disp && disp < 0x80 && !disp_is_oop) {
        // If 8-bit displacement, mode 0x1
        emit_rm(cbuf, 0x1, regenc, 0x4); // *
        emit_rm(cbuf, scale, indexenc, baseenc);
        emit_d8(cbuf, disp);
      } else {
        // If 32-bit displacement
        if (base == 0x04 ) {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, 0x04); // XXX is this valid???
        } else {
          emit_rm(cbuf, 0x2, regenc, 0x4);
          emit_rm(cbuf, scale, indexenc, baseenc); // *
        }
        if (disp_is_oop) {
          emit_d32_reloc(cbuf, disp, relocInfo::oop_type, RELOC_DISP32);
        } else {
          emit_d32(cbuf, disp);
        }
      }
    }
  }
}

// This could be in MacroAssembler but it's fairly C2 specific
void emit_cmpfp_fixup(MacroAssembler& _masm) {
  Label exit;
  __ jccb(Assembler::noParity, exit);
  __ pushf();
  //
  // comiss/ucomiss instructions set ZF,PF,CF flags and
  // zero OF,AF,SF for NaN values.
  // Fixup flags by zeroing ZF,PF so that compare of NaN
  // values returns 'less than' result (CF is set).
  // Leave the rest of flags unchanged.
  //
  //    7 6 5 4 3 2 1 0
  //   |S|Z|r|A|r|P|r|C|  (r - reserved bit)
  //    0 0 1 0 1 0 1 1   (0x2B)
  //
  __ andq(Address(rsp, 0), 0xffffff2b);
  __ popf();
  __ bind(exit);
}

void emit_cmpfp3(MacroAssembler& _masm, Register dst) {
  Label done;
  __ movl(dst, -1);
  __ jcc(Assembler::parity, done);
  __ jcc(Assembler::below, done);
  __ setb(Assembler::notEqual, dst);
  __ movzbl(dst, dst);
  __ bind(done);
}


//=============================================================================
const RegMask& MachConstantBaseNode::_out_RegMask = RegMask::Empty;

int Compile::ConstantTable::calculate_table_base_offset() const {
  return 0;  // absolute addressing, no offset
}

bool MachConstantBaseNode::requires_late_expand() const { return false; }
void MachConstantBaseNode::lateExpand(GrowableArray <Node *> *nodes, PhaseRegAlloc *ra_) {
  ShouldNotReachHere();
}

void MachConstantBaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  // Empty encoding
}

uint MachConstantBaseNode::size(PhaseRegAlloc* ra_) const {
  return 0;
}

#ifndef PRODUCT
void MachConstantBaseNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  st->print("# MachConstantBaseNode (empty encoding)");
}
#endif


//=============================================================================
#ifndef PRODUCT
void MachPrologNode::format(PhaseRegAlloc* ra_, outputStream* st) const {
  Compile* C = ra_->C;

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove wordSize for return addr which is already pushed.
  framesize -= wordSize;

  if (C->need_stack_bang(framesize)) {
    framesize -= wordSize;
    st->print("# stack bang");
    st->print("\n\t");
    st->print("pushq   rbp\t# Save rbp");
    if (framesize) {
      st->print("\n\t");
      st->print("subq    rsp, #%d\t# Create frame",framesize);
    }
  } else {
    st->print("subq    rsp, #%d\t# Create frame",framesize);
    st->print("\n\t");
    framesize -= wordSize;
    st->print("movq    [rsp + #%d], rbp\t# Save rbp",framesize);
  }

  if (VerifyStackAtCalls) {
    st->print("\n\t");
    framesize -= wordSize;
    st->print("movq    [rsp + #%d], 0xbadb100d\t# Majik cookie for stack depth check",framesize);
#ifdef ASSERT
    st->print("\n\t");
    st->print("# stack alignment check");
#endif
  }
  st->cr();
}
#endif

void MachPrologNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  Compile* C = ra_->C;
  MacroAssembler _masm(&cbuf);

  int framesize = C->frame_slots() << LogBytesPerInt;

  __ verified_entry(framesize, C->need_stack_bang(framesize), false);

  C->set_frame_complete(cbuf.insts_size());

  if (C->has_mach_constant_base_node()) {
    // NOTE: We set the table base offset here because users might be
    // emitted before MachConstantBaseNode.
    Compile::ConstantTable& constant_table = C->constant_table();
    constant_table.set_table_base_offset(constant_table.calculate_table_base_offset());
  }
}

uint MachPrologNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachPrologNode::reloc() const
{
  return 0; // a large enough number
}

//=============================================================================
#ifndef PRODUCT
void MachEpilogNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  Compile* C = ra_->C;
  if (C->max_vector_size() > 16) {
    st->print("vzeroupper");
    st->cr(); st->print("\t");
  }

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  if (framesize) {
    st->print_cr("addq    rsp, %d\t# Destroy frame", framesize);
    st->print("\t");
  }

  st->print_cr("popq   rbp");
  if (do_polling() && C->is_method_compilation()) {
    st->print("\t");
    if (Assembler::is_polling_page_far()) {
      st->print_cr("movq   rscratch1, #polling_page_address\n\t"
                   "testl  rax, [rscratch1]\t"
                   "# Safepoint: poll for GC");
    } else {
      st->print_cr("testl  rax, [rip + #offset_to_poll_page]\t"
                   "# Safepoint: poll for GC");
    }
  }
}
#endif

void MachEpilogNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  Compile* C = ra_->C;
  if (C->max_vector_size() > 16) {
    // Clear upper bits of YMM registers when current compiled code uses
    // wide vectors to avoid AVX <-> SSE transition penalty during call.
    MacroAssembler _masm(&cbuf);
    __ vzeroupper();
  }

  int framesize = C->frame_slots() << LogBytesPerInt;
  assert((framesize & (StackAlignmentInBytes-1)) == 0, "frame size not aligned");
  // Remove word for return adr already pushed
  // and RBP
  framesize -= 2*wordSize;

  // Note that VerifyStackAtCalls' Majik cookie does not change the frame size popped here

  if (framesize) {
    emit_opcode(cbuf, Assembler::REX_W);
    if (framesize < 0x80) {
      emit_opcode(cbuf, 0x83); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d8(cbuf, framesize);
    } else {
      emit_opcode(cbuf, 0x81); // addq rsp, #framesize
      emit_rm(cbuf, 0x3, 0x00, RSP_enc);
      emit_d32(cbuf, framesize);
    }
  }

  // popq rbp
  emit_opcode(cbuf, 0x58 | RBP_enc);

  if (do_polling() && C->is_method_compilation()) {
    MacroAssembler _masm(&cbuf);
    AddressLiteral polling_page(os::get_polling_page(), relocInfo::poll_return_type);
    if (Assembler::is_polling_page_far()) {
      __ lea(rscratch1, polling_page);
      __ relocate(relocInfo::poll_return_type);
      __ testl(rax, Address(rscratch1, 0));
    } else {
      __ testl(rax, polling_page);
    }
  }
}

uint MachEpilogNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}

int MachEpilogNode::reloc() const
{
  return 2; // a large enough number
}

const Pipeline* MachEpilogNode::pipeline() const
{
  return MachNode::pipeline_class();
}

int MachEpilogNode::safepoint_offset() const
{
  return 0;
}

//=============================================================================

enum RC {
  rc_bad,
  rc_int,
  rc_float,
  rc_stack
};

static enum RC rc_class(OptoReg::Name reg)
{
  if( !OptoReg::is_valid(reg)  ) return rc_bad;

  if (OptoReg::is_stack(reg)) return rc_stack;

  VMReg r = OptoReg::as_VMReg(reg);

  if (r->is_Register()) return rc_int;

  assert(r->is_XMMRegister(), "must be");
  return rc_float;
}

// Next two methods are shared by 32- and 64-bit VM. They are defined in x86.ad.
static int vec_mov_helper(CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st);

static int vec_spill_helper(CodeBuffer *cbuf, bool do_size, bool is_load,
                            int stack_offset, int reg, uint ireg, outputStream* st);

static void vec_stack_to_stack_helper(CodeBuffer *cbuf, int src_offset,
                                      int dst_offset, uint ireg, outputStream* st) {
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    switch (ireg) {
    case Op_VecS:
      __ movq(Address(rsp, -8), rax);
      __ movl(rax, Address(rsp, src_offset));
      __ movl(Address(rsp, dst_offset), rax);
      __ movq(rax, Address(rsp, -8));
      break;
    case Op_VecD:
      __ pushq(Address(rsp, src_offset));
      __ popq (Address(rsp, dst_offset));
      break;
    case Op_VecX:
      __ pushq(Address(rsp, src_offset));
      __ popq (Address(rsp, dst_offset));
      __ pushq(Address(rsp, src_offset+8));
      __ popq (Address(rsp, dst_offset+8));
      break;
    case Op_VecY:
      __ vmovdqu(Address(rsp, -32), xmm0);
      __ vmovdqu(xmm0, Address(rsp, src_offset));
      __ vmovdqu(Address(rsp, dst_offset), xmm0);
      __ vmovdqu(xmm0, Address(rsp, -32));
      break;
    default:
      ShouldNotReachHere();
    }
#ifndef PRODUCT
  } else {
    switch (ireg) {
    case Op_VecS:
      st->print("movq    [rsp - #8], rax\t# 32-bit mem-mem spill\n\t"
                "movl    rax, [rsp + #%d]\n\t"
                "movl    [rsp + #%d], rax\n\t"
                "movq    rax, [rsp - #8]",
                src_offset, dst_offset);
      break;
    case Op_VecD:
      st->print("pushq   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                "popq    [rsp + #%d]",
                src_offset, dst_offset);
      break;
     case Op_VecX:
      st->print("pushq   [rsp + #%d]\t# 128-bit mem-mem spill\n\t"
                "popq    [rsp + #%d]\n\t"
                "pushq   [rsp + #%d]\n\t"
                "popq    [rsp + #%d]",
                src_offset, dst_offset, src_offset+8, dst_offset+8);
      break;
    case Op_VecY:
      st->print("vmovdqu [rsp - #32], xmm0\t# 256-bit mem-mem spill\n\t"
                "vmovdqu xmm0, [rsp + #%d]\n\t"
                "vmovdqu [rsp + #%d], xmm0\n\t"
                "vmovdqu xmm0, [rsp - #32]",
                src_offset, dst_offset);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
}

uint MachSpillCopyNode::implementation(CodeBuffer* cbuf,
                                       PhaseRegAlloc* ra_,
                                       bool do_size,
                                       outputStream* st) const {
  assert(cbuf != NULL || st  != NULL, "sanity");
  // Get registers to move
  OptoReg::Name src_second = ra_->get_reg_second(in(1));
  OptoReg::Name src_first = ra_->get_reg_first(in(1));
  OptoReg::Name dst_second = ra_->get_reg_second(this);
  OptoReg::Name dst_first = ra_->get_reg_first(this);

  enum RC src_second_rc = rc_class(src_second);
  enum RC src_first_rc = rc_class(src_first);
  enum RC dst_second_rc = rc_class(dst_second);
  enum RC dst_first_rc = rc_class(dst_first);

  assert(OptoReg::is_valid(src_first) && OptoReg::is_valid(dst_first),
         "must move at least 1 register" );

  if (src_first == dst_first && src_second == dst_second) {
    // Self copy, no move
    return 0;
  }
  if (bottom_type()->isa_vect() != NULL) {
    uint ireg = ideal_reg();
    assert((src_first_rc != rc_int && dst_first_rc != rc_int), "sanity");
    assert((ireg == Op_VecS || ireg == Op_VecD || ireg == Op_VecX || ireg == Op_VecY), "sanity");
    if( src_first_rc == rc_stack && dst_first_rc == rc_stack ) {
      // mem -> mem
      int src_offset = ra_->reg2offset(src_first);
      int dst_offset = ra_->reg2offset(dst_first);
      vec_stack_to_stack_helper(cbuf, src_offset, dst_offset, ireg, st);
    } else if (src_first_rc == rc_float && dst_first_rc == rc_float ) {
      vec_mov_helper(cbuf, false, src_first, dst_first, src_second, dst_second, ireg, st);
    } else if (src_first_rc == rc_float && dst_first_rc == rc_stack ) {
      int stack_offset = ra_->reg2offset(dst_first);
      vec_spill_helper(cbuf, false, false, stack_offset, src_first, ireg, st);
    } else if (src_first_rc == rc_stack && dst_first_rc == rc_float ) {
      int stack_offset = ra_->reg2offset(src_first);
      vec_spill_helper(cbuf, false, true,  stack_offset, dst_first, ireg, st);
    } else {
      ShouldNotReachHere();
    }
    return 0;
  }
  if (src_first_rc == rc_stack) {
    // mem ->
    if (dst_first_rc == rc_stack) {
      // mem -> mem
      assert(src_second != dst_first, "overlap");
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ pushq(Address(rsp, src_offset));
          __ popq (Address(rsp, dst_offset));
#ifndef PRODUCT
        } else {
          st->print("pushq   [rsp + #%d]\t# 64-bit mem-mem spill\n\t"
                    "popq    [rsp + #%d]",
                     src_offset, dst_offset);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        // No pushl/popl, so:
        int src_offset = ra_->reg2offset(src_first);
        int dst_offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movq(Address(rsp, -8), rax);
          __ movl(rax, Address(rsp, src_offset));
          __ movl(Address(rsp, dst_offset), rax);
          __ movq(rax, Address(rsp, -8));
#ifndef PRODUCT
        } else {
          st->print("movq    [rsp - #8], rax\t# 32-bit mem-mem spill\n\t"
                    "movl    rax, [rsp + #%d]\n\t"
                    "movl    [rsp + #%d], rax\n\t"
                    "movq    rax, [rsp - #8]",
                     src_offset, dst_offset);
#endif
        }
      }
      return 0;
    } else if (dst_first_rc == rc_int) {
      // mem -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movq(as_Register(Matcher::_regEncode[dst_first]), Address(rsp, offset));
#ifndef PRODUCT
        } else {
          st->print("movq    %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movl(as_Register(Matcher::_regEncode[dst_first]), Address(rsp, offset));
#ifndef PRODUCT
        } else {
          st->print("movl    %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
      }
      return 0;
    } else if (dst_first_rc == rc_float) {
      // mem-> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdbl( as_XMMRegister(Matcher::_regEncode[dst_first]), Address(rsp, offset));
#ifndef PRODUCT
        } else {
          st->print("%s  %s, [rsp + #%d]\t# spill",
                     UseXmmLoadAndClearUpper ? "movsd " : "movlpd",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(src_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movflt( as_XMMRegister(Matcher::_regEncode[dst_first]), Address(rsp, offset));
#ifndef PRODUCT
        } else {
          st->print("movss   %s, [rsp + #%d]\t# spill",
                     Matcher::regName[dst_first],
                     offset);
#endif
        }
      }
      return 0;
    }
  } else if (src_first_rc == rc_int) {
    // gpr ->
    if (dst_first_rc == rc_stack) {
      // gpr -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movq(Address(rsp, offset), as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movq    [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movl(Address(rsp, offset), as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movl    [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
      }
      return 0;
    } else if (dst_first_rc == rc_int) {
      // gpr -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movq(as_Register(Matcher::_regEncode[dst_first]),
                  as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movq    %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 0;
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movl(as_Register(Matcher::_regEncode[dst_first]),
                  as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movl    %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
        return 0;
      }
    } else if (dst_first_rc == rc_float) {
      // gpr -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdq( as_XMMRegister(Matcher::_regEncode[dst_first]), as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movdq   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdl( as_XMMRegister(Matcher::_regEncode[dst_first]), as_Register(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movdl   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      }
      return 0;
    }
  } else if (src_first_rc == rc_float) {
    // xmm ->
    if (dst_first_rc == rc_stack) {
      // xmm -> mem
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdbl( Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movsd   [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        int offset = ra_->reg2offset(dst_first);
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movflt(Address(rsp, offset), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movss   [rsp + #%d], %s\t# spill",
                     offset,
                     Matcher::regName[src_first]);
#endif
        }
      }
      return 0;
    } else if (dst_first_rc == rc_int) {
      // xmm -> gpr
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdq( as_Register(Matcher::_regEncode[dst_first]), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movdq   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdl( as_Register(Matcher::_regEncode[dst_first]), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("movdl   %s, %s\t# spill",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      }
      return 0;
    } else if (dst_first_rc == rc_float) {
      // xmm -> xmm
      if ((src_first & 1) == 0 && src_first + 1 == src_second &&
          (dst_first & 1) == 0 && dst_first + 1 == dst_second) {
        // 64-bit
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movdbl( as_XMMRegister(Matcher::_regEncode[dst_first]), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("%s  %s, %s\t# spill",
                     UseXmmRegToRegMoveAll ? "movapd" : "movsd ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      } else {
        // 32-bit
        assert(!((src_first & 1) == 0 && src_first + 1 == src_second), "no transform");
        assert(!((dst_first & 1) == 0 && dst_first + 1 == dst_second), "no transform");
        if (cbuf) {
          MacroAssembler _masm(cbuf);
          __ movflt( as_XMMRegister(Matcher::_regEncode[dst_first]), as_XMMRegister(Matcher::_regEncode[src_first]));
#ifndef PRODUCT
        } else {
          st->print("%s  %s, %s\t# spill",
                     UseXmmRegToRegMoveAll ? "movaps" : "movss ",
                     Matcher::regName[dst_first],
                     Matcher::regName[src_first]);
#endif
        }
      }
      return 0;
    }
  }

  assert(0," foo ");
  Unimplemented();
  return 0;
}

#ifndef PRODUCT
void MachSpillCopyNode::format(PhaseRegAlloc *ra_, outputStream* st) const {
  implementation(NULL, ra_, false, st);
}
#endif

void MachSpillCopyNode::emit(CodeBuffer &cbuf, PhaseRegAlloc *ra_) const {
  implementation(&cbuf, ra_, false, NULL);
}

uint MachSpillCopyNode::size(PhaseRegAlloc *ra_) const {
  return MachNode::size(ra_);
}

//=============================================================================
#ifndef PRODUCT
void BoxLockNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_reg_first(this);
  st->print("leaq    %s, [rsp + #%d]\t# box lock",
            Matcher::regName[reg], offset);
}
#endif

void BoxLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  int reg = ra_->get_encode(this);
  if (offset >= 0x80) {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x2, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d32(cbuf, offset);
  } else {
    emit_opcode(cbuf, reg < 8 ? Assembler::REX_W : Assembler::REX_WR);
    emit_opcode(cbuf, 0x8D); // LEA  reg,[SP+offset]
    emit_rm(cbuf, 0x1, reg & 7, 0x04);
    emit_rm(cbuf, 0x0, 0x04, RSP_enc);
    emit_d8(cbuf, offset);
  }
}

uint BoxLockNode::size(PhaseRegAlloc *ra_) const
{
  int offset = ra_->reg2offset(in_RegMask(0).find_first_elem());
  return (offset < 0x80) ? 5 : 8; // REX
}

//=============================================================================

// Offset from start of compiled java to interpreter stub to the load
// constant that loads the inline cache (IC) (0 on amd64).
const int CompiledStaticCall::comp_to_int_load_offset = 0;

// emit call stub, compiled java to interpreter
void emit_java_to_interp(CodeBuffer& cbuf)
{
  // Stub is fixed up when the corresponding call is converted from
  // calling compiled code to calling interpreted code.
  // movq rbx, 0
  // jmp -5 # to self

  address mark = cbuf.insts_mark();  // get mark within main instrs section

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a stub.
  MacroAssembler _masm(&cbuf);

  address base =
  __ start_a_stub(Compile::MAX_stubs_size);
  if (base == NULL)  return;  // CodeBuffer::expand failed
  // static stub relocation stores the instruction address of the call
  __ relocate(static_stub_Relocation::spec(mark), RELOC_IMM64);
  // static stub relocation also tags the methodOop in the code-stream.
  __ movoop(rbx, (jobject) NULL);  // method is zapped till fixup time
  // This is recognized as unresolved by relocs/nativeinst/ic code
  __ jump(RuntimeAddress(__ pc()));

  // Update current stubs pointer and restore insts_end.
  __ end_a_stub();
}

// size of call stub, compiled java to interpretor
uint size_java_to_interp()
{
  return 15;  // movq (1+1+8); jmp (1+4)
}

// relocation entries for call stub, compiled java to interpretor
uint reloc_java_to_interp()
{
  return 4; // 3 in emit_java_to_interp + 1 in Java_Static_Call
}

//=============================================================================
#ifndef PRODUCT
void MachUEPNode::format(PhaseRegAlloc* ra_, outputStream* st) const
{
  if (UseCompressedOops) {
    st->print_cr("movl    rscratch1, [j_rarg0 + oopDesc::klass_offset_in_bytes()]\t# compressed klass");
    if (Universe::narrow_oop_shift() != 0) {
      st->print_cr("\tdecode_heap_oop_not_null rscratch1, rscratch1");
    }
    st->print_cr("\tcmpq    rax, rscratch1\t # Inline cache check");
  } else {
    st->print_cr("\tcmpq    rax, [j_rarg0 + oopDesc::klass_offset_in_bytes()]\t"
                 "# Inline cache check");
  }
  st->print_cr("\tjne     SharedRuntime::_ic_miss_stub");
  st->print_cr("\tnop\t# nops to align entry point");
}
#endif

void MachUEPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const
{
  MacroAssembler masm(&cbuf);
  uint insts_size = cbuf.insts_size();
  if (UseCompressedOops) {
    masm.load_klass(rscratch1, j_rarg0);
    masm.cmpptr(rax, rscratch1);
  } else {
    masm.cmpptr(rax, Address(j_rarg0, oopDesc::klass_offset_in_bytes()));
  }

  masm.jump_cc(Assembler::notEqual, RuntimeAddress(SharedRuntime::get_ic_miss_stub()));

  /* WARNING these NOPs are critical so that verified entry point is properly
     4 bytes aligned for patching by NativeJump::patch_verified_entry() */
  int nops_cnt = 4 - ((cbuf.insts_size() - insts_size) & 0x3);
  if (OptoBreakpoint) {
    // Leave space for int3
    nops_cnt -= 1;
  }
  nops_cnt &= 0x3; // Do not add nops if code is aligned.
  if (nops_cnt > 0)
    masm.nop(nops_cnt);
}

uint MachUEPNode::size(PhaseRegAlloc* ra_) const
{
  return MachNode::size(ra_); // too many variables; just compute it
                              // the hard way
}


//=============================================================================
uint size_exception_handler()
{
  // NativeCall instruction size is the same as NativeJump.
  // Note that this value is also credited (in output.cpp) to
  // the size of the code section.
  return NativeJump::instruction_size;
}

// Emit exception handler code.
int emit_exception_handler(CodeBuffer& cbuf)
{

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_exception_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  __ jump(RuntimeAddress(OptoRuntime::exception_blob()->entry_point()));
  assert(__ offset() - offset <= (int) size_exception_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

uint size_deopt_handler()
{
  // three 5 byte instructions
  return 15;
}

// Emit deopt handler code.
int emit_deopt_handler(CodeBuffer& cbuf)
{

  // Note that the code buffer's insts_mark is always relative to insts.
  // That's why we must use the macroassembler to generate a handler.
  MacroAssembler _masm(&cbuf);
  address base =
  __ start_a_stub(size_deopt_handler());
  if (base == NULL)  return 0;  // CodeBuffer::expand failed
  int offset = __ offset();
  address the_pc = (address) __ pc();
  Label next;
  // push a "the_pc" on the stack without destroying any registers
  // as they all may be live.

  // push address of "next"
  __ call(next, relocInfo::none); // reloc none is fine since it is a disp32
  __ bind(next);
  // adjust it so it matches "the_pc"
  __ subptr(Address(rsp, 0), __ offset() - offset);
  __ jump(RuntimeAddress(SharedRuntime::deopt_blob()->unpack()));
  assert(__ offset() - offset <= (int) size_deopt_handler(), "overflow");
  __ end_a_stub();
  return offset;
}

int Matcher::regnum_to_fpu_offset(int regnum)
{
  return regnum - 32; // The FP registers are in the second chunk
}

// This is UltraSparc specific, true just means we have fast l2f conversion
const bool Matcher::convL2FSupported(void) {
  return true;
}

// Is this branch offset short enough that a short branch can be used?
//
// NOTE: If the platform does not provide any short branch variants, then
//       this method should return false for offset 0.
bool Matcher::is_short_branch_offset(int rule, int br_size, int offset) {
  // The passed offset is relative to address of the branch.
  // On 86 a branch displacement is calculated relative to address
  // of a next instruction.
  offset -= br_size;

  // the short version of jmpConUCF2 contains multiple branches,
  // making the reach slightly less
  if (rule == jmpConUCF2_rule)
    return (-126 <= offset && offset <= 125);
  return (-128 <= offset && offset <= 127);
}

const bool Matcher::isSimpleConstant64(jlong value) {
  // Will one (StoreL ConL) be cheaper than two (StoreI ConI)?.
  //return value == (int) value;  // Cf. storeImmL and immL32.

  // Probably always true, even if a temp register is required.
  return true;
}

// The ecx parameter to rep stosq for the ClearArray node is in words.
const bool Matcher::init_array_count_is_in_bytes = false;

// Threshold size for cleararray.
const int Matcher::init_array_short_size = 8 * BytesPerLong;

// No additional cost for CMOVL.
const int Matcher::long_cmove_cost() { return 0; }

// No CMOVF/CMOVD with SSE2
const int Matcher::float_cmove_cost() { return ConditionalMoveLimit; }

// Does the CPU require late expand (see block.cpp for description of late expand)?
const bool Matcher::require_late_expand = false;

// Should the Matcher clone shifts on addressing modes, expecting them
// to be subsumed into complex addressing expressions or compute them
// into registers?  True for Intel but false for most RISCs
const bool Matcher::clone_shift_expressions = true;

// Do we need to mask the count passed to shift instructions or does
// the cpu only look at the lower 5/6 bits anyway?
const bool Matcher::need_masked_shift_count = false;

bool Matcher::narrow_oop_use_complex_address() {
  assert(UseCompressedOops, "only for compressed oops code");
  return (LogMinObjAlignmentInBytes <= 3);
}

// Is it better to copy float constants, or load them directly from
// memory?  Intel can load a float constant from a direct address,
// requiring no extra registers.  Most RISCs will have to materialize
// an address into a register first, so they would do better to copy
// the constant from stack.
const bool Matcher::rematerialize_float_constants = true; // XXX

// If CPU can load and store mis-aligned doubles directly then no
// fixup is needed.  Else we split the double into 2 integer pieces
// and move it piece-by-piece.  Only happens when passing doubles into
// C code as the Java calling convention forces doubles to be aligned.
const bool Matcher::misaligned_doubles_ok = true;

// No-op on amd64
void Matcher::pd_implicit_null_fixup(MachNode *node, uint idx) {}

// Advertise here if the CPU requires explicit rounding operations to
// implement the UseStrictFP mode.
const bool Matcher::strict_fp_requires_explicit_rounding = true;

// Are floats conerted to double when stored to stack during deoptimization?
// On x64 it is stored without convertion so we can use normal access.
bool Matcher::float_in_double() { return false; }

// Do ints take an entire long register or just half?
const bool Matcher::int_in_long = true;

// Return whether or not this register is ever used as an argument.
// This function is used on startup to build the trampoline stubs in
// generateOptoStub.  Registers not mentioned will be killed by the VM
// call in the trampoline, and arguments in those registers not be
// available to the callee.
bool Matcher::can_be_java_arg(int reg)
{
  return
    reg ==  RDI_num || reg == RDI_H_num ||
    reg ==  RSI_num || reg == RSI_H_num ||
    reg ==  RDX_num || reg == RDX_H_num ||
    reg ==  RCX_num || reg == RCX_H_num ||
    reg ==   R8_num || reg ==  R8_H_num ||
    reg ==   R9_num || reg ==  R9_H_num ||
    reg ==  R12_num || reg == R12_H_num ||
    reg == XMM0_num || reg == XMM0b_num ||
    reg == XMM1_num || reg == XMM1b_num ||
    reg == XMM2_num || reg == XMM2b_num ||
    reg == XMM3_num || reg == XMM3b_num ||
    reg == XMM4_num || reg == XMM4b_num ||
    reg == XMM5_num || reg == XMM5b_num ||
    reg == XMM6_num || reg == XMM6b_num ||
    reg == XMM7_num || reg == XMM7b_num;
}

bool Matcher::is_spillable_arg(int reg)
{
  return can_be_java_arg(reg);
}

bool Matcher::use_asm_for_ldiv_by_con( jlong divisor ) {
  // In 64 bit mode a code which use multiply when
  // devisor is constant is faster than hardware
  // DIV instruction (it uses MulHiL).
  return false;
}

// Register for DIVI projection of divmodI
RegMask Matcher::divI_proj_mask() {
  return INT_RAX_REG_mask();
}

// Register for MODI projection of divmodI
RegMask Matcher::modI_proj_mask() {
  return INT_RDX_REG_mask();
}

// Register for DIVL projection of divmodL
RegMask Matcher::divL_proj_mask() {
  return LONG_RAX_REG_mask();
}

// Register for MODL projection of divmodL
RegMask Matcher::modL_proj_mask() {
  return LONG_RDX_REG_mask();
}

const RegMask Matcher::method_handle_invoke_SP_save_mask() {
  return PTR_RBP_REG_mask();
}

static Address build_address(int b, int i, int s, int d) {
  Register index = as_Register(i);
  Address::ScaleFactor scale = (Address::ScaleFactor)s;
  if (index == rsp) {
    index = noreg;
    scale = Address::no_scale;
  }
  Address addr(as_Register(b), index, scale, d);
  return addr;
}


#line 1378 "../generated/adfiles/ad_x86_64.cpp"


//SourceForm

#line 477 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

  // Float masks come from different places depending on platform.
#ifdef _LP64
  static address float_signmask()  { return StubRoutines::x86::float_sign_mask(); }
  static address float_signflip()  { return StubRoutines::x86::float_sign_flip(); }
  static address double_signmask() { return StubRoutines::x86::double_sign_mask(); }
  static address double_signflip() { return StubRoutines::x86::double_sign_flip(); }
#else
  static address float_signmask()  { return (address)float_signmask_pool; }
  static address float_signflip()  { return (address)float_signflip_pool; }
  static address double_signmask() { return (address)double_signmask_pool; }
  static address double_signflip() { return (address)double_signflip_pool; }
#endif

// Map Types to machine register types
const int Matcher::base2reg[Type::lastype] = {
  Node::NotAMachineReg,0,0, Op_RegI, Op_RegL, 0, Op_RegN,
  Node::NotAMachineReg, Node::NotAMachineReg, /* tuple, array */
  Op_VecS, Op_VecD, Op_VecX, Op_VecY, /* Vectors */
  Op_RegP, Op_RegP, Op_RegP, Op_RegP, Op_RegP, Op_RegP, /* the pointers */
  0, 0/*abio*/,
  Op_RegP /* Return address */, 0, /* the memories */
  Op_RegF, Op_RegF, Op_RegF, Op_RegD, Op_RegD, Op_RegD,
  0  /*bottom*/
};

const bool Matcher::match_rule_supported(int opcode) {
  if (!has_match_rule(opcode))
    return false;

  switch (opcode) {
    case Op_PopCountI:
    case Op_PopCountL:
      if (!UsePopCountInstruction)
        return false;
    break;
    case Op_MulVI:
      if ((UseSSE < 4) && (UseAVX < 1)) // only with SSE4_1 or AVX
        return false;
    break;
    case Op_CompareAndSwapL:
#ifdef _LP64
    case Op_CompareAndSwapP:
#endif
      if (!VM_Version::supports_cx8())
        return false;
    break;
  }

  return true;  // Per default match rules are supported.
}

// Max vector size in bytes. 0 if not supported.
const int Matcher::vector_width_in_bytes(BasicType bt) {
  assert(is_java_primitive(bt), "only primitive type vectors");
  if (UseSSE < 2) return 0;
  // SSE2 supports 128bit vectors for all types.
  // AVX2 supports 256bit vectors for all types.
  int size = (UseAVX > 1) ? 32 : 16;
  // AVX1 supports 256bit vectors only for FLOAT and DOUBLE.
  if (UseAVX > 0 && (bt == T_FLOAT || bt == T_DOUBLE))
    size = 32;
  // Use flag to limit vector size.
  size = MIN2(size,(int)MaxVectorSize);
  // Minimum 2 values in vector (or 4 for bytes).
  switch (bt) {
  case T_DOUBLE:
  case T_LONG:
    if (size < 16) return 0;
  case T_FLOAT:
  case T_INT:
    if (size < 8) return 0;
  case T_BOOLEAN:
  case T_BYTE:
  case T_CHAR:
  case T_SHORT:
    if (size < 4) return 0;
    break;
  default:
    ShouldNotReachHere();
  }
  return size;
}

// Limits on vector size (number of elements) loaded into vector.
const int Matcher::max_vector_size(const BasicType bt) {
  return vector_width_in_bytes(bt)/type2aelembytes(bt);
}
const int Matcher::min_vector_size(const BasicType bt) {
  int max_size = max_vector_size(bt);
  // Min size which can be loaded into vector is 4 bytes.
  int size = (type2aelembytes(bt) == 1) ? 4 : 2;
  return MIN2(size,max_size);
}

// Vector ideal reg corresponding to specidied size in bytes
const int Matcher::vector_ideal_reg(int size) {
  assert(MaxVectorSize >= size, "");
  switch(size) {
    case  4: return Op_VecS;
    case  8: return Op_VecD;
    case 16: return Op_VecX;
    case 32: return Op_VecY;
  }
  ShouldNotReachHere();
  return 0;
}

// Only lowest bits of xmm reg are used for vector shift count.
const int Matcher::vector_shift_count_ideal_reg(int size) {
  return Op_VecS;
}

// x86 supports misaligned vectors store/load.
const bool Matcher::misaligned_vectors_ok() {
  return !AlignVector; // can be changed by flag
}

// x86 AES instructions are compatible with SunJCE expanded
// keys, hence we do not need to pass the original key to stubs
const bool Matcher::pass_original_key_for_aes() {
  return false;
}

// Helper methods for MachSpillCopyNode::implementation().
static int vec_mov_helper(CodeBuffer *cbuf, bool do_size, int src_lo, int dst_lo,
                          int src_hi, int dst_hi, uint ireg, outputStream* st) {
  // In 64-bit VM size calculation is very complex. Emitting instructions
  // into scratch buffer is used to get size in 64-bit VM.
  LP64_ONLY( assert(!do_size, "this method calculates size only for 32-bit VM"); )
  assert(ireg == Op_VecS || // 32bit vector
         (src_lo & 1) == 0 && (src_lo + 1) == src_hi &&
         (dst_lo & 1) == 0 && (dst_lo + 1) == dst_hi,
         "no non-adjacent vector moves" );
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    int offset = __ offset();
    switch (ireg) {
    case Op_VecS: // copy whole register
    case Op_VecD:
    case Op_VecX:
      __ movdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      break;
    case Op_VecY:
      __ vmovdqu(as_XMMRegister(Matcher::_regEncode[dst_lo]), as_XMMRegister(Matcher::_regEncode[src_lo]));
      break;
    default:
      ShouldNotReachHere();
    }
    int size = __ offset() - offset;
#ifdef ASSERT
    // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
    assert(!do_size || size == 4, "incorrect size calculattion");
#endif
    return size;
#ifndef PRODUCT
  } else if (!do_size) {
    switch (ireg) {
    case Op_VecS:
    case Op_VecD:
    case Op_VecX:
      st->print("movdqu  %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    case Op_VecY:
      st->print("vmovdqu %s,%s\t# spill",Matcher::regName[dst_lo],Matcher::regName[src_lo]);
      break;
    default:
      ShouldNotReachHere();
    }
#endif
  }
  // VEX_2bytes prefix is used if UseAVX > 0, and it takes the same 2 bytes as SIMD prefix.
  return 4;
}

static int vec_spill_helper(CodeBuffer *cbuf, bool do_size, bool is_load,
                            int stack_offset, int reg, uint ireg, outputStream* st) {
  // In 64-bit VM size calculation is very complex. Emitting instructions
  // into scratch buffer is used to get size in 64-bit VM.
  LP64_ONLY( assert(!do_size, "this method calculates size only for 32-bit VM"); )
  if (cbuf) {
    MacroAssembler _masm(cbuf);
    int offset = __ offset();
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        __ movdl(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecD:
        __ movq(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecX:
        __ movdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      case Op_VecY:
        __ vmovdqu(as_XMMRegister(Matcher::_regEncode[reg]), Address(rsp, stack_offset));
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        __ movdl(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecD:
        __ movq(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecX:
        __ movdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      case Op_VecY:
        __ vmovdqu(Address(rsp, stack_offset), as_XMMRegister(Matcher::_regEncode[reg]));
        break;
      default:
        ShouldNotReachHere();
      }
    }
    int size = __ offset() - offset;
#ifdef ASSERT
    int offset_size = (stack_offset == 0) ? 0 : ((stack_offset < 0x80) ? 1 : 4);
    // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
    assert(!do_size || size == (5+offset_size), "incorrect size calculattion");
#endif
    return size;
#ifndef PRODUCT
  } else if (!do_size) {
    if (is_load) {
      switch (ireg) {
      case Op_VecS:
        st->print("movd    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecD:
        st->print("movq    %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
       case Op_VecX:
        st->print("movdqu  %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      case Op_VecY:
        st->print("vmovdqu %s,[rsp + %d]\t# spill", Matcher::regName[reg], stack_offset);
        break;
      default:
        ShouldNotReachHere();
      }
    } else { // store
      switch (ireg) {
      case Op_VecS:
        st->print("movd    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecD:
        st->print("movq    [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
       case Op_VecX:
        st->print("movdqu  [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      case Op_VecY:
        st->print("vmovdqu [rsp + %d],%s\t# spill", stack_offset, Matcher::regName[reg]);
        break;
      default:
        ShouldNotReachHere();
      }
    }
#endif
  }
  int offset_size = (stack_offset == 0) ? 0 : ((stack_offset < 0x80) ? 1 : 4);
  // VEX_2bytes prefix is used if UseAVX > 0, so it takes the same 2 bytes as SIMD prefix.
  return 5+offset_size;
}

static inline jfloat replicate4_imm(int con, int width) {
  // Load a constant of "width" (in bytes) and replicate it to fill 32bit.
  assert(width == 1 || width == 2, "only byte or short types here");
  int bit_width = width * 8;
  jint val = con;
  val &= (1 << bit_width) - 1;  // mask off sign bits
  while(bit_width < 32) {
    val |= (val << bit_width);
    bit_width <<= 1;
  }
  jfloat fval = *((jfloat*) &val);  // coerce to float type
  return fval;
}

static inline jdouble replicate8_imm(int con, int width) {
  // Load a constant of "width" (in bytes) and replicate it to fill 64bit.
  assert(width == 1 || width == 2 || width == 4, "only byte, short or int types here");
  int bit_width = width * 8;
  jlong val = con;
  val &= (((jlong) 1) << bit_width) - 1;  // mask off sign bits
  while(bit_width < 64) {
    val |= (val << bit_width);
    bit_width <<= 1;
  }
  jdouble dval = *((jdouble*) &val);  // coerce to double type
  return dval;
}

#ifndef PRODUCT
  void MachNopNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("nop \t# %d bytes pad for loops and calls", _count);
  }
#endif

  void MachNopNode::emit(CodeBuffer &cbuf, PhaseRegAlloc*) const {
    MacroAssembler _masm(&cbuf);
    __ nop(_count);
  }

  uint MachNopNode::size(PhaseRegAlloc*) const {
    return _count;
  }

#ifndef PRODUCT
  void MachBreakpointNode::format(PhaseRegAlloc*, outputStream* st) const {
    st->print("# breakpoint");
  }
#endif

  void MachBreakpointNode::emit(CodeBuffer &cbuf, PhaseRegAlloc* ra_) const {
    MacroAssembler _masm(&cbuf);
    __ int3();
  }

  uint MachBreakpointNode::size(PhaseRegAlloc* ra_) const {
    return MachNode::size(ra_);
  }


#line 1712 "../generated/adfiles/ad_x86_64.cpp"


//SourceForm

#line 63 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/os_cpu/linux_x86/vm/linux_x86_64.ad"



#line 1721 "../generated/adfiles/ad_x86_64.cpp"


#ifndef PRODUCT
void Compile::adlc_verification() {

  // Following assertions generated from definition section
}
#endif

// Map from machine-independent register number to register_save_policy
const        char register_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R8
  'C', // R8_H
  'C', // R9
  'C', // R9_H
  'C', // R12
  'C', // R12_H
  'C', // RCX
  'C', // RCX_H
  'C', // RBX
  'C', // RBX_H
  'C', // RDI
  'C', // RDI_H
  'C', // RDX
  'C', // RDX_H
  'C', // RSI
  'C', // RSI_H
  'C', // RAX
  'C', // RAX_H
  'N', // RBP
  'N', // RBP_H
  'C', // R13
  'C', // R13_H
  'C', // R14
  'C', // R14_H
  'C', // R15
  'C', // R15_H
  'N', // RSP
  'N', // RSP_H
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM8
  'C', // XMM8b
  'C', // XMM8c
  'C', // XMM8d
  'C', // XMM8e
  'C', // XMM8f
  'C', // XMM8g
  'C', // XMM8h
  'C', // XMM9
  'C', // XMM9b
  'C', // XMM9c
  'C', // XMM9d
  'C', // XMM9e
  'C', // XMM9f
  'C', // XMM9g
  'C', // XMM9h
  'C', // XMM10
  'C', // XMM10b
  'C', // XMM10c
  'C', // XMM10d
  'C', // XMM10e
  'C', // XMM10f
  'C', // XMM10g
  'C', // XMM10h
  'C', // XMM11
  'C', // XMM11b
  'C', // XMM11c
  'C', // XMM11d
  'C', // XMM11e
  'C', // XMM11f
  'C', // XMM11g
  'C', // XMM11h
  'C', // XMM12
  'C', // XMM12b
  'C', // XMM12c
  'C', // XMM12d
  'C', // XMM12e
  'C', // XMM12f
  'C', // XMM12g
  'C', // XMM12h
  'C', // XMM13
  'C', // XMM13b
  'C', // XMM13c
  'C', // XMM13d
  'C', // XMM13e
  'C', // XMM13f
  'C', // XMM13g
  'C', // XMM13h
  'C', // XMM14
  'C', // XMM14b
  'C', // XMM14c
  'C', // XMM14d
  'C', // XMM14e
  'C', // XMM14f
  'C', // XMM14g
  'C', // XMM14h
  'C', // XMM15
  'C', // XMM15b
  'C', // XMM15c
  'C', // XMM15d
  'C', // XMM15e
  'C', // XMM15f
  'C', // XMM15g
  'C', // XMM15h
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to c_reg_save_policy
const        char c_reg_save_policy[] = {
  'C', // R10
  'C', // R10_H
  'C', // R11
  'C', // R11_H
  'C', // R8
  'C', // R8_H
  'C', // R9
  'C', // R9_H
  'E', // R12
  'E', // R12_H
  'C', // RCX
  'C', // RCX_H
  'E', // RBX
  'E', // RBX_H
  'C', // RDI
  'C', // RDI_H
  'C', // RDX
  'C', // RDX_H
  'C', // RSI
  'C', // RSI_H
  'C', // RAX
  'C', // RAX_H
  'E', // RBP
  'E', // RBP_H
  'E', // R13
  'E', // R13_H
  'E', // R14
  'E', // R14_H
  'E', // R15
  'E', // R15_H
  'N', // RSP
  'N', // RSP_H
  'C', // XMM0
  'C', // XMM0b
  'C', // XMM0c
  'C', // XMM0d
  'C', // XMM0e
  'C', // XMM0f
  'C', // XMM0g
  'C', // XMM0h
  'C', // XMM1
  'C', // XMM1b
  'C', // XMM1c
  'C', // XMM1d
  'C', // XMM1e
  'C', // XMM1f
  'C', // XMM1g
  'C', // XMM1h
  'C', // XMM2
  'C', // XMM2b
  'C', // XMM2c
  'C', // XMM2d
  'C', // XMM2e
  'C', // XMM2f
  'C', // XMM2g
  'C', // XMM2h
  'C', // XMM3
  'C', // XMM3b
  'C', // XMM3c
  'C', // XMM3d
  'C', // XMM3e
  'C', // XMM3f
  'C', // XMM3g
  'C', // XMM3h
  'C', // XMM4
  'C', // XMM4b
  'C', // XMM4c
  'C', // XMM4d
  'C', // XMM4e
  'C', // XMM4f
  'C', // XMM4g
  'C', // XMM4h
  'C', // XMM5
  'C', // XMM5b
  'C', // XMM5c
  'C', // XMM5d
  'C', // XMM5e
  'C', // XMM5f
  'C', // XMM5g
  'C', // XMM5h
  'C', // XMM6
  'C', // XMM6b
  'C', // XMM6c
  'C', // XMM6d
  'C', // XMM6e
  'C', // XMM6f
  'C', // XMM6g
  'C', // XMM6h
  'C', // XMM7
  'C', // XMM7b
  'C', // XMM7c
  'C', // XMM7d
  'C', // XMM7e
  'C', // XMM7f
  'C', // XMM7g
  'C', // XMM7h
  'C', // XMM8
  'C', // XMM8b
  'C', // XMM8c
  'C', // XMM8d
  'C', // XMM8e
  'C', // XMM8f
  'C', // XMM8g
  'C', // XMM8h
  'C', // XMM9
  'C', // XMM9b
  'C', // XMM9c
  'C', // XMM9d
  'C', // XMM9e
  'C', // XMM9f
  'C', // XMM9g
  'C', // XMM9h
  'C', // XMM10
  'C', // XMM10b
  'C', // XMM10c
  'C', // XMM10d
  'C', // XMM10e
  'C', // XMM10f
  'C', // XMM10g
  'C', // XMM10h
  'C', // XMM11
  'C', // XMM11b
  'C', // XMM11c
  'C', // XMM11d
  'C', // XMM11e
  'C', // XMM11f
  'C', // XMM11g
  'C', // XMM11h
  'C', // XMM12
  'C', // XMM12b
  'C', // XMM12c
  'C', // XMM12d
  'C', // XMM12e
  'C', // XMM12f
  'C', // XMM12g
  'C', // XMM12h
  'C', // XMM13
  'C', // XMM13b
  'C', // XMM13c
  'C', // XMM13d
  'C', // XMM13e
  'C', // XMM13f
  'C', // XMM13g
  'C', // XMM13h
  'C', // XMM14
  'C', // XMM14b
  'C', // XMM14c
  'C', // XMM14d
  'C', // XMM14e
  'C', // XMM14f
  'C', // XMM14g
  'C', // XMM14h
  'C', // XMM15
  'C', // XMM15b
  'C', // XMM15c
  'C', // XMM15d
  'C', // XMM15e
  'C', // XMM15f
  'C', // XMM15g
  'C', // XMM15h
  'C' // no trailing comma // RFLAGS
};

// Map from machine-independent register number to register_save_type
const        int register_save_type[] = {
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegI,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  Op_RegF,
  0 // no trailing comma
};


const        int   reduceOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  immI_rule,
  /*    9 */  immI0_rule,
  /*   10 */  immI1_rule,
  /*   11 */  immI_M1_rule,
  /*   12 */  immI2_rule,
  /*   13 */  immI8_rule,
  /*   14 */  immI16_rule,
  /*   15 */  immU31_rule,
  /*   16 */  immI_32_rule,
  /*   17 */  immI_64_rule,
  /*   18 */  immP_rule,
  /*   19 */  immP0_rule,
  /*   20 */  immN_rule,
  /*   21 */  immN0_rule,
  /*   22 */  immP31_rule,
  /*   23 */  immL_rule,
  /*   24 */  immL8_rule,
  /*   25 */  immUL32_rule,
  /*   26 */  immL32_rule,
  /*   27 */  immL0_rule,
  /*   28 */  immL1_rule,
  /*   29 */  immL_M1_rule,
  /*   30 */  immL10_rule,
  /*   31 */  immL_127_rule,
  /*   32 */  immL_32bits_rule,
  /*   33 */  immF0_rule,
  /*   34 */  immF_rule,
  /*   35 */  immD0_rule,
  /*   36 */  immD_rule,
  /*   37 */  immI_16_rule,
  /*   38 */  immI_24_rule,
  /*   39 */  immI_255_rule,
  /*   40 */  immI_65535_rule,
  /*   41 */  immL_255_rule,
  /*   42 */  immL_65535_rule,
  /*   43 */  rRegI_rule,
  /*   44 */  rax_RegI_rule,
  /*   45 */  rbx_RegI_rule,
  /*   46 */  rcx_RegI_rule,
  /*   47 */  rdx_RegI_rule,
  /*   48 */  rdi_RegI_rule,
  /*   49 */  no_rcx_RegI_rule,
  /*   50 */  no_rax_rdx_RegI_rule,
  /*   51 */  any_RegP_rule,
  /*   52 */  rRegP_rule,
  /*   53 */  rRegN_rule,
  /*   54 */  no_rax_RegP_rule,
  /*   55 */  no_rbp_RegP_rule,
  /*   56 */  no_rax_rbx_RegP_rule,
  /*   57 */  rax_RegP_rule,
  /*   58 */  rax_RegN_rule,
  /*   59 */  rbx_RegP_rule,
  /*   60 */  rsi_RegP_rule,
  /*   61 */  rdi_RegP_rule,
  /*   62 */  rbp_RegP_rule,
  /*   63 */  r15_RegP_rule,
  /*   64 */  rRegL_rule,
  /*   65 */  no_rax_rdx_RegL_rule,
  /*   66 */  no_rax_RegL_rule,
  /*   67 */  no_rcx_RegL_rule,
  /*   68 */  rax_RegL_rule,
  /*   69 */  rcx_RegL_rule,
  /*   70 */  rdx_RegL_rule,
  /*   71 */  rFlagsReg_rule,
  /*   72 */  rFlagsRegU_rule,
  /*   73 */  rFlagsRegUCF_rule,
  /*   74 */  regF_rule,
  /*   75 */  regD_rule,
  /*   76 */  indirect_rule,
  /*   77 */  indOffset8_rule,
  /*   78 */  indOffset32_rule,
  /*   79 */  indIndexOffset_rule,
  /*   80 */  indIndex_rule,
  /*   81 */  indIndexScale_rule,
  /*   82 */  indIndexScaleOffset_rule,
  /*   83 */  indPosIndexOffset_rule,
  /*   84 */  indPosIndexScaleOffset_rule,
  /*   85 */  indCompressedOopOffset_rule,
  /*   86 */  indirectNarrow_rule,
  /*   87 */  indOffset8Narrow_rule,
  /*   88 */  indOffset32Narrow_rule,
  /*   89 */  indIndexOffsetNarrow_rule,
  /*   90 */  indIndexNarrow_rule,
  /*   91 */  indIndexScaleNarrow_rule,
  /*   92 */  indIndexScaleOffsetNarrow_rule,
  /*   93 */  indPosIndexOffsetNarrow_rule,
  /*   94 */  indPosIndexScaleOffsetNarrow_rule,
  /*   95 */  stackSlotP_rule,
  /*   96 */  stackSlotI_rule,
  /*   97 */  stackSlotF_rule,
  /*   98 */  stackSlotD_rule,
  /*   99 */  stackSlotL_rule,
  /*  100 */  cmpOp_rule,
  /*  101 */  cmpOpU_rule,
  /*  102 */  cmpOpUCF_rule,
  /*  103 */  cmpOpUCF2_rule,
  /*  104 */  vecS_rule,
  /*  105 */  vecD_rule,
  /*  106 */  vecX_rule,
  /*  107 */  vecY_rule,
  // last operand
  /*  108 */  memory_rule,
  // last operand class
  /*  109 */  _AddP_any_RegP_rRegL_rule,
  /*  110 */  _LShiftL_rRegL_immI2_rule,
  /*  111 */  _AddP_any_RegP__LShiftL_rRegL_immI2_rule,
  /*  112 */  _ConvI2L_rRegI__rule,
  /*  113 */  _AddP_any_RegP__ConvI2L_rRegI__rule,
  /*  114 */  _LShiftL__ConvI2L_rRegI__immI2_rule,
  /*  115 */  _AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2_rule,
  /*  116 */  _DecodeN_rRegN__rule,
  /*  117 */  _AddP__DecodeN_rRegN__rRegL_rule,
  /*  118 */  _AddP__DecodeN_rRegN___LShiftL_rRegL_immI2_rule,
  /*  119 */  _AddP__DecodeN_rRegN___ConvI2L_rRegI__rule,
  /*  120 */  _AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2_rule,
  /*  121 */  _LoadB_memory__rule,
  /*  122 */  _LoadUB_memory__rule,
  /*  123 */  _AndI__LoadUB_memory__immI8_rule,
  /*  124 */  _LoadS_memory__rule,
  /*  125 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  126 */  _LoadUS_memory__rule,
  /*  127 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  128 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  129 */  _AndI__LoadUS_memory__immI16_rule,
  /*  130 */  _LoadI_memory__rule,
  /*  131 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  132 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  133 */  _AndI__LoadI_memory__immI_255_rule,
  /*  134 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  135 */  _AndI__LoadI_memory__immU31_rule,
  /*  136 */  _ConvI2L__LoadI_memory___rule,
  /*  137 */  _LoadL_memory__rule,
  /*  138 */  _CastP2X_rRegP__rule,
  /*  139 */  _CastP2X__DecodeN_rRegN___rule,
  /*  140 */  _AddL__LShiftL_rRegL_immI2_immL32_rule,
  /*  141 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  142 */  _Binary_rRegI_rRegI_rule,
  /*  143 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  144 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  145 */  _Binary_rRegI__LoadI_memory__rule,
  /*  146 */  _Binary_rRegN_rRegN_rule,
  /*  147 */  _Binary_rRegP_rRegP_rule,
  /*  148 */  _Binary_rRegL_rRegL_rule,
  /*  149 */  _Binary_rRegL__LoadL_memory__rule,
  /*  150 */  _Binary_regF_regF_rule,
  /*  151 */  _Binary_regD_regD_rule,
  /*  152 */  _AddI__LoadI_memory__rRegI_rule,
  /*  153 */  _AddI_rRegI__LoadI_memory__rule,
  /*  154 */  _AddI__LoadI_memory__immI_rule,
  /*  155 */  _AddI__LoadI_memory__immI1_rule,
  /*  156 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  157 */  _AddL__LoadL_memory__rRegL_rule,
  /*  158 */  _AddL_rRegL__LoadL_memory__rule,
  /*  159 */  _AddL__LoadL_memory__immL32_rule,
  /*  160 */  _AddL__LoadL_memory__immL1_rule,
  /*  161 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  162 */  _Binary_rax_RegP_rRegP_rule,
  /*  163 */  _Binary_rax_RegI_rRegI_rule,
  /*  164 */  _Binary_rax_RegL_rRegL_rule,
  /*  165 */  _Binary_rax_RegN_rRegN_rule,
  /*  166 */  _SubI__LoadI_memory__rRegI_rule,
  /*  167 */  _SubI__LoadI_memory__immI_rule,
  /*  168 */  _SubL__LoadL_memory__rRegL_rule,
  /*  169 */  _SubL__LoadL_memory__immL32_rule,
  /*  170 */  _SubI_immI0_rRegI_rule,
  /*  171 */  _SubI_immI0__LoadI_memory__rule,
  /*  172 */  _SubL_immL0__LoadL_memory__rule,
  /*  173 */  _LShiftI__LoadI_memory__immI1_rule,
  /*  174 */  _LShiftI__LoadI_memory__immI8_rule,
  /*  175 */  _LShiftI__LoadI_memory__rcx_RegI_rule,
  /*  176 */  _RShiftI__LoadI_memory__immI1_rule,
  /*  177 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  178 */  _RShiftI__LoadI_memory__rcx_RegI_rule,
  /*  179 */  _URShiftI__LoadI_memory__immI1_rule,
  /*  180 */  _URShiftI__LoadI_memory__immI8_rule,
  /*  181 */  _URShiftI__LoadI_memory__rcx_RegI_rule,
  /*  182 */  _LShiftL__LoadL_memory__immI1_rule,
  /*  183 */  _LShiftL__LoadL_memory__immI8_rule,
  /*  184 */  _LShiftL__LoadL_memory__rcx_RegI_rule,
  /*  185 */  _RShiftL__LoadL_memory__immI1_rule,
  /*  186 */  _RShiftL__LoadL_memory__immI8_rule,
  /*  187 */  _RShiftL__LoadL_memory__rcx_RegI_rule,
  /*  188 */  _URShiftL__LoadL_memory__immI1_rule,
  /*  189 */  _URShiftL__LoadL_memory__immI8_rule,
  /*  190 */  _URShiftL__LoadL_memory__rcx_RegI_rule,
  /*  191 */  _LShiftI_rRegI_immI_24_rule,
  /*  192 */  _LShiftI_rRegI_immI_16_rule,
  /*  193 */  _LShiftI_rRegI_immI1_rule,
  /*  194 */  _URShiftI_rRegI_immI_M1_rule,
  /*  195 */  _LShiftI_rRegI_immI8_rule,
  /*  196 */  _URShiftI_rRegI_immI8_rule,
  /*  197 */  _LShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  198 */  _SubI_immI0_rcx_RegI_rule,
  /*  199 */  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  200 */  _SubI_immI_32_rcx_RegI_rule,
  /*  201 */  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  202 */  _URShiftI_rRegI_immI1_rule,
  /*  203 */  _LShiftI_rRegI_immI_M1_rule,
  /*  204 */  _URShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  205 */  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  206 */  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  207 */  _LShiftL_rRegL_immI1_rule,
  /*  208 */  _URShiftL_rRegL_immI_M1_rule,
  /*  209 */  _LShiftL_rRegL_immI8_rule,
  /*  210 */  _URShiftL_rRegL_immI8_rule,
  /*  211 */  _LShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  212 */  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  213 */  _SubI_immI_64_rcx_RegI_rule,
  /*  214 */  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  215 */  _URShiftL_rRegL_immI1_rule,
  /*  216 */  _LShiftL_rRegL_immI_M1_rule,
  /*  217 */  _URShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  218 */  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  219 */  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  220 */  _AndI_rRegI_immI_255_rule,
  /*  221 */  _AndI_rRegI_immI_65535_rule,
  /*  222 */  _AndI__LoadI_memory__rRegI_rule,
  /*  223 */  _AndI_rRegI__LoadI_memory__rule,
  /*  224 */  _AndI__LoadI_memory__immI_rule,
  /*  225 */  _OrI__LoadI_memory__rRegI_rule,
  /*  226 */  _OrI_rRegI__LoadI_memory__rule,
  /*  227 */  _OrI__LoadI_memory__immI_rule,
  /*  228 */  _XorI__LoadI_memory__rRegI_rule,
  /*  229 */  _XorI_rRegI__LoadI_memory__rule,
  /*  230 */  _XorI__LoadI_memory__immI_rule,
  /*  231 */  _AndL__LoadL_memory__rRegL_rule,
  /*  232 */  _AndL_rRegL__LoadL_memory__rule,
  /*  233 */  _AndL__LoadL_memory__immL32_rule,
  /*  234 */  _CastP2X_any_RegP__rule,
  /*  235 */  _OrL__LoadL_memory__rRegL_rule,
  /*  236 */  _OrL_rRegL__LoadL_memory__rule,
  /*  237 */  _OrL__LoadL_memory__immL32_rule,
  /*  238 */  _XorL__LoadL_memory__rRegL_rule,
  /*  239 */  _XorL_rRegL__LoadL_memory__rule,
  /*  240 */  _XorL__LoadL_memory__immL32_rule,
  /*  241 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  242 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  243 */  _SubI_rRegI_rRegI_rule,
  /*  244 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  245 */  _LoadF_memory__rule,
  /*  246 */  _LoadD_memory__rule,
  /*  247 */  _Binary_rdi_RegP_rcx_RegI_rule,
  /*  248 */  _Binary_rsi_RegP_rdx_RegI_rule,
  /*  249 */  _Binary_rdi_RegP_rdx_RegI_rule,
  /*  250 */  _Binary_rsi_RegP_immI_rule,
  /*  251 */  _Binary_rsi_RegP_rax_RegI_rule,
  /*  252 */  _Binary_rdi_RegP_rsi_RegP_rule,
  /*  253 */  _AndI_rRegI_immI_rule,
  /*  254 */  _LoadP_memory__rule,
  /*  255 */  _LoadN_memory__rule,
  /*  256 */  _AndL_rRegL_immL32_rule,
  /*  257 */  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule,
  /*  258 */  _ConvF2D_regF__rule,
  /*  259 */  _SqrtD__ConvF2D_regF___rule,
  /*  260 */  _ConvF2D__LoadF_memory___rule,
  /*  261 */  _SqrtD__ConvF2D__LoadF_memory____rule,
  /*  262 */  _ConvF2D_immF__rule,
  /*  263 */  _SqrtD__ConvF2D_immF___rule,
  /*  264 */  _LoadVector_memory__rule,
  // last internally defined operand
  /*  265 */  stackSlotI_rule,
  /*  266 */  stackSlotL_rule,
  /*  267 */  stackSlotP_rule,
  /*  268 */  stackSlotF_rule,
  /*  269 */  stackSlotD_rule,
  /*  270 */  rRegP_rule,
  /*  271 */  rRegP_rule,
  /*  272 */  rRegP_rule,
  /*  273 */  rRegP_rule,
  /*  274 */  rRegP_rule,
  /*  275 */  rRegP_rule,
  /*  276 */  rRegP_rule,
  /*  277 */  rRegP_rule,
  /*  278 */  rRegP_rule,
  /*  279 */  rRegP_rule,
  /*  280 */  rRegP_rule,
  /*  281 */  rRegP_rule,
  /*  282 */  rRegP_rule,
  /*  283 */  rRegP_rule,
  /*  284 */  rRegP_rule,
  /*  285 */  rRegI_rule,
  /*  286 */  rRegI_rule,
  /*  287 */  rRegL_rule,
  /*  288 */  rRegL_rule,
  /*  289 */  rRegL_rule,
  /*  290 */  rRegL_rule,
  /*  291 */  rRegP_rule,
  /*  292 */  rRegP_rule,
  /*  293 */  rRegP_rule,
  /*  294 */  regF_rule,
  /*  295 */  rRegN_rule,
  /*  296 */  rRegN_rule,
  /*  297 */  regF_rule,
  /*  298 */  regD_rule,
  /*  299 */  regD_rule,
  /*  300 */  rRegI_rule,
  /*  301 */  rRegL_rule,
  /*  302 */  rRegP_rule,
  /*  303 */  regF_rule,
  /*  304 */  regD_rule,
  /*  305 */  Universe_rule,
  /*  306 */  rRegP_rule,
  /*  307 */  rRegL_rule,
  /*  308 */  rRegP_rule,
  /*  309 */  rRegP_rule,
  /*  310 */  rRegI_rule,
  /*  311 */  rFlagsReg_rule,
  /*  312 */  rFlagsReg_rule,
  /*  313 */  rFlagsReg_rule,
  /*  314 */  rFlagsRegU_rule,
  /*  315 */  rFlagsRegUCF_rule,
  /*  316 */  rFlagsRegU_rule,
  /*  317 */  rFlagsRegUCF_rule,
  /*  318 */  rFlagsRegU_rule,
  /*  319 */  rFlagsRegUCF_rule,
  /*  320 */  rFlagsRegU_rule,
  /*  321 */  rFlagsRegUCF_rule,
  /*  322 */  rFlagsRegU_rule,
  /*  323 */  rFlagsRegUCF_rule,
  /*  324 */  rFlagsRegU_rule,
  /*  325 */  rFlagsRegUCF_rule,
  /*  326 */  regF_rule,
  /*  327 */  regD_rule,
  /*  328 */  rRegI_rule,
  /*  329 */  regF_rule,
  /*  330 */  rRegL_rule,
  /*  331 */  regD_rule,
  /*  332 */  regD_rule,
  /*  333 */  rRegI_rule,
  /*  334 */  rRegL_rule,
  /*  335 */  regF_rule,
  /*  336 */  regD_rule,
  /*  337 */  rFlagsReg_rule,
  /*  338 */  rFlagsReg_rule,
  /*  339 */  rFlagsReg_rule,
  /*  340 */  rFlagsReg_rule,
  /*  341 */  rFlagsReg_rule,
  /*  342 */  rFlagsReg_rule,
  /*  343 */  rFlagsReg_rule,
  /*  344 */  rFlagsRegU_rule,
  /*  345 */  rFlagsRegU_rule,
  /*  346 */  rFlagsRegU_rule,
  /*  347 */  rFlagsRegU_rule,
  /*  348 */  rFlagsRegU_rule,
  /*  349 */  rFlagsRegU_rule,
  /*  350 */  rFlagsRegU_rule,
  /*  351 */  rFlagsReg_rule,
  /*  352 */  rFlagsReg_rule,
  /*  353 */  rFlagsReg_rule,
  /*  354 */  rFlagsRegU_rule,
  /*  355 */  rFlagsRegU_rule,
  /*  356 */  rFlagsRegU_rule,
  /*  357 */  rFlagsRegU_rule,
  /*  358 */  rFlagsReg_rule,
  /*  359 */  rFlagsReg_rule,
  /*  360 */  rFlagsReg_rule,
  /*  361 */  rFlagsReg_rule,
  /*  362 */  rFlagsReg_rule,
  /*  363 */  rFlagsReg_rule,
  /*  364 */  rFlagsReg_rule,
  /*  365 */  rFlagsReg_rule,
  /*  366 */  rFlagsReg_rule,
  /*  367 */  rFlagsReg_rule,
  /*  368 */  rFlagsRegU_rule,
  /*  369 */  rFlagsRegU_rule,
  /*  370 */  rFlagsRegU_rule,
  /*  371 */  rFlagsRegU_rule,
  /*  372 */  rFlagsReg_rule,
  /*  373 */  rFlagsReg_rule,
  /*  374 */  rFlagsReg_rule,
  /*  375 */  Universe_rule,
  /*  376 */  r15_RegP_rule,
  /*  377 */  regF_rule,
  /*  378 */  regF_rule,
  /*  379 */  regD_rule,
  /*  380 */  regD_rule,
  /*  381 */  regF_rule,
  /*  382 */  regF_rule,
  /*  383 */  regD_rule,
  /*  384 */  regD_rule,
  /*  385 */  vecS_rule,
  /*  386 */  vecS_rule,
  /*  387 */  rRegI_rule,
  /*  388 */  rRegL_rule,
  /*  389 */  rRegI_rule,
  /*  390 */  rRegL_rule,
  /*  391 */  rRegL_rule,
  /*  392 */  rRegI_rule,
  /*  393 */  rRegI_rule,
  /*  394 */  rRegL_rule,
  /*  395 */  rRegI_rule,
  /*  396 */  rRegI_rule,
  /*  397 */  rRegL_rule,
  /*  398 */  rRegL_rule,
  /*  399 */  rRegL_rule,
  /*  400 */  rRegI_rule,
  /*  401 */  rRegI_rule,
  /*  402 */  rRegI_rule,
  /*  403 */  rRegI_rule,
  /*  404 */  rRegI_rule,
  /*  405 */  rRegL_rule,
  /*  406 */  rRegL_rule,
  /*  407 */  rRegL_rule,
  /*  408 */  rRegL_rule,
  /*  409 */  rRegL_rule,
  /*  410 */  rRegL_rule,
  /*  411 */  rRegI_rule,
  /*  412 */  rRegP_rule,
  /*  413 */  rRegN_rule,
  /*  414 */  rRegP_rule,
  /*  415 */  rRegN_rule,
  /*  416 */  regF_rule,
  /*  417 */  regD_rule,
  /*  418 */  regD_rule,
  /*  419 */  Universe_rule,
  /*  420 */  Universe_rule,
  /*  421 */  Universe_rule,
  /*  422 */  Universe_rule,
  /*  423 */  Universe_rule,
  /*  424 */  Universe_rule,
  /*  425 */  Universe_rule,
  /*  426 */  Universe_rule,
  /*  427 */  Universe_rule,
  /*  428 */  Universe_rule,
  /*  429 */  Universe_rule,
  /*  430 */  Universe_rule,
  /*  431 */  Universe_rule,
  /*  432 */  Universe_rule,
  /*  433 */  Universe_rule,
  /*  434 */  Universe_rule,
  /*  435 */  Universe_rule,
  /*  436 */  Universe_rule,
  /*  437 */  Universe_rule,
  /*  438 */  Universe_rule,
  /*  439 */  Universe_rule,
  /*  440 */  Universe_rule,
  /*  441 */  Universe_rule,
  /*  442 */  Universe_rule,
  /*  443 */  Universe_rule,
  /*  444 */  Universe_rule,
  /*  445 */  Universe_rule,
  /*  446 */  Universe_rule,
  /*  447 */  Universe_rule,
  /*  448 */  Universe_rule,
  /*  449 */  Universe_rule,
  /*  450 */  Universe_rule,
  /*  451 */  Universe_rule,
  /*  452 */  Universe_rule,
  /*  453 */  Universe_rule,
  /*  454 */  rRegI_rule,
  /*  455 */  rRegL_rule,
  /*  456 */  rRegI_rule,
  /*  457 */  rRegI_rule,
  /*  458 */  rRegI_rule,
  /*  459 */  rRegI_rule,
  /*  460 */  rRegI_rule,
  /*  461 */  rRegI_rule,
  /*  462 */  rRegI_rule,
  /*  463 */  rRegI_rule,
  /*  464 */  rRegI_rule,
  /*  465 */  rRegI_rule,
  /*  466 */  rRegI_rule,
  /*  467 */  rRegI_rule,
  /*  468 */  Universe_rule,
  /*  469 */  Universe_rule,
  /*  470 */  Universe_rule,
  /*  471 */  Universe_rule,
  /*  472 */  Universe_rule,
  /*  473 */  Universe_rule,
  /*  474 */  rRegI_rule,
  /*  475 */  rRegI_rule,
  /*  476 */  rRegN_rule,
  /*  477 */  rRegN_rule,
  /*  478 */  rRegP_rule,
  /*  479 */  rRegP_rule,
  /*  480 */  Universe_rule,
  /*  481 */  Universe_rule,
  /*  482 */  Universe_rule,
  /*  483 */  rRegI_rule,
  /*  484 */  rRegI_rule,
  /*  485 */  rRegI_rule,
  /*  486 */  rRegI_rule,
  /*  487 */  rRegI_rule,
  /*  488 */  rRegI_rule,
  /*  489 */  rRegN_rule,
  /*  490 */  rRegN_rule,
  /*  491 */  rRegN_rule,
  /*  492 */  rRegP_rule,
  /*  493 */  rRegP_rule,
  /*  494 */  rRegP_rule,
  /*  495 */  rRegL_rule,
  /*  496 */  rRegL_rule,
  /*  497 */  rRegL_rule,
  /*  498 */  rRegL_rule,
  /*  499 */  rRegL_rule,
  /*  500 */  rRegL_rule,
  /*  501 */  regF_rule,
  /*  502 */  regF_rule,
  /*  503 */  regF_rule,
  /*  504 */  regD_rule,
  /*  505 */  regD_rule,
  /*  506 */  regD_rule,
  /*  507 */  rRegI_rule,
  /*  508 */  rRegI_rule,
  /*  509 */  rRegI_rule,
  /*  510 */  rRegI_rule,
  /*  511 */  Universe_rule,
  /*  512 */  Universe_rule,
  /*  513 */  Universe_rule,
  /*  514 */  rRegI_rule,
  /*  515 */  Universe_rule,
  /*  516 */  rRegI_rule,
  /*  517 */  Universe_rule,
  /*  518 */  rRegI_rule,
  /*  519 */  rRegL_rule,
  /*  520 */  rRegL_rule,
  /*  521 */  rRegL_rule,
  /*  522 */  rRegL_rule,
  /*  523 */  Universe_rule,
  /*  524 */  Universe_rule,
  /*  525 */  Universe_rule,
  /*  526 */  rRegI_rule,
  /*  527 */  Universe_rule,
  /*  528 */  rRegL_rule,
  /*  529 */  Universe_rule,
  /*  530 */  rRegL_rule,
  /*  531 */  rRegP_rule,
  /*  532 */  rRegP_rule,
  /*  533 */  rRegP_rule,
  /*  534 */  rRegP_rule,
  /*  535 */  rRegI_rule,
  /*  536 */  rRegI_rule,
  /*  537 */  rRegI_rule,
  /*  538 */  Universe_rule,
  /*  539 */  rRegI_rule,
  /*  540 */  Universe_rule,
  /*  541 */  rRegL_rule,
  /*  542 */  rRegI_rule,
  /*  543 */  rRegL_rule,
  /*  544 */  rRegP_rule,
  /*  545 */  rRegN_rule,
  /*  546 */  rRegI_rule,
  /*  547 */  rRegI_rule,
  /*  548 */  rRegI_rule,
  /*  549 */  rRegI_rule,
  /*  550 */  Universe_rule,
  /*  551 */  Universe_rule,
  /*  552 */  rRegL_rule,
  /*  553 */  rRegI_rule,
  /*  554 */  rRegL_rule,
  /*  555 */  Universe_rule,
  /*  556 */  Universe_rule,
  /*  557 */  rRegP_rule,
  /*  558 */  rRegI_rule,
  /*  559 */  Universe_rule,
  /*  560 */  rRegL_rule,
  /*  561 */  Universe_rule,
  /*  562 */  rRegI_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  rRegI_rule,
  /*  565 */  rRegI_rule,
  /*  566 */  rRegI_rule,
  /*  567 */  rRegL_rule,
  /*  568 */  rRegL_rule,
  /*  569 */  rRegL_rule,
  /*  570 */  rRegL_rule,
  /*  571 */  rRegL_rule,
  /*  572 */  rdx_RegL_rule,
  /*  573 */  rax_RegI_rule,
  /*  574 */  rax_RegL_rule,
  /*  575 */  Universe_rule,
  /*  576 */  Universe_rule,
  /*  577 */  0,
  /*  578 */  0,
  /*  579 */  0,
  /*  580 */  0,
  /*  581 */  rdx_RegL_rule,
  /*  582 */  rdx_RegI_rule,
  /*  583 */  rdx_RegL_rule,
  /*  584 */  rRegI_rule,
  /*  585 */  Universe_rule,
  /*  586 */  rRegI_rule,
  /*  587 */  Universe_rule,
  /*  588 */  rRegI_rule,
  /*  589 */  Universe_rule,
  /*  590 */  rRegI_rule,
  /*  591 */  Universe_rule,
  /*  592 */  rRegI_rule,
  /*  593 */  Universe_rule,
  /*  594 */  rRegI_rule,
  /*  595 */  Universe_rule,
  /*  596 */  rRegI_rule,
  /*  597 */  Universe_rule,
  /*  598 */  rRegI_rule,
  /*  599 */  Universe_rule,
  /*  600 */  rRegI_rule,
  /*  601 */  Universe_rule,
  /*  602 */  rRegL_rule,
  /*  603 */  Universe_rule,
  /*  604 */  rRegL_rule,
  /*  605 */  Universe_rule,
  /*  606 */  rRegL_rule,
  /*  607 */  Universe_rule,
  /*  608 */  rRegL_rule,
  /*  609 */  Universe_rule,
  /*  610 */  rRegL_rule,
  /*  611 */  Universe_rule,
  /*  612 */  rRegL_rule,
  /*  613 */  Universe_rule,
  /*  614 */  rRegL_rule,
  /*  615 */  Universe_rule,
  /*  616 */  rRegL_rule,
  /*  617 */  Universe_rule,
  /*  618 */  rRegL_rule,
  /*  619 */  Universe_rule,
  /*  620 */  rRegI_rule,
  /*  621 */  rRegI_rule,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  rRegI_rule,
  /*  626 */  rRegI_rule,
  /*  627 */  rRegI_rule,
  /*  628 */  rRegI_rule,
  /*  629 */  no_rcx_RegI_rule,
  /*  630 */  no_rcx_RegI_rule,
  /*  631 */  no_rcx_RegI_rule,
  /*  632 */  no_rcx_RegI_rule,
  /*  633 */  0,
  /*  634 */  0,
  /*  635 */  0,
  /*  636 */  rRegI_rule,
  /*  637 */  rRegI_rule,
  /*  638 */  rRegI_rule,
  /*  639 */  rRegI_rule,
  /*  640 */  no_rcx_RegI_rule,
  /*  641 */  no_rcx_RegI_rule,
  /*  642 */  no_rcx_RegI_rule,
  /*  643 */  no_rcx_RegI_rule,
  /*  644 */  0,
  /*  645 */  0,
  /*  646 */  0,
  /*  647 */  rRegL_rule,
  /*  648 */  rRegL_rule,
  /*  649 */  rRegL_rule,
  /*  650 */  rRegL_rule,
  /*  651 */  no_rcx_RegL_rule,
  /*  652 */  no_rcx_RegL_rule,
  /*  653 */  no_rcx_RegL_rule,
  /*  654 */  no_rcx_RegL_rule,
  /*  655 */  0,
  /*  656 */  0,
  /*  657 */  0,
  /*  658 */  rRegL_rule,
  /*  659 */  rRegL_rule,
  /*  660 */  rRegL_rule,
  /*  661 */  rRegL_rule,
  /*  662 */  no_rcx_RegL_rule,
  /*  663 */  no_rcx_RegL_rule,
  /*  664 */  no_rcx_RegL_rule,
  /*  665 */  no_rcx_RegL_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  rRegI_rule,
  /*  668 */  rRegL_rule,
  /*  669 */  rRegI_rule,
  /*  670 */  rRegL_rule,
  /*  671 */  rRegI_rule,
  /*  672 */  rRegI_rule,
  /*  673 */  rRegI_rule,
  /*  674 */  Universe_rule,
  /*  675 */  Universe_rule,
  /*  676 */  Universe_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  rRegI_rule,
  /*  679 */  rRegI_rule,
  /*  680 */  rRegI_rule,
  /*  681 */  Universe_rule,
  /*  682 */  Universe_rule,
  /*  683 */  Universe_rule,
  /*  684 */  rRegI_rule,
  /*  685 */  rRegI_rule,
  /*  686 */  rRegI_rule,
  /*  687 */  rRegI_rule,
  /*  688 */  rRegI_rule,
  /*  689 */  Universe_rule,
  /*  690 */  Universe_rule,
  /*  691 */  Universe_rule,
  /*  692 */  rRegL_rule,
  /*  693 */  rRegL_rule,
  /*  694 */  rRegL_rule,
  /*  695 */  rRegL_rule,
  /*  696 */  rRegL_rule,
  /*  697 */  rRegL_rule,
  /*  698 */  Universe_rule,
  /*  699 */  Universe_rule,
  /*  700 */  Universe_rule,
  /*  701 */  rRegL_rule,
  /*  702 */  rRegL_rule,
  /*  703 */  rRegL_rule,
  /*  704 */  rRegL_rule,
  /*  705 */  rRegL_rule,
  /*  706 */  rRegL_rule,
  /*  707 */  Universe_rule,
  /*  708 */  Universe_rule,
  /*  709 */  Universe_rule,
  /*  710 */  rRegL_rule,
  /*  711 */  rRegL_rule,
  /*  712 */  rRegL_rule,
  /*  713 */  rRegL_rule,
  /*  714 */  rRegL_rule,
  /*  715 */  Universe_rule,
  /*  716 */  Universe_rule,
  /*  717 */  Universe_rule,
  /*  718 */  rRegI_rule,
  /*  719 */  rRegI_rule,
  /*  720 */  rRegI_rule,
  /*  721 */  rRegI_rule,
  /*  722 */  rRegI_rule,
  /*  723 */  rRegI_rule,
  /*  724 */  rRegI_rule,
  /*  725 */  rRegI_rule,
  /*  726 */  rRegI_rule,
  /*  727 */  rRegI_rule,
  /*  728 */  rRegI_rule,
  /*  729 */  rRegI_rule,
  /*  730 */  rRegI_rule,
  /*  731 */  rRegI_rule,
  /*  732 */  rRegI_rule,
  /*  733 */  rRegI_rule,
  /*  734 */  regD_rule,
  /*  735 */  regD_rule,
  /*  736 */  regD_rule,
  /*  737 */  regD_rule,
  /*  738 */  regD_rule,
  /*  739 */  regD_rule,
  /*  740 */  regD_rule,
  /*  741 */  regD_rule,
  /*  742 */  regD_rule,
  /*  743 */  regF_rule,
  /*  744 */  regF_rule,
  /*  745 */  rRegI_rule,
  /*  746 */  rRegL_rule,
  /*  747 */  rRegI_rule,
  /*  748 */  rRegL_rule,
  /*  749 */  regF_rule,
  /*  750 */  regF_rule,
  /*  751 */  regD_rule,
  /*  752 */  regD_rule,
  /*  753 */  regF_rule,
  /*  754 */  regD_rule,
  /*  755 */  regF_rule,
  /*  756 */  regF_rule,
  /*  757 */  regD_rule,
  /*  758 */  regD_rule,
  /*  759 */  rRegL_rule,
  /*  760 */  rRegL_rule,
  /*  761 */  rRegL_rule,
  /*  762 */  rRegL_rule,
  /*  763 */  rRegI_rule,
  /*  764 */  stackSlotI_rule,
  /*  765 */  stackSlotF_rule,
  /*  766 */  stackSlotL_rule,
  /*  767 */  stackSlotD_rule,
  /*  768 */  Universe_rule,
  /*  769 */  Universe_rule,
  /*  770 */  rax_RegI_rule,
  /*  771 */  rbx_RegI_rule,
  /*  772 */  rbx_RegI_rule,
  /*  773 */  rax_RegI_rule,
  /*  774 */  rax_RegI_rule,
  /*  775 */  rRegI_rule,
  /*  776 */  0,
  /*  777 */  rRegI_rule,
  /*  778 */  0,
  /*  779 */  rRegI_rule,
  /*  780 */  Universe_rule,
  /*  781 */  Universe_rule,
  /*  782 */  Universe_rule,
  /*  783 */  Universe_rule,
  /*  784 */  Universe_rule,
  /*  785 */  Universe_rule,
  /*  786 */  Universe_rule,
  /*  787 */  Universe_rule,
  /*  788 */  rdi_RegP_rule,
  /*  789 */  Universe_rule,
  /*  790 */  Universe_rule,
  /*  791 */  Universe_rule,
  /*  792 */  Universe_rule,
  /*  793 */  Universe_rule,
  /*  794 */  Universe_rule,
  /*  795 */  Universe_rule,
  /*  796 */  Universe_rule,
  /*  797 */  Universe_rule,
  /*  798 */  Universe_rule,
  /*  799 */  Universe_rule,
  /*  800 */  Universe_rule,
  /*  801 */  Universe_rule,
  /*  802 */  Universe_rule,
  /*  803 */  Universe_rule,
  /*  804 */  Universe_rule,
  /*  805 */  Universe_rule,
  /*  806 */  Universe_rule,
  /*  807 */  rax_RegP_rule,
  /*  808 */  Universe_rule,
  /*  809 */  Universe_rule,
  /*  810 */  regF_rule,
  /*  811 */  regF_rule,
  /*  812 */  regF_rule,
  /*  813 */  regF_rule,
  /*  814 */  regF_rule,
  /*  815 */  regF_rule,
  /*  816 */  regF_rule,
  /*  817 */  regF_rule,
  /*  818 */  regD_rule,
  /*  819 */  regD_rule,
  /*  820 */  regD_rule,
  /*  821 */  regD_rule,
  /*  822 */  regD_rule,
  /*  823 */  regD_rule,
  /*  824 */  regD_rule,
  /*  825 */  regD_rule,
  /*  826 */  regF_rule,
  /*  827 */  regF_rule,
  /*  828 */  regF_rule,
  /*  829 */  regF_rule,
  /*  830 */  regF_rule,
  /*  831 */  regF_rule,
  /*  832 */  regD_rule,
  /*  833 */  regD_rule,
  /*  834 */  regD_rule,
  /*  835 */  regD_rule,
  /*  836 */  regD_rule,
  /*  837 */  regD_rule,
  /*  838 */  regF_rule,
  /*  839 */  regF_rule,
  /*  840 */  regF_rule,
  /*  841 */  regF_rule,
  /*  842 */  regF_rule,
  /*  843 */  regF_rule,
  /*  844 */  regF_rule,
  /*  845 */  regF_rule,
  /*  846 */  regD_rule,
  /*  847 */  regD_rule,
  /*  848 */  regD_rule,
  /*  849 */  regD_rule,
  /*  850 */  regD_rule,
  /*  851 */  regD_rule,
  /*  852 */  regD_rule,
  /*  853 */  regD_rule,
  /*  854 */  regF_rule,
  /*  855 */  regF_rule,
  /*  856 */  regF_rule,
  /*  857 */  regF_rule,
  /*  858 */  regF_rule,
  /*  859 */  regF_rule,
  /*  860 */  regD_rule,
  /*  861 */  regD_rule,
  /*  862 */  regD_rule,
  /*  863 */  regD_rule,
  /*  864 */  regD_rule,
  /*  865 */  regD_rule,
  /*  866 */  regF_rule,
  /*  867 */  regF_rule,
  /*  868 */  regF_rule,
  /*  869 */  regD_rule,
  /*  870 */  regD_rule,
  /*  871 */  regD_rule,
  /*  872 */  vecS_rule,
  /*  873 */  vecD_rule,
  /*  874 */  vecX_rule,
  /*  875 */  vecY_rule,
  /*  876 */  Universe_rule,
  /*  877 */  Universe_rule,
  /*  878 */  Universe_rule,
  /*  879 */  Universe_rule,
  /*  880 */  vecS_rule,
  /*  881 */  vecD_rule,
  /*  882 */  vecX_rule,
  /*  883 */  vecY_rule,
  /*  884 */  vecS_rule,
  /*  885 */  vecD_rule,
  /*  886 */  vecX_rule,
  /*  887 */  vecY_rule,
  /*  888 */  vecS_rule,
  /*  889 */  vecD_rule,
  /*  890 */  vecX_rule,
  /*  891 */  vecY_rule,
  /*  892 */  vecS_rule,
  /*  893 */  vecD_rule,
  /*  894 */  vecX_rule,
  /*  895 */  vecY_rule,
  /*  896 */  vecS_rule,
  /*  897 */  vecD_rule,
  /*  898 */  vecX_rule,
  /*  899 */  vecY_rule,
  /*  900 */  vecS_rule,
  /*  901 */  vecD_rule,
  /*  902 */  vecX_rule,
  /*  903 */  vecY_rule,
  /*  904 */  vecD_rule,
  /*  905 */  vecX_rule,
  /*  906 */  vecY_rule,
  /*  907 */  vecD_rule,
  /*  908 */  vecX_rule,
  /*  909 */  vecY_rule,
  /*  910 */  vecD_rule,
  /*  911 */  vecX_rule,
  /*  912 */  vecY_rule,
  /*  913 */  vecD_rule,
  /*  914 */  vecX_rule,
  /*  915 */  vecY_rule,
  /*  916 */  vecX_rule,
  /*  917 */  vecY_rule,
  /*  918 */  vecX_rule,
  /*  919 */  vecY_rule,
  /*  920 */  vecX_rule,
  /*  921 */  vecY_rule,
  /*  922 */  vecX_rule,
  /*  923 */  vecY_rule,
  /*  924 */  vecD_rule,
  /*  925 */  vecX_rule,
  /*  926 */  vecY_rule,
  /*  927 */  vecD_rule,
  /*  928 */  vecX_rule,
  /*  929 */  vecY_rule,
  /*  930 */  vecX_rule,
  /*  931 */  vecY_rule,
  /*  932 */  vecX_rule,
  /*  933 */  vecY_rule,
  /*  934 */  vecS_rule,
  /*  935 */  vecS_rule,
  /*  936 */  vecD_rule,
  /*  937 */  vecD_rule,
  /*  938 */  vecX_rule,
  /*  939 */  vecX_rule,
  /*  940 */  vecX_rule,
  /*  941 */  vecY_rule,
  /*  942 */  vecY_rule,
  /*  943 */  vecS_rule,
  /*  944 */  vecS_rule,
  /*  945 */  vecD_rule,
  /*  946 */  vecD_rule,
  /*  947 */  vecX_rule,
  /*  948 */  vecX_rule,
  /*  949 */  vecX_rule,
  /*  950 */  vecY_rule,
  /*  951 */  vecY_rule,
  /*  952 */  vecD_rule,
  /*  953 */  vecD_rule,
  /*  954 */  vecX_rule,
  /*  955 */  vecX_rule,
  /*  956 */  vecX_rule,
  /*  957 */  vecY_rule,
  /*  958 */  vecY_rule,
  /*  959 */  vecX_rule,
  /*  960 */  vecX_rule,
  /*  961 */  vecX_rule,
  /*  962 */  vecY_rule,
  /*  963 */  vecY_rule,
  /*  964 */  vecD_rule,
  /*  965 */  vecD_rule,
  /*  966 */  vecX_rule,
  /*  967 */  vecX_rule,
  /*  968 */  vecX_rule,
  /*  969 */  vecY_rule,
  /*  970 */  vecY_rule,
  /*  971 */  vecX_rule,
  /*  972 */  vecX_rule,
  /*  973 */  vecX_rule,
  /*  974 */  vecY_rule,
  /*  975 */  vecY_rule,
  /*  976 */  vecS_rule,
  /*  977 */  vecS_rule,
  /*  978 */  vecD_rule,
  /*  979 */  vecD_rule,
  /*  980 */  vecX_rule,
  /*  981 */  vecX_rule,
  /*  982 */  vecX_rule,
  /*  983 */  vecY_rule,
  /*  984 */  vecY_rule,
  /*  985 */  vecS_rule,
  /*  986 */  vecS_rule,
  /*  987 */  vecD_rule,
  /*  988 */  vecD_rule,
  /*  989 */  vecX_rule,
  /*  990 */  vecX_rule,
  /*  991 */  vecX_rule,
  /*  992 */  vecY_rule,
  /*  993 */  vecY_rule,
  /*  994 */  vecD_rule,
  /*  995 */  vecD_rule,
  /*  996 */  vecX_rule,
  /*  997 */  vecX_rule,
  /*  998 */  vecX_rule,
  /*  999 */  vecY_rule,
  /* 1000 */  vecY_rule,
  /* 1001 */  vecX_rule,
  /* 1002 */  vecX_rule,
  /* 1003 */  vecX_rule,
  /* 1004 */  vecY_rule,
  /* 1005 */  vecY_rule,
  /* 1006 */  vecD_rule,
  /* 1007 */  vecD_rule,
  /* 1008 */  vecX_rule,
  /* 1009 */  vecX_rule,
  /* 1010 */  vecX_rule,
  /* 1011 */  vecY_rule,
  /* 1012 */  vecY_rule,
  /* 1013 */  vecX_rule,
  /* 1014 */  vecX_rule,
  /* 1015 */  vecX_rule,
  /* 1016 */  vecY_rule,
  /* 1017 */  vecY_rule,
  /* 1018 */  vecS_rule,
  /* 1019 */  vecS_rule,
  /* 1020 */  vecD_rule,
  /* 1021 */  vecD_rule,
  /* 1022 */  vecX_rule,
  /* 1023 */  vecX_rule,
  /* 1024 */  vecX_rule,
  /* 1025 */  vecY_rule,
  /* 1026 */  vecY_rule,
  /* 1027 */  vecD_rule,
  /* 1028 */  vecD_rule,
  /* 1029 */  vecX_rule,
  /* 1030 */  vecX_rule,
  /* 1031 */  vecX_rule,
  /* 1032 */  vecY_rule,
  /* 1033 */  vecY_rule,
  /* 1034 */  vecD_rule,
  /* 1035 */  vecD_rule,
  /* 1036 */  vecX_rule,
  /* 1037 */  vecX_rule,
  /* 1038 */  vecX_rule,
  /* 1039 */  vecY_rule,
  /* 1040 */  vecY_rule,
  /* 1041 */  vecX_rule,
  /* 1042 */  vecX_rule,
  /* 1043 */  vecX_rule,
  /* 1044 */  vecY_rule,
  /* 1045 */  vecY_rule,
  /* 1046 */  vecD_rule,
  /* 1047 */  vecD_rule,
  /* 1048 */  vecX_rule,
  /* 1049 */  vecX_rule,
  /* 1050 */  vecX_rule,
  /* 1051 */  vecY_rule,
  /* 1052 */  vecY_rule,
  /* 1053 */  vecX_rule,
  /* 1054 */  vecX_rule,
  /* 1055 */  vecX_rule,
  /* 1056 */  vecY_rule,
  /* 1057 */  vecY_rule,
  /* 1058 */  vecS_rule,
  /* 1059 */  vecS_rule,
  /* 1060 */  vecS_rule,
  /* 1061 */  vecS_rule,
  /* 1062 */  vecD_rule,
  /* 1063 */  vecD_rule,
  /* 1064 */  vecD_rule,
  /* 1065 */  vecD_rule,
  /* 1066 */  vecX_rule,
  /* 1067 */  vecX_rule,
  /* 1068 */  vecX_rule,
  /* 1069 */  vecX_rule,
  /* 1070 */  vecY_rule,
  /* 1071 */  vecY_rule,
  /* 1072 */  vecD_rule,
  /* 1073 */  vecD_rule,
  /* 1074 */  vecD_rule,
  /* 1075 */  vecD_rule,
  /* 1076 */  vecX_rule,
  /* 1077 */  vecX_rule,
  /* 1078 */  vecX_rule,
  /* 1079 */  vecX_rule,
  /* 1080 */  vecY_rule,
  /* 1081 */  vecY_rule,
  /* 1082 */  vecX_rule,
  /* 1083 */  vecX_rule,
  /* 1084 */  vecX_rule,
  /* 1085 */  vecX_rule,
  /* 1086 */  vecY_rule,
  /* 1087 */  vecY_rule,
  /* 1088 */  vecS_rule,
  /* 1089 */  vecS_rule,
  /* 1090 */  vecS_rule,
  /* 1091 */  vecS_rule,
  /* 1092 */  vecD_rule,
  /* 1093 */  vecD_rule,
  /* 1094 */  vecD_rule,
  /* 1095 */  vecD_rule,
  /* 1096 */  vecX_rule,
  /* 1097 */  vecX_rule,
  /* 1098 */  vecX_rule,
  /* 1099 */  vecX_rule,
  /* 1100 */  vecY_rule,
  /* 1101 */  vecY_rule,
  /* 1102 */  vecD_rule,
  /* 1103 */  vecD_rule,
  /* 1104 */  vecD_rule,
  /* 1105 */  vecD_rule,
  /* 1106 */  vecX_rule,
  /* 1107 */  vecX_rule,
  /* 1108 */  vecX_rule,
  /* 1109 */  vecX_rule,
  /* 1110 */  vecY_rule,
  /* 1111 */  vecY_rule,
  /* 1112 */  vecX_rule,
  /* 1113 */  vecX_rule,
  /* 1114 */  vecX_rule,
  /* 1115 */  vecX_rule,
  /* 1116 */  vecY_rule,
  /* 1117 */  vecY_rule,
  /* 1118 */  vecS_rule,
  /* 1119 */  vecS_rule,
  /* 1120 */  vecS_rule,
  /* 1121 */  vecS_rule,
  /* 1122 */  vecD_rule,
  /* 1123 */  vecD_rule,
  /* 1124 */  vecD_rule,
  /* 1125 */  vecD_rule,
  /* 1126 */  vecX_rule,
  /* 1127 */  vecX_rule,
  /* 1128 */  vecX_rule,
  /* 1129 */  vecX_rule,
  /* 1130 */  vecY_rule,
  /* 1131 */  vecY_rule,
  /* 1132 */  vecD_rule,
  /* 1133 */  vecD_rule,
  /* 1134 */  vecD_rule,
  /* 1135 */  vecD_rule,
  /* 1136 */  vecX_rule,
  /* 1137 */  vecX_rule,
  /* 1138 */  vecX_rule,
  /* 1139 */  vecX_rule,
  /* 1140 */  vecY_rule,
  /* 1141 */  vecY_rule,
  /* 1142 */  vecS_rule,
  /* 1143 */  vecS_rule,
  /* 1144 */  vecD_rule,
  /* 1145 */  vecD_rule,
  /* 1146 */  vecX_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  vecX_rule,
  /* 1149 */  vecY_rule,
  /* 1150 */  vecY_rule,
  /* 1151 */  vecS_rule,
  /* 1152 */  vecS_rule,
  /* 1153 */  vecD_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecX_rule,
  /* 1157 */  vecX_rule,
  /* 1158 */  vecY_rule,
  /* 1159 */  vecY_rule,
  /* 1160 */  vecS_rule,
  /* 1161 */  vecS_rule,
  /* 1162 */  vecD_rule,
  /* 1163 */  vecD_rule,
  /* 1164 */  vecX_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  vecX_rule,
  /* 1167 */  vecY_rule,
  /* 1168 */  vecY_rule,
  // last instruction
  0 // no trailing comma
};

const        int   leftOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  any_RegP_rule,
  /*   78 */  any_RegP_rule,
  /*   79 */  _AddP_any_RegP_rRegL_rule,
  /*   80 */  any_RegP_rule,
  /*   81 */  any_RegP_rule,
  /*   82 */  _AddP_any_RegP__LShiftL_rRegL_immI2_rule,
  /*   83 */  _AddP_any_RegP__ConvI2L_rRegI__rule,
  /*   84 */  _AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2_rule,
  /*   85 */  _DecodeN_rRegN__rule,
  /*   86 */  rRegN_rule,
  /*   87 */  _DecodeN_rRegN__rule,
  /*   88 */  _DecodeN_rRegN__rule,
  /*   89 */  _AddP__DecodeN_rRegN__rRegL_rule,
  /*   90 */  _DecodeN_rRegN__rule,
  /*   91 */  _DecodeN_rRegN__rule,
  /*   92 */  _AddP__DecodeN_rRegN___LShiftL_rRegL_immI2_rule,
  /*   93 */  _AddP__DecodeN_rRegN___ConvI2L_rRegI__rule,
  /*   94 */  _AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2_rule,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  // last operand
  /*  108 */  0,
  // last operand class
  /*  109 */  any_RegP_rule,
  /*  110 */  rRegL_rule,
  /*  111 */  any_RegP_rule,
  /*  112 */  rRegI_rule,
  /*  113 */  any_RegP_rule,
  /*  114 */  _ConvI2L_rRegI__rule,
  /*  115 */  any_RegP_rule,
  /*  116 */  rRegN_rule,
  /*  117 */  _DecodeN_rRegN__rule,
  /*  118 */  _DecodeN_rRegN__rule,
  /*  119 */  _DecodeN_rRegN__rule,
  /*  120 */  _DecodeN_rRegN__rule,
  /*  121 */  memory_rule,
  /*  122 */  memory_rule,
  /*  123 */  _LoadUB_memory__rule,
  /*  124 */  memory_rule,
  /*  125 */  _LoadS_memory__rule,
  /*  126 */  memory_rule,
  /*  127 */  _LoadUS_memory__rule,
  /*  128 */  _LoadUS_memory__rule,
  /*  129 */  _LoadUS_memory__rule,
  /*  130 */  memory_rule,
  /*  131 */  _LoadI_memory__rule,
  /*  132 */  _LoadI_memory__rule,
  /*  133 */  _LoadI_memory__rule,
  /*  134 */  _LoadI_memory__rule,
  /*  135 */  _LoadI_memory__rule,
  /*  136 */  _LoadI_memory__rule,
  /*  137 */  memory_rule,
  /*  138 */  rRegP_rule,
  /*  139 */  _DecodeN_rRegN__rule,
  /*  140 */  _LShiftL_rRegL_immI2_rule,
  /*  141 */  cmpOp_rule,
  /*  142 */  rRegI_rule,
  /*  143 */  cmpOpU_rule,
  /*  144 */  cmpOpUCF_rule,
  /*  145 */  rRegI_rule,
  /*  146 */  rRegN_rule,
  /*  147 */  rRegP_rule,
  /*  148 */  rRegL_rule,
  /*  149 */  rRegL_rule,
  /*  150 */  regF_rule,
  /*  151 */  regD_rule,
  /*  152 */  _LoadI_memory__rule,
  /*  153 */  rRegI_rule,
  /*  154 */  _LoadI_memory__rule,
  /*  155 */  _LoadI_memory__rule,
  /*  156 */  _LoadI_memory__rule,
  /*  157 */  _LoadL_memory__rule,
  /*  158 */  rRegL_rule,
  /*  159 */  _LoadL_memory__rule,
  /*  160 */  _LoadL_memory__rule,
  /*  161 */  _LoadL_memory__rule,
  /*  162 */  rax_RegP_rule,
  /*  163 */  rax_RegI_rule,
  /*  164 */  rax_RegL_rule,
  /*  165 */  rax_RegN_rule,
  /*  166 */  _LoadI_memory__rule,
  /*  167 */  _LoadI_memory__rule,
  /*  168 */  _LoadL_memory__rule,
  /*  169 */  _LoadL_memory__rule,
  /*  170 */  immI0_rule,
  /*  171 */  immI0_rule,
  /*  172 */  immL0_rule,
  /*  173 */  _LoadI_memory__rule,
  /*  174 */  _LoadI_memory__rule,
  /*  175 */  _LoadI_memory__rule,
  /*  176 */  _LoadI_memory__rule,
  /*  177 */  _LoadI_memory__rule,
  /*  178 */  _LoadI_memory__rule,
  /*  179 */  _LoadI_memory__rule,
  /*  180 */  _LoadI_memory__rule,
  /*  181 */  _LoadI_memory__rule,
  /*  182 */  _LoadL_memory__rule,
  /*  183 */  _LoadL_memory__rule,
  /*  184 */  _LoadL_memory__rule,
  /*  185 */  _LoadL_memory__rule,
  /*  186 */  _LoadL_memory__rule,
  /*  187 */  _LoadL_memory__rule,
  /*  188 */  _LoadL_memory__rule,
  /*  189 */  _LoadL_memory__rule,
  /*  190 */  _LoadL_memory__rule,
  /*  191 */  rRegI_rule,
  /*  192 */  rRegI_rule,
  /*  193 */  rRegI_rule,
  /*  194 */  rRegI_rule,
  /*  195 */  rRegI_rule,
  /*  196 */  rRegI_rule,
  /*  197 */  no_rcx_RegI_rule,
  /*  198 */  immI0_rule,
  /*  199 */  no_rcx_RegI_rule,
  /*  200 */  immI_32_rule,
  /*  201 */  no_rcx_RegI_rule,
  /*  202 */  rRegI_rule,
  /*  203 */  rRegI_rule,
  /*  204 */  no_rcx_RegI_rule,
  /*  205 */  no_rcx_RegI_rule,
  /*  206 */  no_rcx_RegI_rule,
  /*  207 */  rRegL_rule,
  /*  208 */  rRegL_rule,
  /*  209 */  rRegL_rule,
  /*  210 */  rRegL_rule,
  /*  211 */  no_rcx_RegL_rule,
  /*  212 */  no_rcx_RegL_rule,
  /*  213 */  immI_64_rule,
  /*  214 */  no_rcx_RegL_rule,
  /*  215 */  rRegL_rule,
  /*  216 */  rRegL_rule,
  /*  217 */  no_rcx_RegL_rule,
  /*  218 */  no_rcx_RegL_rule,
  /*  219 */  no_rcx_RegL_rule,
  /*  220 */  rRegI_rule,
  /*  221 */  rRegI_rule,
  /*  222 */  _LoadI_memory__rule,
  /*  223 */  rRegI_rule,
  /*  224 */  _LoadI_memory__rule,
  /*  225 */  _LoadI_memory__rule,
  /*  226 */  rRegI_rule,
  /*  227 */  _LoadI_memory__rule,
  /*  228 */  _LoadI_memory__rule,
  /*  229 */  rRegI_rule,
  /*  230 */  _LoadI_memory__rule,
  /*  231 */  _LoadL_memory__rule,
  /*  232 */  rRegL_rule,
  /*  233 */  _LoadL_memory__rule,
  /*  234 */  any_RegP_rule,
  /*  235 */  _LoadL_memory__rule,
  /*  236 */  rRegL_rule,
  /*  237 */  _LoadL_memory__rule,
  /*  238 */  _LoadL_memory__rule,
  /*  239 */  rRegL_rule,
  /*  240 */  _LoadL_memory__rule,
  /*  241 */  rRegI_rule,
  /*  242 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  243 */  rRegI_rule,
  /*  244 */  rRegI_rule,
  /*  245 */  memory_rule,
  /*  246 */  memory_rule,
  /*  247 */  rdi_RegP_rule,
  /*  248 */  rsi_RegP_rule,
  /*  249 */  rdi_RegP_rule,
  /*  250 */  rsi_RegP_rule,
  /*  251 */  rsi_RegP_rule,
  /*  252 */  rdi_RegP_rule,
  /*  253 */  rRegI_rule,
  /*  254 */  memory_rule,
  /*  255 */  memory_rule,
  /*  256 */  rRegL_rule,
  /*  257 */  rsi_RegP_rule,
  /*  258 */  regF_rule,
  /*  259 */  _ConvF2D_regF__rule,
  /*  260 */  _LoadF_memory__rule,
  /*  261 */  _ConvF2D__LoadF_memory___rule,
  /*  262 */  immF_rule,
  /*  263 */  _ConvF2D_immF__rule,
  /*  264 */  memory_rule,
  // last internally defined operand
  /*  265 */  rRegI_rule,
  /*  266 */  rRegL_rule,
  /*  267 */  rRegP_rule,
  /*  268 */  regF_rule,
  /*  269 */  regD_rule,
  /*  270 */  indOffset8_rule,
  /*  271 */  indOffset32_rule,
  /*  272 */  indIndexOffset_rule,
  /*  273 */  indIndexScale_rule,
  /*  274 */  indIndexScaleOffset_rule,
  /*  275 */  indPosIndexOffset_rule,
  /*  276 */  indPosIndexScaleOffset_rule,
  /*  277 */  indCompressedOopOffset_rule,
  /*  278 */  indOffset8Narrow_rule,
  /*  279 */  indOffset32Narrow_rule,
  /*  280 */  indIndexOffsetNarrow_rule,
  /*  281 */  indIndexScaleNarrow_rule,
  /*  282 */  indIndexScaleOffsetNarrow_rule,
  /*  283 */  indPosIndexOffsetNarrow_rule,
  /*  284 */  indPosIndexScaleOffsetNarrow_rule,
  /*  285 */  immI_rule,
  /*  286 */  immI0_rule,
  /*  287 */  immL_rule,
  /*  288 */  immL0_rule,
  /*  289 */  immUL32_rule,
  /*  290 */  immL32_rule,
  /*  291 */  immP_rule,
  /*  292 */  immP0_rule,
  /*  293 */  immP31_rule,
  /*  294 */  immF_rule,
  /*  295 */  immN0_rule,
  /*  296 */  immN_rule,
  /*  297 */  immF0_rule,
  /*  298 */  immD_rule,
  /*  299 */  immD0_rule,
  /*  300 */  stackSlotI_rule,
  /*  301 */  stackSlotL_rule,
  /*  302 */  stackSlotP_rule,
  /*  303 */  stackSlotF_rule,
  /*  304 */  stackSlotD_rule,
  /*  305 */  0,
  /*  306 */  rRegL_rule,
  /*  307 */  rRegP_rule,
  /*  308 */  rRegP_rule,
  /*  309 */  rRegP_rule,
  /*  310 */  rRegI_rule,
  /*  311 */  memory_rule,
  /*  312 */  memory_rule,
  /*  313 */  memory_rule,
  /*  314 */  regF_rule,
  /*  315 */  regF_rule,
  /*  316 */  regF_rule,
  /*  317 */  regF_rule,
  /*  318 */  regF_rule,
  /*  319 */  regF_rule,
  /*  320 */  regD_rule,
  /*  321 */  regD_rule,
  /*  322 */  regD_rule,
  /*  323 */  regD_rule,
  /*  324 */  regD_rule,
  /*  325 */  regD_rule,
  /*  326 */  regF_rule,
  /*  327 */  regD_rule,
  /*  328 */  stackSlotF_rule,
  /*  329 */  stackSlotI_rule,
  /*  330 */  stackSlotD_rule,
  /*  331 */  stackSlotL_rule,
  /*  332 */  stackSlotL_rule,
  /*  333 */  regF_rule,
  /*  334 */  regD_rule,
  /*  335 */  rRegI_rule,
  /*  336 */  rRegL_rule,
  /*  337 */  rRegI_rule,
  /*  338 */  rRegI_rule,
  /*  339 */  rRegI_rule,
  /*  340 */  rRegI_rule,
  /*  341 */  _AndI_rRegI_immI_rule,
  /*  342 */  _AndI_rRegI__LoadI_memory__rule,
  /*  343 */  _AndI__LoadI_memory__rRegI_rule,
  /*  344 */  rRegI_rule,
  /*  345 */  rRegI_rule,
  /*  346 */  rRegI_rule,
  /*  347 */  rRegI_rule,
  /*  348 */  rRegP_rule,
  /*  349 */  rRegP_rule,
  /*  350 */  rRegP_rule,
  /*  351 */  rRegP_rule,
  /*  352 */  _LoadP_memory__rule,
  /*  353 */  _LoadP_memory__rule,
  /*  354 */  rRegN_rule,
  /*  355 */  rRegN_rule,
  /*  356 */  rRegN_rule,
  /*  357 */  immN_rule,
  /*  358 */  rRegN_rule,
  /*  359 */  _LoadN_memory__rule,
  /*  360 */  _LoadN_memory__rule,
  /*  361 */  rRegL_rule,
  /*  362 */  rRegL_rule,
  /*  363 */  rRegL_rule,
  /*  364 */  rRegL_rule,
  /*  365 */  _AndL_rRegL_immL32_rule,
  /*  366 */  _AndL_rRegL__LoadL_memory__rule,
  /*  367 */  _AndL__LoadL_memory__rRegL_rule,
  /*  368 */  rRegL_rule,
  /*  369 */  rRegL_rule,
  /*  370 */  rRegL_rule,
  /*  371 */  rRegL_rule,
  /*  372 */  _PartialSubtypeCheck_rsi_RegP_rax_RegP_rule,
  /*  373 */  rRegP_rule,
  /*  374 */  rRegP_rule,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  regF_rule,
  /*  378 */  regF_rule,
  /*  379 */  regD_rule,
  /*  380 */  regD_rule,
  /*  381 */  regF_rule,
  /*  382 */  regF_rule,
  /*  383 */  regD_rule,
  /*  384 */  regD_rule,
  /*  385 */  rRegI_rule,
  /*  386 */  rRegI_rule,
  /*  387 */  memory_rule,
  /*  388 */  _LoadB_memory__rule,
  /*  389 */  memory_rule,
  /*  390 */  _LoadUB_memory__rule,
  /*  391 */  _AndI__LoadUB_memory__immI8_rule,
  /*  392 */  memory_rule,
  /*  393 */  _LShiftI__LoadS_memory__immI_24_rule,
  /*  394 */  _LoadS_memory__rule,
  /*  395 */  memory_rule,
  /*  396 */  _LShiftI__LoadUS_memory__immI_24_rule,
  /*  397 */  _LoadUS_memory__rule,
  /*  398 */  _AndI__LoadUS_memory__immI_255_rule,
  /*  399 */  _AndI__LoadUS_memory__immI16_rule,
  /*  400 */  memory_rule,
  /*  401 */  _LShiftI__LoadI_memory__immI_24_rule,
  /*  402 */  _LoadI_memory__rule,
  /*  403 */  _LShiftI__LoadI_memory__immI_16_rule,
  /*  404 */  _LoadI_memory__rule,
  /*  405 */  _LoadI_memory__rule,
  /*  406 */  _AndI__LoadI_memory__immI_255_rule,
  /*  407 */  _AndI__LoadI_memory__immI_65535_rule,
  /*  408 */  _AndI__LoadI_memory__immU31_rule,
  /*  409 */  _ConvI2L__LoadI_memory___rule,
  /*  410 */  memory_rule,
  /*  411 */  memory_rule,
  /*  412 */  memory_rule,
  /*  413 */  memory_rule,
  /*  414 */  memory_rule,
  /*  415 */  memory_rule,
  /*  416 */  memory_rule,
  /*  417 */  memory_rule,
  /*  418 */  memory_rule,
  /*  419 */  memory_rule,
  /*  420 */  memory_rule,
  /*  421 */  memory_rule,
  /*  422 */  memory_rule,
  /*  423 */  memory_rule,
  /*  424 */  memory_rule,
  /*  425 */  memory_rule,
  /*  426 */  memory_rule,
  /*  427 */  memory_rule,
  /*  428 */  memory_rule,
  /*  429 */  memory_rule,
  /*  430 */  memory_rule,
  /*  431 */  memory_rule,
  /*  432 */  memory_rule,
  /*  433 */  memory_rule,
  /*  434 */  memory_rule,
  /*  435 */  memory_rule,
  /*  436 */  memory_rule,
  /*  437 */  memory_rule,
  /*  438 */  memory_rule,
  /*  439 */  memory_rule,
  /*  440 */  memory_rule,
  /*  441 */  memory_rule,
  /*  442 */  memory_rule,
  /*  443 */  memory_rule,
  /*  444 */  memory_rule,
  /*  445 */  memory_rule,
  /*  446 */  memory_rule,
  /*  447 */  memory_rule,
  /*  448 */  memory_rule,
  /*  449 */  memory_rule,
  /*  450 */  memory_rule,
  /*  451 */  memory_rule,
  /*  452 */  memory_rule,
  /*  453 */  memory_rule,
  /*  454 */  rRegI_rule,
  /*  455 */  rRegL_rule,
  /*  456 */  rRegI_rule,
  /*  457 */  rRegI_rule,
  /*  458 */  rRegI_rule,
  /*  459 */  rRegI_rule,
  /*  460 */  rRegL_rule,
  /*  461 */  rRegL_rule,
  /*  462 */  rRegI_rule,
  /*  463 */  rRegL_rule,
  /*  464 */  rRegI_rule,
  /*  465 */  _LoadI_memory__rule,
  /*  466 */  rRegL_rule,
  /*  467 */  _LoadL_memory__rule,
  /*  468 */  0,
  /*  469 */  0,
  /*  470 */  0,
  /*  471 */  0,
  /*  472 */  0,
  /*  473 */  0,
  /*  474 */  _CastP2X_rRegP__rule,
  /*  475 */  _CastP2X__DecodeN_rRegN___rule,
  /*  476 */  rRegP_rule,
  /*  477 */  rRegP_rule,
  /*  478 */  rRegN_rule,
  /*  479 */  rRegN_rule,
  /*  480 */  _LShiftL_rRegL_immI2_rule,
  /*  481 */  _AddL__LShiftL_rRegL_immI2_immL32_rule,
  /*  482 */  rRegL_rule,
  /*  483 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  484 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  485 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  486 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  487 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  488 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  489 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  490 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  491 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  492 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  493 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  494 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  495 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  496 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  497 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  498 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  499 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  500 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  501 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  502 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  503 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  504 */  _Binary_cmpOp_rFlagsReg_rule,
  /*  505 */  _Binary_cmpOpU_rFlagsRegU_rule,
  /*  506 */  _Binary_cmpOpUCF_rFlagsRegUCF_rule,
  /*  507 */  rRegI_rule,
  /*  508 */  rRegI_rule,
  /*  509 */  rRegI_rule,
  /*  510 */  _LoadI_memory__rule,
  /*  511 */  memory_rule,
  /*  512 */  memory_rule,
  /*  513 */  memory_rule,
  /*  514 */  rRegI_rule,
  /*  515 */  memory_rule,
  /*  516 */  rRegI_rule,
  /*  517 */  memory_rule,
  /*  518 */  rRegI_rule,
  /*  519 */  rRegL_rule,
  /*  520 */  rRegL_rule,
  /*  521 */  rRegL_rule,
  /*  522 */  _LoadL_memory__rule,
  /*  523 */  memory_rule,
  /*  524 */  memory_rule,
  /*  525 */  memory_rule,
  /*  526 */  rRegI_rule,
  /*  527 */  memory_rule,
  /*  528 */  rRegL_rule,
  /*  529 */  memory_rule,
  /*  530 */  rRegL_rule,
  /*  531 */  rRegP_rule,
  /*  532 */  rRegP_rule,
  /*  533 */  rRegP_rule,
  /*  534 */  memory_rule,
  /*  535 */  memory_rule,
  /*  536 */  memory_rule,
  /*  537 */  memory_rule,
  /*  538 */  memory_rule,
  /*  539 */  memory_rule,
  /*  540 */  memory_rule,
  /*  541 */  memory_rule,
  /*  542 */  memory_rule,
  /*  543 */  memory_rule,
  /*  544 */  memory_rule,
  /*  545 */  memory_rule,
  /*  546 */  memory_rule,
  /*  547 */  rRegI_rule,
  /*  548 */  rRegI_rule,
  /*  549 */  rRegI_rule,
  /*  550 */  memory_rule,
  /*  551 */  memory_rule,
  /*  552 */  rRegL_rule,
  /*  553 */  rRegI_rule,
  /*  554 */  rRegL_rule,
  /*  555 */  memory_rule,
  /*  556 */  memory_rule,
  /*  557 */  rRegP_rule,
  /*  558 */  immI0_rule,
  /*  559 */  memory_rule,
  /*  560 */  immL0_rule,
  /*  561 */  memory_rule,
  /*  562 */  rRegI_rule,
  /*  563 */  rRegI_rule,
  /*  564 */  rRegI_rule,
  /*  565 */  _LoadI_memory__rule,
  /*  566 */  _LoadI_memory__rule,
  /*  567 */  rRegL_rule,
  /*  568 */  rRegL_rule,
  /*  569 */  rRegL_rule,
  /*  570 */  _LoadL_memory__rule,
  /*  571 */  _LoadL_memory__rule,
  /*  572 */  no_rax_RegL_rule,
  /*  573 */  rax_RegI_rule,
  /*  574 */  rax_RegL_rule,
  /*  575 */  rax_RegI_rule,
  /*  576 */  rax_RegL_rule,
  /*  577 */  0,
  /*  578 */  0,
  /*  579 */  0,
  /*  580 */  0,
  /*  581 */  no_rax_RegL_rule,
  /*  582 */  rax_RegI_rule,
  /*  583 */  rax_RegL_rule,
  /*  584 */  rRegI_rule,
  /*  585 */  memory_rule,
  /*  586 */  rRegI_rule,
  /*  587 */  memory_rule,
  /*  588 */  rRegI_rule,
  /*  589 */  memory_rule,
  /*  590 */  rRegI_rule,
  /*  591 */  memory_rule,
  /*  592 */  rRegI_rule,
  /*  593 */  memory_rule,
  /*  594 */  rRegI_rule,
  /*  595 */  memory_rule,
  /*  596 */  rRegI_rule,
  /*  597 */  memory_rule,
  /*  598 */  rRegI_rule,
  /*  599 */  memory_rule,
  /*  600 */  rRegI_rule,
  /*  601 */  memory_rule,
  /*  602 */  rRegL_rule,
  /*  603 */  memory_rule,
  /*  604 */  rRegL_rule,
  /*  605 */  memory_rule,
  /*  606 */  rRegL_rule,
  /*  607 */  memory_rule,
  /*  608 */  rRegL_rule,
  /*  609 */  memory_rule,
  /*  610 */  rRegL_rule,
  /*  611 */  memory_rule,
  /*  612 */  rRegL_rule,
  /*  613 */  memory_rule,
  /*  614 */  rRegL_rule,
  /*  615 */  memory_rule,
  /*  616 */  rRegL_rule,
  /*  617 */  memory_rule,
  /*  618 */  rRegL_rule,
  /*  619 */  memory_rule,
  /*  620 */  _LShiftI_rRegI_immI_24_rule,
  /*  621 */  _LShiftI_rRegI_immI_16_rule,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  _LShiftI_rRegI_immI1_rule,
  /*  626 */  _URShiftI_rRegI_immI_M1_rule,
  /*  627 */  _LShiftI_rRegI_immI8_rule,
  /*  628 */  _URShiftI_rRegI_immI8_rule,
  /*  629 */  _LShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  630 */  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  631 */  _LShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  632 */  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  633 */  0,
  /*  634 */  0,
  /*  635 */  0,
  /*  636 */  _URShiftI_rRegI_immI1_rule,
  /*  637 */  _LShiftI_rRegI_immI_M1_rule,
  /*  638 */  _URShiftI_rRegI_immI8_rule,
  /*  639 */  _LShiftI_rRegI_immI8_rule,
  /*  640 */  _URShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  641 */  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  642 */  _URShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  643 */  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  644 */  0,
  /*  645 */  0,
  /*  646 */  0,
  /*  647 */  _LShiftL_rRegL_immI1_rule,
  /*  648 */  _URShiftL_rRegL_immI_M1_rule,
  /*  649 */  _LShiftL_rRegL_immI8_rule,
  /*  650 */  _URShiftL_rRegL_immI8_rule,
  /*  651 */  _LShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  652 */  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  653 */  _LShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  654 */  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  655 */  0,
  /*  656 */  0,
  /*  657 */  0,
  /*  658 */  _URShiftL_rRegL_immI1_rule,
  /*  659 */  _LShiftL_rRegL_immI_M1_rule,
  /*  660 */  _URShiftL_rRegL_immI8_rule,
  /*  661 */  _LShiftL_rRegL_immI8_rule,
  /*  662 */  _URShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  663 */  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  664 */  _URShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  665 */  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  rRegI_rule,
  /*  668 */  _AndI_rRegI_immI_255_rule,
  /*  669 */  rRegI_rule,
  /*  670 */  _AndI_rRegI_immI_65535_rule,
  /*  671 */  rRegI_rule,
  /*  672 */  rRegI_rule,
  /*  673 */  _LoadI_memory__rule,
  /*  674 */  memory_rule,
  /*  675 */  memory_rule,
  /*  676 */  memory_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  rRegI_rule,
  /*  679 */  rRegI_rule,
  /*  680 */  _LoadI_memory__rule,
  /*  681 */  memory_rule,
  /*  682 */  memory_rule,
  /*  683 */  memory_rule,
  /*  684 */  rRegI_rule,
  /*  685 */  rRegI_rule,
  /*  686 */  rRegI_rule,
  /*  687 */  rRegI_rule,
  /*  688 */  _LoadI_memory__rule,
  /*  689 */  memory_rule,
  /*  690 */  memory_rule,
  /*  691 */  memory_rule,
  /*  692 */  rRegL_rule,
  /*  693 */  rRegL_rule,
  /*  694 */  rRegL_rule,
  /*  695 */  rRegL_rule,
  /*  696 */  rRegL_rule,
  /*  697 */  _LoadL_memory__rule,
  /*  698 */  memory_rule,
  /*  699 */  memory_rule,
  /*  700 */  memory_rule,
  /*  701 */  rRegL_rule,
  /*  702 */  rRegL_rule,
  /*  703 */  _CastP2X_any_RegP__rule,
  /*  704 */  rRegL_rule,
  /*  705 */  rRegL_rule,
  /*  706 */  _LoadL_memory__rule,
  /*  707 */  memory_rule,
  /*  708 */  memory_rule,
  /*  709 */  memory_rule,
  /*  710 */  rRegL_rule,
  /*  711 */  rRegL_rule,
  /*  712 */  rRegL_rule,
  /*  713 */  rRegL_rule,
  /*  714 */  _LoadL_memory__rule,
  /*  715 */  memory_rule,
  /*  716 */  memory_rule,
  /*  717 */  memory_rule,
  /*  718 */  rRegI_rule,
  /*  719 */  rRegP_rule,
  /*  720 */  rRegI_rule,
  /*  721 */  rRegI_rule,
  /*  722 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  723 */  _SubI_rRegI_rRegI_rule,
  /*  724 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  725 */  _SubI_rRegI_rRegI_rule,
  /*  726 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  727 */  rRegI_rule,
  /*  728 */  regF_rule,
  /*  729 */  regF_rule,
  /*  730 */  regF_rule,
  /*  731 */  regD_rule,
  /*  732 */  regD_rule,
  /*  733 */  regD_rule,
  /*  734 */  regD_rule,
  /*  735 */  regD_rule,
  /*  736 */  regD_rule,
  /*  737 */  regD_rule,
  /*  738 */  regD_rule,
  /*  739 */  regD_rule,
  /*  740 */  regD_rule,
  /*  741 */  regF_rule,
  /*  742 */  _LoadF_memory__rule,
  /*  743 */  regD_rule,
  /*  744 */  _LoadD_memory__rule,
  /*  745 */  regF_rule,
  /*  746 */  regF_rule,
  /*  747 */  regD_rule,
  /*  748 */  regD_rule,
  /*  749 */  rRegI_rule,
  /*  750 */  _LoadI_memory__rule,
  /*  751 */  rRegI_rule,
  /*  752 */  _LoadI_memory__rule,
  /*  753 */  rRegI_rule,
  /*  754 */  rRegI_rule,
  /*  755 */  rRegL_rule,
  /*  756 */  _LoadL_memory__rule,
  /*  757 */  rRegL_rule,
  /*  758 */  _LoadL_memory__rule,
  /*  759 */  rRegI_rule,
  /*  760 */  _ConvI2L_rRegI__rule,
  /*  761 */  _ConvI2L__LoadI_memory___rule,
  /*  762 */  rRegL_rule,
  /*  763 */  rRegL_rule,
  /*  764 */  regF_rule,
  /*  765 */  rRegI_rule,
  /*  766 */  regD_rule,
  /*  767 */  rRegL_rule,
  /*  768 */  rcx_RegL_rule,
  /*  769 */  rcx_RegL_rule,
  /*  770 */  _Binary_rdi_RegP_rcx_RegI_rule,
  /*  771 */  _Binary_rdi_RegP_rdx_RegI_rule,
  /*  772 */  _Binary_rdi_RegP_rdx_RegI_rule,
  /*  773 */  _Binary_rdi_RegP_rsi_RegP_rule,
  /*  774 */  rdi_RegP_rule,
  /*  775 */  rRegL_rule,
  /*  776 */  0,
  /*  777 */  rRegI_rule,
  /*  778 */  0,
  /*  779 */  rRegI_rule,
  /*  780 */  0,
  /*  781 */  cmpOp_rule,
  /*  782 */  cmpOp_rule,
  /*  783 */  cmpOpU_rule,
  /*  784 */  cmpOpUCF_rule,
  /*  785 */  cmpOpU_rule,
  /*  786 */  cmpOpUCF_rule,
  /*  787 */  cmpOpUCF2_rule,
  /*  788 */  rsi_RegP_rule,
  /*  789 */  0,
  /*  790 */  cmpOp_rule,
  /*  791 */  cmpOp_rule,
  /*  792 */  cmpOpU_rule,
  /*  793 */  cmpOpUCF_rule,
  /*  794 */  cmpOpU_rule,
  /*  795 */  cmpOpUCF_rule,
  /*  796 */  cmpOpUCF2_rule,
  /*  797 */  rRegP_rule,
  /*  798 */  0,
  /*  799 */  0,
  /*  800 */  0,
  /*  801 */  0,
  /*  802 */  0,
  /*  803 */  0,
  /*  804 */  0,
  /*  805 */  no_rbp_RegP_rule,
  /*  806 */  no_rbp_RegP_rule,
  /*  807 */  0,
  /*  808 */  0,
  /*  809 */  0,
  /*  810 */  regF_rule,
  /*  811 */  regF_rule,
  /*  812 */  _LoadF_memory__rule,
  /*  813 */  regF_rule,
  /*  814 */  regF_rule,
  /*  815 */  regF_rule,
  /*  816 */  _LoadF_memory__rule,
  /*  817 */  regF_rule,
  /*  818 */  regD_rule,
  /*  819 */  regD_rule,
  /*  820 */  _LoadD_memory__rule,
  /*  821 */  regD_rule,
  /*  822 */  regD_rule,
  /*  823 */  regD_rule,
  /*  824 */  _LoadD_memory__rule,
  /*  825 */  regD_rule,
  /*  826 */  regF_rule,
  /*  827 */  regF_rule,
  /*  828 */  regF_rule,
  /*  829 */  regF_rule,
  /*  830 */  regF_rule,
  /*  831 */  regF_rule,
  /*  832 */  regD_rule,
  /*  833 */  regD_rule,
  /*  834 */  regD_rule,
  /*  835 */  regD_rule,
  /*  836 */  regD_rule,
  /*  837 */  regD_rule,
  /*  838 */  regF_rule,
  /*  839 */  regF_rule,
  /*  840 */  _LoadF_memory__rule,
  /*  841 */  regF_rule,
  /*  842 */  regF_rule,
  /*  843 */  regF_rule,
  /*  844 */  _LoadF_memory__rule,
  /*  845 */  regF_rule,
  /*  846 */  regD_rule,
  /*  847 */  regD_rule,
  /*  848 */  _LoadD_memory__rule,
  /*  849 */  regD_rule,
  /*  850 */  regD_rule,
  /*  851 */  regD_rule,
  /*  852 */  _LoadD_memory__rule,
  /*  853 */  regD_rule,
  /*  854 */  regF_rule,
  /*  855 */  regF_rule,
  /*  856 */  regF_rule,
  /*  857 */  regF_rule,
  /*  858 */  regF_rule,
  /*  859 */  regF_rule,
  /*  860 */  regD_rule,
  /*  861 */  regD_rule,
  /*  862 */  regD_rule,
  /*  863 */  regD_rule,
  /*  864 */  regD_rule,
  /*  865 */  regD_rule,
  /*  866 */  _SqrtD__ConvF2D_regF___rule,
  /*  867 */  _SqrtD__ConvF2D__LoadF_memory____rule,
  /*  868 */  _SqrtD__ConvF2D_immF___rule,
  /*  869 */  regD_rule,
  /*  870 */  _LoadD_memory__rule,
  /*  871 */  immD_rule,
  /*  872 */  memory_rule,
  /*  873 */  memory_rule,
  /*  874 */  memory_rule,
  /*  875 */  memory_rule,
  /*  876 */  memory_rule,
  /*  877 */  memory_rule,
  /*  878 */  memory_rule,
  /*  879 */  memory_rule,
  /*  880 */  rRegI_rule,
  /*  881 */  rRegI_rule,
  /*  882 */  rRegI_rule,
  /*  883 */  rRegI_rule,
  /*  884 */  immI_rule,
  /*  885 */  immI_rule,
  /*  886 */  immI_rule,
  /*  887 */  immI_rule,
  /*  888 */  immI0_rule,
  /*  889 */  immI0_rule,
  /*  890 */  immI0_rule,
  /*  891 */  immI0_rule,
  /*  892 */  rRegI_rule,
  /*  893 */  rRegI_rule,
  /*  894 */  rRegI_rule,
  /*  895 */  rRegI_rule,
  /*  896 */  immI_rule,
  /*  897 */  immI_rule,
  /*  898 */  immI_rule,
  /*  899 */  immI_rule,
  /*  900 */  immI0_rule,
  /*  901 */  immI0_rule,
  /*  902 */  immI0_rule,
  /*  903 */  immI0_rule,
  /*  904 */  rRegI_rule,
  /*  905 */  rRegI_rule,
  /*  906 */  rRegI_rule,
  /*  907 */  immI_rule,
  /*  908 */  immI_rule,
  /*  909 */  immI_rule,
  /*  910 */  _LoadI_memory__rule,
  /*  911 */  _LoadI_memory__rule,
  /*  912 */  _LoadI_memory__rule,
  /*  913 */  immI0_rule,
  /*  914 */  immI0_rule,
  /*  915 */  immI0_rule,
  /*  916 */  rRegL_rule,
  /*  917 */  rRegL_rule,
  /*  918 */  immL_rule,
  /*  919 */  immL_rule,
  /*  920 */  _LoadL_memory__rule,
  /*  921 */  _LoadL_memory__rule,
  /*  922 */  immL0_rule,
  /*  923 */  immL0_rule,
  /*  924 */  regF_rule,
  /*  925 */  regF_rule,
  /*  926 */  regF_rule,
  /*  927 */  immF0_rule,
  /*  928 */  immF0_rule,
  /*  929 */  immF0_rule,
  /*  930 */  regD_rule,
  /*  931 */  regD_rule,
  /*  932 */  immD0_rule,
  /*  933 */  immD0_rule,
  /*  934 */  vecS_rule,
  /*  935 */  vecS_rule,
  /*  936 */  vecD_rule,
  /*  937 */  vecD_rule,
  /*  938 */  vecX_rule,
  /*  939 */  vecX_rule,
  /*  940 */  vecX_rule,
  /*  941 */  vecY_rule,
  /*  942 */  vecY_rule,
  /*  943 */  vecS_rule,
  /*  944 */  vecS_rule,
  /*  945 */  vecD_rule,
  /*  946 */  vecD_rule,
  /*  947 */  vecX_rule,
  /*  948 */  vecX_rule,
  /*  949 */  vecX_rule,
  /*  950 */  vecY_rule,
  /*  951 */  vecY_rule,
  /*  952 */  vecD_rule,
  /*  953 */  vecD_rule,
  /*  954 */  vecX_rule,
  /*  955 */  vecX_rule,
  /*  956 */  vecX_rule,
  /*  957 */  vecY_rule,
  /*  958 */  vecY_rule,
  /*  959 */  vecX_rule,
  /*  960 */  vecX_rule,
  /*  961 */  vecX_rule,
  /*  962 */  vecY_rule,
  /*  963 */  vecY_rule,
  /*  964 */  vecD_rule,
  /*  965 */  vecD_rule,
  /*  966 */  vecX_rule,
  /*  967 */  vecX_rule,
  /*  968 */  vecX_rule,
  /*  969 */  vecY_rule,
  /*  970 */  vecY_rule,
  /*  971 */  vecX_rule,
  /*  972 */  vecX_rule,
  /*  973 */  vecX_rule,
  /*  974 */  vecY_rule,
  /*  975 */  vecY_rule,
  /*  976 */  vecS_rule,
  /*  977 */  vecS_rule,
  /*  978 */  vecD_rule,
  /*  979 */  vecD_rule,
  /*  980 */  vecX_rule,
  /*  981 */  vecX_rule,
  /*  982 */  vecX_rule,
  /*  983 */  vecY_rule,
  /*  984 */  vecY_rule,
  /*  985 */  vecS_rule,
  /*  986 */  vecS_rule,
  /*  987 */  vecD_rule,
  /*  988 */  vecD_rule,
  /*  989 */  vecX_rule,
  /*  990 */  vecX_rule,
  /*  991 */  vecX_rule,
  /*  992 */  vecY_rule,
  /*  993 */  vecY_rule,
  /*  994 */  vecD_rule,
  /*  995 */  vecD_rule,
  /*  996 */  vecX_rule,
  /*  997 */  vecX_rule,
  /*  998 */  vecX_rule,
  /*  999 */  vecY_rule,
  /* 1000 */  vecY_rule,
  /* 1001 */  vecX_rule,
  /* 1002 */  vecX_rule,
  /* 1003 */  vecX_rule,
  /* 1004 */  vecY_rule,
  /* 1005 */  vecY_rule,
  /* 1006 */  vecD_rule,
  /* 1007 */  vecD_rule,
  /* 1008 */  vecX_rule,
  /* 1009 */  vecX_rule,
  /* 1010 */  vecX_rule,
  /* 1011 */  vecY_rule,
  /* 1012 */  vecY_rule,
  /* 1013 */  vecX_rule,
  /* 1014 */  vecX_rule,
  /* 1015 */  vecX_rule,
  /* 1016 */  vecY_rule,
  /* 1017 */  vecY_rule,
  /* 1018 */  vecS_rule,
  /* 1019 */  vecS_rule,
  /* 1020 */  vecD_rule,
  /* 1021 */  vecD_rule,
  /* 1022 */  vecX_rule,
  /* 1023 */  vecX_rule,
  /* 1024 */  vecX_rule,
  /* 1025 */  vecY_rule,
  /* 1026 */  vecY_rule,
  /* 1027 */  vecD_rule,
  /* 1028 */  vecD_rule,
  /* 1029 */  vecX_rule,
  /* 1030 */  vecX_rule,
  /* 1031 */  vecX_rule,
  /* 1032 */  vecY_rule,
  /* 1033 */  vecY_rule,
  /* 1034 */  vecD_rule,
  /* 1035 */  vecD_rule,
  /* 1036 */  vecX_rule,
  /* 1037 */  vecX_rule,
  /* 1038 */  vecX_rule,
  /* 1039 */  vecY_rule,
  /* 1040 */  vecY_rule,
  /* 1041 */  vecX_rule,
  /* 1042 */  vecX_rule,
  /* 1043 */  vecX_rule,
  /* 1044 */  vecY_rule,
  /* 1045 */  vecY_rule,
  /* 1046 */  vecD_rule,
  /* 1047 */  vecD_rule,
  /* 1048 */  vecX_rule,
  /* 1049 */  vecX_rule,
  /* 1050 */  vecX_rule,
  /* 1051 */  vecY_rule,
  /* 1052 */  vecY_rule,
  /* 1053 */  vecX_rule,
  /* 1054 */  vecX_rule,
  /* 1055 */  vecX_rule,
  /* 1056 */  vecY_rule,
  /* 1057 */  vecY_rule,
  /* 1058 */  vecS_rule,
  /* 1059 */  vecS_rule,
  /* 1060 */  vecS_rule,
  /* 1061 */  vecS_rule,
  /* 1062 */  vecD_rule,
  /* 1063 */  vecD_rule,
  /* 1064 */  vecD_rule,
  /* 1065 */  vecD_rule,
  /* 1066 */  vecX_rule,
  /* 1067 */  vecX_rule,
  /* 1068 */  vecX_rule,
  /* 1069 */  vecX_rule,
  /* 1070 */  vecY_rule,
  /* 1071 */  vecY_rule,
  /* 1072 */  vecD_rule,
  /* 1073 */  vecD_rule,
  /* 1074 */  vecD_rule,
  /* 1075 */  vecD_rule,
  /* 1076 */  vecX_rule,
  /* 1077 */  vecX_rule,
  /* 1078 */  vecX_rule,
  /* 1079 */  vecX_rule,
  /* 1080 */  vecY_rule,
  /* 1081 */  vecY_rule,
  /* 1082 */  vecX_rule,
  /* 1083 */  vecX_rule,
  /* 1084 */  vecX_rule,
  /* 1085 */  vecX_rule,
  /* 1086 */  vecY_rule,
  /* 1087 */  vecY_rule,
  /* 1088 */  vecS_rule,
  /* 1089 */  vecS_rule,
  /* 1090 */  vecS_rule,
  /* 1091 */  vecS_rule,
  /* 1092 */  vecD_rule,
  /* 1093 */  vecD_rule,
  /* 1094 */  vecD_rule,
  /* 1095 */  vecD_rule,
  /* 1096 */  vecX_rule,
  /* 1097 */  vecX_rule,
  /* 1098 */  vecX_rule,
  /* 1099 */  vecX_rule,
  /* 1100 */  vecY_rule,
  /* 1101 */  vecY_rule,
  /* 1102 */  vecD_rule,
  /* 1103 */  vecD_rule,
  /* 1104 */  vecD_rule,
  /* 1105 */  vecD_rule,
  /* 1106 */  vecX_rule,
  /* 1107 */  vecX_rule,
  /* 1108 */  vecX_rule,
  /* 1109 */  vecX_rule,
  /* 1110 */  vecY_rule,
  /* 1111 */  vecY_rule,
  /* 1112 */  vecX_rule,
  /* 1113 */  vecX_rule,
  /* 1114 */  vecX_rule,
  /* 1115 */  vecX_rule,
  /* 1116 */  vecY_rule,
  /* 1117 */  vecY_rule,
  /* 1118 */  vecS_rule,
  /* 1119 */  vecS_rule,
  /* 1120 */  vecS_rule,
  /* 1121 */  vecS_rule,
  /* 1122 */  vecD_rule,
  /* 1123 */  vecD_rule,
  /* 1124 */  vecD_rule,
  /* 1125 */  vecD_rule,
  /* 1126 */  vecX_rule,
  /* 1127 */  vecX_rule,
  /* 1128 */  vecX_rule,
  /* 1129 */  vecX_rule,
  /* 1130 */  vecY_rule,
  /* 1131 */  vecY_rule,
  /* 1132 */  vecD_rule,
  /* 1133 */  vecD_rule,
  /* 1134 */  vecD_rule,
  /* 1135 */  vecD_rule,
  /* 1136 */  vecX_rule,
  /* 1137 */  vecX_rule,
  /* 1138 */  vecX_rule,
  /* 1139 */  vecX_rule,
  /* 1140 */  vecY_rule,
  /* 1141 */  vecY_rule,
  /* 1142 */  vecS_rule,
  /* 1143 */  vecS_rule,
  /* 1144 */  vecD_rule,
  /* 1145 */  vecD_rule,
  /* 1146 */  vecX_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  vecX_rule,
  /* 1149 */  vecY_rule,
  /* 1150 */  vecY_rule,
  /* 1151 */  vecS_rule,
  /* 1152 */  vecS_rule,
  /* 1153 */  vecD_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecX_rule,
  /* 1157 */  vecX_rule,
  /* 1158 */  vecY_rule,
  /* 1159 */  vecY_rule,
  /* 1160 */  vecS_rule,
  /* 1161 */  vecS_rule,
  /* 1162 */  vecD_rule,
  /* 1163 */  vecD_rule,
  /* 1164 */  vecX_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  vecX_rule,
  /* 1167 */  vecY_rule,
  /* 1168 */  vecY_rule,
  // last instruction
  0 // no trailing comma
};

const        int   rightOp[] = {
  /*    0 */  0,
  /*    1 */  0,
  /*    2 */  0,
  /*    3 */  0,
  /*    4 */  0,
  /*    5 */  0,
  /*    6 */  0,
  /*    7 */  0,
  /*    8 */  0,
  /*    9 */  0,
  /*   10 */  0,
  /*   11 */  0,
  /*   12 */  0,
  /*   13 */  0,
  /*   14 */  0,
  /*   15 */  0,
  /*   16 */  0,
  /*   17 */  0,
  /*   18 */  0,
  /*   19 */  0,
  /*   20 */  0,
  /*   21 */  0,
  /*   22 */  0,
  /*   23 */  0,
  /*   24 */  0,
  /*   25 */  0,
  /*   26 */  0,
  /*   27 */  0,
  /*   28 */  0,
  /*   29 */  0,
  /*   30 */  0,
  /*   31 */  0,
  /*   32 */  0,
  /*   33 */  0,
  /*   34 */  0,
  /*   35 */  0,
  /*   36 */  0,
  /*   37 */  0,
  /*   38 */  0,
  /*   39 */  0,
  /*   40 */  0,
  /*   41 */  0,
  /*   42 */  0,
  /*   43 */  0,
  /*   44 */  0,
  /*   45 */  0,
  /*   46 */  0,
  /*   47 */  0,
  /*   48 */  0,
  /*   49 */  0,
  /*   50 */  0,
  /*   51 */  0,
  /*   52 */  0,
  /*   53 */  0,
  /*   54 */  0,
  /*   55 */  0,
  /*   56 */  0,
  /*   57 */  0,
  /*   58 */  0,
  /*   59 */  0,
  /*   60 */  0,
  /*   61 */  0,
  /*   62 */  0,
  /*   63 */  0,
  /*   64 */  0,
  /*   65 */  0,
  /*   66 */  0,
  /*   67 */  0,
  /*   68 */  0,
  /*   69 */  0,
  /*   70 */  0,
  /*   71 */  0,
  /*   72 */  0,
  /*   73 */  0,
  /*   74 */  0,
  /*   75 */  0,
  /*   76 */  0,
  /*   77 */  immL8_rule,
  /*   78 */  immL32_rule,
  /*   79 */  immL32_rule,
  /*   80 */  rRegL_rule,
  /*   81 */  _LShiftL_rRegL_immI2_rule,
  /*   82 */  immL32_rule,
  /*   83 */  immL32_rule,
  /*   84 */  immL32_rule,
  /*   85 */  immL32_rule,
  /*   86 */  0,
  /*   87 */  immL8_rule,
  /*   88 */  immL32_rule,
  /*   89 */  immL32_rule,
  /*   90 */  rRegL_rule,
  /*   91 */  _LShiftL_rRegL_immI2_rule,
  /*   92 */  immL32_rule,
  /*   93 */  immL32_rule,
  /*   94 */  immL32_rule,
  /*   95 */  0,
  /*   96 */  0,
  /*   97 */  0,
  /*   98 */  0,
  /*   99 */  0,
  /*  100 */  0,
  /*  101 */  0,
  /*  102 */  0,
  /*  103 */  0,
  /*  104 */  0,
  /*  105 */  0,
  /*  106 */  0,
  /*  107 */  0,
  // last operand
  /*  108 */  0,
  // last operand class
  /*  109 */  rRegL_rule,
  /*  110 */  immI2_rule,
  /*  111 */  _LShiftL_rRegL_immI2_rule,
  /*  112 */  0,
  /*  113 */  _ConvI2L_rRegI__rule,
  /*  114 */  immI2_rule,
  /*  115 */  _LShiftL__ConvI2L_rRegI__immI2_rule,
  /*  116 */  0,
  /*  117 */  rRegL_rule,
  /*  118 */  _LShiftL_rRegL_immI2_rule,
  /*  119 */  _ConvI2L_rRegI__rule,
  /*  120 */  _LShiftL__ConvI2L_rRegI__immI2_rule,
  /*  121 */  0,
  /*  122 */  0,
  /*  123 */  immI8_rule,
  /*  124 */  0,
  /*  125 */  immI_24_rule,
  /*  126 */  0,
  /*  127 */  immI_24_rule,
  /*  128 */  immI_255_rule,
  /*  129 */  immI16_rule,
  /*  130 */  0,
  /*  131 */  immI_24_rule,
  /*  132 */  immI_16_rule,
  /*  133 */  immI_255_rule,
  /*  134 */  immI_65535_rule,
  /*  135 */  immU31_rule,
  /*  136 */  0,
  /*  137 */  0,
  /*  138 */  0,
  /*  139 */  0,
  /*  140 */  immL32_rule,
  /*  141 */  rFlagsReg_rule,
  /*  142 */  rRegI_rule,
  /*  143 */  rFlagsRegU_rule,
  /*  144 */  rFlagsRegUCF_rule,
  /*  145 */  _LoadI_memory__rule,
  /*  146 */  rRegN_rule,
  /*  147 */  rRegP_rule,
  /*  148 */  rRegL_rule,
  /*  149 */  _LoadL_memory__rule,
  /*  150 */  regF_rule,
  /*  151 */  regD_rule,
  /*  152 */  rRegI_rule,
  /*  153 */  _LoadI_memory__rule,
  /*  154 */  immI_rule,
  /*  155 */  immI1_rule,
  /*  156 */  immI_M1_rule,
  /*  157 */  rRegL_rule,
  /*  158 */  _LoadL_memory__rule,
  /*  159 */  immL32_rule,
  /*  160 */  immL1_rule,
  /*  161 */  immL_M1_rule,
  /*  162 */  rRegP_rule,
  /*  163 */  rRegI_rule,
  /*  164 */  rRegL_rule,
  /*  165 */  rRegN_rule,
  /*  166 */  rRegI_rule,
  /*  167 */  immI_rule,
  /*  168 */  rRegL_rule,
  /*  169 */  immL32_rule,
  /*  170 */  rRegI_rule,
  /*  171 */  _LoadI_memory__rule,
  /*  172 */  _LoadL_memory__rule,
  /*  173 */  immI1_rule,
  /*  174 */  immI8_rule,
  /*  175 */  rcx_RegI_rule,
  /*  176 */  immI1_rule,
  /*  177 */  immI8_rule,
  /*  178 */  rcx_RegI_rule,
  /*  179 */  immI1_rule,
  /*  180 */  immI8_rule,
  /*  181 */  rcx_RegI_rule,
  /*  182 */  immI1_rule,
  /*  183 */  immI8_rule,
  /*  184 */  rcx_RegI_rule,
  /*  185 */  immI1_rule,
  /*  186 */  immI8_rule,
  /*  187 */  rcx_RegI_rule,
  /*  188 */  immI1_rule,
  /*  189 */  immI8_rule,
  /*  190 */  rcx_RegI_rule,
  /*  191 */  immI_24_rule,
  /*  192 */  immI_16_rule,
  /*  193 */  immI1_rule,
  /*  194 */  immI_M1_rule,
  /*  195 */  immI8_rule,
  /*  196 */  immI8_rule,
  /*  197 */  rcx_RegI_rule,
  /*  198 */  rcx_RegI_rule,
  /*  199 */  _SubI_immI0_rcx_RegI_rule,
  /*  200 */  rcx_RegI_rule,
  /*  201 */  _SubI_immI_32_rcx_RegI_rule,
  /*  202 */  immI1_rule,
  /*  203 */  immI_M1_rule,
  /*  204 */  rcx_RegI_rule,
  /*  205 */  _SubI_immI0_rcx_RegI_rule,
  /*  206 */  _SubI_immI_32_rcx_RegI_rule,
  /*  207 */  immI1_rule,
  /*  208 */  immI_M1_rule,
  /*  209 */  immI8_rule,
  /*  210 */  immI8_rule,
  /*  211 */  rcx_RegI_rule,
  /*  212 */  _SubI_immI0_rcx_RegI_rule,
  /*  213 */  rcx_RegI_rule,
  /*  214 */  _SubI_immI_64_rcx_RegI_rule,
  /*  215 */  immI1_rule,
  /*  216 */  immI_M1_rule,
  /*  217 */  rcx_RegI_rule,
  /*  218 */  _SubI_immI0_rcx_RegI_rule,
  /*  219 */  _SubI_immI_64_rcx_RegI_rule,
  /*  220 */  immI_255_rule,
  /*  221 */  immI_65535_rule,
  /*  222 */  rRegI_rule,
  /*  223 */  _LoadI_memory__rule,
  /*  224 */  immI_rule,
  /*  225 */  rRegI_rule,
  /*  226 */  _LoadI_memory__rule,
  /*  227 */  immI_rule,
  /*  228 */  rRegI_rule,
  /*  229 */  _LoadI_memory__rule,
  /*  230 */  immI_rule,
  /*  231 */  rRegL_rule,
  /*  232 */  _LoadL_memory__rule,
  /*  233 */  immL32_rule,
  /*  234 */  0,
  /*  235 */  rRegL_rule,
  /*  236 */  _LoadL_memory__rule,
  /*  237 */  immL32_rule,
  /*  238 */  rRegL_rule,
  /*  239 */  _LoadL_memory__rule,
  /*  240 */  immL32_rule,
  /*  241 */  rRegI_rule,
  /*  242 */  rRegI_rule,
  /*  243 */  rRegI_rule,
  /*  244 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  245 */  0,
  /*  246 */  0,
  /*  247 */  rcx_RegI_rule,
  /*  248 */  rdx_RegI_rule,
  /*  249 */  rdx_RegI_rule,
  /*  250 */  immI_rule,
  /*  251 */  rax_RegI_rule,
  /*  252 */  rsi_RegP_rule,
  /*  253 */  immI_rule,
  /*  254 */  0,
  /*  255 */  0,
  /*  256 */  immL32_rule,
  /*  257 */  rax_RegP_rule,
  /*  258 */  0,
  /*  259 */  0,
  /*  260 */  0,
  /*  261 */  0,
  /*  262 */  0,
  /*  263 */  0,
  /*  264 */  0,
  // last internally defined operand
  /*  265 */  0,
  /*  266 */  0,
  /*  267 */  0,
  /*  268 */  0,
  /*  269 */  0,
  /*  270 */  0,
  /*  271 */  0,
  /*  272 */  0,
  /*  273 */  0,
  /*  274 */  0,
  /*  275 */  0,
  /*  276 */  0,
  /*  277 */  0,
  /*  278 */  0,
  /*  279 */  0,
  /*  280 */  0,
  /*  281 */  0,
  /*  282 */  0,
  /*  283 */  0,
  /*  284 */  0,
  /*  285 */  0,
  /*  286 */  0,
  /*  287 */  0,
  /*  288 */  0,
  /*  289 */  0,
  /*  290 */  0,
  /*  291 */  0,
  /*  292 */  0,
  /*  293 */  0,
  /*  294 */  0,
  /*  295 */  0,
  /*  296 */  0,
  /*  297 */  0,
  /*  298 */  0,
  /*  299 */  0,
  /*  300 */  0,
  /*  301 */  0,
  /*  302 */  0,
  /*  303 */  0,
  /*  304 */  0,
  /*  305 */  0,
  /*  306 */  0,
  /*  307 */  0,
  /*  308 */  0,
  /*  309 */  0,
  /*  310 */  0,
  /*  311 */  _Binary_rax_RegP_rRegP_rule,
  /*  312 */  _Binary_rax_RegI_rRegI_rule,
  /*  313 */  _Binary_rax_RegL_rRegL_rule,
  /*  314 */  regF_rule,
  /*  315 */  regF_rule,
  /*  316 */  _LoadF_memory__rule,
  /*  317 */  _LoadF_memory__rule,
  /*  318 */  immF_rule,
  /*  319 */  immF_rule,
  /*  320 */  regD_rule,
  /*  321 */  regD_rule,
  /*  322 */  _LoadD_memory__rule,
  /*  323 */  _LoadD_memory__rule,
  /*  324 */  immD_rule,
  /*  325 */  immD_rule,
  /*  326 */  0,
  /*  327 */  0,
  /*  328 */  0,
  /*  329 */  0,
  /*  330 */  0,
  /*  331 */  0,
  /*  332 */  0,
  /*  333 */  0,
  /*  334 */  0,
  /*  335 */  0,
  /*  336 */  0,
  /*  337 */  rRegI_rule,
  /*  338 */  immI_rule,
  /*  339 */  _LoadI_memory__rule,
  /*  340 */  immI0_rule,
  /*  341 */  immI0_rule,
  /*  342 */  immI0_rule,
  /*  343 */  immI0_rule,
  /*  344 */  rRegI_rule,
  /*  345 */  immI_rule,
  /*  346 */  _LoadI_memory__rule,
  /*  347 */  immI0_rule,
  /*  348 */  rRegP_rule,
  /*  349 */  _LoadP_memory__rule,
  /*  350 */  _LoadP_memory__rule,
  /*  351 */  immP0_rule,
  /*  352 */  immP0_rule,
  /*  353 */  immP0_rule,
  /*  354 */  rRegN_rule,
  /*  355 */  _LoadN_memory__rule,
  /*  356 */  immN_rule,
  /*  357 */  _LoadN_memory__rule,
  /*  358 */  immN0_rule,
  /*  359 */  immN0_rule,
  /*  360 */  immN0_rule,
  /*  361 */  rRegL_rule,
  /*  362 */  immL32_rule,
  /*  363 */  _LoadL_memory__rule,
  /*  364 */  immL0_rule,
  /*  365 */  immL0_rule,
  /*  366 */  immL0_rule,
  /*  367 */  immL0_rule,
  /*  368 */  rRegL_rule,
  /*  369 */  immL32_rule,
  /*  370 */  _LoadL_memory__rule,
  /*  371 */  immL0_rule,
  /*  372 */  immP0_rule,
  /*  373 */  rbx_RegP_rule,
  /*  374 */  rax_RegP_rule,
  /*  375 */  0,
  /*  376 */  0,
  /*  377 */  0,
  /*  378 */  0,
  /*  379 */  0,
  /*  380 */  0,
  /*  381 */  0,
  /*  382 */  0,
  /*  383 */  0,
  /*  384 */  0,
  /*  385 */  0,
  /*  386 */  0,
  /*  387 */  0,
  /*  388 */  0,
  /*  389 */  0,
  /*  390 */  0,
  /*  391 */  0,
  /*  392 */  0,
  /*  393 */  immI_24_rule,
  /*  394 */  0,
  /*  395 */  0,
  /*  396 */  immI_24_rule,
  /*  397 */  0,
  /*  398 */  0,
  /*  399 */  0,
  /*  400 */  0,
  /*  401 */  immI_24_rule,
  /*  402 */  immI_255_rule,
  /*  403 */  immI_16_rule,
  /*  404 */  immI_65535_rule,
  /*  405 */  0,
  /*  406 */  0,
  /*  407 */  0,
  /*  408 */  0,
  /*  409 */  immL_32bits_rule,
  /*  410 */  0,
  /*  411 */  0,
  /*  412 */  0,
  /*  413 */  0,
  /*  414 */  0,
  /*  415 */  0,
  /*  416 */  0,
  /*  417 */  0,
  /*  418 */  0,
  /*  419 */  0,
  /*  420 */  0,
  /*  421 */  0,
  /*  422 */  0,
  /*  423 */  0,
  /*  424 */  0,
  /*  425 */  0,
  /*  426 */  0,
  /*  427 */  0,
  /*  428 */  rRegI_rule,
  /*  429 */  rRegI_rule,
  /*  430 */  rRegI_rule,
  /*  431 */  rRegL_rule,
  /*  432 */  any_RegP_rule,
  /*  433 */  immP0_rule,
  /*  434 */  immP31_rule,
  /*  435 */  rRegN_rule,
  /*  436 */  immN0_rule,
  /*  437 */  immN_rule,
  /*  438 */  immI0_rule,
  /*  439 */  immI_rule,
  /*  440 */  immL0_rule,
  /*  441 */  immL32_rule,
  /*  442 */  immI0_rule,
  /*  443 */  immI16_rule,
  /*  444 */  immI0_rule,
  /*  445 */  immI8_rule,
  /*  446 */  immI0_rule,
  /*  447 */  immI0_rule,
  /*  448 */  regF_rule,
  /*  449 */  immF0_rule,
  /*  450 */  immF_rule,
  /*  451 */  regD_rule,
  /*  452 */  immD0_rule,
  /*  453 */  immD0_rule,
  /*  454 */  0,
  /*  455 */  0,
  /*  456 */  0,
  /*  457 */  0,
  /*  458 */  0,
  /*  459 */  0,
  /*  460 */  0,
  /*  461 */  0,
  /*  462 */  0,
  /*  463 */  0,
  /*  464 */  0,
  /*  465 */  0,
  /*  466 */  0,
  /*  467 */  0,
  /*  468 */  0,
  /*  469 */  0,
  /*  470 */  0,
  /*  471 */  0,
  /*  472 */  0,
  /*  473 */  0,
  /*  474 */  0,
  /*  475 */  0,
  /*  476 */  0,
  /*  477 */  0,
  /*  478 */  0,
  /*  479 */  0,
  /*  480 */  0,
  /*  481 */  0,
  /*  482 */  0,
  /*  483 */  _Binary_rRegI_rRegI_rule,
  /*  484 */  _Binary_rRegI_rRegI_rule,
  /*  485 */  _Binary_rRegI_rRegI_rule,
  /*  486 */  _Binary_rRegI__LoadI_memory__rule,
  /*  487 */  _Binary_rRegI__LoadI_memory__rule,
  /*  488 */  _Binary_rRegI__LoadI_memory__rule,
  /*  489 */  _Binary_rRegN_rRegN_rule,
  /*  490 */  _Binary_rRegN_rRegN_rule,
  /*  491 */  _Binary_rRegN_rRegN_rule,
  /*  492 */  _Binary_rRegP_rRegP_rule,
  /*  493 */  _Binary_rRegP_rRegP_rule,
  /*  494 */  _Binary_rRegP_rRegP_rule,
  /*  495 */  _Binary_rRegL_rRegL_rule,
  /*  496 */  _Binary_rRegL__LoadL_memory__rule,
  /*  497 */  _Binary_rRegL_rRegL_rule,
  /*  498 */  _Binary_rRegL_rRegL_rule,
  /*  499 */  _Binary_rRegL__LoadL_memory__rule,
  /*  500 */  _Binary_rRegL__LoadL_memory__rule,
  /*  501 */  _Binary_regF_regF_rule,
  /*  502 */  _Binary_regF_regF_rule,
  /*  503 */  _Binary_regF_regF_rule,
  /*  504 */  _Binary_regD_regD_rule,
  /*  505 */  _Binary_regD_regD_rule,
  /*  506 */  _Binary_regD_regD_rule,
  /*  507 */  rRegI_rule,
  /*  508 */  immI_rule,
  /*  509 */  _LoadI_memory__rule,
  /*  510 */  rRegI_rule,
  /*  511 */  _AddI__LoadI_memory__rRegI_rule,
  /*  512 */  _AddI_rRegI__LoadI_memory__rule,
  /*  513 */  _AddI__LoadI_memory__immI_rule,
  /*  514 */  immI1_rule,
  /*  515 */  _AddI__LoadI_memory__immI1_rule,
  /*  516 */  immI_M1_rule,
  /*  517 */  _AddI__LoadI_memory__immI_M1_rule,
  /*  518 */  immI_rule,
  /*  519 */  rRegL_rule,
  /*  520 */  immL32_rule,
  /*  521 */  _LoadL_memory__rule,
  /*  522 */  rRegL_rule,
  /*  523 */  _AddL__LoadL_memory__rRegL_rule,
  /*  524 */  _AddL_rRegL__LoadL_memory__rule,
  /*  525 */  _AddL__LoadL_memory__immL32_rule,
  /*  526 */  immL1_rule,
  /*  527 */  _AddL__LoadL_memory__immL1_rule,
  /*  528 */  immL_M1_rule,
  /*  529 */  _AddL__LoadL_memory__immL_M1_rule,
  /*  530 */  immL32_rule,
  /*  531 */  rRegL_rule,
  /*  532 */  immL32_rule,
  /*  533 */  immL32_rule,
  /*  534 */  0,
  /*  535 */  _Binary_rax_RegP_rRegP_rule,
  /*  536 */  _Binary_rax_RegL_rRegL_rule,
  /*  537 */  _Binary_rax_RegI_rRegI_rule,
  /*  538 */  immI_rule,
  /*  539 */  rRegI_rule,
  /*  540 */  immL32_rule,
  /*  541 */  rRegL_rule,
  /*  542 */  rRegI_rule,
  /*  543 */  rRegL_rule,
  /*  544 */  rRegP_rule,
  /*  545 */  rRegN_rule,
  /*  546 */  _Binary_rax_RegN_rRegN_rule,
  /*  547 */  rRegI_rule,
  /*  548 */  immI_rule,
  /*  549 */  _LoadI_memory__rule,
  /*  550 */  _SubI__LoadI_memory__rRegI_rule,
  /*  551 */  _SubI__LoadI_memory__immI_rule,
  /*  552 */  rRegL_rule,
  /*  553 */  immL32_rule,
  /*  554 */  _LoadL_memory__rule,
  /*  555 */  _SubL__LoadL_memory__rRegL_rule,
  /*  556 */  _SubL__LoadL_memory__immL32_rule,
  /*  557 */  _SubI_immI0_rRegI_rule,
  /*  558 */  rRegI_rule,
  /*  559 */  _SubI_immI0__LoadI_memory__rule,
  /*  560 */  rRegL_rule,
  /*  561 */  _SubL_immL0__LoadL_memory__rule,
  /*  562 */  rRegI_rule,
  /*  563 */  immI_rule,
  /*  564 */  _LoadI_memory__rule,
  /*  565 */  rRegI_rule,
  /*  566 */  immI_rule,
  /*  567 */  rRegL_rule,
  /*  568 */  immL32_rule,
  /*  569 */  _LoadL_memory__rule,
  /*  570 */  rRegL_rule,
  /*  571 */  immL32_rule,
  /*  572 */  rax_RegL_rule,
  /*  573 */  no_rax_rdx_RegI_rule,
  /*  574 */  no_rax_rdx_RegL_rule,
  /*  575 */  no_rax_rdx_RegI_rule,
  /*  576 */  no_rax_rdx_RegL_rule,
  /*  577 */  0,
  /*  578 */  0,
  /*  579 */  0,
  /*  580 */  0,
  /*  581 */  immL10_rule,
  /*  582 */  no_rax_rdx_RegI_rule,
  /*  583 */  no_rax_rdx_RegL_rule,
  /*  584 */  immI1_rule,
  /*  585 */  _LShiftI__LoadI_memory__immI1_rule,
  /*  586 */  immI8_rule,
  /*  587 */  _LShiftI__LoadI_memory__immI8_rule,
  /*  588 */  rcx_RegI_rule,
  /*  589 */  _LShiftI__LoadI_memory__rcx_RegI_rule,
  /*  590 */  immI1_rule,
  /*  591 */  _RShiftI__LoadI_memory__immI1_rule,
  /*  592 */  immI8_rule,
  /*  593 */  _RShiftI__LoadI_memory__immI8_rule,
  /*  594 */  rcx_RegI_rule,
  /*  595 */  _RShiftI__LoadI_memory__rcx_RegI_rule,
  /*  596 */  immI1_rule,
  /*  597 */  _URShiftI__LoadI_memory__immI1_rule,
  /*  598 */  immI8_rule,
  /*  599 */  _URShiftI__LoadI_memory__immI8_rule,
  /*  600 */  rcx_RegI_rule,
  /*  601 */  _URShiftI__LoadI_memory__rcx_RegI_rule,
  /*  602 */  immI1_rule,
  /*  603 */  _LShiftL__LoadL_memory__immI1_rule,
  /*  604 */  immI8_rule,
  /*  605 */  _LShiftL__LoadL_memory__immI8_rule,
  /*  606 */  rcx_RegI_rule,
  /*  607 */  _LShiftL__LoadL_memory__rcx_RegI_rule,
  /*  608 */  immI1_rule,
  /*  609 */  _RShiftL__LoadL_memory__immI1_rule,
  /*  610 */  immI8_rule,
  /*  611 */  _RShiftL__LoadL_memory__immI8_rule,
  /*  612 */  rcx_RegI_rule,
  /*  613 */  _RShiftL__LoadL_memory__rcx_RegI_rule,
  /*  614 */  immI1_rule,
  /*  615 */  _URShiftL__LoadL_memory__immI1_rule,
  /*  616 */  immI8_rule,
  /*  617 */  _URShiftL__LoadL_memory__immI8_rule,
  /*  618 */  rcx_RegI_rule,
  /*  619 */  _URShiftL__LoadL_memory__rcx_RegI_rule,
  /*  620 */  immI_24_rule,
  /*  621 */  immI_16_rule,
  /*  622 */  0,
  /*  623 */  0,
  /*  624 */  0,
  /*  625 */  _URShiftI_rRegI_immI_M1_rule,
  /*  626 */  _LShiftI_rRegI_immI1_rule,
  /*  627 */  _URShiftI_rRegI_immI8_rule,
  /*  628 */  _LShiftI_rRegI_immI8_rule,
  /*  629 */  _URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  630 */  _LShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  631 */  _URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  632 */  _LShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  633 */  0,
  /*  634 */  0,
  /*  635 */  0,
  /*  636 */  _LShiftI_rRegI_immI_M1_rule,
  /*  637 */  _URShiftI_rRegI_immI1_rule,
  /*  638 */  _LShiftI_rRegI_immI8_rule,
  /*  639 */  _URShiftI_rRegI_immI8_rule,
  /*  640 */  _LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI_rule,
  /*  641 */  _URShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  642 */  _LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI_rule,
  /*  643 */  _URShiftI_no_rcx_RegI_rcx_RegI_rule,
  /*  644 */  0,
  /*  645 */  0,
  /*  646 */  0,
  /*  647 */  _URShiftL_rRegL_immI_M1_rule,
  /*  648 */  _LShiftL_rRegL_immI1_rule,
  /*  649 */  _URShiftL_rRegL_immI8_rule,
  /*  650 */  _LShiftL_rRegL_immI8_rule,
  /*  651 */  _URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  652 */  _LShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  653 */  _URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  654 */  _LShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  655 */  0,
  /*  656 */  0,
  /*  657 */  0,
  /*  658 */  _LShiftL_rRegL_immI_M1_rule,
  /*  659 */  _URShiftL_rRegL_immI1_rule,
  /*  660 */  _LShiftL_rRegL_immI8_rule,
  /*  661 */  _URShiftL_rRegL_immI8_rule,
  /*  662 */  _LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI_rule,
  /*  663 */  _URShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  664 */  _LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI_rule,
  /*  665 */  _URShiftL_no_rcx_RegL_rcx_RegI_rule,
  /*  666 */  rRegI_rule,
  /*  667 */  immI_255_rule,
  /*  668 */  0,
  /*  669 */  immI_65535_rule,
  /*  670 */  0,
  /*  671 */  immI_rule,
  /*  672 */  _LoadI_memory__rule,
  /*  673 */  rRegI_rule,
  /*  674 */  _AndI__LoadI_memory__rRegI_rule,
  /*  675 */  _AndI_rRegI__LoadI_memory__rule,
  /*  676 */  _AndI__LoadI_memory__immI_rule,
  /*  677 */  rRegI_rule,
  /*  678 */  immI_rule,
  /*  679 */  _LoadI_memory__rule,
  /*  680 */  rRegI_rule,
  /*  681 */  _OrI__LoadI_memory__rRegI_rule,
  /*  682 */  _OrI_rRegI__LoadI_memory__rule,
  /*  683 */  _OrI__LoadI_memory__immI_rule,
  /*  684 */  rRegI_rule,
  /*  685 */  immI_M1_rule,
  /*  686 */  immI_rule,
  /*  687 */  _LoadI_memory__rule,
  /*  688 */  rRegI_rule,
  /*  689 */  _XorI__LoadI_memory__rRegI_rule,
  /*  690 */  _XorI_rRegI__LoadI_memory__rule,
  /*  691 */  _XorI__LoadI_memory__immI_rule,
  /*  692 */  rRegL_rule,
  /*  693 */  immL_255_rule,
  /*  694 */  immL_65535_rule,
  /*  695 */  immL32_rule,
  /*  696 */  _LoadL_memory__rule,
  /*  697 */  rRegL_rule,
  /*  698 */  _AndL__LoadL_memory__rRegL_rule,
  /*  699 */  _AndL_rRegL__LoadL_memory__rule,
  /*  700 */  _AndL__LoadL_memory__immL32_rule,
  /*  701 */  rRegL_rule,
  /*  702 */  _CastP2X_any_RegP__rule,
  /*  703 */  rRegL_rule,
  /*  704 */  immL32_rule,
  /*  705 */  _LoadL_memory__rule,
  /*  706 */  rRegL_rule,
  /*  707 */  _OrL__LoadL_memory__rRegL_rule,
  /*  708 */  _OrL_rRegL__LoadL_memory__rule,
  /*  709 */  _OrL__LoadL_memory__immL32_rule,
  /*  710 */  rRegL_rule,
  /*  711 */  immL_M1_rule,
  /*  712 */  immL32_rule,
  /*  713 */  _LoadL_memory__rule,
  /*  714 */  rRegL_rule,
  /*  715 */  _XorL__LoadL_memory__rRegL_rule,
  /*  716 */  _XorL_rRegL__LoadL_memory__rule,
  /*  717 */  _XorL__LoadL_memory__immL32_rule,
  /*  718 */  0,
  /*  719 */  0,
  /*  720 */  rRegI_rule,
  /*  721 */  immI0_rule,
  /*  722 */  _SubI_rRegI_rRegI_rule,
  /*  723 */  _AndI__CmpLTMask_rRegI_rRegI_rRegI_rule,
  /*  724 */  _SubI_rRegI_rRegI_rule,
  /*  725 */  _AndI_rRegI__CmpLTMask_rRegI_rRegI_rule,
  /*  726 */  rRegI_rule,
  /*  727 */  _CmpLTMask_rRegI_rRegI_rule,
  /*  728 */  regF_rule,
  /*  729 */  _LoadF_memory__rule,
  /*  730 */  immF_rule,
  /*  731 */  regD_rule,
  /*  732 */  _LoadD_memory__rule,
  /*  733 */  immD_rule,
  /*  734 */  0,
  /*  735 */  0,
  /*  736 */  0,
  /*  737 */  0,
  /*  738 */  0,
  /*  739 */  regD_rule,
  /*  740 */  0,
  /*  741 */  0,
  /*  742 */  0,
  /*  743 */  0,
  /*  744 */  0,
  /*  745 */  0,
  /*  746 */  0,
  /*  747 */  0,
  /*  748 */  0,
  /*  749 */  0,
  /*  750 */  0,
  /*  751 */  0,
  /*  752 */  0,
  /*  753 */  0,
  /*  754 */  0,
  /*  755 */  0,
  /*  756 */  0,
  /*  757 */  0,
  /*  758 */  0,
  /*  759 */  0,
  /*  760 */  immL_32bits_rule,
  /*  761 */  immL_32bits_rule,
  /*  762 */  immL_32bits_rule,
  /*  763 */  0,
  /*  764 */  0,
  /*  765 */  0,
  /*  766 */  0,
  /*  767 */  0,
  /*  768 */  rdi_RegP_rule,
  /*  769 */  rdi_RegP_rule,
  /*  770 */  _Binary_rsi_RegP_rdx_RegI_rule,
  /*  771 */  _Binary_rsi_RegP_immI_rule,
  /*  772 */  _Binary_rsi_RegP_rax_RegI_rule,
  /*  773 */  rcx_RegI_rule,
  /*  774 */  rsi_RegP_rule,
  /*  775 */  rRegL_rule,
  /*  776 */  0,
  /*  777 */  rRegI_rule,
  /*  778 */  0,
  /*  779 */  rRegI_rule,
  /*  780 */  0,
  /*  781 */  rFlagsReg_rule,
  /*  782 */  rFlagsReg_rule,
  /*  783 */  rFlagsRegU_rule,
  /*  784 */  rFlagsRegUCF_rule,
  /*  785 */  rFlagsRegU_rule,
  /*  786 */  rFlagsRegUCF_rule,
  /*  787 */  rFlagsRegUCF_rule,
  /*  788 */  rax_RegP_rule,
  /*  789 */  0,
  /*  790 */  rFlagsReg_rule,
  /*  791 */  rFlagsReg_rule,
  /*  792 */  rFlagsRegU_rule,
  /*  793 */  rFlagsRegUCF_rule,
  /*  794 */  rFlagsRegU_rule,
  /*  795 */  rFlagsRegUCF_rule,
  /*  796 */  rFlagsRegUCF_rule,
  /*  797 */  0,
  /*  798 */  0,
  /*  799 */  0,
  /*  800 */  0,
  /*  801 */  0,
  /*  802 */  0,
  /*  803 */  0,
  /*  804 */  0,
  /*  805 */  rbx_RegP_rule,
  /*  806 */  rax_RegP_rule,
  /*  807 */  0,
  /*  808 */  0,
  /*  809 */  0,
  /*  810 */  regF_rule,
  /*  811 */  _LoadF_memory__rule,
  /*  812 */  regF_rule,
  /*  813 */  immF_rule,
  /*  814 */  regF_rule,
  /*  815 */  _LoadF_memory__rule,
  /*  816 */  regF_rule,
  /*  817 */  immF_rule,
  /*  818 */  regD_rule,
  /*  819 */  _LoadD_memory__rule,
  /*  820 */  regD_rule,
  /*  821 */  immD_rule,
  /*  822 */  regD_rule,
  /*  823 */  _LoadD_memory__rule,
  /*  824 */  regD_rule,
  /*  825 */  immD_rule,
  /*  826 */  regF_rule,
  /*  827 */  _LoadF_memory__rule,
  /*  828 */  immF_rule,
  /*  829 */  regF_rule,
  /*  830 */  _LoadF_memory__rule,
  /*  831 */  immF_rule,
  /*  832 */  regD_rule,
  /*  833 */  _LoadD_memory__rule,
  /*  834 */  immD_rule,
  /*  835 */  regD_rule,
  /*  836 */  _LoadD_memory__rule,
  /*  837 */  immD_rule,
  /*  838 */  regF_rule,
  /*  839 */  _LoadF_memory__rule,
  /*  840 */  regF_rule,
  /*  841 */  immF_rule,
  /*  842 */  regF_rule,
  /*  843 */  _LoadF_memory__rule,
  /*  844 */  regF_rule,
  /*  845 */  immF_rule,
  /*  846 */  regD_rule,
  /*  847 */  _LoadD_memory__rule,
  /*  848 */  regD_rule,
  /*  849 */  immD_rule,
  /*  850 */  regD_rule,
  /*  851 */  _LoadD_memory__rule,
  /*  852 */  regD_rule,
  /*  853 */  immD_rule,
  /*  854 */  regF_rule,
  /*  855 */  _LoadF_memory__rule,
  /*  856 */  immF_rule,
  /*  857 */  regF_rule,
  /*  858 */  _LoadF_memory__rule,
  /*  859 */  immF_rule,
  /*  860 */  regD_rule,
  /*  861 */  _LoadD_memory__rule,
  /*  862 */  immD_rule,
  /*  863 */  regD_rule,
  /*  864 */  _LoadD_memory__rule,
  /*  865 */  immD_rule,
  /*  866 */  0,
  /*  867 */  0,
  /*  868 */  0,
  /*  869 */  0,
  /*  870 */  0,
  /*  871 */  0,
  /*  872 */  0,
  /*  873 */  0,
  /*  874 */  0,
  /*  875 */  0,
  /*  876 */  vecS_rule,
  /*  877 */  vecD_rule,
  /*  878 */  vecX_rule,
  /*  879 */  vecY_rule,
  /*  880 */  0,
  /*  881 */  0,
  /*  882 */  0,
  /*  883 */  0,
  /*  884 */  0,
  /*  885 */  0,
  /*  886 */  0,
  /*  887 */  0,
  /*  888 */  0,
  /*  889 */  0,
  /*  890 */  0,
  /*  891 */  0,
  /*  892 */  0,
  /*  893 */  0,
  /*  894 */  0,
  /*  895 */  0,
  /*  896 */  0,
  /*  897 */  0,
  /*  898 */  0,
  /*  899 */  0,
  /*  900 */  0,
  /*  901 */  0,
  /*  902 */  0,
  /*  903 */  0,
  /*  904 */  0,
  /*  905 */  0,
  /*  906 */  0,
  /*  907 */  0,
  /*  908 */  0,
  /*  909 */  0,
  /*  910 */  0,
  /*  911 */  0,
  /*  912 */  0,
  /*  913 */  0,
  /*  914 */  0,
  /*  915 */  0,
  /*  916 */  0,
  /*  917 */  0,
  /*  918 */  0,
  /*  919 */  0,
  /*  920 */  0,
  /*  921 */  0,
  /*  922 */  0,
  /*  923 */  0,
  /*  924 */  0,
  /*  925 */  0,
  /*  926 */  0,
  /*  927 */  0,
  /*  928 */  0,
  /*  929 */  0,
  /*  930 */  0,
  /*  931 */  0,
  /*  932 */  0,
  /*  933 */  0,
  /*  934 */  vecS_rule,
  /*  935 */  vecS_rule,
  /*  936 */  vecD_rule,
  /*  937 */  vecD_rule,
  /*  938 */  vecX_rule,
  /*  939 */  vecX_rule,
  /*  940 */  _LoadVector_memory__rule,
  /*  941 */  vecY_rule,
  /*  942 */  _LoadVector_memory__rule,
  /*  943 */  vecS_rule,
  /*  944 */  vecS_rule,
  /*  945 */  vecD_rule,
  /*  946 */  vecD_rule,
  /*  947 */  vecX_rule,
  /*  948 */  vecX_rule,
  /*  949 */  _LoadVector_memory__rule,
  /*  950 */  vecY_rule,
  /*  951 */  _LoadVector_memory__rule,
  /*  952 */  vecD_rule,
  /*  953 */  vecD_rule,
  /*  954 */  vecX_rule,
  /*  955 */  vecX_rule,
  /*  956 */  _LoadVector_memory__rule,
  /*  957 */  vecY_rule,
  /*  958 */  _LoadVector_memory__rule,
  /*  959 */  vecX_rule,
  /*  960 */  vecX_rule,
  /*  961 */  _LoadVector_memory__rule,
  /*  962 */  vecY_rule,
  /*  963 */  _LoadVector_memory__rule,
  /*  964 */  vecD_rule,
  /*  965 */  vecD_rule,
  /*  966 */  vecX_rule,
  /*  967 */  vecX_rule,
  /*  968 */  _LoadVector_memory__rule,
  /*  969 */  vecY_rule,
  /*  970 */  _LoadVector_memory__rule,
  /*  971 */  vecX_rule,
  /*  972 */  vecX_rule,
  /*  973 */  _LoadVector_memory__rule,
  /*  974 */  vecY_rule,
  /*  975 */  _LoadVector_memory__rule,
  /*  976 */  vecS_rule,
  /*  977 */  vecS_rule,
  /*  978 */  vecD_rule,
  /*  979 */  vecD_rule,
  /*  980 */  vecX_rule,
  /*  981 */  vecX_rule,
  /*  982 */  _LoadVector_memory__rule,
  /*  983 */  vecY_rule,
  /*  984 */  _LoadVector_memory__rule,
  /*  985 */  vecS_rule,
  /*  986 */  vecS_rule,
  /*  987 */  vecD_rule,
  /*  988 */  vecD_rule,
  /*  989 */  vecX_rule,
  /*  990 */  vecX_rule,
  /*  991 */  _LoadVector_memory__rule,
  /*  992 */  vecY_rule,
  /*  993 */  _LoadVector_memory__rule,
  /*  994 */  vecD_rule,
  /*  995 */  vecD_rule,
  /*  996 */  vecX_rule,
  /*  997 */  vecX_rule,
  /*  998 */  _LoadVector_memory__rule,
  /*  999 */  vecY_rule,
  /* 1000 */  _LoadVector_memory__rule,
  /* 1001 */  vecX_rule,
  /* 1002 */  vecX_rule,
  /* 1003 */  _LoadVector_memory__rule,
  /* 1004 */  vecY_rule,
  /* 1005 */  _LoadVector_memory__rule,
  /* 1006 */  vecD_rule,
  /* 1007 */  vecD_rule,
  /* 1008 */  vecX_rule,
  /* 1009 */  vecX_rule,
  /* 1010 */  _LoadVector_memory__rule,
  /* 1011 */  vecY_rule,
  /* 1012 */  _LoadVector_memory__rule,
  /* 1013 */  vecX_rule,
  /* 1014 */  vecX_rule,
  /* 1015 */  _LoadVector_memory__rule,
  /* 1016 */  vecY_rule,
  /* 1017 */  _LoadVector_memory__rule,
  /* 1018 */  vecS_rule,
  /* 1019 */  vecS_rule,
  /* 1020 */  vecD_rule,
  /* 1021 */  vecD_rule,
  /* 1022 */  vecX_rule,
  /* 1023 */  vecX_rule,
  /* 1024 */  _LoadVector_memory__rule,
  /* 1025 */  vecY_rule,
  /* 1026 */  _LoadVector_memory__rule,
  /* 1027 */  vecD_rule,
  /* 1028 */  vecD_rule,
  /* 1029 */  vecX_rule,
  /* 1030 */  vecX_rule,
  /* 1031 */  _LoadVector_memory__rule,
  /* 1032 */  vecY_rule,
  /* 1033 */  _LoadVector_memory__rule,
  /* 1034 */  vecD_rule,
  /* 1035 */  vecD_rule,
  /* 1036 */  vecX_rule,
  /* 1037 */  vecX_rule,
  /* 1038 */  _LoadVector_memory__rule,
  /* 1039 */  vecY_rule,
  /* 1040 */  _LoadVector_memory__rule,
  /* 1041 */  vecX_rule,
  /* 1042 */  vecX_rule,
  /* 1043 */  _LoadVector_memory__rule,
  /* 1044 */  vecY_rule,
  /* 1045 */  _LoadVector_memory__rule,
  /* 1046 */  vecD_rule,
  /* 1047 */  vecD_rule,
  /* 1048 */  vecX_rule,
  /* 1049 */  vecX_rule,
  /* 1050 */  _LoadVector_memory__rule,
  /* 1051 */  vecY_rule,
  /* 1052 */  _LoadVector_memory__rule,
  /* 1053 */  vecX_rule,
  /* 1054 */  vecX_rule,
  /* 1055 */  _LoadVector_memory__rule,
  /* 1056 */  vecY_rule,
  /* 1057 */  _LoadVector_memory__rule,
  /* 1058 */  vecS_rule,
  /* 1059 */  immI8_rule,
  /* 1060 */  vecS_rule,
  /* 1061 */  immI8_rule,
  /* 1062 */  vecS_rule,
  /* 1063 */  immI8_rule,
  /* 1064 */  vecS_rule,
  /* 1065 */  immI8_rule,
  /* 1066 */  vecS_rule,
  /* 1067 */  immI8_rule,
  /* 1068 */  vecS_rule,
  /* 1069 */  immI8_rule,
  /* 1070 */  vecS_rule,
  /* 1071 */  immI8_rule,
  /* 1072 */  vecS_rule,
  /* 1073 */  immI8_rule,
  /* 1074 */  vecS_rule,
  /* 1075 */  immI8_rule,
  /* 1076 */  vecS_rule,
  /* 1077 */  immI8_rule,
  /* 1078 */  vecS_rule,
  /* 1079 */  immI8_rule,
  /* 1080 */  vecS_rule,
  /* 1081 */  immI8_rule,
  /* 1082 */  vecS_rule,
  /* 1083 */  immI8_rule,
  /* 1084 */  vecS_rule,
  /* 1085 */  immI8_rule,
  /* 1086 */  vecS_rule,
  /* 1087 */  immI8_rule,
  /* 1088 */  vecS_rule,
  /* 1089 */  immI8_rule,
  /* 1090 */  vecS_rule,
  /* 1091 */  immI8_rule,
  /* 1092 */  vecS_rule,
  /* 1093 */  immI8_rule,
  /* 1094 */  vecS_rule,
  /* 1095 */  immI8_rule,
  /* 1096 */  vecS_rule,
  /* 1097 */  immI8_rule,
  /* 1098 */  vecS_rule,
  /* 1099 */  immI8_rule,
  /* 1100 */  vecS_rule,
  /* 1101 */  immI8_rule,
  /* 1102 */  vecS_rule,
  /* 1103 */  immI8_rule,
  /* 1104 */  vecS_rule,
  /* 1105 */  immI8_rule,
  /* 1106 */  vecS_rule,
  /* 1107 */  immI8_rule,
  /* 1108 */  vecS_rule,
  /* 1109 */  immI8_rule,
  /* 1110 */  vecS_rule,
  /* 1111 */  immI8_rule,
  /* 1112 */  vecS_rule,
  /* 1113 */  immI8_rule,
  /* 1114 */  vecS_rule,
  /* 1115 */  immI8_rule,
  /* 1116 */  vecS_rule,
  /* 1117 */  immI8_rule,
  /* 1118 */  vecS_rule,
  /* 1119 */  immI8_rule,
  /* 1120 */  vecS_rule,
  /* 1121 */  immI8_rule,
  /* 1122 */  vecS_rule,
  /* 1123 */  immI8_rule,
  /* 1124 */  vecS_rule,
  /* 1125 */  immI8_rule,
  /* 1126 */  vecS_rule,
  /* 1127 */  immI8_rule,
  /* 1128 */  vecS_rule,
  /* 1129 */  immI8_rule,
  /* 1130 */  vecS_rule,
  /* 1131 */  immI8_rule,
  /* 1132 */  vecS_rule,
  /* 1133 */  immI8_rule,
  /* 1134 */  vecS_rule,
  /* 1135 */  immI8_rule,
  /* 1136 */  vecS_rule,
  /* 1137 */  immI8_rule,
  /* 1138 */  vecS_rule,
  /* 1139 */  immI8_rule,
  /* 1140 */  vecS_rule,
  /* 1141 */  immI8_rule,
  /* 1142 */  vecS_rule,
  /* 1143 */  vecS_rule,
  /* 1144 */  vecD_rule,
  /* 1145 */  vecD_rule,
  /* 1146 */  vecX_rule,
  /* 1147 */  vecX_rule,
  /* 1148 */  _LoadVector_memory__rule,
  /* 1149 */  vecY_rule,
  /* 1150 */  _LoadVector_memory__rule,
  /* 1151 */  vecS_rule,
  /* 1152 */  vecS_rule,
  /* 1153 */  vecD_rule,
  /* 1154 */  vecD_rule,
  /* 1155 */  vecX_rule,
  /* 1156 */  vecX_rule,
  /* 1157 */  _LoadVector_memory__rule,
  /* 1158 */  vecY_rule,
  /* 1159 */  _LoadVector_memory__rule,
  /* 1160 */  vecS_rule,
  /* 1161 */  vecS_rule,
  /* 1162 */  vecD_rule,
  /* 1163 */  vecD_rule,
  /* 1164 */  vecX_rule,
  /* 1165 */  vecX_rule,
  /* 1166 */  _LoadVector_memory__rule,
  /* 1167 */  vecY_rule,
  /* 1168 */  _LoadVector_memory__rule,
  // last instruction
  0 // no trailing comma
};

const char        *ruleName[] = {
  /*    0 */  "UNIVERSE",
  /*    1 */  "LABEL",
  /*    2 */  "SREGI",
  /*    3 */  "SREGP",
  /*    4 */  "SREGF",
  /*    5 */  "SREGD",
  /*    6 */  "SREGL",
  /*    7 */  "METHOD",
  /*    8 */  "IMMI",
  /*    9 */  "IMMI0",
  /*   10 */  "IMMI1",
  /*   11 */  "IMMI_M1",
  /*   12 */  "IMMI2",
  /*   13 */  "IMMI8",
  /*   14 */  "IMMI16",
  /*   15 */  "IMMU31",
  /*   16 */  "IMMI_32",
  /*   17 */  "IMMI_64",
  /*   18 */  "IMMP",
  /*   19 */  "IMMP0",
  /*   20 */  "IMMN",
  /*   21 */  "IMMN0",
  /*   22 */  "IMMP31",
  /*   23 */  "IMML",
  /*   24 */  "IMML8",
  /*   25 */  "IMMUL32",
  /*   26 */  "IMML32",
  /*   27 */  "IMML0",
  /*   28 */  "IMML1",
  /*   29 */  "IMML_M1",
  /*   30 */  "IMML10",
  /*   31 */  "IMML_127",
  /*   32 */  "IMML_32BITS",
  /*   33 */  "IMMF0",
  /*   34 */  "IMMF",
  /*   35 */  "IMMD0",
  /*   36 */  "IMMD",
  /*   37 */  "IMMI_16",
  /*   38 */  "IMMI_24",
  /*   39 */  "IMMI_255",
  /*   40 */  "IMMI_65535",
  /*   41 */  "IMML_255",
  /*   42 */  "IMML_65535",
  /*   43 */  "RREGI",
  /*   44 */  "RAX_REGI",
  /*   45 */  "RBX_REGI",
  /*   46 */  "RCX_REGI",
  /*   47 */  "RDX_REGI",
  /*   48 */  "RDI_REGI",
  /*   49 */  "NO_RCX_REGI",
  /*   50 */  "NO_RAX_RDX_REGI",
  /*   51 */  "ANY_REGP",
  /*   52 */  "RREGP",
  /*   53 */  "RREGN",
  /*   54 */  "NO_RAX_REGP",
  /*   55 */  "NO_RBP_REGP",
  /*   56 */  "NO_RAX_RBX_REGP",
  /*   57 */  "RAX_REGP",
  /*   58 */  "RAX_REGN",
  /*   59 */  "RBX_REGP",
  /*   60 */  "RSI_REGP",
  /*   61 */  "RDI_REGP",
  /*   62 */  "RBP_REGP",
  /*   63 */  "R15_REGP",
  /*   64 */  "RREGL",
  /*   65 */  "NO_RAX_RDX_REGL",
  /*   66 */  "NO_RAX_REGL",
  /*   67 */  "NO_RCX_REGL",
  /*   68 */  "RAX_REGL",
  /*   69 */  "RCX_REGL",
  /*   70 */  "RDX_REGL",
  /*   71 */  "RFLAGSREG",
  /*   72 */  "RFLAGSREGU",
  /*   73 */  "RFLAGSREGUCF",
  /*   74 */  "REGF",
  /*   75 */  "REGD",
  /*   76 */  "INDIRECT",
  /*   77 */  "INDOFFSET8",
  /*   78 */  "INDOFFSET32",
  /*   79 */  "INDINDEXOFFSET",
  /*   80 */  "INDINDEX",
  /*   81 */  "INDINDEXSCALE",
  /*   82 */  "INDINDEXSCALEOFFSET",
  /*   83 */  "INDPOSINDEXOFFSET",
  /*   84 */  "INDPOSINDEXSCALEOFFSET",
  /*   85 */  "INDCOMPRESSEDOOPOFFSET",
  /*   86 */  "INDIRECTNARROW",
  /*   87 */  "INDOFFSET8NARROW",
  /*   88 */  "INDOFFSET32NARROW",
  /*   89 */  "INDINDEXOFFSETNARROW",
  /*   90 */  "INDINDEXNARROW",
  /*   91 */  "INDINDEXSCALENARROW",
  /*   92 */  "INDINDEXSCALEOFFSETNARROW",
  /*   93 */  "INDPOSINDEXOFFSETNARROW",
  /*   94 */  "INDPOSINDEXSCALEOFFSETNARROW",
  /*   95 */  "STACKSLOTP",
  /*   96 */  "STACKSLOTI",
  /*   97 */  "STACKSLOTF",
  /*   98 */  "STACKSLOTD",
  /*   99 */  "STACKSLOTL",
  /*  100 */  "CMPOP",
  /*  101 */  "CMPOPU",
  /*  102 */  "CMPOPUCF",
  /*  103 */  "CMPOPUCF2",
  /*  104 */  "VECS",
  /*  105 */  "VECD",
  /*  106 */  "VECX",
  /*  107 */  "VECY",
  // last operand
  /*  108 */  "MEMORY",
  // last operand class
  /*  109 */  "_AddP_any_RegP_rRegL",
  /*  110 */  "_LShiftL_rRegL_immI2",
  /*  111 */  "_AddP_any_RegP__LShiftL_rRegL_immI2",
  /*  112 */  "_ConvI2L_rRegI_",
  /*  113 */  "_AddP_any_RegP__ConvI2L_rRegI_",
  /*  114 */  "_LShiftL__ConvI2L_rRegI__immI2",
  /*  115 */  "_AddP_any_RegP__LShiftL__ConvI2L_rRegI__immI2",
  /*  116 */  "_DecodeN_rRegN_",
  /*  117 */  "_AddP__DecodeN_rRegN__rRegL",
  /*  118 */  "_AddP__DecodeN_rRegN___LShiftL_rRegL_immI2",
  /*  119 */  "_AddP__DecodeN_rRegN___ConvI2L_rRegI_",
  /*  120 */  "_AddP__DecodeN_rRegN___LShiftL__ConvI2L_rRegI__immI2",
  /*  121 */  "_LoadB_memory_",
  /*  122 */  "_LoadUB_memory_",
  /*  123 */  "_AndI__LoadUB_memory__immI8",
  /*  124 */  "_LoadS_memory_",
  /*  125 */  "_LShiftI__LoadS_memory__immI_24",
  /*  126 */  "_LoadUS_memory_",
  /*  127 */  "_LShiftI__LoadUS_memory__immI_24",
  /*  128 */  "_AndI__LoadUS_memory__immI_255",
  /*  129 */  "_AndI__LoadUS_memory__immI16",
  /*  130 */  "_LoadI_memory_",
  /*  131 */  "_LShiftI__LoadI_memory__immI_24",
  /*  132 */  "_LShiftI__LoadI_memory__immI_16",
  /*  133 */  "_AndI__LoadI_memory__immI_255",
  /*  134 */  "_AndI__LoadI_memory__immI_65535",
  /*  135 */  "_AndI__LoadI_memory__immU31",
  /*  136 */  "_ConvI2L__LoadI_memory__",
  /*  137 */  "_LoadL_memory_",
  /*  138 */  "_CastP2X_rRegP_",
  /*  139 */  "_CastP2X__DecodeN_rRegN__",
  /*  140 */  "_AddL__LShiftL_rRegL_immI2_immL32",
  /*  141 */  "_Binary_cmpOp_rFlagsReg",
  /*  142 */  "_Binary_rRegI_rRegI",
  /*  143 */  "_Binary_cmpOpU_rFlagsRegU",
  /*  144 */  "_Binary_cmpOpUCF_rFlagsRegUCF",
  /*  145 */  "_Binary_rRegI__LoadI_memory_",
  /*  146 */  "_Binary_rRegN_rRegN",
  /*  147 */  "_Binary_rRegP_rRegP",
  /*  148 */  "_Binary_rRegL_rRegL",
  /*  149 */  "_Binary_rRegL__LoadL_memory_",
  /*  150 */  "_Binary_regF_regF",
  /*  151 */  "_Binary_regD_regD",
  /*  152 */  "_AddI__LoadI_memory__rRegI",
  /*  153 */  "_AddI_rRegI__LoadI_memory_",
  /*  154 */  "_AddI__LoadI_memory__immI",
  /*  155 */  "_AddI__LoadI_memory__immI1",
  /*  156 */  "_AddI__LoadI_memory__immI_M1",
  /*  157 */  "_AddL__LoadL_memory__rRegL",
  /*  158 */  "_AddL_rRegL__LoadL_memory_",
  /*  159 */  "_AddL__LoadL_memory__immL32",
  /*  160 */  "_AddL__LoadL_memory__immL1",
  /*  161 */  "_AddL__LoadL_memory__immL_M1",
  /*  162 */  "_Binary_rax_RegP_rRegP",
  /*  163 */  "_Binary_rax_RegI_rRegI",
  /*  164 */  "_Binary_rax_RegL_rRegL",
  /*  165 */  "_Binary_rax_RegN_rRegN",
  /*  166 */  "_SubI__LoadI_memory__rRegI",
  /*  167 */  "_SubI__LoadI_memory__immI",
  /*  168 */  "_SubL__LoadL_memory__rRegL",
  /*  169 */  "_SubL__LoadL_memory__immL32",
  /*  170 */  "_SubI_immI0_rRegI",
  /*  171 */  "_SubI_immI0__LoadI_memory_",
  /*  172 */  "_SubL_immL0__LoadL_memory_",
  /*  173 */  "_LShiftI__LoadI_memory__immI1",
  /*  174 */  "_LShiftI__LoadI_memory__immI8",
  /*  175 */  "_LShiftI__LoadI_memory__rcx_RegI",
  /*  176 */  "_RShiftI__LoadI_memory__immI1",
  /*  177 */  "_RShiftI__LoadI_memory__immI8",
  /*  178 */  "_RShiftI__LoadI_memory__rcx_RegI",
  /*  179 */  "_URShiftI__LoadI_memory__immI1",
  /*  180 */  "_URShiftI__LoadI_memory__immI8",
  /*  181 */  "_URShiftI__LoadI_memory__rcx_RegI",
  /*  182 */  "_LShiftL__LoadL_memory__immI1",
  /*  183 */  "_LShiftL__LoadL_memory__immI8",
  /*  184 */  "_LShiftL__LoadL_memory__rcx_RegI",
  /*  185 */  "_RShiftL__LoadL_memory__immI1",
  /*  186 */  "_RShiftL__LoadL_memory__immI8",
  /*  187 */  "_RShiftL__LoadL_memory__rcx_RegI",
  /*  188 */  "_URShiftL__LoadL_memory__immI1",
  /*  189 */  "_URShiftL__LoadL_memory__immI8",
  /*  190 */  "_URShiftL__LoadL_memory__rcx_RegI",
  /*  191 */  "_LShiftI_rRegI_immI_24",
  /*  192 */  "_LShiftI_rRegI_immI_16",
  /*  193 */  "_LShiftI_rRegI_immI1",
  /*  194 */  "_URShiftI_rRegI_immI_M1",
  /*  195 */  "_LShiftI_rRegI_immI8",
  /*  196 */  "_URShiftI_rRegI_immI8",
  /*  197 */  "_LShiftI_no_rcx_RegI_rcx_RegI",
  /*  198 */  "_SubI_immI0_rcx_RegI",
  /*  199 */  "_URShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI",
  /*  200 */  "_SubI_immI_32_rcx_RegI",
  /*  201 */  "_URShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI",
  /*  202 */  "_URShiftI_rRegI_immI1",
  /*  203 */  "_LShiftI_rRegI_immI_M1",
  /*  204 */  "_URShiftI_no_rcx_RegI_rcx_RegI",
  /*  205 */  "_LShiftI_no_rcx_RegI__SubI_immI0_rcx_RegI",
  /*  206 */  "_LShiftI_no_rcx_RegI__SubI_immI_32_rcx_RegI",
  /*  207 */  "_LShiftL_rRegL_immI1",
  /*  208 */  "_URShiftL_rRegL_immI_M1",
  /*  209 */  "_LShiftL_rRegL_immI8",
  /*  210 */  "_URShiftL_rRegL_immI8",
  /*  211 */  "_LShiftL_no_rcx_RegL_rcx_RegI",
  /*  212 */  "_URShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI",
  /*  213 */  "_SubI_immI_64_rcx_RegI",
  /*  214 */  "_URShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI",
  /*  215 */  "_URShiftL_rRegL_immI1",
  /*  216 */  "_LShiftL_rRegL_immI_M1",
  /*  217 */  "_URShiftL_no_rcx_RegL_rcx_RegI",
  /*  218 */  "_LShiftL_no_rcx_RegL__SubI_immI0_rcx_RegI",
  /*  219 */  "_LShiftL_no_rcx_RegL__SubI_immI_64_rcx_RegI",
  /*  220 */  "_AndI_rRegI_immI_255",
  /*  221 */  "_AndI_rRegI_immI_65535",
  /*  222 */  "_AndI__LoadI_memory__rRegI",
  /*  223 */  "_AndI_rRegI__LoadI_memory_",
  /*  224 */  "_AndI__LoadI_memory__immI",
  /*  225 */  "_OrI__LoadI_memory__rRegI",
  /*  226 */  "_OrI_rRegI__LoadI_memory_",
  /*  227 */  "_OrI__LoadI_memory__immI",
  /*  228 */  "_XorI__LoadI_memory__rRegI",
  /*  229 */  "_XorI_rRegI__LoadI_memory_",
  /*  230 */  "_XorI__LoadI_memory__immI",
  /*  231 */  "_AndL__LoadL_memory__rRegL",
  /*  232 */  "_AndL_rRegL__LoadL_memory_",
  /*  233 */  "_AndL__LoadL_memory__immL32",
  /*  234 */  "_CastP2X_any_RegP_",
  /*  235 */  "_OrL__LoadL_memory__rRegL",
  /*  236 */  "_OrL_rRegL__LoadL_memory_",
  /*  237 */  "_OrL__LoadL_memory__immL32",
  /*  238 */  "_XorL__LoadL_memory__rRegL",
  /*  239 */  "_XorL_rRegL__LoadL_memory_",
  /*  240 */  "_XorL__LoadL_memory__immL32",
  /*  241 */  "_CmpLTMask_rRegI_rRegI",
  /*  242 */  "_AndI__CmpLTMask_rRegI_rRegI_rRegI",
  /*  243 */  "_SubI_rRegI_rRegI",
  /*  244 */  "_AndI_rRegI__CmpLTMask_rRegI_rRegI",
  /*  245 */  "_LoadF_memory_",
  /*  246 */  "_LoadD_memory_",
  /*  247 */  "_Binary_rdi_RegP_rcx_RegI",
  /*  248 */  "_Binary_rsi_RegP_rdx_RegI",
  /*  249 */  "_Binary_rdi_RegP_rdx_RegI",
  /*  250 */  "_Binary_rsi_RegP_immI",
  /*  251 */  "_Binary_rsi_RegP_rax_RegI",
  /*  252 */  "_Binary_rdi_RegP_rsi_RegP",
  /*  253 */  "_AndI_rRegI_immI",
  /*  254 */  "_LoadP_memory_",
  /*  255 */  "_LoadN_memory_",
  /*  256 */  "_AndL_rRegL_immL32",
  /*  257 */  "_PartialSubtypeCheck_rsi_RegP_rax_RegP",
  /*  258 */  "_ConvF2D_regF_",
  /*  259 */  "_SqrtD__ConvF2D_regF__",
  /*  260 */  "_ConvF2D__LoadF_memory__",
  /*  261 */  "_SqrtD__ConvF2D__LoadF_memory___",
  /*  262 */  "_ConvF2D_immF_",
  /*  263 */  "_SqrtD__ConvF2D_immF__",
  /*  264 */  "_LoadVector_memory_",
  // last internally defined operand
  /*  265 */  "storeSSI",
  /*  266 */  "storeSSL",
  /*  267 */  "storeSSP",
  /*  268 */  "storeSSF",
  /*  269 */  "storeSSD",
  /*  270 */  "leaP8",
  /*  271 */  "leaP32",
  /*  272 */  "leaPIdxOff",
  /*  273 */  "leaPIdxScale",
  /*  274 */  "leaPIdxScaleOff",
  /*  275 */  "leaPPosIdxOff",
  /*  276 */  "leaPPosIdxScaleOff",
  /*  277 */  "leaPCompressedOopOffset",
  /*  278 */  "leaP8Narrow",
  /*  279 */  "leaP32Narrow",
  /*  280 */  "leaPIdxOffNarrow",
  /*  281 */  "leaPIdxScaleNarrow",
  /*  282 */  "leaPIdxScaleOffNarrow",
  /*  283 */  "leaPPosIdxOffNarrow",
  /*  284 */  "leaPPosIdxScaleOffNarrow",
  /*  285 */  "loadConI",
  /*  286 */  "loadConI0",
  /*  287 */  "loadConL",
  /*  288 */  "loadConL0",
  /*  289 */  "loadConUL32",
  /*  290 */  "loadConL32",
  /*  291 */  "loadConP",
  /*  292 */  "loadConP0",
  /*  293 */  "loadConP31",
  /*  294 */  "loadConF",
  /*  295 */  "loadConN0",
  /*  296 */  "loadConN",
  /*  297 */  "loadConF0",
  /*  298 */  "loadConD",
  /*  299 */  "loadConD0",
  /*  300 */  "loadSSI",
  /*  301 */  "loadSSL",
  /*  302 */  "loadSSP",
  /*  303 */  "loadSSF",
  /*  304 */  "loadSSD",
  /*  305 */  "membar_volatile",
  /*  306 */  "castX2P",
  /*  307 */  "castP2X",
  /*  308 */  "checkCastPP",
  /*  309 */  "castPP",
  /*  310 */  "castII",
  /*  311 */  "storePConditional",
  /*  312 */  "storeIConditional",
  /*  313 */  "storeLConditional",
  /*  314 */  "cmpF_cc_reg",
  /*  315 */  "cmpF_cc_reg_CF",
  /*  316 */  "cmpF_cc_mem",
  /*  317 */  "cmpF_cc_memCF",
  /*  318 */  "cmpF_cc_imm",
  /*  319 */  "cmpF_cc_immCF",
  /*  320 */  "cmpD_cc_reg",
  /*  321 */  "cmpD_cc_reg_CF",
  /*  322 */  "cmpD_cc_mem",
  /*  323 */  "cmpD_cc_memCF",
  /*  324 */  "cmpD_cc_imm",
  /*  325 */  "cmpD_cc_immCF",
  /*  326 */  "roundFloat_nop",
  /*  327 */  "roundDouble_nop",
  /*  328 */  "MoveF2I_stack_reg",
  /*  329 */  "MoveI2F_stack_reg",
  /*  330 */  "MoveD2L_stack_reg",
  /*  331 */  "MoveL2D_stack_reg_partial",
  /*  332 */  "MoveL2D_stack_reg",
  /*  333 */  "MoveF2I_reg_reg",
  /*  334 */  "MoveD2L_reg_reg",
  /*  335 */  "MoveI2F_reg_reg",
  /*  336 */  "MoveL2D_reg_reg",
  /*  337 */  "compI_rReg",
  /*  338 */  "compI_rReg_imm",
  /*  339 */  "compI_rReg_mem",
  /*  340 */  "testI_reg",
  /*  341 */  "testI_reg_imm",
  /*  342 */  "testI_reg_mem",
  /*  343 */  "testI_reg_mem_0",
  /*  344 */  "compU_rReg",
  /*  345 */  "compU_rReg_imm",
  /*  346 */  "compU_rReg_mem",
  /*  347 */  "testU_reg",
  /*  348 */  "compP_rReg",
  /*  349 */  "compP_rReg_mem",
  /*  350 */  "compP_mem_rReg",
  /*  351 */  "testP_reg",
  /*  352 */  "testP_mem",
  /*  353 */  "testP_mem_reg0",
  /*  354 */  "compN_rReg",
  /*  355 */  "compN_rReg_mem",
  /*  356 */  "compN_rReg_imm",
  /*  357 */  "compN_mem_imm",
  /*  358 */  "testN_reg",
  /*  359 */  "testN_mem",
  /*  360 */  "testN_mem_reg0",
  /*  361 */  "compL_rReg",
  /*  362 */  "compL_rReg_imm",
  /*  363 */  "compL_rReg_mem",
  /*  364 */  "testL_reg",
  /*  365 */  "testL_reg_imm",
  /*  366 */  "testL_reg_mem",
  /*  367 */  "testL_reg_mem_0",
  /*  368 */  "compUL_rReg",
  /*  369 */  "compUL_rReg_imm",
  /*  370 */  "compUL_rReg_mem",
  /*  371 */  "testUL_reg",
  /*  372 */  "partialSubtypeCheck_vs_Zero",
  /*  373 */  "cmpFastLock",
  /*  374 */  "cmpFastUnlock",
  /*  375 */  "safePoint_poll",
  /*  376 */  "tlsLoadP",
  /*  377 */  "absF_reg",
  /*  378 */  "absF_reg_reg",
  /*  379 */  "absD_reg",
  /*  380 */  "absD_reg_reg",
  /*  381 */  "negF_reg",
  /*  382 */  "negF_reg_reg",
  /*  383 */  "negD_reg",
  /*  384 */  "negD_reg_reg",
  /*  385 */  "vshiftcnt",
  /*  386 */  "vshiftcnt_0",
  /*  387 */  "loadB",
  /*  388 */  "loadB2L",
  /*  389 */  "loadUB",
  /*  390 */  "loadUB2L",
  /*  391 */  "loadUB2L_immI8",
  /*  392 */  "loadS",
  /*  393 */  "loadS2B",
  /*  394 */  "loadS2L",
  /*  395 */  "loadUS",
  /*  396 */  "loadUS2B",
  /*  397 */  "loadUS2L",
  /*  398 */  "loadUS2L_immI_255",
  /*  399 */  "loadUS2L_immI16",
  /*  400 */  "loadI",
  /*  401 */  "loadI2B",
  /*  402 */  "loadI2UB",
  /*  403 */  "loadI2S",
  /*  404 */  "loadI2US",
  /*  405 */  "loadI2L",
  /*  406 */  "loadI2L_immI_255",
  /*  407 */  "loadI2L_immI_65535",
  /*  408 */  "loadI2L_immU31",
  /*  409 */  "loadUI2L",
  /*  410 */  "loadL",
  /*  411 */  "loadRange",
  /*  412 */  "loadP",
  /*  413 */  "loadN",
  /*  414 */  "loadKlass",
  /*  415 */  "loadNKlass",
  /*  416 */  "loadF",
  /*  417 */  "loadD_partial",
  /*  418 */  "loadD",
  /*  419 */  "prefetchr",
  /*  420 */  "prefetchrNTA",
  /*  421 */  "prefetchrT0",
  /*  422 */  "prefetchrT2",
  /*  423 */  "prefetchwNTA",
  /*  424 */  "prefetchAlloc",
  /*  425 */  "prefetchAllocNTA",
  /*  426 */  "prefetchAllocT0",
  /*  427 */  "prefetchAllocT2",
  /*  428 */  "storeB",
  /*  429 */  "storeC",
  /*  430 */  "storeI",
  /*  431 */  "storeL",
  /*  432 */  "storeP",
  /*  433 */  "storeImmP0",
  /*  434 */  "storeImmP",
  /*  435 */  "storeN",
  /*  436 */  "storeImmN0",
  /*  437 */  "storeImmN",
  /*  438 */  "storeImmI0",
  /*  439 */  "storeImmI",
  /*  440 */  "storeImmL0",
  /*  441 */  "storeImmL",
  /*  442 */  "storeImmC0",
  /*  443 */  "storeImmI16",
  /*  444 */  "storeImmB0",
  /*  445 */  "storeImmB",
  /*  446 */  "storeImmCM0_reg",
  /*  447 */  "storeImmCM0",
  /*  448 */  "storeF",
  /*  449 */  "storeF0",
  /*  450 */  "storeF_imm",
  /*  451 */  "storeD",
  /*  452 */  "storeD0_imm",
  /*  453 */  "storeD0",
  /*  454 */  "bytes_reverse_int",
  /*  455 */  "bytes_reverse_long",
  /*  456 */  "bytes_reverse_unsigned_short",
  /*  457 */  "bytes_reverse_short",
  /*  458 */  "countLeadingZerosI",
  /*  459 */  "countLeadingZerosI_bsr",
  /*  460 */  "countLeadingZerosL",
  /*  461 */  "countLeadingZerosL_bsr",
  /*  462 */  "countTrailingZerosI",
  /*  463 */  "countTrailingZerosL",
  /*  464 */  "popCountI",
  /*  465 */  "popCountI_mem",
  /*  466 */  "popCountL",
  /*  467 */  "popCountL_mem",
  /*  468 */  "membar_acquire",
  /*  469 */  "membar_acquire_lock",
  /*  470 */  "membar_release",
  /*  471 */  "membar_release_lock",
  /*  472 */  "unnecessary_membar_volatile",
  /*  473 */  "membar_storestore",
  /*  474 */  "convP2I",
  /*  475 */  "convN2I",
  /*  476 */  "encodeHeapOop",
  /*  477 */  "encodeHeapOop_not_null",
  /*  478 */  "decodeHeapOop",
  /*  479 */  "decodeHeapOop_not_null",
  /*  480 */  "jumpXtnd_offset",
  /*  481 */  "jumpXtnd_addr",
  /*  482 */  "jumpXtnd",
  /*  483 */  "cmovI_reg",
  /*  484 */  "cmovI_regU",
  /*  485 */  "cmovI_regUCF",
  /*  486 */  "cmovI_mem",
  /*  487 */  "cmovI_memU",
  /*  488 */  "cmovI_memUCF",
  /*  489 */  "cmovN_reg",
  /*  490 */  "cmovN_regU",
  /*  491 */  "cmovN_regUCF",
  /*  492 */  "cmovP_reg",
  /*  493 */  "cmovP_regU",
  /*  494 */  "cmovP_regUCF",
  /*  495 */  "cmovL_reg",
  /*  496 */  "cmovL_mem",
  /*  497 */  "cmovL_regU",
  /*  498 */  "cmovL_regUCF",
  /*  499 */  "cmovL_memU",
  /*  500 */  "cmovL_memUCF",
  /*  501 */  "cmovF_reg",
  /*  502 */  "cmovF_regU",
  /*  503 */  "cmovF_regUCF",
  /*  504 */  "cmovD_reg",
  /*  505 */  "cmovD_regU",
  /*  506 */  "cmovD_regUCF",
  /*  507 */  "addI_rReg",
  /*  508 */  "addI_rReg_imm",
  /*  509 */  "addI_rReg_mem",
  /*  510 */  "addI_rReg_mem_0",
  /*  511 */  "addI_mem_rReg",
  /*  512 */  "addI_mem_rReg_0",
  /*  513 */  "addI_mem_imm",
  /*  514 */  "incI_rReg",
  /*  515 */  "incI_mem",
  /*  516 */  "decI_rReg",
  /*  517 */  "decI_mem",
  /*  518 */  "leaI_rReg_immI",
  /*  519 */  "addL_rReg",
  /*  520 */  "addL_rReg_imm",
  /*  521 */  "addL_rReg_mem",
  /*  522 */  "addL_rReg_mem_0",
  /*  523 */  "addL_mem_rReg",
  /*  524 */  "addL_mem_rReg_0",
  /*  525 */  "addL_mem_imm",
  /*  526 */  "incL_rReg",
  /*  527 */  "incL_mem",
  /*  528 */  "decL_rReg",
  /*  529 */  "decL_mem",
  /*  530 */  "leaL_rReg_immL",
  /*  531 */  "addP_rReg",
  /*  532 */  "addP_rReg_imm",
  /*  533 */  "leaP_rReg_imm",
  /*  534 */  "loadPLocked",
  /*  535 */  "compareAndSwapP",
  /*  536 */  "compareAndSwapL",
  /*  537 */  "compareAndSwapI",
  /*  538 */  "xaddI_no_res",
  /*  539 */  "xaddI",
  /*  540 */  "xaddL_no_res",
  /*  541 */  "xaddL",
  /*  542 */  "xchgI",
  /*  543 */  "xchgL",
  /*  544 */  "xchgP",
  /*  545 */  "xchgN",
  /*  546 */  "compareAndSwapN",
  /*  547 */  "subI_rReg",
  /*  548 */  "subI_rReg_imm",
  /*  549 */  "subI_rReg_mem",
  /*  550 */  "subI_mem_rReg",
  /*  551 */  "subI_mem_imm",
  /*  552 */  "subL_rReg",
  /*  553 */  "subL_rReg_imm",
  /*  554 */  "subL_rReg_mem",
  /*  555 */  "subL_mem_rReg",
  /*  556 */  "subL_mem_imm",
  /*  557 */  "subP_rReg",
  /*  558 */  "negI_rReg",
  /*  559 */  "negI_mem",
  /*  560 */  "negL_rReg",
  /*  561 */  "negL_mem",
  /*  562 */  "mulI_rReg",
  /*  563 */  "mulI_rReg_imm",
  /*  564 */  "mulI_mem",
  /*  565 */  "mulI_mem_0",
  /*  566 */  "mulI_mem_imm",
  /*  567 */  "mulL_rReg",
  /*  568 */  "mulL_rReg_imm",
  /*  569 */  "mulL_mem",
  /*  570 */  "mulL_mem_0",
  /*  571 */  "mulL_mem_imm",
  /*  572 */  "mulHiL_rReg",
  /*  573 */  "divI_rReg",
  /*  574 */  "divL_rReg",
  /*  575 */  "divModI_rReg_divmod",
  /*  576 */  "divModL_rReg_divmod",
  /*  577 */  "loadConL_0x6666666666666667",
  /*  578 */  "mul_hi",
  /*  579 */  "sarL_rReg_63",
  /*  580 */  "sarL_rReg_2",
  /*  581 */  "divL_10",
  /*  582 */  "modI_rReg",
  /*  583 */  "modL_rReg",
  /*  584 */  "salI_rReg_1",
  /*  585 */  "salI_mem_1",
  /*  586 */  "salI_rReg_imm",
  /*  587 */  "salI_mem_imm",
  /*  588 */  "salI_rReg_CL",
  /*  589 */  "salI_mem_CL",
  /*  590 */  "sarI_rReg_1",
  /*  591 */  "sarI_mem_1",
  /*  592 */  "sarI_rReg_imm",
  /*  593 */  "sarI_mem_imm",
  /*  594 */  "sarI_rReg_CL",
  /*  595 */  "sarI_mem_CL",
  /*  596 */  "shrI_rReg_1",
  /*  597 */  "shrI_mem_1",
  /*  598 */  "shrI_rReg_imm",
  /*  599 */  "shrI_mem_imm",
  /*  600 */  "shrI_rReg_CL",
  /*  601 */  "shrI_mem_CL",
  /*  602 */  "salL_rReg_1",
  /*  603 */  "salL_mem_1",
  /*  604 */  "salL_rReg_imm",
  /*  605 */  "salL_mem_imm",
  /*  606 */  "salL_rReg_CL",
  /*  607 */  "salL_mem_CL",
  /*  608 */  "sarL_rReg_1",
  /*  609 */  "sarL_mem_1",
  /*  610 */  "sarL_rReg_imm",
  /*  611 */  "sarL_mem_imm",
  /*  612 */  "sarL_rReg_CL",
  /*  613 */  "sarL_mem_CL",
  /*  614 */  "shrL_rReg_1",
  /*  615 */  "shrL_mem_1",
  /*  616 */  "shrL_rReg_imm",
  /*  617 */  "shrL_mem_imm",
  /*  618 */  "shrL_rReg_CL",
  /*  619 */  "shrL_mem_CL",
  /*  620 */  "i2b",
  /*  621 */  "i2s",
  /*  622 */  "rolI_rReg_imm1",
  /*  623 */  "rolI_rReg_imm8",
  /*  624 */  "rolI_rReg_CL",
  /*  625 */  "rolI_rReg_i1",
  /*  626 */  "rolI_rReg_i1_0",
  /*  627 */  "rolI_rReg_i8",
  /*  628 */  "rolI_rReg_i8_0",
  /*  629 */  "rolI_rReg_Var_C0",
  /*  630 */  "rolI_rReg_Var_C0_0",
  /*  631 */  "rolI_rReg_Var_C32",
  /*  632 */  "rolI_rReg_Var_C32_0",
  /*  633 */  "rorI_rReg_imm1",
  /*  634 */  "rorI_rReg_imm8",
  /*  635 */  "rorI_rReg_CL",
  /*  636 */  "rorI_rReg_i1",
  /*  637 */  "rorI_rReg_i1_0",
  /*  638 */  "rorI_rReg_i8",
  /*  639 */  "rorI_rReg_i8_0",
  /*  640 */  "rorI_rReg_Var_C0",
  /*  641 */  "rorI_rReg_Var_C0_0",
  /*  642 */  "rorI_rReg_Var_C32",
  /*  643 */  "rorI_rReg_Var_C32_0",
  /*  644 */  "rolL_rReg_imm1",
  /*  645 */  "rolL_rReg_imm8",
  /*  646 */  "rolL_rReg_CL",
  /*  647 */  "rolL_rReg_i1",
  /*  648 */  "rolL_rReg_i1_0",
  /*  649 */  "rolL_rReg_i8",
  /*  650 */  "rolL_rReg_i8_0",
  /*  651 */  "rolL_rReg_Var_C0",
  /*  652 */  "rolL_rReg_Var_C0_0",
  /*  653 */  "rolL_rReg_Var_C64",
  /*  654 */  "rolL_rReg_Var_C64_0",
  /*  655 */  "rorL_rReg_imm1",
  /*  656 */  "rorL_rReg_imm8",
  /*  657 */  "rorL_rReg_CL",
  /*  658 */  "rorL_rReg_i1",
  /*  659 */  "rorL_rReg_i1_0",
  /*  660 */  "rorL_rReg_i8",
  /*  661 */  "rorL_rReg_i8_0",
  /*  662 */  "rorL_rReg_Var_C0",
  /*  663 */  "rorL_rReg_Var_C0_0",
  /*  664 */  "rorL_rReg_Var_C64",
  /*  665 */  "rorL_rReg_Var_C64_0",
  /*  666 */  "andI_rReg",
  /*  667 */  "andI_rReg_imm255",
  /*  668 */  "andI2L_rReg_imm255",
  /*  669 */  "andI_rReg_imm65535",
  /*  670 */  "andI2L_rReg_imm65535",
  /*  671 */  "andI_rReg_imm",
  /*  672 */  "andI_rReg_mem",
  /*  673 */  "andI_rReg_mem_0",
  /*  674 */  "andI_mem_rReg",
  /*  675 */  "andI_mem_rReg_0",
  /*  676 */  "andI_mem_imm",
  /*  677 */  "orI_rReg",
  /*  678 */  "orI_rReg_imm",
  /*  679 */  "orI_rReg_mem",
  /*  680 */  "orI_rReg_mem_0",
  /*  681 */  "orI_mem_rReg",
  /*  682 */  "orI_mem_rReg_0",
  /*  683 */  "orI_mem_imm",
  /*  684 */  "xorI_rReg",
  /*  685 */  "xorI_rReg_im1",
  /*  686 */  "xorI_rReg_imm",
  /*  687 */  "xorI_rReg_mem",
  /*  688 */  "xorI_rReg_mem_0",
  /*  689 */  "xorI_mem_rReg",
  /*  690 */  "xorI_mem_rReg_0",
  /*  691 */  "xorI_mem_imm",
  /*  692 */  "andL_rReg",
  /*  693 */  "andL_rReg_imm255",
  /*  694 */  "andL_rReg_imm65535",
  /*  695 */  "andL_rReg_imm",
  /*  696 */  "andL_rReg_mem",
  /*  697 */  "andL_rReg_mem_0",
  /*  698 */  "andL_mem_rReg",
  /*  699 */  "andL_mem_rReg_0",
  /*  700 */  "andL_mem_imm",
  /*  701 */  "orL_rReg",
  /*  702 */  "orL_rReg_castP2X",
  /*  703 */  "orL_rReg_castP2X_0",
  /*  704 */  "orL_rReg_imm",
  /*  705 */  "orL_rReg_mem",
  /*  706 */  "orL_rReg_mem_0",
  /*  707 */  "orL_mem_rReg",
  /*  708 */  "orL_mem_rReg_0",
  /*  709 */  "orL_mem_imm",
  /*  710 */  "xorL_rReg",
  /*  711 */  "xorL_rReg_im1",
  /*  712 */  "xorL_rReg_imm",
  /*  713 */  "xorL_rReg_mem",
  /*  714 */  "xorL_rReg_mem_0",
  /*  715 */  "xorL_mem_rReg",
  /*  716 */  "xorL_mem_rReg_0",
  /*  717 */  "xorL_mem_imm",
  /*  718 */  "convI2B",
  /*  719 */  "convP2B",
  /*  720 */  "cmpLTMask",
  /*  721 */  "cmpLTMask0",
  /*  722 */  "cadd_cmpLTMask",
  /*  723 */  "cadd_cmpLTMask_1",
  /*  724 */  "cadd_cmpLTMask_0",
  /*  725 */  "cadd_cmpLTMask_2",
  /*  726 */  "and_cmpLTMask",
  /*  727 */  "and_cmpLTMask_0",
  /*  728 */  "cmpF_reg",
  /*  729 */  "cmpF_mem",
  /*  730 */  "cmpF_imm",
  /*  731 */  "cmpD_reg",
  /*  732 */  "cmpD_mem",
  /*  733 */  "cmpD_imm",
  /*  734 */  "cosD_reg",
  /*  735 */  "sinD_reg",
  /*  736 */  "tanD_reg",
  /*  737 */  "log10D_reg",
  /*  738 */  "logD_reg",
  /*  739 */  "powD_reg",
  /*  740 */  "expD_reg",
  /*  741 */  "convF2D_reg_reg",
  /*  742 */  "convF2D_reg_mem",
  /*  743 */  "convD2F_reg_reg",
  /*  744 */  "convD2F_reg_mem",
  /*  745 */  "convF2I_reg_reg",
  /*  746 */  "convF2L_reg_reg",
  /*  747 */  "convD2I_reg_reg",
  /*  748 */  "convD2L_reg_reg",
  /*  749 */  "convI2F_reg_reg",
  /*  750 */  "convI2F_reg_mem",
  /*  751 */  "convI2D_reg_reg",
  /*  752 */  "convI2D_reg_mem",
  /*  753 */  "convXI2F_reg",
  /*  754 */  "convXI2D_reg",
  /*  755 */  "convL2F_reg_reg",
  /*  756 */  "convL2F_reg_mem",
  /*  757 */  "convL2D_reg_reg",
  /*  758 */  "convL2D_reg_mem",
  /*  759 */  "convI2L_reg_reg",
  /*  760 */  "convI2L_reg_reg_zex",
  /*  761 */  "convI2L_reg_mem_zex",
  /*  762 */  "zerox_long_reg_reg",
  /*  763 */  "convL2I_reg_reg",
  /*  764 */  "MoveF2I_reg_stack",
  /*  765 */  "MoveI2F_reg_stack",
  /*  766 */  "MoveD2L_reg_stack",
  /*  767 */  "MoveL2D_reg_stack",
  /*  768 */  "rep_stos",
  /*  769 */  "rep_fast_stosb",
  /*  770 */  "string_compare",
  /*  771 */  "string_indexof_con",
  /*  772 */  "string_indexof",
  /*  773 */  "string_equals",
  /*  774 */  "array_equals",
  /*  775 */  "cmpL3_reg_reg",
  /*  776 */  "cmovI_reg_g",
  /*  777 */  "minI_rReg",
  /*  778 */  "cmovI_reg_l",
  /*  779 */  "maxI_rReg",
  /*  780 */  "jmpDir",
  /*  781 */  "jmpCon",
  /*  782 */  "jmpLoopEnd",
  /*  783 */  "jmpLoopEndU",
  /*  784 */  "jmpLoopEndUCF",
  /*  785 */  "jmpConU",
  /*  786 */  "jmpConUCF",
  /*  787 */  "jmpConUCF2",
  /*  788 */  "partialSubtypeCheck",
  /*  789 */  "jmpDir_short",
  /*  790 */  "jmpCon_short",
  /*  791 */  "jmpLoopEnd_short",
  /*  792 */  "jmpLoopEndU_short",
  /*  793 */  "jmpLoopEndUCF_short",
  /*  794 */  "jmpConU_short",
  /*  795 */  "jmpConUCF_short",
  /*  796 */  "jmpConUCF2_short",
  /*  797 */  "safePoint_poll_far",
  /*  798 */  "CallStaticJavaDirect",
  /*  799 */  "CallStaticJavaHandle",
  /*  800 */  "CallDynamicJavaDirect",
  /*  801 */  "CallRuntimeDirect",
  /*  802 */  "CallLeafDirect",
  /*  803 */  "CallLeafNoFPDirect",
  /*  804 */  "Ret",
  /*  805 */  "TailCalljmpInd",
  /*  806 */  "tailjmpInd",
  /*  807 */  "CreateException",
  /*  808 */  "RethrowException",
  /*  809 */  "ShouldNotReachHere",
  /*  810 */  "addF_reg",
  /*  811 */  "addF_mem",
  /*  812 */  "addF_mem_0",
  /*  813 */  "addF_imm",
  /*  814 */  "addF_reg_reg",
  /*  815 */  "addF_reg_mem",
  /*  816 */  "addF_reg_mem_0",
  /*  817 */  "addF_reg_imm",
  /*  818 */  "addD_reg",
  /*  819 */  "addD_mem",
  /*  820 */  "addD_mem_0",
  /*  821 */  "addD_imm",
  /*  822 */  "addD_reg_reg",
  /*  823 */  "addD_reg_mem",
  /*  824 */  "addD_reg_mem_0",
  /*  825 */  "addD_reg_imm",
  /*  826 */  "subF_reg",
  /*  827 */  "subF_mem",
  /*  828 */  "subF_imm",
  /*  829 */  "subF_reg_reg",
  /*  830 */  "subF_reg_mem",
  /*  831 */  "subF_reg_imm",
  /*  832 */  "subD_reg",
  /*  833 */  "subD_mem",
  /*  834 */  "subD_imm",
  /*  835 */  "subD_reg_reg",
  /*  836 */  "subD_reg_mem",
  /*  837 */  "subD_reg_imm",
  /*  838 */  "mulF_reg",
  /*  839 */  "mulF_mem",
  /*  840 */  "mulF_mem_0",
  /*  841 */  "mulF_imm",
  /*  842 */  "mulF_reg_reg",
  /*  843 */  "mulF_reg_mem",
  /*  844 */  "mulF_reg_mem_0",
  /*  845 */  "mulF_reg_imm",
  /*  846 */  "mulD_reg",
  /*  847 */  "mulD_mem",
  /*  848 */  "mulD_mem_0",
  /*  849 */  "mulD_imm",
  /*  850 */  "mulD_reg_reg",
  /*  851 */  "mulD_reg_mem",
  /*  852 */  "mulD_reg_mem_0",
  /*  853 */  "mulD_reg_imm",
  /*  854 */  "divF_reg",
  /*  855 */  "divF_mem",
  /*  856 */  "divF_imm",
  /*  857 */  "divF_reg_reg",
  /*  858 */  "divF_reg_mem",
  /*  859 */  "divF_reg_imm",
  /*  860 */  "divD_reg",
  /*  861 */  "divD_mem",
  /*  862 */  "divD_imm",
  /*  863 */  "divD_reg_reg",
  /*  864 */  "divD_reg_mem",
  /*  865 */  "divD_reg_imm",
  /*  866 */  "sqrtF_reg",
  /*  867 */  "sqrtF_mem",
  /*  868 */  "sqrtF_imm",
  /*  869 */  "sqrtD_reg",
  /*  870 */  "sqrtD_mem",
  /*  871 */  "sqrtD_imm",
  /*  872 */  "loadV4",
  /*  873 */  "loadV8",
  /*  874 */  "loadV16",
  /*  875 */  "loadV32",
  /*  876 */  "storeV4",
  /*  877 */  "storeV8",
  /*  878 */  "storeV16",
  /*  879 */  "storeV32",
  /*  880 */  "Repl4B",
  /*  881 */  "Repl8B",
  /*  882 */  "Repl16B",
  /*  883 */  "Repl32B",
  /*  884 */  "Repl4B_imm",
  /*  885 */  "Repl8B_imm",
  /*  886 */  "Repl16B_imm",
  /*  887 */  "Repl32B_imm",
  /*  888 */  "Repl4B_zero",
  /*  889 */  "Repl8B_zero",
  /*  890 */  "Repl16B_zero",
  /*  891 */  "Repl32B_zero",
  /*  892 */  "Repl2S",
  /*  893 */  "Repl4S",
  /*  894 */  "Repl8S",
  /*  895 */  "Repl16S",
  /*  896 */  "Repl2S_imm",
  /*  897 */  "Repl4S_imm",
  /*  898 */  "Repl8S_imm",
  /*  899 */  "Repl16S_imm",
  /*  900 */  "Repl2S_zero",
  /*  901 */  "Repl4S_zero",
  /*  902 */  "Repl8S_zero",
  /*  903 */  "Repl16S_zero",
  /*  904 */  "Repl2I",
  /*  905 */  "Repl4I",
  /*  906 */  "Repl8I",
  /*  907 */  "Repl2I_imm",
  /*  908 */  "Repl4I_imm",
  /*  909 */  "Repl8I_imm",
  /*  910 */  "Repl2I_mem",
  /*  911 */  "Repl4I_mem",
  /*  912 */  "Repl8I_mem",
  /*  913 */  "Repl2I_zero",
  /*  914 */  "Repl4I_zero",
  /*  915 */  "Repl8I_zero",
  /*  916 */  "Repl2L",
  /*  917 */  "Repl4L",
  /*  918 */  "Repl2L_imm",
  /*  919 */  "Repl4L_imm",
  /*  920 */  "Repl2L_mem",
  /*  921 */  "Repl4L_mem",
  /*  922 */  "Repl2L_zero",
  /*  923 */  "Repl4L_zero",
  /*  924 */  "Repl2F",
  /*  925 */  "Repl4F",
  /*  926 */  "Repl8F",
  /*  927 */  "Repl2F_zero",
  /*  928 */  "Repl4F_zero",
  /*  929 */  "Repl8F_zero",
  /*  930 */  "Repl2D",
  /*  931 */  "Repl4D",
  /*  932 */  "Repl2D_zero",
  /*  933 */  "Repl4D_zero",
  /*  934 */  "vadd4B",
  /*  935 */  "vadd4B_reg",
  /*  936 */  "vadd8B",
  /*  937 */  "vadd8B_reg",
  /*  938 */  "vadd16B",
  /*  939 */  "vadd16B_reg",
  /*  940 */  "vadd16B_mem",
  /*  941 */  "vadd32B_reg",
  /*  942 */  "vadd32B_mem",
  /*  943 */  "vadd2S",
  /*  944 */  "vadd2S_reg",
  /*  945 */  "vadd4S",
  /*  946 */  "vadd4S_reg",
  /*  947 */  "vadd8S",
  /*  948 */  "vadd8S_reg",
  /*  949 */  "vadd8S_mem",
  /*  950 */  "vadd16S_reg",
  /*  951 */  "vadd16S_mem",
  /*  952 */  "vadd2I",
  /*  953 */  "vadd2I_reg",
  /*  954 */  "vadd4I",
  /*  955 */  "vadd4I_reg",
  /*  956 */  "vadd4I_mem",
  /*  957 */  "vadd8I_reg",
  /*  958 */  "vadd8I_mem",
  /*  959 */  "vadd2L",
  /*  960 */  "vadd2L_reg",
  /*  961 */  "vadd2L_mem",
  /*  962 */  "vadd4L_reg",
  /*  963 */  "vadd4L_mem",
  /*  964 */  "vadd2F",
  /*  965 */  "vadd2F_reg",
  /*  966 */  "vadd4F",
  /*  967 */  "vadd4F_reg",
  /*  968 */  "vadd4F_mem",
  /*  969 */  "vadd8F_reg",
  /*  970 */  "vadd8F_mem",
  /*  971 */  "vadd2D",
  /*  972 */  "vadd2D_reg",
  /*  973 */  "vadd2D_mem",
  /*  974 */  "vadd4D_reg",
  /*  975 */  "vadd4D_mem",
  /*  976 */  "vsub4B",
  /*  977 */  "vsub4B_reg",
  /*  978 */  "vsub8B",
  /*  979 */  "vsub8B_reg",
  /*  980 */  "vsub16B",
  /*  981 */  "vsub16B_reg",
  /*  982 */  "vsub16B_mem",
  /*  983 */  "vsub32B_reg",
  /*  984 */  "vsub32B_mem",
  /*  985 */  "vsub2S",
  /*  986 */  "vsub2S_reg",
  /*  987 */  "vsub4S",
  /*  988 */  "vsub4S_reg",
  /*  989 */  "vsub8S",
  /*  990 */  "vsub8S_reg",
  /*  991 */  "vsub8S_mem",
  /*  992 */  "vsub16S_reg",
  /*  993 */  "vsub16S_mem",
  /*  994 */  "vsub2I",
  /*  995 */  "vsub2I_reg",
  /*  996 */  "vsub4I",
  /*  997 */  "vsub4I_reg",
  /*  998 */  "vsub4I_mem",
  /*  999 */  "vsub8I_reg",
  /* 1000 */  "vsub8I_mem",
  /* 1001 */  "vsub2L",
  /* 1002 */  "vsub2L_reg",
  /* 1003 */  "vsub2L_mem",
  /* 1004 */  "vsub4L_reg",
  /* 1005 */  "vsub4L_mem",
  /* 1006 */  "vsub2F",
  /* 1007 */  "vsub2F_reg",
  /* 1008 */  "vsub4F",
  /* 1009 */  "vsub4F_reg",
  /* 1010 */  "vsub4F_mem",
  /* 1011 */  "vsub8F_reg",
  /* 1012 */  "vsub8F_mem",
  /* 1013 */  "vsub2D",
  /* 1014 */  "vsub2D_reg",
  /* 1015 */  "vsub2D_mem",
  /* 1016 */  "vsub4D_reg",
  /* 1017 */  "vsub4D_mem",
  /* 1018 */  "vmul2S",
  /* 1019 */  "vmul2S_reg",
  /* 1020 */  "vmul4S",
  /* 1021 */  "vmul4S_reg",
  /* 1022 */  "vmul8S",
  /* 1023 */  "vmul8S_reg",
  /* 1024 */  "vmul8S_mem",
  /* 1025 */  "vmul16S_reg",
  /* 1026 */  "vmul16S_mem",
  /* 1027 */  "vmul2I",
  /* 1028 */  "vmul2I_reg",
  /* 1029 */  "vmul4I",
  /* 1030 */  "vmul4I_reg",
  /* 1031 */  "vmul4I_mem",
  /* 1032 */  "vmul8I_reg",
  /* 1033 */  "vmul8I_mem",
  /* 1034 */  "vmul2F",
  /* 1035 */  "vmul2F_reg",
  /* 1036 */  "vmul4F",
  /* 1037 */  "vmul4F_reg",
  /* 1038 */  "vmul4F_mem",
  /* 1039 */  "vmul8F_reg",
  /* 1040 */  "vmul8F_mem",
  /* 1041 */  "vmul2D",
  /* 1042 */  "vmul2D_reg",
  /* 1043 */  "vmul2D_mem",
  /* 1044 */  "vmul4D_reg",
  /* 1045 */  "vmul4D_mem",
  /* 1046 */  "vdiv2F",
  /* 1047 */  "vdiv2F_reg",
  /* 1048 */  "vdiv4F",
  /* 1049 */  "vdiv4F_reg",
  /* 1050 */  "vdiv4F_mem",
  /* 1051 */  "vdiv8F_reg",
  /* 1052 */  "vdiv8F_mem",
  /* 1053 */  "vdiv2D",
  /* 1054 */  "vdiv2D_reg",
  /* 1055 */  "vdiv2D_mem",
  /* 1056 */  "vdiv4D_reg",
  /* 1057 */  "vdiv4D_mem",
  /* 1058 */  "vsll2S",
  /* 1059 */  "vsll2S_imm",
  /* 1060 */  "vsll2S_reg",
  /* 1061 */  "vsll2S_reg_imm",
  /* 1062 */  "vsll4S",
  /* 1063 */  "vsll4S_imm",
  /* 1064 */  "vsll4S_reg",
  /* 1065 */  "vsll4S_reg_imm",
  /* 1066 */  "vsll8S",
  /* 1067 */  "vsll8S_imm",
  /* 1068 */  "vsll8S_reg",
  /* 1069 */  "vsll8S_reg_imm",
  /* 1070 */  "vsll16S_reg",
  /* 1071 */  "vsll16S_reg_imm",
  /* 1072 */  "vsll2I",
  /* 1073 */  "vsll2I_imm",
  /* 1074 */  "vsll2I_reg",
  /* 1075 */  "vsll2I_reg_imm",
  /* 1076 */  "vsll4I",
  /* 1077 */  "vsll4I_imm",
  /* 1078 */  "vsll4I_reg",
  /* 1079 */  "vsll4I_reg_imm",
  /* 1080 */  "vsll8I_reg",
  /* 1081 */  "vsll8I_reg_imm",
  /* 1082 */  "vsll2L",
  /* 1083 */  "vsll2L_imm",
  /* 1084 */  "vsll2L_reg",
  /* 1085 */  "vsll2L_reg_imm",
  /* 1086 */  "vsll4L_reg",
  /* 1087 */  "vsll4L_reg_imm",
  /* 1088 */  "vsrl2S",
  /* 1089 */  "vsrl2S_imm",
  /* 1090 */  "vsrl2S_reg",
  /* 1091 */  "vsrl2S_reg_imm",
  /* 1092 */  "vsrl4S",
  /* 1093 */  "vsrl4S_imm",
  /* 1094 */  "vsrl4S_reg",
  /* 1095 */  "vsrl4S_reg_imm",
  /* 1096 */  "vsrl8S",
  /* 1097 */  "vsrl8S_imm",
  /* 1098 */  "vsrl8S_reg",
  /* 1099 */  "vsrl8S_reg_imm",
  /* 1100 */  "vsrl16S_reg",
  /* 1101 */  "vsrl16S_reg_imm",
  /* 1102 */  "vsrl2I",
  /* 1103 */  "vsrl2I_imm",
  /* 1104 */  "vsrl2I_reg",
  /* 1105 */  "vsrl2I_reg_imm",
  /* 1106 */  "vsrl4I",
  /* 1107 */  "vsrl4I_imm",
  /* 1108 */  "vsrl4I_reg",
  /* 1109 */  "vsrl4I_reg_imm",
  /* 1110 */  "vsrl8I_reg",
  /* 1111 */  "vsrl8I_reg_imm",
  /* 1112 */  "vsrl2L",
  /* 1113 */  "vsrl2L_imm",
  /* 1114 */  "vsrl2L_reg",
  /* 1115 */  "vsrl2L_reg_imm",
  /* 1116 */  "vsrl4L_reg",
  /* 1117 */  "vsrl4L_reg_imm",
  /* 1118 */  "vsra2S",
  /* 1119 */  "vsra2S_imm",
  /* 1120 */  "vsra2S_reg",
  /* 1121 */  "vsra2S_reg_imm",
  /* 1122 */  "vsra4S",
  /* 1123 */  "vsra4S_imm",
  /* 1124 */  "vsra4S_reg",
  /* 1125 */  "vsra4S_reg_imm",
  /* 1126 */  "vsra8S",
  /* 1127 */  "vsra8S_imm",
  /* 1128 */  "vsra8S_reg",
  /* 1129 */  "vsra8S_reg_imm",
  /* 1130 */  "vsra16S_reg",
  /* 1131 */  "vsra16S_reg_imm",
  /* 1132 */  "vsra2I",
  /* 1133 */  "vsra2I_imm",
  /* 1134 */  "vsra2I_reg",
  /* 1135 */  "vsra2I_reg_imm",
  /* 1136 */  "vsra4I",
  /* 1137 */  "vsra4I_imm",
  /* 1138 */  "vsra4I_reg",
  /* 1139 */  "vsra4I_reg_imm",
  /* 1140 */  "vsra8I_reg",
  /* 1141 */  "vsra8I_reg_imm",
  /* 1142 */  "vand4B",
  /* 1143 */  "vand4B_reg",
  /* 1144 */  "vand8B",
  /* 1145 */  "vand8B_reg",
  /* 1146 */  "vand16B",
  /* 1147 */  "vand16B_reg",
  /* 1148 */  "vand16B_mem",
  /* 1149 */  "vand32B_reg",
  /* 1150 */  "vand32B_mem",
  /* 1151 */  "vor4B",
  /* 1152 */  "vor4B_reg",
  /* 1153 */  "vor8B",
  /* 1154 */  "vor8B_reg",
  /* 1155 */  "vor16B",
  /* 1156 */  "vor16B_reg",
  /* 1157 */  "vor16B_mem",
  /* 1158 */  "vor32B_reg",
  /* 1159 */  "vor32B_mem",
  /* 1160 */  "vxor4B",
  /* 1161 */  "vxor4B_reg",
  /* 1162 */  "vxor8B",
  /* 1163 */  "vxor8B_reg",
  /* 1164 */  "vxor16B",
  /* 1165 */  "vxor16B_reg",
  /* 1166 */  "vxor16B_mem",
  /* 1167 */  "vxor32B_reg",
  /* 1168 */  "vxor32B_mem",
  // last instruction
  "invalid rule name" // no trailing comma
};

const        bool  swallowed[] = {
  /*    0 */  false,
  /*    1 */  false,
  /*    2 */  false,
  /*    3 */  false,
  /*    4 */  false,
  /*    5 */  false,
  /*    6 */  false,
  /*    7 */  false,
  /*    8 */  true,
  /*    9 */  true,
  /*   10 */  true,
  /*   11 */  true,
  /*   12 */  true,
  /*   13 */  true,
  /*   14 */  true,
  /*   15 */  true,
  /*   16 */  true,
  /*   17 */  true,
  /*   18 */  true,
  /*   19 */  true,
  /*   20 */  true,
  /*   21 */  true,
  /*   22 */  true,
  /*   23 */  true,
  /*   24 */  true,
  /*   25 */  true,
  /*   26 */  true,
  /*   27 */  true,
  /*   28 */  true,
  /*   29 */  true,
  /*   30 */  true,
  /*   31 */  true,
  /*   32 */  true,
  /*   33 */  true,
  /*   34 */  true,
  /*   35 */  true,
  /*   36 */  true,
  /*   37 */  true,
  /*   38 */  true,
  /*   39 */  true,
  /*   40 */  true,
  /*   41 */  true,
  /*   42 */  true,
  /*   43 */  false,
  /*   44 */  false,
  /*   45 */  false,
  /*   46 */  false,
  /*   47 */  false,
  /*   48 */  false,
  /*   49 */  false,
  /*   50 */  false,
  /*   51 */  false,
  /*   52 */  false,
  /*   53 */  false,
  /*   54 */  false,
  /*   55 */  false,
  /*   56 */  false,
  /*   57 */  false,
  /*   58 */  false,
  /*   59 */  false,
  /*   60 */  false,
  /*   61 */  false,
  /*   62 */  false,
  /*   63 */  false,
  /*   64 */  false,
  /*   65 */  false,
  /*   66 */  false,
  /*   67 */  false,
  /*   68 */  false,
  /*   69 */  false,
  /*   70 */  false,
  /*   71 */  false,
  /*   72 */  false,
  /*   73 */  false,
  /*   74 */  false,
  /*   75 */  false,
  /*   76 */  false,
  /*   77 */  false,
  /*   78 */  false,
  /*   79 */  false,
  /*   80 */  false,
  /*   81 */  false,
  /*   82 */  false,
  /*   83 */  false,
  /*   84 */  false,
  /*   85 */  false,
  /*   86 */  false,
  /*   87 */  false,
  /*   88 */  false,
  /*   89 */  false,
  /*   90 */  false,
  /*   91 */  false,
  /*   92 */  false,
  /*   93 */  false,
  /*   94 */  false,
  /*   95 */  false,
  /*   96 */  false,
  /*   97 */  false,
  /*   98 */  false,
  /*   99 */  false,
  /*  100 */  true,
  /*  101 */  true,
  /*  102 */  true,
  /*  103 */  true,
  /*  104 */  false,
  /*  105 */  false,
  /*  106 */  false,
  /*  107 */  false,
  // last operand
  /*  108 */  false,
  // last operand class
  /*  109 */  false,
  /*  110 */  false,
  /*  111 */  false,
  /*  112 */  false,
  /*  113 */  false,
  /*  114 */  false,
  /*  115 */  false,
  /*  116 */  false,
  /*  117 */  false,
  /*  118 */  false,
  /*  119 */  false,
  /*  120 */  false,
  /*  121 */  false,
  /*  122 */  false,
  /*  123 */  false,
  /*  124 */  false,
  /*  125 */  false,
  /*  126 */  false,
  /*  127 */  false,
  /*  128 */  false,
  /*  129 */  false,
  /*  130 */  false,
  /*  131 */  false,
  /*  132 */  false,
  /*  133 */  false,
  /*  134 */  false,
  /*  135 */  false,
  /*  136 */  false,
  /*  137 */  false,
  /*  138 */  false,
  /*  139 */  false,
  /*  140 */  false,
  /*  141 */  false,
  /*  142 */  false,
  /*  143 */  false,
  /*  144 */  false,
  /*  145 */  false,
  /*  146 */  false,
  /*  147 */  false,
  /*  148 */  false,
  /*  149 */  false,
  /*  150 */  false,
  /*  151 */  false,
  /*  152 */  false,
  /*  153 */  false,
  /*  154 */  false,
  /*  155 */  false,
  /*  156 */  false,
  /*  157 */  false,
  /*  158 */  false,
  /*  159 */  false,
  /*  160 */  false,
  /*  161 */  false,
  /*  162 */  false,
  /*  163 */  false,
  /*  164 */  false,
  /*  165 */  false,
  /*  166 */  false,
  /*  167 */  false,
  /*  168 */  false,
  /*  169 */  false,
  /*  170 */  false,
  /*  171 */  false,
  /*  172 */  false,
  /*  173 */  false,
  /*  174 */  false,
  /*  175 */  false,
  /*  176 */  false,
  /*  177 */  false,
  /*  178 */  false,
  /*  179 */  false,
  /*  180 */  false,
  /*  181 */  false,
  /*  182 */  false,
  /*  183 */  false,
  /*  184 */  false,
  /*  185 */  false,
  /*  186 */  false,
  /*  187 */  false,
  /*  188 */  false,
  /*  189 */  false,
  /*  190 */  false,
  /*  191 */  false,
  /*  192 */  false,
  /*  193 */  false,
  /*  194 */  false,
  /*  195 */  false,
  /*  196 */  false,
  /*  197 */  false,
  /*  198 */  false,
  /*  199 */  false,
  /*  200 */  false,
  /*  201 */  false,
  /*  202 */  false,
  /*  203 */  false,
  /*  204 */  false,
  /*  205 */  false,
  /*  206 */  false,
  /*  207 */  false,
  /*  208 */  false,
  /*  209 */  false,
  /*  210 */  false,
  /*  211 */  false,
  /*  212 */  false,
  /*  213 */  false,
  /*  214 */  false,
  /*  215 */  false,
  /*  216 */  false,
  /*  217 */  false,
  /*  218 */  false,
  /*  219 */  false,
  /*  220 */  false,
  /*  221 */  false,
  /*  222 */  false,
  /*  223 */  false,
  /*  224 */  false,
  /*  225 */  false,
  /*  226 */  false,
  /*  227 */  false,
  /*  228 */  false,
  /*  229 */  false,
  /*  230 */  false,
  /*  231 */  false,
  /*  232 */  false,
  /*  233 */  false,
  /*  234 */  false,
  /*  235 */  false,
  /*  236 */  false,
  /*  237 */  false,
  /*  238 */  false,
  /*  239 */  false,
  /*  240 */  false,
  /*  241 */  false,
  /*  242 */  false,
  /*  243 */  false,
  /*  244 */  false,
  /*  245 */  false,
  /*  246 */  false,
  /*  247 */  false,
  /*  248 */  false,
  /*  249 */  false,
  /*  250 */  false,
  /*  251 */  false,
  /*  252 */  false,
  /*  253 */  false,
  /*  254 */  false,
  /*  255 */  false,
  /*  256 */  false,
  /*  257 */  false,
  /*  258 */  false,
  /*  259 */  false,
  /*  260 */  false,
  /*  261 */  false,
  /*  262 */  false,
  /*  263 */  false,
  /*  264 */  false,
  // last internally defined operand
  /*  265 */  false,
  /*  266 */  false,
  /*  267 */  false,
  /*  268 */  false,
  /*  269 */  false,
  /*  270 */  false,
  /*  271 */  false,
  /*  272 */  false,
  /*  273 */  false,
  /*  274 */  false,
  /*  275 */  false,
  /*  276 */  false,
  /*  277 */  false,
  /*  278 */  false,
  /*  279 */  false,
  /*  280 */  false,
  /*  281 */  false,
  /*  282 */  false,
  /*  283 */  false,
  /*  284 */  false,
  /*  285 */  false,
  /*  286 */  false,
  /*  287 */  false,
  /*  288 */  false,
  /*  289 */  false,
  /*  290 */  false,
  /*  291 */  false,
  /*  292 */  false,
  /*  293 */  false,
  /*  294 */  false,
  /*  295 */  false,
  /*  296 */  false,
  /*  297 */  false,
  /*  298 */  false,
  /*  299 */  false,
  /*  300 */  false,
  /*  301 */  false,
  /*  302 */  false,
  /*  303 */  false,
  /*  304 */  false,
  /*  305 */  false,
  /*  306 */  false,
  /*  307 */  false,
  /*  308 */  false,
  /*  309 */  false,
  /*  310 */  false,
  /*  311 */  false,
  /*  312 */  false,
  /*  313 */  false,
  /*  314 */  false,
  /*  315 */  false,
  /*  316 */  false,
  /*  317 */  false,
  /*  318 */  false,
  /*  319 */  false,
  /*  320 */  false,
  /*  321 */  false,
  /*  322 */  false,
  /*  323 */  false,
  /*  324 */  false,
  /*  325 */  false,
  /*  326 */  false,
  /*  327 */  false,
  /*  328 */  false,
  /*  329 */  false,
  /*  330 */  false,
  /*  331 */  false,
  /*  332 */  false,
  /*  333 */  false,
  /*  334 */  false,
  /*  335 */  false,
  /*  336 */  false,
  /*  337 */  false,
  /*  338 */  false,
  /*  339 */  false,
  /*  340 */  false,
  /*  341 */  false,
  /*  342 */  false,
  /*  343 */  false,
  /*  344 */  false,
  /*  345 */  false,
  /*  346 */  false,
  /*  347 */  false,
  /*  348 */  false,
  /*  349 */  false,
  /*  350 */  false,
  /*  351 */  false,
  /*  352 */  false,
  /*  353 */  false,
  /*  354 */  false,
  /*  355 */  false,
  /*  356 */  false,
  /*  357 */  false,
  /*  358 */  false,
  /*  359 */  false,
  /*  360 */  false,
  /*  361 */  false,
  /*  362 */  false,
  /*  363 */  false,
  /*  364 */  false,
  /*  365 */  false,
  /*  366 */  false,
  /*  367 */  false,
  /*  368 */  false,
  /*  369 */  false,
  /*  370 */  false,
  /*  371 */  false,
  /*  372 */  false,
  /*  373 */  false,
  /*  374 */  false,
  /*  375 */  false,
  /*  376 */  false,
  /*  377 */  false,
  /*  378 */  false,
  /*  379 */  false,
  /*  380 */  false,
  /*  381 */  false,
  /*  382 */  false,
  /*  383 */  false,
  /*  384 */  false,
  /*  385 */  false,
  /*  386 */  false,
  /*  387 */  false,
  /*  388 */  false,
  /*  389 */  false,
  /*  390 */  false,
  /*  391 */  false,
  /*  392 */  false,
  /*  393 */  false,
  /*  394 */  false,
  /*  395 */  false,
  /*  396 */  false,
  /*  397 */  false,
  /*  398 */  false,
  /*  399 */  false,
  /*  400 */  false,
  /*  401 */  false,
  /*  402 */  false,
  /*  403 */  false,
  /*  404 */  false,
  /*  405 */  false,
  /*  406 */  false,
  /*  407 */  false,
  /*  408 */  false,
  /*  409 */  false,
  /*  410 */  false,
  /*  411 */  false,
  /*  412 */  false,
  /*  413 */  false,
  /*  414 */  false,
  /*  415 */  false,
  /*  416 */  false,
  /*  417 */  false,
  /*  418 */  false,
  /*  419 */  false,
  /*  420 */  false,
  /*  421 */  false,
  /*  422 */  false,
  /*  423 */  false,
  /*  424 */  false,
  /*  425 */  false,
  /*  426 */  false,
  /*  427 */  false,
  /*  428 */  false,
  /*  429 */  false,
  /*  430 */  false,
  /*  431 */  false,
  /*  432 */  false,
  /*  433 */  false,
  /*  434 */  false,
  /*  435 */  false,
  /*  436 */  false,
  /*  437 */  false,
  /*  438 */  false,
  /*  439 */  false,
  /*  440 */  false,
  /*  441 */  false,
  /*  442 */  false,
  /*  443 */  false,
  /*  444 */  false,
  /*  445 */  false,
  /*  446 */  false,
  /*  447 */  false,
  /*  448 */  false,
  /*  449 */  false,
  /*  450 */  false,
  /*  451 */  false,
  /*  452 */  false,
  /*  453 */  false,
  /*  454 */  false,
  /*  455 */  false,
  /*  456 */  false,
  /*  457 */  false,
  /*  458 */  false,
  /*  459 */  false,
  /*  460 */  false,
  /*  461 */  false,
  /*  462 */  false,
  /*  463 */  false,
  /*  464 */  false,
  /*  465 */  false,
  /*  466 */  false,
  /*  467 */  false,
  /*  468 */  false,
  /*  469 */  false,
  /*  470 */  false,
  /*  471 */  false,
  /*  472 */  false,
  /*  473 */  false,
  /*  474 */  false,
  /*  475 */  false,
  /*  476 */  false,
  /*  477 */  false,
  /*  478 */  false,
  /*  479 */  false,
  /*  480 */  false,
  /*  481 */  false,
  /*  482 */  false,
  /*  483 */  false,
  /*  484 */  false,
  /*  485 */  false,
  /*  486 */  false,
  /*  487 */  false,
  /*  488 */  false,
  /*  489 */  false,
  /*  490 */  false,
  /*  491 */  false,
  /*  492 */  false,
  /*  493 */  false,
  /*  494 */  false,
  /*  495 */  false,
  /*  496 */  false,
  /*  497 */  false,
  /*  498 */  false,
  /*  499 */  false,
  /*  500 */  false,
  /*  501 */  false,
  /*  502 */  false,
  /*  503 */  false,
  /*  504 */  false,
  /*  505 */  false,
  /*  506 */  false,
  /*  507 */  false,
  /*  508 */  false,
  /*  509 */  false,
  /*  510 */  false,
  /*  511 */  false,
  /*  512 */  false,
  /*  513 */  false,
  /*  514 */  false,
  /*  515 */  false,
  /*  516 */  false,
  /*  517 */  false,
  /*  518 */  false,
  /*  519 */  false,
  /*  520 */  false,
  /*  521 */  false,
  /*  522 */  false,
  /*  523 */  false,
  /*  524 */  false,
  /*  525 */  false,
  /*  526 */  false,
  /*  527 */  false,
  /*  528 */  false,
  /*  529 */  false,
  /*  530 */  false,
  /*  531 */  false,
  /*  532 */  false,
  /*  533 */  false,
  /*  534 */  false,
  /*  535 */  false,
  /*  536 */  false,
  /*  537 */  false,
  /*  538 */  false,
  /*  539 */  false,
  /*  540 */  false,
  /*  541 */  false,
  /*  542 */  false,
  /*  543 */  false,
  /*  544 */  false,
  /*  545 */  false,
  /*  546 */  false,
  /*  547 */  false,
  /*  548 */  false,
  /*  549 */  false,
  /*  550 */  false,
  /*  551 */  false,
  /*  552 */  false,
  /*  553 */  false,
  /*  554 */  false,
  /*  555 */  false,
  /*  556 */  false,
  /*  557 */  false,
  /*  558 */  false,
  /*  559 */  false,
  /*  560 */  false,
  /*  561 */  false,
  /*  562 */  false,
  /*  563 */  false,
  /*  564 */  false,
  /*  565 */  false,
  /*  566 */  false,
  /*  567 */  false,
  /*  568 */  false,
  /*  569 */  false,
  /*  570 */  false,
  /*  571 */  false,
  /*  572 */  false,
  /*  573 */  false,
  /*  574 */  false,
  /*  575 */  false,
  /*  576 */  false,
  /*  577 */  false,
  /*  578 */  false,
  /*  579 */  false,
  /*  580 */  false,
  /*  581 */  false,
  /*  582 */  false,
  /*  583 */  false,
  /*  584 */  false,
  /*  585 */  false,
  /*  586 */  false,
  /*  587 */  false,
  /*  588 */  false,
  /*  589 */  false,
  /*  590 */  false,
  /*  591 */  false,
  /*  592 */  false,
  /*  593 */  false,
  /*  594 */  false,
  /*  595 */  false,
  /*  596 */  false,
  /*  597 */  false,
  /*  598 */  false,
  /*  599 */  false,
  /*  600 */  false,
  /*  601 */  false,
  /*  602 */  false,
  /*  603 */  false,
  /*  604 */  false,
  /*  605 */  false,
  /*  606 */  false,
  /*  607 */  false,
  /*  608 */  false,
  /*  609 */  false,
  /*  610 */  false,
  /*  611 */  false,
  /*  612 */  false,
  /*  613 */  false,
  /*  614 */  false,
  /*  615 */  false,
  /*  616 */  false,
  /*  617 */  false,
  /*  618 */  false,
  /*  619 */  false,
  /*  620 */  false,
  /*  621 */  false,
  /*  622 */  false,
  /*  623 */  false,
  /*  624 */  false,
  /*  625 */  false,
  /*  626 */  false,
  /*  627 */  false,
  /*  628 */  false,
  /*  629 */  false,
  /*  630 */  false,
  /*  631 */  false,
  /*  632 */  false,
  /*  633 */  false,
  /*  634 */  false,
  /*  635 */  false,
  /*  636 */  false,
  /*  637 */  false,
  /*  638 */  false,
  /*  639 */  false,
  /*  640 */  false,
  /*  641 */  false,
  /*  642 */  false,
  /*  643 */  false,
  /*  644 */  false,
  /*  645 */  false,
  /*  646 */  false,
  /*  647 */  false,
  /*  648 */  false,
  /*  649 */  false,
  /*  650 */  false,
  /*  651 */  false,
  /*  652 */  false,
  /*  653 */  false,
  /*  654 */  false,
  /*  655 */  false,
  /*  656 */  false,
  /*  657 */  false,
  /*  658 */  false,
  /*  659 */  false,
  /*  660 */  false,
  /*  661 */  false,
  /*  662 */  false,
  /*  663 */  false,
  /*  664 */  false,
  /*  665 */  false,
  /*  666 */  false,
  /*  667 */  false,
  /*  668 */  false,
  /*  669 */  false,
  /*  670 */  false,
  /*  671 */  false,
  /*  672 */  false,
  /*  673 */  false,
  /*  674 */  false,
  /*  675 */  false,
  /*  676 */  false,
  /*  677 */  false,
  /*  678 */  false,
  /*  679 */  false,
  /*  680 */  false,
  /*  681 */  false,
  /*  682 */  false,
  /*  683 */  false,
  /*  684 */  false,
  /*  685 */  false,
  /*  686 */  false,
  /*  687 */  false,
  /*  688 */  false,
  /*  689 */  false,
  /*  690 */  false,
  /*  691 */  false,
  /*  692 */  false,
  /*  693 */  false,
  /*  694 */  false,
  /*  695 */  false,
  /*  696 */  false,
  /*  697 */  false,
  /*  698 */  false,
  /*  699 */  false,
  /*  700 */  false,
  /*  701 */  false,
  /*  702 */  false,
  /*  703 */  false,
  /*  704 */  false,
  /*  705 */  false,
  /*  706 */  false,
  /*  707 */  false,
  /*  708 */  false,
  /*  709 */  false,
  /*  710 */  false,
  /*  711 */  false,
  /*  712 */  false,
  /*  713 */  false,
  /*  714 */  false,
  /*  715 */  false,
  /*  716 */  false,
  /*  717 */  false,
  /*  718 */  false,
  /*  719 */  false,
  /*  720 */  false,
  /*  721 */  false,
  /*  722 */  false,
  /*  723 */  false,
  /*  724 */  false,
  /*  725 */  false,
  /*  726 */  false,
  /*  727 */  false,
  /*  728 */  false,
  /*  729 */  false,
  /*  730 */  false,
  /*  731 */  false,
  /*  732 */  false,
  /*  733 */  false,
  /*  734 */  false,
  /*  735 */  false,
  /*  736 */  false,
  /*  737 */  false,
  /*  738 */  false,
  /*  739 */  false,
  /*  740 */  false,
  /*  741 */  false,
  /*  742 */  false,
  /*  743 */  false,
  /*  744 */  false,
  /*  745 */  false,
  /*  746 */  false,
  /*  747 */  false,
  /*  748 */  false,
  /*  749 */  false,
  /*  750 */  false,
  /*  751 */  false,
  /*  752 */  false,
  /*  753 */  false,
  /*  754 */  false,
  /*  755 */  false,
  /*  756 */  false,
  /*  757 */  false,
  /*  758 */  false,
  /*  759 */  false,
  /*  760 */  false,
  /*  761 */  false,
  /*  762 */  false,
  /*  763 */  false,
  /*  764 */  false,
  /*  765 */  false,
  /*  766 */  false,
  /*  767 */  false,
  /*  768 */  false,
  /*  769 */  false,
  /*  770 */  false,
  /*  771 */  false,
  /*  772 */  false,
  /*  773 */  false,
  /*  774 */  false,
  /*  775 */  false,
  /*  776 */  false,
  /*  777 */  false,
  /*  778 */  false,
  /*  779 */  false,
  /*  780 */  false,
  /*  781 */  false,
  /*  782 */  false,
  /*  783 */  false,
  /*  784 */  false,
  /*  785 */  false,
  /*  786 */  false,
  /*  787 */  false,
  /*  788 */  false,
  /*  789 */  false,
  /*  790 */  false,
  /*  791 */  false,
  /*  792 */  false,
  /*  793 */  false,
  /*  794 */  false,
  /*  795 */  false,
  /*  796 */  false,
  /*  797 */  false,
  /*  798 */  false,
  /*  799 */  false,
  /*  800 */  false,
  /*  801 */  false,
  /*  802 */  false,
  /*  803 */  false,
  /*  804 */  false,
  /*  805 */  false,
  /*  806 */  false,
  /*  807 */  false,
  /*  808 */  false,
  /*  809 */  false,
  /*  810 */  false,
  /*  811 */  false,
  /*  812 */  false,
  /*  813 */  false,
  /*  814 */  false,
  /*  815 */  false,
  /*  816 */  false,
  /*  817 */  false,
  /*  818 */  false,
  /*  819 */  false,
  /*  820 */  false,
  /*  821 */  false,
  /*  822 */  false,
  /*  823 */  false,
  /*  824 */  false,
  /*  825 */  false,
  /*  826 */  false,
  /*  827 */  false,
  /*  828 */  false,
  /*  829 */  false,
  /*  830 */  false,
  /*  831 */  false,
  /*  832 */  false,
  /*  833 */  false,
  /*  834 */  false,
  /*  835 */  false,
  /*  836 */  false,
  /*  837 */  false,
  /*  838 */  false,
  /*  839 */  false,
  /*  840 */  false,
  /*  841 */  false,
  /*  842 */  false,
  /*  843 */  false,
  /*  844 */  false,
  /*  845 */  false,
  /*  846 */  false,
  /*  847 */  false,
  /*  848 */  false,
  /*  849 */  false,
  /*  850 */  false,
  /*  851 */  false,
  /*  852 */  false,
  /*  853 */  false,
  /*  854 */  false,
  /*  855 */  false,
  /*  856 */  false,
  /*  857 */  false,
  /*  858 */  false,
  /*  859 */  false,
  /*  860 */  false,
  /*  861 */  false,
  /*  862 */  false,
  /*  863 */  false,
  /*  864 */  false,
  /*  865 */  false,
  /*  866 */  false,
  /*  867 */  false,
  /*  868 */  false,
  /*  869 */  false,
  /*  870 */  false,
  /*  871 */  false,
  /*  872 */  false,
  /*  873 */  false,
  /*  874 */  false,
  /*  875 */  false,
  /*  876 */  false,
  /*  877 */  false,
  /*  878 */  false,
  /*  879 */  false,
  /*  880 */  false,
  /*  881 */  false,
  /*  882 */  false,
  /*  883 */  false,
  /*  884 */  false,
  /*  885 */  false,
  /*  886 */  false,
  /*  887 */  false,
  /*  888 */  false,
  /*  889 */  false,
  /*  890 */  false,
  /*  891 */  false,
  /*  892 */  false,
  /*  893 */  false,
  /*  894 */  false,
  /*  895 */  false,
  /*  896 */  false,
  /*  897 */  false,
  /*  898 */  false,
  /*  899 */  false,
  /*  900 */  false,
  /*  901 */  false,
  /*  902 */  false,
  /*  903 */  false,
  /*  904 */  false,
  /*  905 */  false,
  /*  906 */  false,
  /*  907 */  false,
  /*  908 */  false,
  /*  909 */  false,
  /*  910 */  false,
  /*  911 */  false,
  /*  912 */  false,
  /*  913 */  false,
  /*  914 */  false,
  /*  915 */  false,
  /*  916 */  false,
  /*  917 */  false,
  /*  918 */  false,
  /*  919 */  false,
  /*  920 */  false,
  /*  921 */  false,
  /*  922 */  false,
  /*  923 */  false,
  /*  924 */  false,
  /*  925 */  false,
  /*  926 */  false,
  /*  927 */  false,
  /*  928 */  false,
  /*  929 */  false,
  /*  930 */  false,
  /*  931 */  false,
  /*  932 */  false,
  /*  933 */  false,
  /*  934 */  false,
  /*  935 */  false,
  /*  936 */  false,
  /*  937 */  false,
  /*  938 */  false,
  /*  939 */  false,
  /*  940 */  false,
  /*  941 */  false,
  /*  942 */  false,
  /*  943 */  false,
  /*  944 */  false,
  /*  945 */  false,
  /*  946 */  false,
  /*  947 */  false,
  /*  948 */  false,
  /*  949 */  false,
  /*  950 */  false,
  /*  951 */  false,
  /*  952 */  false,
  /*  953 */  false,
  /*  954 */  false,
  /*  955 */  false,
  /*  956 */  false,
  /*  957 */  false,
  /*  958 */  false,
  /*  959 */  false,
  /*  960 */  false,
  /*  961 */  false,
  /*  962 */  false,
  /*  963 */  false,
  /*  964 */  false,
  /*  965 */  false,
  /*  966 */  false,
  /*  967 */  false,
  /*  968 */  false,
  /*  969 */  false,
  /*  970 */  false,
  /*  971 */  false,
  /*  972 */  false,
  /*  973 */  false,
  /*  974 */  false,
  /*  975 */  false,
  /*  976 */  false,
  /*  977 */  false,
  /*  978 */  false,
  /*  979 */  false,
  /*  980 */  false,
  /*  981 */  false,
  /*  982 */  false,
  /*  983 */  false,
  /*  984 */  false,
  /*  985 */  false,
  /*  986 */  false,
  /*  987 */  false,
  /*  988 */  false,
  /*  989 */  false,
  /*  990 */  false,
  /*  991 */  false,
  /*  992 */  false,
  /*  993 */  false,
  /*  994 */  false,
  /*  995 */  false,
  /*  996 */  false,
  /*  997 */  false,
  /*  998 */  false,
  /*  999 */  false,
  /* 1000 */  false,
  /* 1001 */  false,
  /* 1002 */  false,
  /* 1003 */  false,
  /* 1004 */  false,
  /* 1005 */  false,
  /* 1006 */  false,
  /* 1007 */  false,
  /* 1008 */  false,
  /* 1009 */  false,
  /* 1010 */  false,
  /* 1011 */  false,
  /* 1012 */  false,
  /* 1013 */  false,
  /* 1014 */  false,
  /* 1015 */  false,
  /* 1016 */  false,
  /* 1017 */  false,
  /* 1018 */  false,
  /* 1019 */  false,
  /* 1020 */  false,
  /* 1021 */  false,
  /* 1022 */  false,
  /* 1023 */  false,
  /* 1024 */  false,
  /* 1025 */  false,
  /* 1026 */  false,
  /* 1027 */  false,
  /* 1028 */  false,
  /* 1029 */  false,
  /* 1030 */  false,
  /* 1031 */  false,
  /* 1032 */  false,
  /* 1033 */  false,
  /* 1034 */  false,
  /* 1035 */  false,
  /* 1036 */  false,
  /* 1037 */  false,
  /* 1038 */  false,
  /* 1039 */  false,
  /* 1040 */  false,
  /* 1041 */  false,
  /* 1042 */  false,
  /* 1043 */  false,
  /* 1044 */  false,
  /* 1045 */  false,
  /* 1046 */  false,
  /* 1047 */  false,
  /* 1048 */  false,
  /* 1049 */  false,
  /* 1050 */  false,
  /* 1051 */  false,
  /* 1052 */  false,
  /* 1053 */  false,
  /* 1054 */  false,
  /* 1055 */  false,
  /* 1056 */  false,
  /* 1057 */  false,
  /* 1058 */  false,
  /* 1059 */  false,
  /* 1060 */  false,
  /* 1061 */  false,
  /* 1062 */  false,
  /* 1063 */  false,
  /* 1064 */  false,
  /* 1065 */  false,
  /* 1066 */  false,
  /* 1067 */  false,
  /* 1068 */  false,
  /* 1069 */  false,
  /* 1070 */  false,
  /* 1071 */  false,
  /* 1072 */  false,
  /* 1073 */  false,
  /* 1074 */  false,
  /* 1075 */  false,
  /* 1076 */  false,
  /* 1077 */  false,
  /* 1078 */  false,
  /* 1079 */  false,
  /* 1080 */  false,
  /* 1081 */  false,
  /* 1082 */  false,
  /* 1083 */  false,
  /* 1084 */  false,
  /* 1085 */  false,
  /* 1086 */  false,
  /* 1087 */  false,
  /* 1088 */  false,
  /* 1089 */  false,
  /* 1090 */  false,
  /* 1091 */  false,
  /* 1092 */  false,
  /* 1093 */  false,
  /* 1094 */  false,
  /* 1095 */  false,
  /* 1096 */  false,
  /* 1097 */  false,
  /* 1098 */  false,
  /* 1099 */  false,
  /* 1100 */  false,
  /* 1101 */  false,
  /* 1102 */  false,
  /* 1103 */  false,
  /* 1104 */  false,
  /* 1105 */  false,
  /* 1106 */  false,
  /* 1107 */  false,
  /* 1108 */  false,
  /* 1109 */  false,
  /* 1110 */  false,
  /* 1111 */  false,
  /* 1112 */  false,
  /* 1113 */  false,
  /* 1114 */  false,
  /* 1115 */  false,
  /* 1116 */  false,
  /* 1117 */  false,
  /* 1118 */  false,
  /* 1119 */  false,
  /* 1120 */  false,
  /* 1121 */  false,
  /* 1122 */  false,
  /* 1123 */  false,
  /* 1124 */  false,
  /* 1125 */  false,
  /* 1126 */  false,
  /* 1127 */  false,
  /* 1128 */  false,
  /* 1129 */  false,
  /* 1130 */  false,
  /* 1131 */  false,
  /* 1132 */  false,
  /* 1133 */  false,
  /* 1134 */  false,
  /* 1135 */  false,
  /* 1136 */  false,
  /* 1137 */  false,
  /* 1138 */  false,
  /* 1139 */  false,
  /* 1140 */  false,
  /* 1141 */  false,
  /* 1142 */  false,
  /* 1143 */  false,
  /* 1144 */  false,
  /* 1145 */  false,
  /* 1146 */  false,
  /* 1147 */  false,
  /* 1148 */  false,
  /* 1149 */  false,
  /* 1150 */  false,
  /* 1151 */  false,
  /* 1152 */  false,
  /* 1153 */  false,
  /* 1154 */  false,
  /* 1155 */  false,
  /* 1156 */  false,
  /* 1157 */  false,
  /* 1158 */  false,
  /* 1159 */  false,
  /* 1160 */  false,
  /* 1161 */  false,
  /* 1162 */  false,
  /* 1163 */  false,
  /* 1164 */  false,
  /* 1165 */  false,
  /* 1166 */  false,
  /* 1167 */  false,
  /* 1168 */  false,
  // last instruction
  false // no trailing comma
};

// Mapping from machine-independent opcode to boolean
const        char must_clone[] = {
  0, // Node: 0
  0, // Set: 1
  0, // RegN: 2
  0, // RegI: 3
  0, // RegP: 4
  0, // RegF: 5
  0, // RegD: 6
  0, // RegL: 7
  0, // RegFlags: 8
  0, // VecS: 9
  0, // VecD: 10
  0, // VecX: 11
  0, // VecY: 12
  0, // _last_machine_leaf: 13
  0, // AbsD: 14
  0, // AbsF: 15
  0, // AbsI: 16
  0, // AddD: 17
  0, // AddF: 18
  0, // AddI: 19
  0, // AddL: 20
  0, // AddP: 21
  0, // Allocate: 22
  0, // AllocateArray: 23
  0, // AndI: 24
  0, // AndL: 25
  0, // AryEq: 26
  0, // AtanD: 27
  1, // Binary: 28
  1, // Bool: 29
  0, // BoxLock: 30
  0, // ReverseBytesI: 31
  0, // ReverseBytesL: 32
  0, // ReverseBytesUS: 33
  0, // ReverseBytesS: 34
  0, // CProj: 35
  0, // CallDynamicJava: 36
  0, // CallJava: 37
  0, // CallLeaf: 38
  0, // CallLeafNoFP: 39
  0, // CallRuntime: 40
  0, // CallStaticJava: 41
  0, // CastII: 42
  0, // CastX2P: 43
  0, // CastP2X: 44
  0, // CastPP: 45
  0, // Catch: 46
  0, // CatchProj: 47
  0, // CheckCastPP: 48
  0, // ClearArray: 49
  0, // ConstraintCast: 50
  0, // CMoveD: 51
  0, // CMoveF: 52
  0, // CMoveI: 53
  0, // CMoveL: 54
  0, // CMoveP: 55
  0, // CMoveN: 56
  1, // CmpN: 57
  1, // CmpD: 58
  0, // CmpD3: 59
  1, // CmpF: 60
  0, // CmpF3: 61
  1, // CmpI: 62
  1, // CmpL: 63
  0, // CmpL3: 64
  0, // CmpLTMask: 65
  1, // CmpP: 66
  1, // CmpU: 67
  1, // CmpUL: 68
  0, // CompareAndSwapI: 69
  0, // CompareAndSwapL: 70
  0, // CompareAndSwapP: 71
  0, // CompareAndSwapN: 72
  0, // GetAndAddI: 73
  0, // GetAndAddL: 74
  0, // GetAndSetI: 75
  0, // GetAndSetL: 76
  0, // GetAndSetP: 77
  0, // GetAndSetN: 78
  0, // Con: 79
  0, // ConN: 80
  0, // ConD: 81
  0, // ConF: 82
  0, // ConI: 83
  0, // ConL: 84
  0, // ConP: 85
  0, // Conv2B: 86
  0, // ConvD2F: 87
  0, // ConvD2I: 88
  0, // ConvD2L: 89
  0, // ConvF2D: 90
  0, // ConvF2I: 91
  0, // ConvF2L: 92
  0, // ConvI2D: 93
  0, // ConvI2F: 94
  0, // ConvI2L: 95
  0, // ConvL2D: 96
  0, // ConvL2F: 97
  0, // ConvL2I: 98
  0, // CosD: 99
  0, // CountedLoop: 100
  0, // CountedLoopEnd: 101
  0, // CountLeadingZerosI: 102
  0, // CountLeadingZerosL: 103
  0, // CountTrailingZerosI: 104
  0, // CountTrailingZerosL: 105
  0, // CreateEx: 106
  0, // DecodeN: 107
  0, // DivD: 108
  0, // DivF: 109
  0, // DivI: 110
  0, // DivL: 111
  0, // DivMod: 112
  0, // DivModI: 113
  0, // DivModL: 114
  0, // EncodeP: 115
  0, // ExpD: 116
  1, // FastLock: 117
  1, // FastUnlock: 118
  0, // Goto: 119
  0, // Halt: 120
  0, // If: 121
  0, // IfFalse: 122
  0, // IfTrue: 123
  0, // Initialize: 124
  0, // JProj: 125
  0, // Jump: 126
  0, // JumpProj: 127
  0, // LShiftI: 128
  0, // LShiftL: 129
  0, // LoadB: 130
  0, // LoadUB: 131
  0, // LoadUS: 132
  0, // LoadD: 133
  0, // LoadD_unaligned: 134
  0, // LoadF: 135
  0, // LoadI: 136
  0, // LoadKlass: 137
  0, // LoadNKlass: 138
  0, // LoadL: 139
  0, // LoadL_unaligned: 140
  0, // LoadPLocked: 141
  0, // LoadP: 142
  0, // LoadN: 143
  0, // LoadRange: 144
  0, // LoadS: 145
  0, // Lock: 146
  0, // LogD: 147
  0, // Log10D: 148
  0, // Loop: 149
  0, // LoopLimit: 150
  0, // Mach: 151
  0, // MachProj: 152
  0, // MaxI: 153
  0, // MemBarAcquire: 154
  0, // MemBarAcquireLock: 155
  0, // MemBarCPUOrder: 156
  0, // MemBarRelease: 157
  0, // MemBarReleaseLock: 158
  0, // MemBarVolatile: 159
  0, // MemBarStoreStore: 160
  0, // MergeMem: 161
  0, // MinI: 162
  0, // ModD: 163
  0, // ModF: 164
  0, // ModI: 165
  0, // ModL: 166
  0, // MoveI2F: 167
  0, // MoveF2I: 168
  0, // MoveL2D: 169
  0, // MoveD2L: 170
  0, // MulD: 171
  0, // MulF: 172
  0, // MulHiL: 173
  0, // MulI: 174
  0, // MulL: 175
  0, // Multi: 176
  0, // NegD: 177
  0, // NegF: 178
  0, // NeverBranch: 179
  0, // Opaque1: 180
  0, // Opaque2: 181
  0, // OrI: 182
  0, // OrL: 183
  0, // PCTable: 184
  0, // Parm: 185
  0, // PartialSubtypeCheck: 186
  0, // Phi: 187
  0, // PopCountI: 188
  0, // PopCountL: 189
  0, // PowD: 190
  0, // PrefetchAllocation: 191
  0, // PrefetchRead: 192
  0, // PrefetchWrite: 193
  0, // Proj: 194
  0, // RShiftI: 195
  0, // RShiftL: 196
  0, // Region: 197
  0, // Rethrow: 198
  0, // Return: 199
  0, // Root: 200
  0, // RoundDouble: 201
  0, // RoundFloat: 202
  0, // SafePoint: 203
  0, // SafePointScalarObject: 204
  0, // SCMemProj: 205
  0, // SinD: 206
  0, // SqrtD: 207
  0, // Start: 208
  0, // StartOSR: 209
  0, // StoreB: 210
  0, // StoreC: 211
  0, // StoreCM: 212
  0, // StorePConditional: 213
  0, // StoreIConditional: 214
  0, // StoreLConditional: 215
  0, // StoreD: 216
  0, // StoreF: 217
  0, // StoreI: 218
  0, // StoreL: 219
  0, // StoreP: 220
  0, // StoreN: 221
  0, // StrComp: 222
  0, // StrEquals: 223
  0, // StrIndexOf: 224
  0, // SubD: 225
  0, // SubF: 226
  0, // SubI: 227
  0, // SubL: 228
  0, // TailCall: 229
  0, // TailJump: 230
  0, // TanD: 231
  0, // ThreadLocal: 232
  0, // Unlock: 233
  0, // URShiftI: 234
  0, // URShiftL: 235
  0, // XorI: 236
  0, // XorL: 237
  0, // Vector: 238
  0, // AddVB: 239
  0, // AddVS: 240
  0, // AddVI: 241
  0, // AddVL: 242
  0, // AddVF: 243
  0, // AddVD: 244
  0, // SubVB: 245
  0, // SubVS: 246
  0, // SubVI: 247
  0, // SubVL: 248
  0, // SubVF: 249
  0, // SubVD: 250
  0, // MulVS: 251
  0, // MulVI: 252
  0, // MulVF: 253
  0, // MulVD: 254
  0, // DivVF: 255
  0, // DivVD: 256
  0, // LShiftCntV: 257
  0, // RShiftCntV: 258
  0, // LShiftVB: 259
  0, // LShiftVS: 260
  0, // LShiftVI: 261
  0, // LShiftVL: 262
  0, // RShiftVB: 263
  0, // RShiftVS: 264
  0, // RShiftVI: 265
  0, // RShiftVL: 266
  0, // URShiftVB: 267
  0, // URShiftVS: 268
  0, // URShiftVI: 269
  0, // URShiftVL: 270
  0, // AndV: 271
  0, // OrV: 272
  0, // XorV: 273
  0, // LoadVector: 274
  0, // StoreVector: 275
  0, // Pack: 276
  0, // PackB: 277
  0, // PackS: 278
  0, // PackI: 279
  0, // PackL: 280
  0, // PackF: 281
  0, // PackD: 282
  0, // Pack2L: 283
  0, // Pack2D: 284
  0, // ReplicateB: 285
  0, // ReplicateS: 286
  0, // ReplicateI: 287
  0, // ReplicateL: 288
  0, // ReplicateF: 289
  0, // ReplicateD: 290
  0, // Extract: 291
  0, // ExtractB: 292
  0, // ExtractUB: 293
  0, // ExtractC: 294
  0, // ExtractS: 295
  0, // ExtractI: 296
  0, // ExtractL: 297
  0, // ExtractF: 298
  0 // no trailing comma // ExtractD: 299
};
//  The following instructions can cisc-spill
//  popCountI can cisc-spill operand 1 to popCountI_mem
//  popCountL can cisc-spill operand 1 to popCountL_mem
//  cmovI_reg can cisc-spill operand 4 to cmovI_mem
//  cmovI_regU can cisc-spill operand 4 to cmovI_memU
//  cmovI_regUCF can cisc-spill operand 4 to cmovI_memUCF
//  cmovL_reg can cisc-spill operand 4 to cmovL_mem
//  cmovL_regU can cisc-spill operand 4 to cmovL_memU
//  cmovL_regUCF can cisc-spill operand 4 to cmovL_memUCF
//  addI_rReg can cisc-spill operand 2 to addI_rReg_mem
//  addL_rReg can cisc-spill operand 2 to addL_rReg_mem
//  subI_rReg can cisc-spill operand 2 to subI_rReg_mem
//  subL_rReg can cisc-spill operand 2 to subL_rReg_mem
//  mulI_rReg can cisc-spill operand 2 to mulI_mem
//  mulI_rReg_imm can cisc-spill operand 1 to mulI_mem_imm
//  mulL_rReg can cisc-spill operand 2 to mulL_mem
//  mulL_rReg_imm can cisc-spill operand 1 to mulL_mem_imm
//  i2b can cisc-spill operand 1 to loadI2B
//  i2s can cisc-spill operand 1 to loadI2S
//  andI_rReg can cisc-spill operand 2 to andI_rReg_mem
//  andI2L_rReg_imm255 can cisc-spill operand 1 to loadI2L_immI_255
//  andI2L_rReg_imm65535 can cisc-spill operand 1 to loadI2L_immI_65535
//  orI_rReg can cisc-spill operand 2 to orI_rReg_mem
//  xorI_rReg can cisc-spill operand 2 to xorI_rReg_mem
//  andL_rReg can cisc-spill operand 2 to andL_rReg_mem
//  orL_rReg can cisc-spill operand 2 to orL_rReg_mem
//  xorL_rReg can cisc-spill operand 2 to xorL_rReg_mem
//  cmpF_cc_reg can cisc-spill operand 2 to cmpF_cc_mem
//  cmpF_cc_reg_CF can cisc-spill operand 2 to cmpF_cc_memCF
//  cmpD_cc_reg can cisc-spill operand 2 to cmpD_cc_mem
//  cmpD_cc_reg_CF can cisc-spill operand 2 to cmpD_cc_memCF
//  cmpF_reg can cisc-spill operand 2 to cmpF_mem
//  cmpD_reg can cisc-spill operand 2 to cmpD_mem
//  convF2D_reg_reg can cisc-spill operand 1 to convF2D_reg_mem
//  convD2F_reg_reg can cisc-spill operand 1 to convD2F_reg_mem
//  convL2F_reg_reg can cisc-spill operand 1 to convL2F_reg_mem
//  convL2D_reg_reg can cisc-spill operand 1 to convL2D_reg_mem
//  convI2L_reg_reg can cisc-spill operand 1 to loadI2L
//  convI2L_reg_reg_zex can cisc-spill operand 1 to loadUI2L
//  MoveF2I_reg_reg can cisc-spill operand 1 to MoveF2I_stack_reg
//  MoveD2L_reg_reg can cisc-spill operand 1 to MoveD2L_stack_reg
//  MoveI2F_reg_reg can cisc-spill operand 1 to MoveI2F_stack_reg
//  compI_rReg can cisc-spill operand 2 to compI_rReg_mem
//  compU_rReg can cisc-spill operand 2 to compU_rReg_mem
//  compP_rReg can cisc-spill operand 2 to compP_rReg_mem
//  compL_rReg can cisc-spill operand 2 to compL_rReg_mem
//  compUL_rReg can cisc-spill operand 2 to compUL_rReg_mem
//  addF_reg can cisc-spill operand 2 to addF_mem
//  addF_reg_reg can cisc-spill operand 2 to addF_reg_mem
//  addD_reg can cisc-spill operand 2 to addD_mem
//  addD_reg_reg can cisc-spill operand 2 to addD_reg_mem
//  subF_reg can cisc-spill operand 2 to subF_mem
//  subF_reg_reg can cisc-spill operand 2 to subF_reg_mem
//  subD_reg can cisc-spill operand 2 to subD_mem
//  subD_reg_reg can cisc-spill operand 2 to subD_reg_mem
//  mulF_reg can cisc-spill operand 2 to mulF_mem
//  mulF_reg_reg can cisc-spill operand 2 to mulF_reg_mem
//  mulD_reg can cisc-spill operand 2 to mulD_mem
//  mulD_reg_reg can cisc-spill operand 2 to mulD_reg_mem
//  divF_reg can cisc-spill operand 2 to divF_mem
//  divF_reg_reg can cisc-spill operand 2 to divF_reg_mem
//  divD_reg can cisc-spill operand 2 to divD_mem
//  divD_reg_reg can cisc-spill operand 2 to divD_reg_mem
//  sqrtF_reg can cisc-spill operand 1 to sqrtF_mem
//  sqrtD_reg can cisc-spill operand 1 to sqrtD_mem
//  Repl2I can cisc-spill operand 1 to Repl2I_mem
//  Repl4I can cisc-spill operand 1 to Repl4I_mem
//  Repl8I can cisc-spill operand 1 to Repl8I_mem
//  Repl2L can cisc-spill operand 1 to Repl2L_mem
//  Repl4L can cisc-spill operand 1 to Repl4L_mem



// An array of character pointers to machine register names.
const char *Matcher::regName[REG_COUNT] = {
  "R10",
  "R10_H",
  "R11",
  "R11_H",
  "R8",
  "R8_H",
  "R9",
  "R9_H",
  "R12",
  "R12_H",
  "RCX",
  "RCX_H",
  "RBX",
  "RBX_H",
  "RDI",
  "RDI_H",
  "RDX",
  "RDX_H",
  "RSI",
  "RSI_H",
  "RAX",
  "RAX_H",
  "RBP",
  "RBP_H",
  "R13",
  "R13_H",
  "R14",
  "R14_H",
  "R15",
  "R15_H",
  "RSP",
  "RSP_H",
  "XMM0",
  "XMM0b",
  "XMM0c",
  "XMM0d",
  "XMM0e",
  "XMM0f",
  "XMM0g",
  "XMM0h",
  "XMM1",
  "XMM1b",
  "XMM1c",
  "XMM1d",
  "XMM1e",
  "XMM1f",
  "XMM1g",
  "XMM1h",
  "XMM2",
  "XMM2b",
  "XMM2c",
  "XMM2d",
  "XMM2e",
  "XMM2f",
  "XMM2g",
  "XMM2h",
  "XMM3",
  "XMM3b",
  "XMM3c",
  "XMM3d",
  "XMM3e",
  "XMM3f",
  "XMM3g",
  "XMM3h",
  "XMM4",
  "XMM4b",
  "XMM4c",
  "XMM4d",
  "XMM4e",
  "XMM4f",
  "XMM4g",
  "XMM4h",
  "XMM5",
  "XMM5b",
  "XMM5c",
  "XMM5d",
  "XMM5e",
  "XMM5f",
  "XMM5g",
  "XMM5h",
  "XMM6",
  "XMM6b",
  "XMM6c",
  "XMM6d",
  "XMM6e",
  "XMM6f",
  "XMM6g",
  "XMM6h",
  "XMM7",
  "XMM7b",
  "XMM7c",
  "XMM7d",
  "XMM7e",
  "XMM7f",
  "XMM7g",
  "XMM7h",
  "XMM8",
  "XMM8b",
  "XMM8c",
  "XMM8d",
  "XMM8e",
  "XMM8f",
  "XMM8g",
  "XMM8h",
  "XMM9",
  "XMM9b",
  "XMM9c",
  "XMM9d",
  "XMM9e",
  "XMM9f",
  "XMM9g",
  "XMM9h",
  "XMM10",
  "XMM10b",
  "XMM10c",
  "XMM10d",
  "XMM10e",
  "XMM10f",
  "XMM10g",
  "XMM10h",
  "XMM11",
  "XMM11b",
  "XMM11c",
  "XMM11d",
  "XMM11e",
  "XMM11f",
  "XMM11g",
  "XMM11h",
  "XMM12",
  "XMM12b",
  "XMM12c",
  "XMM12d",
  "XMM12e",
  "XMM12f",
  "XMM12g",
  "XMM12h",
  "XMM13",
  "XMM13b",
  "XMM13c",
  "XMM13d",
  "XMM13e",
  "XMM13f",
  "XMM13g",
  "XMM13h",
  "XMM14",
  "XMM14b",
  "XMM14c",
  "XMM14d",
  "XMM14e",
  "XMM14f",
  "XMM14g",
  "XMM14h",
  "XMM15",
  "XMM15b",
  "XMM15c",
  "XMM15d",
  "XMM15e",
  "XMM15f",
  "XMM15g",
  "XMM15h",
  "RFLAGS" // no trailing comma
};

// An array of character pointers to machine register names.
const VMReg OptoReg::opto2vm[REG_COUNT] = {
	r10->as_VMReg(),
	r10->as_VMReg()->next(),
	r11->as_VMReg(),
	r11->as_VMReg()->next(),
	r8->as_VMReg(),
	r8->as_VMReg()->next(),
	r9->as_VMReg(),
	r9->as_VMReg()->next(),
	r12->as_VMReg(),
	r12->as_VMReg()->next(),
	rcx->as_VMReg(),
	rcx->as_VMReg()->next(),
	rbx->as_VMReg(),
	rbx->as_VMReg()->next(),
	rdi->as_VMReg(),
	rdi->as_VMReg()->next(),
	rdx->as_VMReg(),
	rdx->as_VMReg()->next(),
	rsi->as_VMReg(),
	rsi->as_VMReg()->next(),
	rax->as_VMReg(),
	rax->as_VMReg()->next(),
	rbp->as_VMReg(),
	rbp->as_VMReg()->next(),
	r13->as_VMReg(),
	r13->as_VMReg()->next(),
	r14->as_VMReg(),
	r14->as_VMReg()->next(),
	r15->as_VMReg(),
	r15->as_VMReg()->next(),
	rsp->as_VMReg(),
	rsp->as_VMReg()->next(),
	xmm0->as_VMReg(),
	xmm0->as_VMReg()->next(1),
	xmm0->as_VMReg()->next(2),
	xmm0->as_VMReg()->next(3),
	xmm0->as_VMReg()->next(4),
	xmm0->as_VMReg()->next(5),
	xmm0->as_VMReg()->next(6),
	xmm0->as_VMReg()->next(7),
	xmm1->as_VMReg(),
	xmm1->as_VMReg()->next(1),
	xmm1->as_VMReg()->next(2),
	xmm1->as_VMReg()->next(3),
	xmm1->as_VMReg()->next(4),
	xmm1->as_VMReg()->next(5),
	xmm1->as_VMReg()->next(6),
	xmm1->as_VMReg()->next(7),
	xmm2->as_VMReg(),
	xmm2->as_VMReg()->next(1),
	xmm2->as_VMReg()->next(2),
	xmm2->as_VMReg()->next(3),
	xmm2->as_VMReg()->next(4),
	xmm2->as_VMReg()->next(5),
	xmm2->as_VMReg()->next(6),
	xmm2->as_VMReg()->next(7),
	xmm3->as_VMReg(),
	xmm3->as_VMReg()->next(1),
	xmm3->as_VMReg()->next(2),
	xmm3->as_VMReg()->next(3),
	xmm3->as_VMReg()->next(4),
	xmm3->as_VMReg()->next(5),
	xmm3->as_VMReg()->next(6),
	xmm3->as_VMReg()->next(7),
	xmm4->as_VMReg(),
	xmm4->as_VMReg()->next(1),
	xmm4->as_VMReg()->next(2),
	xmm4->as_VMReg()->next(3),
	xmm4->as_VMReg()->next(4),
	xmm4->as_VMReg()->next(5),
	xmm4->as_VMReg()->next(6),
	xmm4->as_VMReg()->next(7),
	xmm5->as_VMReg(),
	xmm5->as_VMReg()->next(1),
	xmm5->as_VMReg()->next(2),
	xmm5->as_VMReg()->next(3),
	xmm5->as_VMReg()->next(4),
	xmm5->as_VMReg()->next(5),
	xmm5->as_VMReg()->next(6),
	xmm5->as_VMReg()->next(7),
	xmm6->as_VMReg(),
	xmm6->as_VMReg()->next(1),
	xmm6->as_VMReg()->next(2),
	xmm6->as_VMReg()->next(3),
	xmm6->as_VMReg()->next(4),
	xmm6->as_VMReg()->next(5),
	xmm6->as_VMReg()->next(6),
	xmm6->as_VMReg()->next(7),
	xmm7->as_VMReg(),
	xmm7->as_VMReg()->next(1),
	xmm7->as_VMReg()->next(2),
	xmm7->as_VMReg()->next(3),
	xmm7->as_VMReg()->next(4),
	xmm7->as_VMReg()->next(5),
	xmm7->as_VMReg()->next(6),
	xmm7->as_VMReg()->next(7),
	xmm8->as_VMReg(),
	xmm8->as_VMReg()->next(1),
	xmm8->as_VMReg()->next(2),
	xmm8->as_VMReg()->next(3),
	xmm8->as_VMReg()->next(4),
	xmm8->as_VMReg()->next(5),
	xmm8->as_VMReg()->next(6),
	xmm8->as_VMReg()->next(7),
	xmm9->as_VMReg(),
	xmm9->as_VMReg()->next(1),
	xmm9->as_VMReg()->next(2),
	xmm9->as_VMReg()->next(3),
	xmm9->as_VMReg()->next(4),
	xmm9->as_VMReg()->next(5),
	xmm9->as_VMReg()->next(6),
	xmm9->as_VMReg()->next(7),
	xmm10->as_VMReg(),
	xmm10->as_VMReg()->next(1),
	xmm10->as_VMReg()->next(2),
	xmm10->as_VMReg()->next(3),
	xmm10->as_VMReg()->next(4),
	xmm10->as_VMReg()->next(5),
	xmm10->as_VMReg()->next(6),
	xmm10->as_VMReg()->next(7),
	xmm11->as_VMReg(),
	xmm11->as_VMReg()->next(1),
	xmm11->as_VMReg()->next(2),
	xmm11->as_VMReg()->next(3),
	xmm11->as_VMReg()->next(4),
	xmm11->as_VMReg()->next(5),
	xmm11->as_VMReg()->next(6),
	xmm11->as_VMReg()->next(7),
	xmm12->as_VMReg(),
	xmm12->as_VMReg()->next(1),
	xmm12->as_VMReg()->next(2),
	xmm12->as_VMReg()->next(3),
	xmm12->as_VMReg()->next(4),
	xmm12->as_VMReg()->next(5),
	xmm12->as_VMReg()->next(6),
	xmm12->as_VMReg()->next(7),
	xmm13->as_VMReg(),
	xmm13->as_VMReg()->next(1),
	xmm13->as_VMReg()->next(2),
	xmm13->as_VMReg()->next(3),
	xmm13->as_VMReg()->next(4),
	xmm13->as_VMReg()->next(5),
	xmm13->as_VMReg()->next(6),
	xmm13->as_VMReg()->next(7),
	xmm14->as_VMReg(),
	xmm14->as_VMReg()->next(1),
	xmm14->as_VMReg()->next(2),
	xmm14->as_VMReg()->next(3),
	xmm14->as_VMReg()->next(4),
	xmm14->as_VMReg()->next(5),
	xmm14->as_VMReg()->next(6),
	xmm14->as_VMReg()->next(7),
	xmm15->as_VMReg(),
	xmm15->as_VMReg()->next(1),
	xmm15->as_VMReg()->next(2),
	xmm15->as_VMReg()->next(3),
	xmm15->as_VMReg()->next(4),
	xmm15->as_VMReg()->next(5),
	xmm15->as_VMReg()->next(6),
	xmm15->as_VMReg()->next(7),
	VMRegImpl::Bad() // no trailing comma
	};

 OptoReg::Name OptoReg::vm2opto[ConcreteRegisterImpl::number_of_registers];

// An array of the machine register encode values
const unsigned char Matcher::_regEncode[REG_COUNT] = {
  (unsigned char)'\xA',  // R10
  (unsigned char)'\xA',  // R10_H
  (unsigned char)'\xB',  // R11
  (unsigned char)'\xB',  // R11_H
  (unsigned char)'\x8',  // R8
  (unsigned char)'\x8',  // R8_H
  (unsigned char)'\x9',  // R9
  (unsigned char)'\x9',  // R9_H
  (unsigned char)'\xC',  // R12
  (unsigned char)'\xC',  // R12_H
  (unsigned char)'\x1',  // RCX
  (unsigned char)'\x1',  // RCX_H
  (unsigned char)'\x3',  // RBX
  (unsigned char)'\x3',  // RBX_H
  (unsigned char)'\x7',  // RDI
  (unsigned char)'\x7',  // RDI_H
  (unsigned char)'\x2',  // RDX
  (unsigned char)'\x2',  // RDX_H
  (unsigned char)'\x6',  // RSI
  (unsigned char)'\x6',  // RSI_H
  (unsigned char)'\x0',  // RAX
  (unsigned char)'\x0',  // RAX_H
  (unsigned char)'\x5',  // RBP
  (unsigned char)'\x5',  // RBP_H
  (unsigned char)'\xD',  // R13
  (unsigned char)'\xD',  // R13_H
  (unsigned char)'\xE',  // R14
  (unsigned char)'\xE',  // R14_H
  (unsigned char)'\xF',  // R15
  (unsigned char)'\xF',  // R15_H
  (unsigned char)'\x4',  // RSP
  (unsigned char)'\x4',  // RSP_H
  (unsigned char)'\x0',  // XMM0
  (unsigned char)'\x0',  // XMM0b
  (unsigned char)'\x0',  // XMM0c
  (unsigned char)'\x0',  // XMM0d
  (unsigned char)'\x0',  // XMM0e
  (unsigned char)'\x0',  // XMM0f
  (unsigned char)'\x0',  // XMM0g
  (unsigned char)'\x0',  // XMM0h
  (unsigned char)'\x1',  // XMM1
  (unsigned char)'\x1',  // XMM1b
  (unsigned char)'\x1',  // XMM1c
  (unsigned char)'\x1',  // XMM1d
  (unsigned char)'\x1',  // XMM1e
  (unsigned char)'\x1',  // XMM1f
  (unsigned char)'\x1',  // XMM1g
  (unsigned char)'\x1',  // XMM1h
  (unsigned char)'\x2',  // XMM2
  (unsigned char)'\x2',  // XMM2b
  (unsigned char)'\x2',  // XMM2c
  (unsigned char)'\x2',  // XMM2d
  (unsigned char)'\x2',  // XMM2e
  (unsigned char)'\x2',  // XMM2f
  (unsigned char)'\x2',  // XMM2g
  (unsigned char)'\x2',  // XMM2h
  (unsigned char)'\x3',  // XMM3
  (unsigned char)'\x3',  // XMM3b
  (unsigned char)'\x3',  // XMM3c
  (unsigned char)'\x3',  // XMM3d
  (unsigned char)'\x3',  // XMM3e
  (unsigned char)'\x3',  // XMM3f
  (unsigned char)'\x3',  // XMM3g
  (unsigned char)'\x3',  // XMM3h
  (unsigned char)'\x4',  // XMM4
  (unsigned char)'\x4',  // XMM4b
  (unsigned char)'\x4',  // XMM4c
  (unsigned char)'\x4',  // XMM4d
  (unsigned char)'\x4',  // XMM4e
  (unsigned char)'\x4',  // XMM4f
  (unsigned char)'\x4',  // XMM4g
  (unsigned char)'\x4',  // XMM4h
  (unsigned char)'\x5',  // XMM5
  (unsigned char)'\x5',  // XMM5b
  (unsigned char)'\x5',  // XMM5c
  (unsigned char)'\x5',  // XMM5d
  (unsigned char)'\x5',  // XMM5e
  (unsigned char)'\x5',  // XMM5f
  (unsigned char)'\x5',  // XMM5g
  (unsigned char)'\x5',  // XMM5h
  (unsigned char)'\x6',  // XMM6
  (unsigned char)'\x6',  // XMM6b
  (unsigned char)'\x6',  // XMM6c
  (unsigned char)'\x6',  // XMM6d
  (unsigned char)'\x6',  // XMM6e
  (unsigned char)'\x6',  // XMM6f
  (unsigned char)'\x6',  // XMM6g
  (unsigned char)'\x6',  // XMM6h
  (unsigned char)'\x7',  // XMM7
  (unsigned char)'\x7',  // XMM7b
  (unsigned char)'\x7',  // XMM7c
  (unsigned char)'\x7',  // XMM7d
  (unsigned char)'\x7',  // XMM7e
  (unsigned char)'\x7',  // XMM7f
  (unsigned char)'\x7',  // XMM7g
  (unsigned char)'\x7',  // XMM7h
  (unsigned char)'\x8',  // XMM8
  (unsigned char)'\x8',  // XMM8b
  (unsigned char)'\x8',  // XMM8c
  (unsigned char)'\x8',  // XMM8d
  (unsigned char)'\x8',  // XMM8e
  (unsigned char)'\x8',  // XMM8f
  (unsigned char)'\x8',  // XMM8g
  (unsigned char)'\x8',  // XMM8h
  (unsigned char)'\x9',  // XMM9
  (unsigned char)'\x9',  // XMM9b
  (unsigned char)'\x9',  // XMM9c
  (unsigned char)'\x9',  // XMM9d
  (unsigned char)'\x9',  // XMM9e
  (unsigned char)'\x9',  // XMM9f
  (unsigned char)'\x9',  // XMM9g
  (unsigned char)'\x9',  // XMM9h
  (unsigned char)'\xA',  // XMM10
  (unsigned char)'\xA',  // XMM10b
  (unsigned char)'\xA',  // XMM10c
  (unsigned char)'\xA',  // XMM10d
  (unsigned char)'\xA',  // XMM10e
  (unsigned char)'\xA',  // XMM10f
  (unsigned char)'\xA',  // XMM10g
  (unsigned char)'\xA',  // XMM10h
  (unsigned char)'\xB',  // XMM11
  (unsigned char)'\xB',  // XMM11b
  (unsigned char)'\xB',  // XMM11c
  (unsigned char)'\xB',  // XMM11d
  (unsigned char)'\xB',  // XMM11e
  (unsigned char)'\xB',  // XMM11f
  (unsigned char)'\xB',  // XMM11g
  (unsigned char)'\xB',  // XMM11h
  (unsigned char)'\xC',  // XMM12
  (unsigned char)'\xC',  // XMM12b
  (unsigned char)'\xC',  // XMM12c
  (unsigned char)'\xC',  // XMM12d
  (unsigned char)'\xC',  // XMM12e
  (unsigned char)'\xC',  // XMM12f
  (unsigned char)'\xC',  // XMM12g
  (unsigned char)'\xC',  // XMM12h
  (unsigned char)'\xD',  // XMM13
  (unsigned char)'\xD',  // XMM13b
  (unsigned char)'\xD',  // XMM13c
  (unsigned char)'\xD',  // XMM13d
  (unsigned char)'\xD',  // XMM13e
  (unsigned char)'\xD',  // XMM13f
  (unsigned char)'\xD',  // XMM13g
  (unsigned char)'\xD',  // XMM13h
  (unsigned char)'\xE',  // XMM14
  (unsigned char)'\xE',  // XMM14b
  (unsigned char)'\xE',  // XMM14c
  (unsigned char)'\xE',  // XMM14d
  (unsigned char)'\xE',  // XMM14e
  (unsigned char)'\xE',  // XMM14f
  (unsigned char)'\xE',  // XMM14g
  (unsigned char)'\xE',  // XMM14h
  (unsigned char)'\xF',  // XMM15
  (unsigned char)'\xF',  // XMM15b
  (unsigned char)'\xF',  // XMM15c
  (unsigned char)'\xF',  // XMM15d
  (unsigned char)'\xF',  // XMM15e
  (unsigned char)'\xF',  // XMM15f
  (unsigned char)'\xF',  // XMM15g
  (unsigned char)'\xF',  // XMM15h
  (unsigned char)'\x10' // no trailing comma  // RFLAGS
};


//------------------Define classes derived from MachOper---------------------
MachOper  *labelOper::clone(Compile* C) const {
  return  new (C) labelOper(_label, _block_num);
}
uint labelOper::opcode() const { return LABEL; }

const RegMask *sRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *sRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

MachOper  *methodOper::clone(Compile* C) const {
  return  new (C) methodOper(_method);
}
uint methodOper::opcode() const { return METHOD; }

const RegMask *rRegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *rax_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RAX_REG_mask();
}

const RegMask *rbx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RBX_REG_mask();
}

const RegMask *rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RCX_REG_mask();
}

const RegMask *rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDX_REG_mask();
}

const RegMask *rdi_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RDI_REG_mask();
}

const RegMask *no_rcx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RCX_REG_mask();
}

const RegMask *no_rax_rdx_RegIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_NO_RAX_RDX_REG_mask();
}

const RegMask *any_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *rRegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *rRegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *no_rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_REG_mask();
}

const RegMask *no_rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RBP_REG_mask();
}

const RegMask *no_rax_rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_NO_RAX_RBX_REG_mask();
}

const RegMask *rax_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RAX_REG_mask();
}

const RegMask *rax_RegNOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_RAX_REG_mask();
}

const RegMask *rbx_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBX_REG_mask();
}

const RegMask *rsi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RSI_REG_mask();
}

const RegMask *rdi_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RDI_REG_mask();
}

const RegMask *rbp_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_RBP_REG_mask();
}

const RegMask *r15_RegPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_R15_REG_mask();
}

const RegMask *rRegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_REG_mask();
}

const RegMask *no_rax_rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask();
}

const RegMask *no_rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RAX_RDX_REG_mask();
}

const RegMask *no_rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_NO_RCX_REG_mask();
}

const RegMask *rax_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RAX_REG_mask();
}

const RegMask *rcx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RCX_REG_mask();
}

const RegMask *rdx_RegLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &LONG_RDX_REG_mask();
}

const RegMask *rFlagsRegOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *rFlagsRegUOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *rFlagsRegUCFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_FLAGS_mask();
}

const RegMask *regFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &FLOAT_REG_mask();
}

const RegMask *regDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &DOUBLE_REG_mask();
}

const RegMask *indirectOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &PTR_REG_mask();
}

const RegMask *indOffset8Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *indOffset32Oper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &ANY_REG_mask();
}

const RegMask *indIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indPosIndexOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &INT_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indPosIndexScaleOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &ANY_REG_mask();
  case 1: return &INT_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indCompressedOopOffsetOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indirectNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset8NarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indOffset32NarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indIndexOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indIndexScaleOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  switch (index) {
  case 0: return &INT_REG_mask();
  case 1: return &LONG_REG_mask();
  }
  ShouldNotReachHere();
  return NULL;
}

const RegMask *indPosIndexOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *indPosIndexScaleOffsetNarrowOper::in_RegMask(int index) const {
  assert(0 <= index && index < 2, "index out of range");
  return &INT_REG_mask();
}

const RegMask *stackSlotPOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotIOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotFOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *stackSlotLOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &(Compile::current()->FIRST_STACK_mask());
}

const RegMask *vecSOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORS_REG_mask();
}

const RegMask *vecDOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORD_REG_mask();
}

const RegMask *vecXOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORX_REG_mask();
}

const RegMask *vecYOper::in_RegMask(int index) const {
  assert(0 <= index && index < 1, "index out of range");
  return &VECTORY_REG_mask();
}

//------------------Define members for classes derived from MachNode----------

void popCountINode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountINode::cisc_version( int offset, Compile* C ) {
  popCountI_memNode *node = new (C) popCountI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void popCountLNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *popCountLNode::cisc_version( int offset, Compile* C ) {
  popCountL_memNode *node = new (C) popCountL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regNode::cisc_version( int offset, Compile* C ) {
  cmovI_memNode *node = new (C) cmovI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUNode *node = new (C) cmovI_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovI_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovI_regUCFNode::cisc_version( int offset, Compile* C ) {
  cmovI_memUCFNode *node = new (C) cmovI_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovL_regNode::cisc_version( int offset, Compile* C ) {
  cmovL_memNode *node = new (C) cmovL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regUNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovL_regUNode::cisc_version( int offset, Compile* C ) {
  cmovL_memUNode *node = new (C) cmovL_memUNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmovL_regUCFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmovL_regUCFNode::cisc_version( int offset, Compile* C ) {
  cmovL_memUCFNode *node = new (C) cmovL_memUCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addI_rRegNode::cisc_version( int offset, Compile* C ) {
  addI_rReg_memNode *node = new (C) addI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *addL_rRegNode::cisc_version( int offset, Compile* C ) {
  addL_rReg_memNode *node = new (C) addL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subI_rRegNode::cisc_version( int offset, Compile* C ) {
  subI_rReg_memNode *node = new (C) subI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *subL_rRegNode::cisc_version( int offset, Compile* C ) {
  subL_rReg_memNode *node = new (C) subL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_rRegNode::cisc_version( int offset, Compile* C ) {
  mulI_memNode *node = new (C) mulI_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulI_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulI_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulI_mem_immNode *node = new (C) mulI_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulL_rRegNode::cisc_version( int offset, Compile* C ) {
  mulL_memNode *node = new (C) mulL_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulL_rReg_immNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulL_rReg_immNode::cisc_version( int offset, Compile* C ) {
  mulL_mem_immNode *node = new (C) mulL_mem_immNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void i2bNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2bNode::cisc_version( int offset, Compile* C ) {
  loadI2BNode *node = new (C) loadI2BNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void i2sNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *i2sNode::cisc_version( int offset, Compile* C ) {
  loadI2SNode *node = new (C) loadI2SNode();
  node->_num_opnds = 3;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI_rRegNode::cisc_version( int offset, Compile* C ) {
  andI_rReg_memNode *node = new (C) andI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI2L_rReg_imm255Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI2L_rReg_imm255Node::cisc_version( int offset, Compile* C ) {
  loadI2L_immI_255Node *node = new (C) loadI2L_immI_255Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andI2L_rReg_imm65535Node::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *andI2L_rReg_imm65535Node::cisc_version( int offset, Compile* C ) {
  loadI2L_immI_65535Node *node = new (C) loadI2L_immI_65535Node();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *orI_rRegNode::cisc_version( int offset, Compile* C ) {
  orI_rReg_memNode *node = new (C) orI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorI_rRegNode::cisc_version( int offset, Compile* C ) {
  xorI_rReg_memNode *node = new (C) xorI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void andL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *andL_rRegNode::cisc_version( int offset, Compile* C ) {
  andL_rReg_memNode *node = new (C) andL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void orL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *orL_rRegNode::cisc_version( int offset, Compile* C ) {
  orL_rReg_memNode *node = new (C) orL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void xorL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *xorL_rRegNode::cisc_version( int offset, Compile* C ) {
  xorL_rReg_memNode *node = new (C) xorL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_cc_memNode *node = new (C) cmpF_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_cc_reg_CFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_cc_reg_CFNode::cisc_version( int offset, Compile* C ) {
  cmpF_cc_memCFNode *node = new (C) cmpF_cc_memCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_cc_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_cc_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_cc_memNode *node = new (C) cmpD_cc_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_cc_reg_CFNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_cc_reg_CFNode::cisc_version( int offset, Compile* C ) {
  cmpD_cc_memCFNode *node = new (C) cmpD_cc_memCFNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpF_regNode::cisc_version( int offset, Compile* C ) {
  cmpF_memNode *node = new (C) cmpF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void cmpD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *cmpD_regNode::cisc_version( int offset, Compile* C ) {
  cmpD_memNode *node = new (C) cmpD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convF2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convF2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convF2D_reg_memNode *node = new (C) convF2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convD2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *convD2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convD2F_reg_memNode *node = new (C) convD2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *convL2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2F_reg_memNode *node = new (C) convL2F_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convL2D_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *convL2D_reg_regNode::cisc_version( int offset, Compile* C ) {
  convL2D_reg_memNode *node = new (C) convL2D_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  loadI2LNode *node = new (C) loadI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void convI2L_reg_reg_zexNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *convI2L_reg_reg_zexNode::cisc_version( int offset, Compile* C ) {
  loadUI2LNode *node = new (C) loadUI2LNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveF2I_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *MoveF2I_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveF2I_stack_regNode *node = new (C) MoveF2I_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveD2L_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *MoveD2L_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveD2L_stack_regNode *node = new (C) MoveD2L_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void MoveI2F_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *MoveI2F_reg_regNode::cisc_version( int offset, Compile* C ) {
  MoveI2F_stack_regNode *node = new (C) MoveI2F_stack_regNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compI_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compI_rRegNode::cisc_version( int offset, Compile* C ) {
  compI_rReg_memNode *node = new (C) compI_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compU_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *compU_rRegNode::cisc_version( int offset, Compile* C ) {
  compU_rReg_memNode *node = new (C) compU_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compP_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_PTR_REG_mask();
}

// Build CISC version of this instruction
MachNode *compP_rRegNode::cisc_version( int offset, Compile* C ) {
  compP_rReg_memNode *node = new (C) compP_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *compL_rRegNode::cisc_version( int offset, Compile* C ) {
  compL_rReg_memNode *node = new (C) compL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void compUL_rRegNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *compUL_rRegNode::cisc_version( int offset, Compile* C ) {
  compUL_rReg_memNode *node = new (C) compUL_rReg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}

// Build short branch version of this instruction
MachNode *jmpDirNode::short_branch_version(Compile* C) {
  jmpDir_shortNode *node = new (C) jmpDir_shortNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConNode::short_branch_version(Compile* C) {
  jmpCon_shortNode *node = new (C) jmpCon_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndNode::short_branch_version(Compile* C) {
  jmpLoopEnd_shortNode *node = new (C) jmpLoopEnd_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUNode::short_branch_version(Compile* C) {
  jmpLoopEndU_shortNode *node = new (C) jmpLoopEndU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpLoopEndUCFNode::short_branch_version(Compile* C) {
  jmpLoopEndUCF_shortNode *node = new (C) jmpLoopEndUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUNode::short_branch_version(Compile* C) {
  jmpConU_shortNode *node = new (C) jmpConU_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCFNode::short_branch_version(Compile* C) {
  jmpConUCF_shortNode *node = new (C) jmpConUCF_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}

// Build short branch version of this instruction
MachNode *jmpConUCF2Node::short_branch_version(Compile* C) {
  jmpConUCF2_shortNode *node = new (C) jmpConUCF2_shortNode();
  node->_prob = _prob;
  node->_fcnt = _fcnt;

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  return node;
}


void addF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addF_regNode::cisc_version( int offset, Compile* C ) {
  addF_memNode *node = new (C) addF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *addF_reg_regNode::cisc_version( int offset, Compile* C ) {
  addF_reg_memNode *node = new (C) addF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *addD_regNode::cisc_version( int offset, Compile* C ) {
  addD_memNode *node = new (C) addD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void addD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *addD_reg_regNode::cisc_version( int offset, Compile* C ) {
  addD_reg_memNode *node = new (C) addD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subF_regNode::cisc_version( int offset, Compile* C ) {
  subF_memNode *node = new (C) subF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *subF_reg_regNode::cisc_version( int offset, Compile* C ) {
  subF_reg_memNode *node = new (C) subF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *subD_regNode::cisc_version( int offset, Compile* C ) {
  subD_memNode *node = new (C) subD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void subD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *subD_reg_regNode::cisc_version( int offset, Compile* C ) {
  subD_reg_memNode *node = new (C) subD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulF_regNode::cisc_version( int offset, Compile* C ) {
  mulF_memNode *node = new (C) mulF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulF_reg_regNode::cisc_version( int offset, Compile* C ) {
  mulF_reg_memNode *node = new (C) mulF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulD_regNode::cisc_version( int offset, Compile* C ) {
  mulD_memNode *node = new (C) mulD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void mulD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *mulD_reg_regNode::cisc_version( int offset, Compile* C ) {
  mulD_reg_memNode *node = new (C) mulD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *divF_regNode::cisc_version( int offset, Compile* C ) {
  divF_memNode *node = new (C) divF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divF_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *divF_reg_regNode::cisc_version( int offset, Compile* C ) {
  divF_reg_memNode *node = new (C) divF_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *divD_regNode::cisc_version( int offset, Compile* C ) {
  divD_memNode *node = new (C) divD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void divD_reg_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *divD_reg_regNode::cisc_version( int offset, Compile* C ) {
  divD_reg_memNode *node = new (C) divD_reg_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtF_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_FLOAT_REG_mask();
}

// Build CISC version of this instruction
MachNode *sqrtF_regNode::cisc_version( int offset, Compile* C ) {
  sqrtF_memNode *node = new (C) sqrtF_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void sqrtD_regNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_DOUBLE_REG_mask();
}

// Build CISC version of this instruction
MachNode *sqrtD_regNode::cisc_version( int offset, Compile* C ) {
  sqrtD_memNode *node = new (C) sqrtD_memNode();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void Repl2INode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *Repl2INode::cisc_version( int offset, Compile* C ) {
  Repl2I_memNode *node = new (C) Repl2I_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void Repl4INode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *Repl4INode::cisc_version( int offset, Compile* C ) {
  Repl4I_memNode *node = new (C) Repl4I_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void Repl8INode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_INT_REG_mask();
}

// Build CISC version of this instruction
MachNode *Repl8INode::cisc_version( int offset, Compile* C ) {
  Repl8I_memNode *node = new (C) Repl8I_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void Repl2LNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *Repl2LNode::cisc_version( int offset, Compile* C ) {
  Repl2L_memNode *node = new (C) Repl2L_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


void Repl4LNode::use_cisc_RegMask() {
  _cisc_RegMask = &STACK_OR_LONG_REG_mask();
}

// Build CISC version of this instruction
MachNode *Repl4LNode::cisc_version( int offset, Compile* C ) {
  Repl4L_memNode *node = new (C) Repl4L_memNode();
  node->_bottom_type = bottom_type();

  // Copy _idx, inputs and operands to new node
  fill_new_machnode(node, C);
  // Construct operand to access [stack_pointer + offset]
  node->set_opnd_array(cisc_operand(), new (C) indOffset32Oper(offset));

  return node;
}


// Copy _idx, inputs and operands to new node
void MachNode::fill_new_machnode( MachNode* node, Compile* C) const {
  // New node must use same node index
  node->set_idx( _idx );
  // Copy machine-independent inputs
  for( uint j = 0; j < req(); j++ ) {
    node->add_req(in(j));
  }
  // Copy my operands, except for cisc position
  int nopnds = num_opnds();
  assert( node->num_opnds() == (uint)nopnds, "Must have same number of operands");
  MachOper **to = node->_opnds;
  for( int i = 0; i < nopnds; i++ ) {
    if( i != cisc_operand() ) 
      to[i] = _opnds[i]->clone(C);
  }
}

void jmpDirNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDirNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCFNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCFNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2Node::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2Node::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpDir_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpDir_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(1));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpCon_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpCon_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEnd_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEnd_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpLoopEndUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpLoopEndUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConU_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConU_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void jmpConUCF2_shortNode::label_set( Label* label, uint block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  oper->_label     = label;
  oper->_block_num = block_num;
}
void jmpConUCF2_shortNode::save_label( Label** label, uint* block_num ) {
  labelOper* oper  = (labelOper*)(opnd_array(3));
  *label = oper->_label;
  *block_num = oper->_block_num;
}
void CallStaticJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallStaticJavaHandleNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallDynamicJavaDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallRuntimeDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

void CallLeafNoFPDirectNode::method_set( intptr_t method ) {
  ((methodOper*)opnd_array(1))->_method = method;
}

int loadBNode::reloc() const {
  return 1;
}

int loadB2LNode::reloc() const {
  return 1;
}

int loadUBNode::reloc() const {
  return 1;
}

int loadUB2LNode::reloc() const {
  return 1;
}

int loadUB2L_immI8Node::reloc() const {
  return 1;
}

int loadSNode::reloc() const {
  return 1;
}

int loadS2BNode::reloc() const {
  return 1;
}

int loadS2LNode::reloc() const {
  return 1;
}

int loadUSNode::reloc() const {
  return 1;
}

int loadUS2BNode::reloc() const {
  return 1;
}

int loadUS2LNode::reloc() const {
  return 1;
}

int loadUS2L_immI_255Node::reloc() const {
  return 1;
}

int loadUS2L_immI16Node::reloc() const {
  return 1;
}

int loadINode::reloc() const {
  return 1;
}

int loadI2BNode::reloc() const {
  return 1;
}

int loadI2UBNode::reloc() const {
  return 1;
}

int loadI2SNode::reloc() const {
  return 1;
}

int loadI2USNode::reloc() const {
  return 1;
}

int loadI2LNode::reloc() const {
  return 1;
}

int loadI2L_immI_255Node::reloc() const {
  return 1;
}

int loadI2L_immI_65535Node::reloc() const {
  return 1;
}

int loadI2L_immU31Node::reloc() const {
  return 1;
}

int loadUI2LNode::reloc() const {
  return 1;
}

int loadLNode::reloc() const {
  return 1;
}

int loadRangeNode::reloc() const {
  return 1;
}

int loadPNode::reloc() const {
  return 1;
}

int loadNNode::reloc() const {
  return 1;
}

int loadKlassNode::reloc() const {
  return 1;
}

int loadNKlassNode::reloc() const {
  return 1;
}

int loadFNode::reloc() const {
  return 1;
}

int loadD_partialNode::reloc() const {
  return 1;
}

int loadDNode::reloc() const {
  return 1;
}

int loadConPNode::reloc() const {
  return 1;
}

int loadConP0Node::reloc() const {
  return 1;
}

int loadConP31Node::reloc() const {
  return 1;
}

int loadConFNode::reloc() const {
  return 2;
}

int loadConF0Node::reloc() const {
  return 2;
}

int loadConDNode::reloc() const {
  return 2;
}

int loadConD0Node::reloc() const {
  return 2;
}

int prefetchrNode::reloc() const {
  return 1;
}

int prefetchrNTANode::reloc() const {
  return 1;
}

int prefetchrT0Node::reloc() const {
  return 1;
}

int prefetchrT2Node::reloc() const {
  return 1;
}

int prefetchwNTANode::reloc() const {
  return 1;
}

int prefetchAllocNode::reloc() const {
  return 1;
}

int prefetchAllocNTANode::reloc() const {
  return 1;
}

int prefetchAllocT0Node::reloc() const {
  return 1;
}

int prefetchAllocT2Node::reloc() const {
  return 1;
}

int storeBNode::reloc() const {
  return 2;
}

int storeCNode::reloc() const {
  return 2;
}

int storeINode::reloc() const {
  return 2;
}

int storeLNode::reloc() const {
  return 2;
}

int storePNode::reloc() const {
  return 2;
}

int storeImmP0Node::reloc() const {
  return 3;
}

int storeImmPNode::reloc() const {
  return 3;
}

int storeNNode::reloc() const {
  return 2;
}

int storeImmN0Node::reloc() const {
  return 2;
}

int storeImmNNode::reloc() const {
  return 2;
}

int storeImmI0Node::reloc() const {
  return 2;
}

int storeImmINode::reloc() const {
  return 2;
}

int storeImmL0Node::reloc() const {
  return 2;
}

int storeImmLNode::reloc() const {
  return 2;
}

int storeImmC0Node::reloc() const {
  return 2;
}

int storeImmI16Node::reloc() const {
  return 2;
}

int storeImmB0Node::reloc() const {
  return 2;
}

int storeImmBNode::reloc() const {
  return 2;
}

int storeImmCM0_regNode::reloc() const {
  return 2;
}

int storeImmCM0Node::reloc() const {
  return 2;
}

int storeFNode::reloc() const {
  return 2;
}

int storeF0Node::reloc() const {
  return 3;
}

int storeF_immNode::reloc() const {
  return 3;
}

int storeDNode::reloc() const {
  return 2;
}

int storeD0_immNode::reloc() const {
  return 3;
}

int storeD0Node::reloc() const {
  return 3;
}

int popCountI_memNode::reloc() const {
  return 1;
}

int popCountL_memNode::reloc() const {
  return 1;
}

int cmovI_memNode::reloc() const {
  return 1;
}

int cmovI_memUNode::reloc() const {
  return 1;
}

int cmovI_memUCFNode::reloc() const {
  return 1;
}

int cmovL_memNode::reloc() const {
  return 1;
}

int cmovL_memUNode::reloc() const {
  return 1;
}

int cmovL_memUCFNode::reloc() const {
  return 1;
}

int addI_rReg_memNode::reloc() const {
  return 1;
}

int addI_rReg_mem_0Node::reloc() const {
  return 1;
}

int addI_mem_rRegNode::reloc() const {
  return 3;
}

int addI_mem_rReg_0Node::reloc() const {
  return 3;
}

int addI_mem_immNode::reloc() const {
  return 3;
}

int incI_memNode::reloc() const {
  return 3;
}

int decI_memNode::reloc() const {
  return 3;
}

int addL_rReg_memNode::reloc() const {
  return 1;
}

int addL_rReg_mem_0Node::reloc() const {
  return 1;
}

int addL_mem_rRegNode::reloc() const {
  return 3;
}

int addL_mem_rReg_0Node::reloc() const {
  return 3;
}

int addL_mem_immNode::reloc() const {
  return 3;
}

int incL_memNode::reloc() const {
  return 3;
}

int decL_memNode::reloc() const {
  return 3;
}

int loadPLockedNode::reloc() const {
  return 1;
}

int storePConditionalNode::reloc() const {
  return 1;
}

int storeIConditionalNode::reloc() const {
  return 1;
}

int storeLConditionalNode::reloc() const {
  return 1;
}

int compareAndSwapPNode::reloc() const {
  return 1;
}

int compareAndSwapLNode::reloc() const {
  return 1;
}

int compareAndSwapINode::reloc() const {
  return 1;
}

int xaddI_no_resNode::reloc() const {
  return 1;
}

int xaddINode::reloc() const {
  return 1;
}

int xaddL_no_resNode::reloc() const {
  return 1;
}

int xaddLNode::reloc() const {
  return 1;
}

int xchgINode::reloc() const {
  return 1;
}

int xchgLNode::reloc() const {
  return 1;
}

int xchgPNode::reloc() const {
  return 1;
}

int xchgNNode::reloc() const {
  return 1;
}

int compareAndSwapNNode::reloc() const {
  return 1;
}

int subI_rReg_memNode::reloc() const {
  return 1;
}

int subI_mem_rRegNode::reloc() const {
  return 3;
}

int subI_mem_immNode::reloc() const {
  return 3;
}

int subL_rReg_memNode::reloc() const {
  return 1;
}

int subL_mem_rRegNode::reloc() const {
  return 3;
}

int subL_mem_immNode::reloc() const {
  return 3;
}

int negI_memNode::reloc() const {
  return 3;
}

int negL_memNode::reloc() const {
  return 3;
}

int mulI_memNode::reloc() const {
  return 1;
}

int mulI_mem_0Node::reloc() const {
  return 1;
}

int mulI_mem_immNode::reloc() const {
  return 1;
}

int mulL_memNode::reloc() const {
  return 1;
}

int mulL_mem_0Node::reloc() const {
  return 1;
}

int mulL_mem_immNode::reloc() const {
  return 1;
}

int salI_mem_1Node::reloc() const {
  return 3;
}

int salI_mem_immNode::reloc() const {
  return 3;
}

int salI_mem_CLNode::reloc() const {
  return 3;
}

int sarI_mem_1Node::reloc() const {
  return 3;
}

int sarI_mem_immNode::reloc() const {
  return 3;
}

int sarI_mem_CLNode::reloc() const {
  return 3;
}

int shrI_mem_1Node::reloc() const {
  return 3;
}

int shrI_mem_immNode::reloc() const {
  return 3;
}

int shrI_mem_CLNode::reloc() const {
  return 3;
}

int salL_mem_1Node::reloc() const {
  return 3;
}

int salL_mem_immNode::reloc() const {
  return 3;
}

int salL_mem_CLNode::reloc() const {
  return 3;
}

int sarL_mem_1Node::reloc() const {
  return 3;
}

int sarL_mem_immNode::reloc() const {
  return 3;
}

int sarL_mem_CLNode::reloc() const {
  return 3;
}

int shrL_mem_1Node::reloc() const {
  return 3;
}

int shrL_mem_immNode::reloc() const {
  return 3;
}

int shrL_mem_CLNode::reloc() const {
  return 3;
}

int andI_rReg_memNode::reloc() const {
  return 1;
}

int andI_rReg_mem_0Node::reloc() const {
  return 1;
}

int andI_mem_rRegNode::reloc() const {
  return 3;
}

int andI_mem_rReg_0Node::reloc() const {
  return 3;
}

int andI_mem_immNode::reloc() const {
  return 3;
}

int orI_rReg_memNode::reloc() const {
  return 1;
}

int orI_rReg_mem_0Node::reloc() const {
  return 1;
}

int orI_mem_rRegNode::reloc() const {
  return 3;
}

int orI_mem_rReg_0Node::reloc() const {
  return 3;
}

int orI_mem_immNode::reloc() const {
  return 3;
}

int xorI_rReg_memNode::reloc() const {
  return 1;
}

int xorI_rReg_mem_0Node::reloc() const {
  return 1;
}

int xorI_mem_rRegNode::reloc() const {
  return 3;
}

int xorI_mem_rReg_0Node::reloc() const {
  return 3;
}

int xorI_mem_immNode::reloc() const {
  return 3;
}

int andL_rReg_memNode::reloc() const {
  return 1;
}

int andL_rReg_mem_0Node::reloc() const {
  return 1;
}

int andL_mem_rRegNode::reloc() const {
  return 3;
}

int andL_mem_rReg_0Node::reloc() const {
  return 3;
}

int andL_mem_immNode::reloc() const {
  return 3;
}

int orL_rReg_memNode::reloc() const {
  return 1;
}

int orL_rReg_mem_0Node::reloc() const {
  return 1;
}

int orL_mem_rRegNode::reloc() const {
  return 3;
}

int orL_mem_rReg_0Node::reloc() const {
  return 3;
}

int orL_mem_immNode::reloc() const {
  return 3;
}

int xorL_rReg_memNode::reloc() const {
  return 1;
}

int xorL_rReg_mem_0Node::reloc() const {
  return 1;
}

int xorL_mem_rRegNode::reloc() const {
  return 3;
}

int xorL_mem_rReg_0Node::reloc() const {
  return 3;
}

int xorL_mem_immNode::reloc() const {
  return 3;
}

int cmpF_cc_memNode::reloc() const {
  return 1;
}

int cmpF_cc_memCFNode::reloc() const {
  return 1;
}

int cmpF_cc_immNode::reloc() const {
  return 1;
}

int cmpF_cc_immCFNode::reloc() const {
  return 1;
}

int cmpD_cc_memNode::reloc() const {
  return 1;
}

int cmpD_cc_memCFNode::reloc() const {
  return 1;
}

int cmpD_cc_immNode::reloc() const {
  return 1;
}

int cmpD_cc_immCFNode::reloc() const {
  return 1;
}

int cmpF_memNode::reloc() const {
  return 1;
}

int cmpF_immNode::reloc() const {
  return 1;
}

int cmpD_memNode::reloc() const {
  return 1;
}

int cmpD_immNode::reloc() const {
  return 1;
}

int convF2D_reg_memNode::reloc() const {
  return 1;
}

int convD2F_reg_memNode::reloc() const {
  return 1;
}

int convI2F_reg_memNode::reloc() const {
  return 1;
}

int convI2D_reg_memNode::reloc() const {
  return 1;
}

int convL2F_reg_memNode::reloc() const {
  return 1;
}

int convL2D_reg_memNode::reloc() const {
  return 1;
}

int convI2L_reg_mem_zexNode::reloc() const {
  return 1;
}

int compI_rReg_memNode::reloc() const {
  return 1;
}

int testI_reg_memNode::reloc() const {
  return 1;
}

int testI_reg_mem_0Node::reloc() const {
  return 1;
}

int compU_rReg_memNode::reloc() const {
  return 1;
}

int compP_rReg_memNode::reloc() const {
  return 1;
}

int compP_mem_rRegNode::reloc() const {
  return 1;
}

int testP_regNode::reloc() const {
  return 1;
}

int testP_memNode::reloc() const {
  return 2;
}

int testP_mem_reg0Node::reloc() const {
  return 2;
}

int compN_rReg_memNode::reloc() const {
  return 1;
}

int compN_mem_immNode::reloc() const {
  return 1;
}

int testN_memNode::reloc() const {
  return 1;
}

int testN_mem_reg0Node::reloc() const {
  return 1;
}

int compL_rReg_memNode::reloc() const {
  return 1;
}

int testL_reg_memNode::reloc() const {
  return 1;
}

int testL_reg_mem_0Node::reloc() const {
  return 1;
}

int compUL_rReg_memNode::reloc() const {
  return 1;
}

int partialSubtypeCheck_vs_ZeroNode::reloc() const {
  return 1;
}

int safePoint_pollNode::reloc() const {
  return 1;
}

int safePoint_poll_farNode::reloc() const {
  return 1;
}

int CallStaticJavaDirectNode::reloc() const {
  return 1;
}

int CallStaticJavaHandleNode::reloc() const {
  return 1;
}

int CallDynamicJavaDirectNode::reloc() const {
  return 1;
}

int CallRuntimeDirectNode::reloc() const {
  return 1;
}

int CallLeafDirectNode::reloc() const {
  return 1;
}

int CallLeafNoFPDirectNode::reloc() const {
  return 1;
}

int RetNode::reloc() const {
  return 1;
}

int TailCalljmpIndNode::reloc() const {
  return 1;
}

int tailjmpIndNode::reloc() const {
  return 1;
}

int RethrowExceptionNode::reloc() const {
  return 1;
}

int addF_memNode::reloc() const {
  return 1;
}

int addF_mem_0Node::reloc() const {
  return 1;
}

int addF_immNode::reloc() const {
  return 1;
}

int addF_reg_memNode::reloc() const {
  return 1;
}

int addF_reg_mem_0Node::reloc() const {
  return 1;
}

int addF_reg_immNode::reloc() const {
  return 1;
}

int addD_memNode::reloc() const {
  return 1;
}

int addD_mem_0Node::reloc() const {
  return 1;
}

int addD_immNode::reloc() const {
  return 1;
}

int addD_reg_memNode::reloc() const {
  return 1;
}

int addD_reg_mem_0Node::reloc() const {
  return 1;
}

int addD_reg_immNode::reloc() const {
  return 1;
}

int subF_memNode::reloc() const {
  return 1;
}

int subF_immNode::reloc() const {
  return 1;
}

int subF_reg_memNode::reloc() const {
  return 1;
}

int subF_reg_immNode::reloc() const {
  return 1;
}

int subD_memNode::reloc() const {
  return 1;
}

int subD_immNode::reloc() const {
  return 1;
}

int subD_reg_memNode::reloc() const {
  return 1;
}

int subD_reg_immNode::reloc() const {
  return 1;
}

int mulF_memNode::reloc() const {
  return 1;
}

int mulF_mem_0Node::reloc() const {
  return 1;
}

int mulF_immNode::reloc() const {
  return 1;
}

int mulF_reg_memNode::reloc() const {
  return 1;
}

int mulF_reg_mem_0Node::reloc() const {
  return 1;
}

int mulF_reg_immNode::reloc() const {
  return 1;
}

int mulD_memNode::reloc() const {
  return 1;
}

int mulD_mem_0Node::reloc() const {
  return 1;
}

int mulD_immNode::reloc() const {
  return 1;
}

int mulD_reg_memNode::reloc() const {
  return 1;
}

int mulD_reg_mem_0Node::reloc() const {
  return 1;
}

int mulD_reg_immNode::reloc() const {
  return 1;
}

int divF_memNode::reloc() const {
  return 1;
}

int divF_immNode::reloc() const {
  return 1;
}

int divF_reg_memNode::reloc() const {
  return 1;
}

int divF_reg_immNode::reloc() const {
  return 1;
}

int divD_memNode::reloc() const {
  return 1;
}

int divD_immNode::reloc() const {
  return 1;
}

int divD_reg_memNode::reloc() const {
  return 1;
}

int divD_reg_immNode::reloc() const {
  return 1;
}

int sqrtF_memNode::reloc() const {
  return 1;
}

int sqrtF_immNode::reloc() const {
  return 1;
}

int sqrtD_memNode::reloc() const {
  return 1;
}

int sqrtD_immNode::reloc() const {
  return 1;
}

int loadV4Node::reloc() const {
  return 1;
}

int loadV8Node::reloc() const {
  return 1;
}

int loadV16Node::reloc() const {
  return 1;
}

int loadV32Node::reloc() const {
  return 1;
}

int storeV4Node::reloc() const {
  return 2;
}

int storeV8Node::reloc() const {
  return 2;
}

int storeV16Node::reloc() const {
  return 2;
}

int storeV32Node::reloc() const {
  return 2;
}

int Repl2I_memNode::reloc() const {
  return 1;
}

int Repl4I_memNode::reloc() const {
  return 1;
}

int Repl8I_memNode::reloc() const {
  return 1;
}

int Repl2L_memNode::reloc() const {
  return 1;
}

int Repl4L_memNode::reloc() const {
  return 1;
}

int Repl2F_zeroNode::reloc() const {
  return 1;
}

int Repl4F_zeroNode::reloc() const {
  return 1;
}

int Repl8F_zeroNode::reloc() const {
  return 1;
}

int Repl2D_zeroNode::reloc() const {
  return 1;
}

int Repl4D_zeroNode::reloc() const {
  return 1;
}

int vadd16B_memNode::reloc() const {
  return 1;
}

int vadd32B_memNode::reloc() const {
  return 1;
}

int vadd8S_memNode::reloc() const {
  return 1;
}

int vadd16S_memNode::reloc() const {
  return 1;
}

int vadd4I_memNode::reloc() const {
  return 1;
}

int vadd8I_memNode::reloc() const {
  return 1;
}

int vadd2L_memNode::reloc() const {
  return 1;
}

int vadd4L_memNode::reloc() const {
  return 1;
}

int vadd4F_memNode::reloc() const {
  return 1;
}

int vadd8F_memNode::reloc() const {
  return 1;
}

int vadd2D_memNode::reloc() const {
  return 1;
}

int vadd4D_memNode::reloc() const {
  return 1;
}

int vsub16B_memNode::reloc() const {
  return 1;
}

int vsub32B_memNode::reloc() const {
  return 1;
}

int vsub8S_memNode::reloc() const {
  return 1;
}

int vsub16S_memNode::reloc() const {
  return 1;
}

int vsub4I_memNode::reloc() const {
  return 1;
}

int vsub8I_memNode::reloc() const {
  return 1;
}

int vsub2L_memNode::reloc() const {
  return 1;
}

int vsub4L_memNode::reloc() const {
  return 1;
}

int vsub4F_memNode::reloc() const {
  return 1;
}

int vsub8F_memNode::reloc() const {
  return 1;
}

int vsub2D_memNode::reloc() const {
  return 1;
}

int vsub4D_memNode::reloc() const {
  return 1;
}

int vmul8S_memNode::reloc() const {
  return 1;
}

int vmul16S_memNode::reloc() const {
  return 1;
}

int vmul4I_memNode::reloc() const {
  return 1;
}

int vmul8I_memNode::reloc() const {
  return 1;
}

int vmul4F_memNode::reloc() const {
  return 1;
}

int vmul8F_memNode::reloc() const {
  return 1;
}

int vmul2D_memNode::reloc() const {
  return 1;
}

int vmul4D_memNode::reloc() const {
  return 1;
}

int vdiv4F_memNode::reloc() const {
  return 1;
}

int vdiv8F_memNode::reloc() const {
  return 1;
}

int vdiv2D_memNode::reloc() const {
  return 1;
}

int vdiv4D_memNode::reloc() const {
  return 1;
}

int vand16B_memNode::reloc() const {
  return 1;
}

int vand32B_memNode::reloc() const {
  return 1;
}

int vor16B_memNode::reloc() const {
  return 1;
}

int vor32B_memNode::reloc() const {
  return 1;
}

int vxor16B_memNode::reloc() const {
  return 1;
}

int vxor32B_memNode::reloc() const {
  return 1;
}


void loadBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4866 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12168 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4881 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12184 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4896 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12200 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUB2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4911 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12216 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUB2L_immI8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 4925 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzbq(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 12235 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4941 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12251 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    MacroAssembler _masm(&cbuf);

#line 4954 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12268 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4968 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12284 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUSNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4983 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12300 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUS2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    MacroAssembler _masm(&cbuf);

#line 4996 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12317 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUS2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5010 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12333 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUS2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5022 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12350 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUS2L_immI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5035 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movzwq(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 12369 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5051 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12385 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {
    MacroAssembler _masm(&cbuf);

#line 5064 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movsbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12402 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2UBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5076 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12419 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {
    MacroAssembler _masm(&cbuf);

#line 5088 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movswl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12436 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2USNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5100 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12453 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5114 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movslq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12469 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2L_immI_255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5126 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzbq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12486 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2L_immI_65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5137 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movzwq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12503 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadI2L_immU31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5150 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    __ movl(Rdst, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ andl(Rdst, opnd_array(2)->constant());
  
#line 12522 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadUI2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 5166 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12539 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5181 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12555 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadRangeNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 12596 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 12604 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12620 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12663 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 12671 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12687 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5219 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
   
#line 12703 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12746 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 12754 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12770 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadNKlassNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5245 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12786 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5258 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12802 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadD_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5272 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12818 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 5285 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 12834 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaP8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12877 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 12885 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12901 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaP32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 12944 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 12952 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 12968 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13011 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13019 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13035 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxScaleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13078 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13086 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13102 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13145 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13153 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13169 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPPosIdxOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13212 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13220 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13236 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPPosIdxScaleOffNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13279 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13287 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13303 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPCompressedOopOffsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13346 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13354 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13370 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaP8NarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13413 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13421 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13437 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaP32NarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13480 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13488 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13504 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13547 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13555 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13571 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxScaleNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13614 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13622 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13638 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPIdxScaleOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13681 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13689 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13705 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPPosIdxOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13748 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13756 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13772 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaPPosIdxScaleOffNarrowNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 13815 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 13823 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 13839 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2176 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constant());
  
#line 13860 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 13885 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 13893 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 13901 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2187 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, opnd_array(1)->constantL());
  
#line 13924 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 13949 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 13957 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 13965 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConUL32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2200 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  
#line 13987 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConL32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2212 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xC7);
    emit_rm(cbuf, 0x03, 0x00, dstenc);
    emit_d32(cbuf, opnd_array(1)->constantL());
  
#line 14011 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {

#line 2238 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    // This next line should be generated from ADLC
    if (opnd_array(1)->constant_is_oop()) {
      emit_d64_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, RELOC_IMM64);
    } else {
      emit_d64(cbuf, opnd_array(1)->constant());
    }
  
#line 14039 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 14064 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 14072 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 14080 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConP31Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2226 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // same as load_immI, but this time we care about zeroes in the high word
    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    
if ( opnd_array(1)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(1)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(1)->constant());
};
  
#line 14107 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 5575 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 14123 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConFNode::eval_constant(Compile* C) {
  {

#line 5576 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 14132 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5585 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xorq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 14147 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5596 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    address con = (address)opnd_array(1)->constant();
    if (con == NULL) {
      ShouldNotReachHere();
    } else {
      __ set_narrow_oop(opnd_array(0)->as_Register(ra_,this)/* dst */, (jobject)opnd_array(1)->constant());
    }
  
#line 14168 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5613 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 14184 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 5624 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 14200 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConDNode::eval_constant(Compile* C) {
  {

#line 5625 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 14209 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadConD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5636 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xorpd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 14224 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14265 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14273 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14289 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14332 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14340 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14356 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14399 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 14407 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14423 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5681 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 14439 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5694 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 14455 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5709 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetchr(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14471 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchrNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5721 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14487 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchrT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5733 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetcht0(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14503 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchrT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5745 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetcht2(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14519 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchwNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5756 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14535 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchAllocNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5770 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetchw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14551 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchAllocNTANode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5782 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetchnta(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14567 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchAllocT0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5794 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetcht0(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14583 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void prefetchAllocT2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 5806 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ prefetcht2(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 14599 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2352 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        } else if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 4) {
          emit_opcode(cbuf, Assembler::REX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14643 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x88 /*primary()*/));
  
#line 14651 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14667 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeCNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1775 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, 0x66);
  
#line 14683 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14717 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 14725 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14741 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 14783 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 14791 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14807 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14851 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 14859 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14875 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storePNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 14919 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 14927 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 14943 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmP0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 5881 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 14960 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 14988 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 14996 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15016 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2256 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    
if ( opnd_array(2)->constant_is_oop() ) {
  emit_d32_reloc(cbuf, opnd_array(2)->constant(), relocInfo::oop_type, 0);
} else {
  emit_d32(cbuf, opnd_array(2)->constant());
};
  
#line 15030 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5906 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_Register(ra_,this,idx2)/* src */);
  
#line 15047 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmN0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 5919 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15064 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 5931 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    address con = (address)opnd_array(2)->constant();
    if (con == NULL) {
      __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (int32_t)0);
    } else {
      __ set_narrow_oop(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (jobject)opnd_array(2)->constant());
    }
  
#line 15086 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmI0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 5950 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15103 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15129 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15137 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15157 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2256 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 15166 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmL0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 5975 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15183 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 15211 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15219 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15239 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2256 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  
#line 15248 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmC0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 6000 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movw(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15265 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmI16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1775 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, 0x66);
  
#line 15281 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15299 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15307 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15327 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2270 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d16(cbuf, opnd_array(2)->constant());
  
#line 15336 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmB0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 6026 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15353 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmBNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15379 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 15387 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15407 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15421 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmCM0_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 6051 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movb(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15438 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeImmCM0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15464 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC6 /*primary()*/));
  
#line 15472 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15492 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 15506 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6075 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 15523 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeF0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 6089 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15540 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 15566 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15574 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15594 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2262 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantF();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  
#line 15605 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6113 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 15622 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeD0_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 15650 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC7 /*primary()*/));
  
#line 15658 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 15678 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2262 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output Float immediate bits
    jfloat jf = opnd_array(2)->constantD();
    jint jf_as_bits = jint_cast(jf);
    emit_d32(cbuf, jf_as_bits);
  
#line 15689 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeD0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 6139 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), r12);
  
#line 15706 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeSSINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 15747 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15755 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15771 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeSSLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 15814 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15822 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15838 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeSSPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(0)->base(ra_,this,idx0) < 8) {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(0)->index(ra_,this,idx0) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 15881 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x89 /*primary()*/));
  
#line 15889 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base = opnd_array(0)->base(ra_,this,idx0);
    int index = opnd_array(0)->index(ra_,this,idx0);
    int scale = opnd_array(0)->scale();
    int disp = opnd_array(0)->disp(ra_,this,0);
    bool disp_is_oop = opnd_array(0)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 15905 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeSSFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6184 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15921 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeSSDNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6196 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 15937 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void bytes_reverse_intNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 15954 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 15962 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1997 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 15971 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void bytes_reverse_longNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 15990 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 15998 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1997 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // BSWAP
    emit_cc(cbuf, (0xC8 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */);
  
#line 16007 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void bytes_reverse_unsigned_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 6227 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ shrl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 16024 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void bytes_reverse_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 6240 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ bswapl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
    __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 16);
  
#line 16041 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countLeadingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6255 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ lzcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16057 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countLeadingZerosI_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6272 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label skip;
    __ bsrl(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerInt - 1);
  
#line 16081 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countLeadingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6292 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ lzcntq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16097 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countLeadingZerosL_bsrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6309 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Register Rsrc = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Label skip;
    __ bsrq(Rdst, Rsrc);
    __ jccb(Assembler::notZero, skip);
    __ movl(Rdst, -1);
    __ bind(skip);
    __ negl(Rdst);
    __ addl(Rdst, BitsPerLong - 1);
  
#line 16121 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countTrailingZerosINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6331 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ bsfl(Rdst, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerInt);
    __ bind(done);
  
#line 16142 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void countTrailingZerosLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6350 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rdst = opnd_array(0)->as_Register(ra_,this)/* dst */;
    Label done;
    __ bsfq(Rdst, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ jccb(Assembler::notZero, done);
    __ movl(Rdst, BitsPerLong);
    __ bind(done);
  
#line 16163 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void popCountINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6370 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16179 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void popCountI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 6382 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 16195 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void popCountLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6395 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16211 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void popCountL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 6408 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ popcntq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 16227 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void membar_acquireNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquireNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_acquire_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_acquire_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_releaseNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_releaseNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_release_lockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_release_lockNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 6475 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ membar(Assembler::StoreLoad);
  
#line 16295 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void unnecessary_membar_volatileNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint unnecessary_membar_volatileNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void membar_storestoreNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint membar_storestoreNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castX2PNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ movptr(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    }
  
#line 16339 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6523 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ movptr(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    }
  
#line 16357 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convP2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6537 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16373 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convN2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6551 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16389 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void encodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6563 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    if (s != d) {
      __ movq(d, s);
    }
    __ encode_heap_oop(d);
  
#line 16410 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void encodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6579 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ encode_heap_oop_not_null(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 16426 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decodeHeapOopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6591 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    if (s != d) {
      __ movq(d, s);
    }
    __ decode_heap_oop(d);
  
#line 16447 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decodeHeapOop_not_nullNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6608 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register s = opnd_array(1)->as_Register(ra_,this,idx1)/* src */;
    Register d = opnd_array(0)->as_Register(ra_,this)/* dst */;
    if (s != d) {
      __ decode_heap_oop_not_null(d, s);
    } else {
      __ decode_heap_oop_not_null(d);
    }
  
#line 16469 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jumpXtnd_offsetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dest
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 6632 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor)$shift$$constant);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(3)->as_Register(ra_,this,idx3)/* dest */, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, (Address::ScaleFactor) opnd_array(2)->constant());
    __ lea(opnd_array(3)->as_Register(ra_,this,idx3)/* dest */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 16495 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jumpXtnd_offsetNode::eval_constant(Compile* C) {
  _constant = C->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void jumpXtnd_addrNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// offset
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dest
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 6652 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, (Address::ScaleFactor) $shift$$constant, (int) $offset$$constant);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(4)->as_Register(ra_,this,idx4)/* dest */, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, (Address::ScaleFactor) opnd_array(2)->constant(), (int) opnd_array(3)->constantL());
    __ lea(opnd_array(4)->as_Register(ra_,this,idx4)/* dest */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 16526 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jumpXtnd_addrNode::eval_constant(Compile* C) {
  _constant = C->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void jumpXtndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  ra_->C->constant_table().fill_jump_table(cbuf, (MachConstantNode*) this, _index2label);
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dest
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 6672 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // We could use jump(ArrayAddress) except that the macro assembler needs to use r10
    // to do that and the compiler is using that register as one it can allocate.
    // So we build it all by hand.
    // Address index(noreg, switch_reg, Address::times_1);
    // ArrayAddress dispatch(table, index);
    Address dispatch(opnd_array(2)->as_Register(ra_,this,idx2)/* dest */, opnd_array(1)->as_Register(ra_,this,idx1)/* switch_val */, Address::times_1);
    __ lea(opnd_array(2)->as_Register(ra_,this,idx2)/* dest */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ jmp(dispatch);
  
#line 16555 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jumpXtndNode::eval_constant(Compile* C) {
  _constant = C->constant_table().add_jump_table(this);
  // User did not define an encode section.
}
void cmovI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 16587 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16597 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16605 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovI_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 16633 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16643 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16651 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 16695 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16705 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16721 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovI_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 16765 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16775 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 16791 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 16819 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16829 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16837 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovN_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 16865 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16875 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16883 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 16913 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16923 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16931 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovP_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 16961 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 16971 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 16979 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 17009 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17019 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 17027 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 17073 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17083 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17099 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovL_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(4)->reg(ra_,this,idx4)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 17129 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17139 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(3)->reg(ra_,this,idx3)/* dst */ & 7, opnd_array(4)->reg(ra_,this,idx4)/* src */ & 7);
  
#line 17147 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovL_memUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* dst */ < 8) {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(4)->base(ra_,this,idx4) < 8) {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(4)->index(ra_,this,idx4) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 17193 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // CMOV
    emit_d8(cbuf, (0x0F /*primary()*/));
    emit_cc(cbuf, (0x40 /*secondary()*/), opnd_array(1)->ccode());
  
#line 17203 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* dst */;
    int base = opnd_array(4)->base(ra_,this,idx4);
    int index = opnd_array(4)->index(ra_,this,idx4);
    int scale = opnd_array(4)->scale();
    int disp = opnd_array(4)->disp(ra_,this,idx4);
    bool disp_is_oop = opnd_array(4)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17219 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6905 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17242 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovF_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6935 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movflt(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17265 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6961 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17288 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovD_regUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// dst
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 6979 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label Lskip;
    // Invert sense of branch from sense of CMOV
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()^1), Lskip);
    __ movdbl(opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* dst */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* src */);
    __ bind(Lskip);
  
#line 17311 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17337 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17345 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 17353 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 17384 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17398 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17448 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17464 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17506 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 17514 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17530 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17572 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 17580 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17596 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 17638 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 17646 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 17662 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 17688 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 17703 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 17723 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 17737 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void incI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 17755 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 17763 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 17772 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void incI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 17798 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 17806 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 17826 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 17844 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 17852 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 17861 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 17887 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 17895 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 17915 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaI_rReg_immINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x67));
  
#line 17931 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 17949 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 17957 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2527 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constant(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 17972 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18000 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 18008 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 18016 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 18049 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 18063 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18107 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 18115 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18131 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18175 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 18183 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18199 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18243 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 18251 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18267 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18311 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x01 /*primary()*/));
  
#line 18319 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18335 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 18363 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 18378 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18398 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 18412 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void incL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 18432 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 18449 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void incL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 18477 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18485 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18505 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 18525 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18533 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 18542 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void decL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 18570 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 18578 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x01);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 18598 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaL_rReg_immLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18626 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 18634 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2527 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 18649 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18677 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x03 /*primary()*/));
  
#line 18685 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 18693 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addP_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), dstenc);
  
#line 18726 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 18740 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void leaP_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src0 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 18768 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8D /*primary()*/));
  
#line 18776 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2527 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int reg_encoding = opnd_array(0)->reg(ra_,this)/* dst */;
    int base         = opnd_array(1)->reg(ra_,this,idx1)/* src0 */;      // 0xFFFFFFFF indicates no base
    int index        = 0x04;            // 0x04 indicates no index
    int scale        = 0x00;            // 0x00 indicates no scale
    int displace     = opnd_array(2)->constantL(); // 0x00 indicates no displacement
    bool disp_is_oop = false;
    encode_RegMem(cbuf, reg_encoding, base, index, scale, displace,
                  disp_is_oop);
  
#line 18791 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void checkCastPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint checkCastPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castPPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castPPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void castIINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

uint castIINode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void loadPLockedNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18873 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x8B /*primary()*/));
  
#line 18881 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18897 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storePConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// heap_top_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 18916 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 18952 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 18960 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 18968 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 18984 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeIConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19003 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 19037 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19045 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19053 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19069 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeLConditionalNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19088 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 19124 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19132 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19140 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19156 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compareAndSwapPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19175 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 19211 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19219 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19227 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19243 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2328 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 19253 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 19261 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 19269 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1780 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19277 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 19295 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 19303 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 19311 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* res */ & 7, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19319 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compareAndSwapLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19338 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 19374 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19382 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19390 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19406 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2328 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 19416 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 19424 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 19432 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1780 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 19458 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 19466 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 19474 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* res */ & 7, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19482 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compareAndSwapINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19501 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 19535 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19543 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19551 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19567 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2328 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 19577 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 19585 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 19593 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1780 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19601 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 19619 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 19627 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 19635 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* res */ & 7, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19643 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xaddI_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    MacroAssembler _masm(&cbuf);

#line 7451 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) { __ lock(); }
    __ addl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->constant());
  
#line 19661 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xaddINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7462 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) { __ lock(); }
    __ xaddl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 19679 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xaddL_no_resNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// add
  {
    MacroAssembler _masm(&cbuf);

#line 7474 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) { __ lock(); }
    __ addq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->constantL());
  
#line 19697 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xaddLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7485 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) { __ lock(); }
    __ xaddq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_Register(ra_,this,idx2)/* newval */);
  
#line 19715 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xchgINode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7495 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 19732 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xchgLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7504 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xchgq(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 19749 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xchgPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7513 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xchgq(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 19766 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xchgNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// newval
  {
    MacroAssembler _masm(&cbuf);

#line 7522 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ xchgl(opnd_array(2)->as_Register(ra_,this,idx2)/* newval */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 19783 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compareAndSwapNNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem_ptr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// oldval
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// newval
  {

#line 2288 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (os::is_MP()) {
      emit_opcode(cbuf, 0xF0); // lock
    }
  
#line 19802 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(3)->reg(ra_,this,idx3)/* newval */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 19836 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 19844 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB1 /*secondary()*/));
  
#line 19852 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(3)->reg(ra_,this,idx3)/* newval */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 19868 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2328 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
      emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
    }
  
#line 19878 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 19886 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x94));
  
#line 19894 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1780 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, 0, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19902 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* res */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* res */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* res */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 19920 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF));
  
#line 19928 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 19936 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* res */ & 7, opnd_array(0)->reg(ra_,this)/* res */ & 7);
  
#line 19944 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 19970 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 19978 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 19986 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  
#line 20017 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20031 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 20073 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20081 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 20097 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 20139 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 20147 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 20163 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20189 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 20204 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20224 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20238 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 20266 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20274 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 20282 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x05 /*secondary()*/), dstenc);
  
#line 20315 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 20329 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 20373 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20381 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 20397 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 20441 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x29 /*primary()*/));
  
#line 20449 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 20465 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 20493 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 20508 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x05);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20528 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 20542 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(3)->reg(ra_,this,idx3)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(3)->reg(ra_,this,idx3)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 20571 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x2B /*primary()*/));
  
#line 20579 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(3)->reg(ra_,this,idx3)/* src */ & 7);
  
#line 20587 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 20605 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 20613 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ & 7);
  
#line 20622 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 20648 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 20656 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20676 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 20696 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 20704 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x03 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* dst */ & 7);
  
#line 20713 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 20741 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 20749 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x03 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 20769 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 20795 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 20803 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 20811 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 20819 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 20845 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 20860 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 20868 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 20882 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulI_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 20924 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 20932 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 20940 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 20956 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulI_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 20998 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21006 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 21014 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 21030 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 21072 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 21087 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 21103 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 21117 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 21145 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21153 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 21161 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 21169 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 21197 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 21212 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 21220 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 21234 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulL_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 21278 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21286 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 21294 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 21310 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulL_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 21354 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 21362 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xAF /*secondary()*/));
  
#line 21370 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 21386 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 21430 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x69 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x69 /*primary()*/));
    }
  
#line 21445 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(0)->reg(ra_,this)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 21461 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 21475 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulHiL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rax
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21495 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21503 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 21512 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1796 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 21587 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21597 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21605 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 21614 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1860 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 21696 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21708 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21716 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 21725 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divModI_rReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// div
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rdx
  {

#line 1796 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 21800 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 21810 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21818 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 21827 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divModL_rReg_divmodNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// div
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rdx
  {

#line 1860 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 21909 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21921 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21929 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 21938 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadConL_0x6666666666666667Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2187 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_opcode(cbuf, 0xB8 | dstenc);
    emit_d64(cbuf, (0x6666666666666667));
  
#line 21961 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mul_hiNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// rax
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 21981 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 21989 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 21998 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_rReg_63Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x3F));
  
#line 22023 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_rReg_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, (0x2));
  
#line 22048 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void modI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1796 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java idiv and irem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_int
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_int
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   3d 00 00 00 80          cmp    $0x80000000,%eax
    //    5:   75 07/08                jne    e <normal>
    //    7:   33 d2                   xor    %edx,%edx
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    9:   83 f9 ff                cmp    $0xffffffffffffffff,$div
    //    c:   74 03/04                je     11 <done>
    // 000000000000000e <normal>:
    //    e:   99                      cltd
    //  [div >= 8 -> offset + 1]
    //  [REX_B]
    //    f:   f7 f9                   idiv   $div
    // 0000000000000011 <done>:

    // cmp    $0x80000000,%eax
    emit_opcode(cbuf, 0x3d);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // jne    e <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x07 : 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,%ecx
    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     11 <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? 0x03 : 0x04);

    // <normal>
    // cltd
    emit_opcode(cbuf, 0x99);

    // idivl (note: must be emitted by the user of this rule)
    // <done>
  
#line 22123 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22133 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 22141 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 22150 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void modL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rax
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// div
  {

#line 1860 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Full implementation of Java ldiv and lrem; checks for
    // special case as described in JVM spec., p.243 & p.271.
    //
    //         normal case                           special case
    //
    // input : rax: dividend                         min_long
    //         reg: divisor                          -1
    //
    // output: rax: quotient  (= rax idiv reg)       min_long
    //         rdx: remainder (= rax irem reg)       0
    //
    //  Code sequnce:
    //
    //    0:   48 ba 00 00 00 00 00    mov    $0x8000000000000000,%rdx
    //    7:   00 00 80
    //    a:   48 39 d0                cmp    %rdx,%rax
    //    d:   75 08                   jne    17 <normal>
    //    f:   33 d2                   xor    %edx,%edx
    //   11:   48 83 f9 ff             cmp    $0xffffffffffffffff,$div
    //   15:   74 05                   je     1c <done>
    // 0000000000000017 <normal>:
    //   17:   48 99                   cqto
    //   19:   48 f7 f9                idiv   $div
    // 000000000000001c <done>:

    // mov    $0x8000000000000000,%rdx
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xBA);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x00);
    emit_d8(cbuf, 0x80);

    // cmp    %rdx,%rax
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x39);
    emit_d8(cbuf, 0xD0);

    // jne    17 <normal>
    emit_opcode(cbuf, 0x75);
    emit_d8(cbuf, 0x08);

    // xor    %edx,%edx
    emit_opcode(cbuf, 0x33);
    emit_d8(cbuf, 0xD2);

    // cmp    $0xffffffffffffffff,$div
    emit_opcode(cbuf, opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8 ? Assembler::REX_W : Assembler::REX_WB);
    emit_opcode(cbuf, 0x83);
    emit_rm(cbuf, 0x3, 0x7, opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
    emit_d8(cbuf, 0xFF);

    // je     1e <done>
    emit_opcode(cbuf, 0x74);
    emit_d8(cbuf, 0x05);

    // <normal>
    // cqto
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0x99);

    // idivq (note: must be emitted by the user of this rule)
    // <done>
  
#line 22232 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* div */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 22244 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 22252 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(2)->reg(ra_,this,idx2)/* div */ & 7);
  
#line 22261 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22279 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22287 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 22296 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22322 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22330 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22350 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 22374 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22400 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 22408 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22428 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22442 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22460 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22468 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 22477 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22503 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22511 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22531 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22549 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22557 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 22566 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22592 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22600 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22620 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 22644 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22670 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 22678 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22698 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22712 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22730 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22738 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 22747 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22773 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 22781 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22801 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 22819 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22827 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 22836 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 22870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22890 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 22914 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 22940 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 22948 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 22968 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 22982 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 23000 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23008 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23017 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrI_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 23043 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23051 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23071 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23091 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23099 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23108 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23136 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23144 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23164 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 23190 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23218 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 23226 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23246 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23260 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23280 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23288 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23297 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void salL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23325 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23333 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23353 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23373 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23381 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23390 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23418 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23426 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23446 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 23472 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23500 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 23508 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23528 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23542 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23562 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23570 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x7 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23579 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sarL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23607 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23615 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x7 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23635 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_rReg_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23655 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23663 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23672 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_mem_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23700 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 23708 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23728 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 23754 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23782 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC1 /*primary()*/));
  
#line 23790 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23810 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 23824 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 23844 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23852 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x5 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 23861 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void shrL_mem_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 23889 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 23897 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x5 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 23917 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void i2bNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// twentyfour
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 23943 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 23951 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xBE /*secondary()*/));
  
#line 23959 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 23967 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void i2sNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// sixteen
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 23993 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 24001 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xBF /*secondary()*/));
  
#line 24009 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 24017 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolI_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 24034 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 24042 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24051 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolI_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 24075 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 24093 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 24101 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24110 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorI_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 24127 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 24135 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24144 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorI_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 24168 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorI_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 24186 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 24194 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24203 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolL_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 24222 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 24230 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24239 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolL_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 24265 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rolL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 24285 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 24293 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x0 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24302 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorL_rReg_imm1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 24321 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD1 /*primary()*/));
  
#line 24329 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24338 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorL_rReg_imm8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // SAL, SAR, SHR
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    emit_d8(cbuf, (0xC1 /*primary()*/));
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), dstenc);
    emit_d8(cbuf, opnd_array(2)->constant());
  
#line 24364 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rorL_rReg_CLNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// shift
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 24384 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD3 /*primary()*/));
  
#line 24392 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x1 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24401 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24427 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 24435 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 24443 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24469 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 24477 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 24485 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24493 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI2L_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 4) {
        emit_opcode(cbuf, opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24519 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 24527 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 24535 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 24543 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24569 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 24577 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 24585 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 24593 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI2L_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 24619 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 24627 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 24635 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 24643 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  
#line 24674 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 24688 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 24730 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 24738 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24754 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 24796 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 24804 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24820 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 24862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 24870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24886 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 24928 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 24936 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 24952 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 24978 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 24993 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 25013 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 25027 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25053 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 25061 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 25069 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  
#line 25100 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 25114 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25156 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 25164 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25180 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25222 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 25230 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25246 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25288 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 25296 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25312 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25354 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 25362 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25378 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 25404 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 25419 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 25439 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 25453 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 25479 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 25487 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 25495 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_rReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 8980 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ notl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
  
#line 25512 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  
#line 25543 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 25557 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25599 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 25607 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25623 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25665 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 25673 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25689 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25731 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 25739 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25755 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 25797 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 25805 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 25821 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorI_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2295 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      } else {
        emit_opcode(cbuf, Assembler::REX_XB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
        emit_opcode(cbuf, Assembler::REX_X);
      }
    }
  
#line 25847 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 25862 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 25882 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 25896 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 25924 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 25932 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 25940 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rReg_imm255Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 25968 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 25976 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6 /*secondary()*/));
  
#line 25984 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 25992 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rReg_imm65535Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 26020 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 26028 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB7 /*secondary()*/));
  
#line 26036 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7);
  
#line 26044 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x04 /*secondary()*/), dstenc);
  
#line 26077 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 26091 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26135 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 26143 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26159 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26203 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x23 /*primary()*/));
  
#line 26211 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26227 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26271 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 26279 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26295 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26339 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x21 /*primary()*/));
  
#line 26347 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26363 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void andL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 26391 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 26406 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x4 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 26426 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 26440 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 26468 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 26476 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 26484 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rReg_castP2XNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 26512 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 26520 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 26528 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rReg_castP2X_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 26556 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 26564 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(2)->reg(ra_,this,idx2)/* dst */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 26572 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x01 /*secondary()*/), dstenc);
  
#line 26605 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 26619 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26663 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 26671 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26687 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26731 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0B /*primary()*/));
  
#line 26739 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26755 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26799 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 26807 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26823 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 26867 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x09 /*primary()*/));
  
#line 26875 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 26891 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void orL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 26919 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 26934 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x1 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 26954 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 26968 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 26996 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 27004 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 27012 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_rReg_im1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// imm
  {
    MacroAssembler _masm(&cbuf);

#line 9223 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ notq(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */);
  
#line 27029 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x06 /*secondary()*/), dstenc);
  
#line 27062 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 27076 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 27120 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 27128 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* dst */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27144 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_rReg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* dst */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 27188 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x33 /*primary()*/));
  
#line 27196 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* dst */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27212 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 27256 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 27264 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27280 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_mem_rReg_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 27324 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x31 /*primary()*/));
  
#line 27332 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 27348 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void xorL_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 27376 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1932 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
  
#line 27391 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x6 /*secondary()*/);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 27411 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 27425 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27450 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1790 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 27459 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2580 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 27477 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* dst */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27495 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 27503 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 27511 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 27519 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convP2BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 27546 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1790 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 27555 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2580 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETNZ $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 27573 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* dst */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27591 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 27599 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 27607 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 27615 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* p */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* q */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* q */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27641 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1790 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B));
    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* p */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* q */ & 7);
  
#line 27650 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2565 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    } else if (dstenc >= 4) {
      emit_opcode(cbuf, Assembler::REX);
    }
    // SETLT $dst
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x9C);
    emit_rm(cbuf, 0x3, 0x0, dstenc);
  
#line 27668 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2336 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
      if (opnd_array(0)->reg(ra_,this)/* dst */ >= 4) {
        emit_opcode(cbuf, opnd_array(0)->reg(ra_,this)/* dst */ < 8 ? Assembler::REX : Assembler::REX_B);
      }
    } else {
      if (opnd_array(0)->reg(ra_,this)/* dst */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 27686 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F));
  
#line 27694 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xB6));
  
#line 27702 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(0)->reg(ra_,this)/* dst */ & 7, opnd_array(0)->reg(ra_,this)/* dst */ & 7);
  
#line 27710 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2539 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // NEG $dst
    emit_opcode(cbuf, 0xF7);
    emit_rm(cbuf, 0x3, 0x03, dstenc);
  
#line 27725 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpLTMask0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 9338 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

  __ sarl(opnd_array(1)->as_Register(ra_,this,idx1)/* dst */, 31);
  
#line 27742 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cadd_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    MacroAssembler _masm(&cbuf);

#line 9354 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 27767 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cadd_cmpLTMask_1Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    MacroAssembler _masm(&cbuf);

#line 9354 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 27792 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cadd_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    MacroAssembler _masm(&cbuf);

#line 9354 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 27817 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cadd_cmpLTMask_2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    MacroAssembler _masm(&cbuf);

#line 9354 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ subl(Rp, Rq);
    __ jccb(Assembler::greaterEqual, done);
    __ addl(Rp, Ry);
    __ bind(done);
  
#line 27842 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void and_cmpLTMaskNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// p
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// q
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// y
  {
    MacroAssembler _masm(&cbuf);

#line 9379 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(1)->as_Register(ra_,this,idx1)/* p */;
    Register Rq = opnd_array(2)->as_Register(ra_,this,idx2)/* q */;
    Register Ry = opnd_array(3)->as_Register(ra_,this,idx3)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 27867 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void and_cmpLTMask_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// y
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// p
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// q
  {
    MacroAssembler _masm(&cbuf);

#line 9379 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rp = opnd_array(2)->as_Register(ra_,this,idx2)/* p */;
    Register Rq = opnd_array(3)->as_Register(ra_,this,idx3)/* q */;
    Register Ry = opnd_array(1)->as_Register(ra_,this,idx1)/* y */;
    Label done;
    __ cmpl(Rp, Rq);
    __ jccb(Assembler::less, done);
    __ xorl(Ry, Ry);
    __ bind(done);
  
#line 27892 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9406 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 27910 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_reg_CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9418 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 27927 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9435 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
    emit_cmpfp_fixup(_masm);
  
#line 27945 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_memCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9447 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 27962 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9463 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    emit_cmpfp_fixup(_masm);
  
#line 27980 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_immNode::eval_constant(Compile* C) {
  {

#line 9464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 27989 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_cc_immCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9474 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28005 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_cc_immCFNode::eval_constant(Compile* C) {
  {

#line 9475 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 28014 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9491 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp_fixup(_masm);
  
#line 28031 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_reg_CFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9503 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 28048 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9520 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
    emit_cmpfp_fixup(_masm);
  
#line 28066 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_memCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9532 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 28083 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9548 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    emit_cmpfp_fixup(_masm);
  
#line 28101 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_immNode::eval_constant(Compile* C) {
  {

#line 9549 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 28110 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_cc_immCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9559 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 28126 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_cc_immCFNode::eval_constant(Compile* C) {
  {

#line 9560 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 28135 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9579 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28152 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9600 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28170 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9620 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomiss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28188 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpF_immNode::eval_constant(Compile* C) {
  {

#line 9621 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 28197 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cmpD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9641 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28214 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 9662 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28232 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 9682 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ ucomisd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    emit_cmpfp3(_masm, opnd_array(0)->as_Register(ra_,this)/* dst */);
  
#line 28250 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpD_immNode::eval_constant(Compile* C) {
  {

#line 9683 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 28259 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void cosD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 2654 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ fld_d(Address(rsp, 0));
  
#line 28276 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28284 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*secondary()*/));
  
#line 28292 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2647 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28303 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sinD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 2654 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ fld_d(Address(rsp, 0));
  
#line 28321 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9 /*primary()*/));
  
#line 28329 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFE /*secondary()*/));
  
#line 28337 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2647 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28348 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void tanD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 2654 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ fld_d(Address(rsp, 0));
  
#line 28366 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28374 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF2));
  
#line 28382 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xDD));
  
#line 28390 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD8));
  
#line 28398 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2647 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28409 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void log10D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28424 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xEC));
  
#line 28432 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2654 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ fld_d(Address(rsp, 0));
  
#line 28443 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28451 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF1));
  
#line 28459 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2647 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28470 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void logD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28485 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xED));
  
#line 28493 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2654 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */);
    __ fld_d(Address(rsp, 0));
  
#line 28504 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xD9));
  
#line 28512 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF1));
  
#line 28520 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2647 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    MacroAssembler _masm(&cbuf);
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28531 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void powD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src0
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 9754 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */);
    __ fld_d(Address(rsp, 0));
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src0 */);
    __ fld_d(Address(rsp, 0));
    __ fast_pow();
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28556 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void expD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9772 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ fld_d(Address(rsp, 0));
    __ fast_exp();
    __ fstp_d(Address(rsp, 0));
    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, 0));
    __ addptr(rsp, 8);
  
#line 28578 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void roundFloat_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void roundDouble_nopNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  // User did not define which encode class to use.
}

void convF2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9809 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtss2sd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 28610 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convF2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9820 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtss2sd (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28626 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convD2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9831 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsd2ss (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 28642 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convD2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9842 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsd2ss (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28658 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9862 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label done;
    __ cvttss2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, done);
    __ subptr(rsp, 8);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::f2i_fixup())));
    __ pop(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 28682 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convF2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9889 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label done;
    __ cvttss2siq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmp64(opnd_array(0)->as_Register(ra_,this)/* dst */,
             ExternalAddress((address) StubRoutines::x86::double_sign_flip()));
    __ jccb(Assembler::notEqual, done);
    __ subptr(rsp, 8);
    __ movflt(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::f2l_fixup())));
    __ pop(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 28707 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convD2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9917 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label done;
    __ cvttsd2sil(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmpl(opnd_array(0)->as_Register(ra_,this)/* dst */, 0x80000000);
    __ jccb(Assembler::notEqual, done);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2i_fixup())));
    __ pop(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 28731 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label done;
    __ cvttsd2siq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ cmp64(opnd_array(0)->as_Register(ra_,this)/* dst */,
             ExternalAddress((address) StubRoutines::x86::double_sign_flip()));
    __ jccb(Assembler::notEqual, done);
    __ subptr(rsp, 8);
    __ movdbl(Address(rsp, 0), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
    __ call(RuntimeAddress(CAST_FROM_FN_PTR(address, StubRoutines::x86::d2l_fixup())));
    __ pop(opnd_array(0)->as_Register(ra_,this)/* dst */);
    __ bind(done);
  
#line 28756 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9965 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2ssl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28772 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9976 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2ssl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28788 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9988 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28804 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 9999 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2sdl (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28820 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convXI2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10012 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2ps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 28837 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convXI2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10026 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ cvtdq2pd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 28854 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convL2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10038 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2ssq (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28870 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convL2F_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10049 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2ssq (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28886 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10060 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2sdq (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28902 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convL2D_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10071 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cvtsi2sdq (opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28918 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10083 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movslq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28934 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2L_reg_reg_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 10112 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(0)->reg(ra_,this)/* dst */!= opnd_array(1)->reg(ra_,this,idx1)/* src */) {
      __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    }
  
#line 28953 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convI2L_reg_mem_zexNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 10126 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 28970 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void zerox_long_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mask
  {
    MacroAssembler _masm(&cbuf);

#line 10137 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 28987 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void convL2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29003 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveF2I_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10161 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29019 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveI2F_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10173 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29035 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveD2L_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10185 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(opnd_array(0)->as_Register(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29051 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveL2D_stack_reg_partialNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10198 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29067 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveL2D_stack_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10211 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address(rsp, opnd_array(1)->disp(ra_,this,idx1)));
  
#line 29083 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveF2I_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10224 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movflt(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29099 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveI2F_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10236 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29115 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveD2L_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10248 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdbl(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29131 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveL2D_reg_stackNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10260 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movq(Address(rsp, opnd_array(0)->disp(ra_,this,0)), opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29147 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveF2I_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10271 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29163 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveD2L_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10282 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdq(opnd_array(0)->as_Register(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 29179 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveI2F_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10293 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29195 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void MoveL2D_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 10304 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

     __ movdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
  
#line 29211 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rep_stosNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    MacroAssembler _masm(&cbuf);

#line 10322 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(RAX_enc));
  
#line 29228 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void rep_fast_stosbNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// cnt
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// base
  {
    MacroAssembler _masm(&cbuf);

#line 10337 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ clear_mem(opnd_array(2)->as_Register(ra_,this,idx2)/* base */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */, as_Register(RAX_enc));
  
#line 29245 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void string_compareNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp1
  {
    MacroAssembler _masm(&cbuf);

#line 10350 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ string_compare(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */, opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp1 */);
  
#line 29267 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void string_indexof_conNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// int_cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec
  {
    MacroAssembler _masm(&cbuf);

#line 10367 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int icnt2 = (int)opnd_array(4)->constant();
    if (icnt2 >= 8) {
      // IndexOf for constant substrings with size >= 8 elements
      // which don't need to be loaded through stack.
      __ string_indexofC8(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                          opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(RAX_enc),
                          icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                          opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec */, as_Register(RCX_enc));
    } else {
      // Small strings are loaded through stack if they cross page boundary.
      __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                        opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, as_Register(RAX_enc),
                        icnt2, opnd_array(0)->as_Register(ra_,this)/* result */,
                        opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec */, as_Register(RCX_enc));
    }
  
#line 29301 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void string_indexofNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cnt1
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// str2
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// cnt2
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// vec
  {
    MacroAssembler _masm(&cbuf);

#line 10395 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ string_indexof(opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(3)->as_Register(ra_,this,idx3)/* str2 */,
                      opnd_array(2)->as_Register(ra_,this,idx2)/* cnt1 */, opnd_array(4)->as_Register(ra_,this,idx4)/* cnt2 */,
                      (-1), opnd_array(0)->as_Register(ra_,this)/* result */,
                      opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* vec */, as_Register(RCX_enc));
  
#line 29324 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void string_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// str1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// str2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// cnt
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp1
  unsigned idx5 = idx4 + opnd_array(4)->num_edges(); 	// tmp2
  {
    MacroAssembler _masm(&cbuf);

#line 10412 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ char_arrays_equals(false, opnd_array(1)->as_Register(ra_,this,idx1)/* str1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* str2 */,
                          opnd_array(3)->as_Register(ra_,this,idx3)/* cnt */, opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(RBX_enc),
                          opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp1 */, opnd_array(5)->as_XMMRegister(ra_,this,idx5)/* tmp2 */);
  
#line 29346 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void array_equalsNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// ary1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// ary2
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp1
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// tmp2
  {
    MacroAssembler _masm(&cbuf);

#line 10429 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ char_arrays_equals(true, opnd_array(1)->as_Register(ra_,this,idx1)/* ary1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* ary2 */,
                          as_Register(RCX_enc), opnd_array(0)->as_Register(ra_,this)/* result */, as_Register(RBX_enc),
                          opnd_array(3)->as_XMMRegister(ra_,this,idx3)/* tmp1 */, opnd_array(4)->as_XMMRegister(ra_,this,idx4)/* tmp2 */);
  
#line 29367 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compI_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29393 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29401 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* op2 */ & 7);
  
#line 29409 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compI_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 29440 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 29454 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compI_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29496 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29504 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29520 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testI_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29546 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29554 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 29562 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testI_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 29581 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 29589 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 29598 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2256 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constant());
  
#line 29607 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testI_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29650 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29658 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29674 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testI_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29717 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29725 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29741 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compU_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29767 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29775 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* op2 */ & 7);
  
#line 29783 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compU_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1944 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 29814 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constant() && opnd_array(2)->constant() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constant());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constant());
    }
  
#line 29828 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compU_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2433 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) >= 8) {
          emit_opcode(cbuf, Assembler::REX_X);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_B);
        } else {
          emit_opcode(cbuf, Assembler::REX_XB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_R);
        } else {
          emit_opcode(cbuf, Assembler::REX_RX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_RB);
        } else {
          emit_opcode(cbuf, Assembler::REX_RXB);
        }
      }
    }
  
#line 29870 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29878 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 29894 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testU_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 29920 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 29928 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 29936 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compP_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 29964 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 29972 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* op2 */ & 7);
  
#line 29980 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compP_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30024 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30032 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30048 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compP_mem_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30092 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30100 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30116 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testP_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30144 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30152 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 30160 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testP_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2310 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->base(ra_,this,idx1) >= 8) {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_WB);
      } else {
        emit_opcode(cbuf, Assembler::REX_WXB);
      }
    } else {
      if (opnd_array(1)->index(ra_,this,idx1) < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WX);
      }
    }
  
#line 30188 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 30196 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2510 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int rm_byte_opcode = (0x00);

    // High registers handle in encode_RegMem
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int displace = opnd_array(1)->disp(ra_,this,idx1);

    bool disp_is_oop = opnd_array(1)->disp_is_oop(); // disp-as-oop when
                                            // working with static
                                            // globals
    encode_RegMem(cbuf, rm_byte_opcode, base, index, scale, displace,
                  disp_is_oop);
  
#line 30216 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2277 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_d32(cbuf,(0xFFFFFFFF));
  
#line 30224 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testP_mem_reg0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10637 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpq(r12, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 30241 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compN_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 10648 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, opnd_array(2)->as_Register(ra_,this,idx2)/* op2 */); 
#line 30256 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compN_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 10657 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 30273 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compN_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {
    MacroAssembler _masm(&cbuf);

#line 10667 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmp_narrow_oop(opnd_array(1)->as_Register(ra_,this,idx1)/* op1 */, (jobject)opnd_array(2)->constant());
  
#line 30290 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compN_mem_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 10678 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmp_narrow_oop(Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), (jobject)opnd_array(1)->constant());
  
#line 30307 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testN_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10688 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"
__ testl(opnd_array(1)->as_Register(ra_,this,idx1)/* src */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */); 
#line 30322 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testN_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10699 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), (int)0xFFFFFFFF);
  
#line 30339 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testN_mem_reg0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 10711 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ cmpl(r12, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 30356 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30384 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30392 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* op2 */ & 7);
  
#line 30400 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 30433 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 30447 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30491 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30499 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30515 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30543 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30551 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 30559 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testL_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2392 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
    }
  
#line 30580 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xF7 /*primary()*/));
  
#line 30588 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x00 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 30597 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2256 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Output immediate
    emit_d32(cbuf, opnd_array(2)->constantL());
  
#line 30606 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testL_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30651 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30659 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* src */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30675 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testL_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(1)->base(ra_,this,idx1) < 8) {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(1)->index(ra_,this,idx1) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30720 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 30728 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(2)->reg(ra_,this,idx2)/* src */;
    int base = opnd_array(1)->base(ra_,this,idx1);
    int index = opnd_array(1)->index(ra_,this,idx1);
    int scale = opnd_array(1)->scale();
    int disp = opnd_array(1)->disp(ra_,this,idx1);
    bool disp_is_oop = opnd_array(1)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30744 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpL3_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {

#line 2597 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    int src1enc = opnd_array(1)->reg(ra_,this,idx1)/* src1 */;
    int src2enc = opnd_array(2)->reg(ra_,this,idx2)/* src2 */;
    int dstenc = opnd_array(0)->reg(ra_,this)/* dst */;

    // cmpq $src1, $src2
    if (src1enc < 8) {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (src2enc < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
    emit_opcode(cbuf, 0x3B);
    emit_rm(cbuf, 0x3, src1enc & 7, src2enc & 7);

    // movl $dst, -1
    if (dstenc >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
    emit_opcode(cbuf, 0xB8 | (dstenc & 7));
    emit_d32(cbuf, -1);

    // jl,s done
    emit_opcode(cbuf, 0x7C);
    emit_d8(cbuf, dstenc < 4 ? 0x06 : 0x08);

    // setne $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_B);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0x95);
    emit_opcode(cbuf, 0xC0 | (dstenc & 7));

    // movzbl $dst, $dst
    if (dstenc >= 4) {
      emit_opcode(cbuf, dstenc < 8 ? Assembler::REX : Assembler::REX_RB);
    }
    emit_opcode(cbuf, 0x0F);
    emit_opcode(cbuf, 0xB6);
    emit_rm(cbuf, 0x3, dstenc & 7, dstenc & 7);
  
#line 30806 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compUL_rRegNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* op2 */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30834 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30842 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* op1 */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* op2 */ & 7);
  
#line 30850 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compUL_rReg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 1964 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // OpcSEr/m
    int dstenc = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    if (dstenc < 8) {
      emit_opcode(cbuf, Assembler::REX_W);
    } else {
      emit_opcode(cbuf, Assembler::REX_WB);
      dstenc -= 8;
    }
    // Emit primary opcode and set sign-extend bit
    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_opcode(cbuf, (0x81 /*primary()*/) | 0x02);
    } else {
      // 32-bit immediate
      emit_opcode(cbuf, (0x81 /*primary()*/));
    }
    // Emit r/m byte with secondary opcode, after primary opcode.
    emit_rm(cbuf, 0x3, (0x07 /*secondary()*/), dstenc);
  
#line 30883 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1986 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // Check for 8-bit immediate, and set sign extend bit in opcode
    if (-0x80 <= opnd_array(2)->constantL() && opnd_array(2)->constantL() < 0x80) {
      emit_d8(cbuf, opnd_array(2)->constantL());
    } else {
      // 32-bit immediate
      emit_d32(cbuf, opnd_array(2)->constantL());
    }
  
#line 30897 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void compUL_rReg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// op1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// op2
  {

#line 2464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* op1 */ < 8) {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_W);
        } else {
          emit_opcode(cbuf, Assembler::REX_WX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WXB);
        }
      }
    } else {
      if (opnd_array(2)->base(ra_,this,idx2) < 8) {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WR);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRX);
        }
      } else {
        if (opnd_array(2)->index(ra_,this,idx2) < 8) {
          emit_opcode(cbuf, Assembler::REX_WRB);
        } else {
          emit_opcode(cbuf, Assembler::REX_WRXB);
        }
      }
    }
  
#line 30941 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x3B /*primary()*/));
  
#line 30949 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2497 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // High registers handle in encode_RegMem
    int reg = opnd_array(1)->reg(ra_,this,idx1)/* op1 */;
    int base = opnd_array(2)->base(ra_,this,idx2);
    int index = opnd_array(2)->index(ra_,this,idx2);
    int scale = opnd_array(2)->scale();
    int disp = opnd_array(2)->disp(ra_,this,idx2);
    bool disp_is_oop = opnd_array(2)->disp_is_oop();

    encode_RegMem(cbuf, reg, base, index, scale, disp, disp_is_oop);
  
#line 30965 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void testUL_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// zero
  {

#line 2416 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_W);
      } else {
        emit_opcode(cbuf, Assembler::REX_WB);
      }
    } else {
      if (opnd_array(1)->reg(ra_,this,idx1)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_WR);
      } else {
        emit_opcode(cbuf, Assembler::REX_WRB);
      }
    }
  
#line 30993 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x85 /*primary()*/));
  
#line 31001 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7, opnd_array(1)->reg(ra_,this,idx1)/* src */ & 7);
  
#line 31009 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovI_reg_gNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 31036 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 31044 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x4F /*secondary()*/));
  
#line 31052 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 31060 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmovI_reg_lNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// cr
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {

#line 2401 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* dst */ < 8) {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ >= 8) {
        emit_opcode(cbuf, Assembler::REX_B);
      }
    } else {
      if (opnd_array(2)->reg(ra_,this,idx2)/* src */ < 8) {
        emit_opcode(cbuf, Assembler::REX_R);
      } else {
        emit_opcode(cbuf, Assembler::REX_RB);
      }
    }
  
#line 31087 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x0F /*primary()*/));
  
#line 31095 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1757 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x4C /*secondary()*/));
  
#line 31103 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1785 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_rm(cbuf, 0x3, opnd_array(1)->reg(ra_,this,idx1)/* dst */ & 7, opnd_array(2)->reg(ra_,this,idx2)/* src */ & 7);
  
#line 31111 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jmpDirNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10901 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(1)->label();
    __ jmp(*L, false); // Always long jump
  
#line 31128 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpDirNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 5, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 5);
}

void jmpConNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10917 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31152 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10933 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31176 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEndNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10948 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31200 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEndUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpLoopEndUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10962 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31224 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEndUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10977 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31248 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConUNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCFNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 10991 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jcc((Assembler::Condition)(opnd_array(1)->ccode()), *L, false); // Always long jump
  
#line 31272 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConUCFNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 6, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 6);
}

void jmpConUCF2Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11013 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jcc(Assembler::parity, *l, false);
      __ jcc(Assembler::notEqual, *l, false);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jcc(Assembler::equal, *l, false);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 31306 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void partialSubtypeCheckNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  {

#line 2022 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label miss;
    const bool set_cond_codes = true;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Rrsi, Rrax, Rrcx, Rrdi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x1 /*primary()*/)) {
      __ xorptr(Rrdi, Rrdi);
    }
    __ bind(miss);
  
#line 31336 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void partialSubtypeCheck_vs_ZeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// sub
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// super
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// zero
  {

#line 2022 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register Rrdi = as_Register(RDI_enc); // result register
    Register Rrax = as_Register(RAX_enc); // super class
    Register Rrcx = as_Register(RCX_enc); // killed
    Register Rrsi = as_Register(RSI_enc); // sub class
    Label miss;
    const bool set_cond_codes = true;

    MacroAssembler _masm(&cbuf);
    __ check_klass_subtype_slow_path(Rrsi, Rrax, Rrcx, Rrdi,
                                     NULL, &miss,
                                     /*set_cond_codes:*/ true);
    if ((0x0 /*primary()*/)) {
      __ xorptr(Rrdi, Rrdi);
    }
    __ bind(miss);
  
#line 31367 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void jmpDir_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11101 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(1)->label();
    __ jmpb(*L);
  
#line 31384 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpDir_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpCon_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11117 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31408 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpCon_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEnd_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cr
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11133 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31432 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEnd_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11149 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31456 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEndU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpLoopEndUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11164 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31480 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpLoopEndUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConU_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11180 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31504 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConU_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11195 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* L = opnd_array(3)->label();
    __ jccb((Assembler::Condition)(opnd_array(1)->ccode()), *L);
  
#line 31528 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConUCF_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 2, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 2);
}

void jmpConUCF2_shortNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cmp
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// labl
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11219 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Label* l = opnd_array(3)->label();
    if (opnd_array(1)->ccode()== Assembler::notEqual) {
      __ jccb(Assembler::parity, *l);
      __ jccb(Assembler::notEqual, *l);
    } else if (opnd_array(1)->ccode()== Assembler::equal) {
      Label done;
      __ jccb(Assembler::parity, done);
      __ jccb(Assembler::equal, *l);
      __ bind(done);
    } else {
       ShouldNotReachHere();
    }
  
#line 31562 "../generated/adfiles/ad_x86_64.cpp"
  }
}

uint jmpConUCF2_shortNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 4, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 4);
}

void cmpFastLockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  unsigned idx4 = idx3 + opnd_array(3)->num_edges(); 	// scr
  {

#line 2674 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    Register objReg = as_Register((int)opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register boxReg = as_Register((int)opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    Register scrReg = as_Register(opnd_array(4)->reg(ra_,this,idx4)/* scr */);
    MacroAssembler masm(&cbuf);

    // Verify uniqueness of register assignments -- necessary but not sufficient
    assert (objReg != boxReg && objReg != tmpReg &&
            objReg != scrReg && tmpReg != scrReg, "invariant") ;

    if (_counters != NULL) {
      masm.atomic_incl(ExternalAddress((address) _counters->total_entry_count_addr()));
    }
    if (EmitSync & 1) {
        // Without cast to int32_t a movptr will destroy r10 which is typically obj
        masm.movptr (Address(boxReg, 0), (int32_t)intptr_t(markOopDesc::unused_mark())) ;
        masm.cmpptr(rsp, (int32_t)NULL_WORD) ;
    } else
    if (EmitSync & 2) {
        Label DONE_LABEL;
        if (UseBiasedLocking) {
           // Note: tmpReg maps to the swap_reg argument and scrReg to the tmp_reg argument.
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, false, DONE_LABEL, NULL, _counters);
        }
        // QQQ was movl...
        masm.movptr(tmpReg, 0x1);
        masm.orptr(tmpReg, Address(objReg, 0));
        masm.movptr(Address(boxReg, 0), tmpReg);
        if (os::is_MP()) {
          masm.lock();
        }
        masm.cmpxchgptr(boxReg, Address(objReg, 0)); // Updates tmpReg
        masm.jcc(Assembler::equal, DONE_LABEL);

        // Recursive locking
        masm.subptr(tmpReg, rsp);
        masm.andptr(tmpReg, 7 - os::vm_page_size());
        masm.movptr(Address(boxReg, 0), tmpReg);

        masm.bind(DONE_LABEL);
        masm.nop(); // avoid branch to branch
    } else {
        Label DONE_LABEL, IsInflated, Egress;

        masm.movptr(tmpReg, Address(objReg, 0)) ;
        masm.testl (tmpReg, 0x02) ;         // inflated vs stack-locked|neutral|biased
        masm.jcc   (Assembler::notZero, IsInflated) ;

        // it's stack-locked, biased or neutral
        // TODO: optimize markword triage order to reduce the number of
        // conditional branches in the most common cases.
        // Beware -- there's a subtle invariant that fetch of the markword
        // at [FETCH], below, will never observe a biased encoding (*101b).
        // If this invariant is not held we'll suffer exclusion (safety) failure.

        if (UseBiasedLocking && !UseOptoBiasInlining) {
          masm.biased_locking_enter(boxReg, objReg, tmpReg, scrReg, true, DONE_LABEL, NULL, _counters);
          masm.movptr(tmpReg, Address(objReg, 0)) ;        // [FETCH]
        }

        // was q will it destroy high?
        masm.orl   (tmpReg, 1) ;
        masm.movptr(Address(boxReg, 0), tmpReg) ;
        if (os::is_MP()) { masm.lock(); }
        masm.cmpxchgptr(boxReg, Address(objReg, 0)); // Updates tmpReg
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jcc   (Assembler::equal, DONE_LABEL);

        // Recursive locking
        masm.subptr(tmpReg, rsp);
        masm.andptr(tmpReg, 7 - os::vm_page_size());
        masm.movptr(Address(boxReg, 0), tmpReg);
        if (_counters != NULL) {
           masm.cond_inc32(Assembler::equal,
                           ExternalAddress((address) _counters->fast_path_entry_count_addr()));
        }
        masm.jmp   (DONE_LABEL) ;

        masm.bind  (IsInflated) ;
        // It's inflated

        // TODO: someday avoid the ST-before-CAS penalty by
        // relocating (deferring) the following ST.
        // We should also think about trying a CAS without having
        // fetched _owner.  If the CAS is successful we may
        // avoid an RTO->RTS upgrade on the $line.
        // Without cast to int32_t a movptr will destroy r10 which is typically obj
        masm.movptr(Address(boxReg, 0), (int32_t)intptr_t(markOopDesc::unused_mark())) ;

        masm.mov    (boxReg, tmpReg) ;
        masm.movptr (tmpReg, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
        masm.testptr(tmpReg, tmpReg) ;
        masm.jcc    (Assembler::notZero, DONE_LABEL) ;

        // It's inflated and appears unlocked
        if (os::is_MP()) { masm.lock(); }
        masm.cmpxchgptr(r15_thread, Address(boxReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
        // Intentional fall-through into DONE_LABEL ...

        masm.bind  (DONE_LABEL) ;
        masm.nop   () ;                 // avoid jmp to jmp
    }
  
#line 31689 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void cmpFastUnlockNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// object
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// box
  unsigned idx3 = idx2 + opnd_array(2)->num_edges(); 	// tmp
  {

#line 2786 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"


    Register objReg = as_Register(opnd_array(1)->reg(ra_,this,idx1)/* object */);
    Register boxReg = as_Register(opnd_array(2)->reg(ra_,this,idx2)/* box */);
    Register tmpReg = as_Register(opnd_array(3)->reg(ra_,this,idx3)/* tmp */);
    MacroAssembler masm(&cbuf);

    if (EmitSync & 4) {
       masm.cmpptr(rsp, 0) ;
    } else
    if (EmitSync & 8) {
       Label DONE_LABEL;
       if (UseBiasedLocking) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }

       // Check whether the displaced header is 0
       //(=> recursive unlock)
       masm.movptr(tmpReg, Address(boxReg, 0));
       masm.testptr(tmpReg, tmpReg);
       masm.jcc(Assembler::zero, DONE_LABEL);

       // If not recursive lock, reset the header to displaced header
       if (os::is_MP()) {
         masm.lock();
       }
       masm.cmpxchgptr(tmpReg, Address(objReg, 0)); // Uses RAX which is box
       masm.bind(DONE_LABEL);
       masm.nop(); // avoid branch to branch
    } else {
       Label DONE_LABEL, Stacked, CheckSucc ;

       if (UseBiasedLocking && !UseOptoBiasInlining) {
         masm.biased_locking_exit(objReg, tmpReg, DONE_LABEL);
       }

       masm.movptr(tmpReg, Address(objReg, 0)) ;
       masm.cmpptr(Address(boxReg, 0), (int32_t)NULL_WORD) ;
       masm.jcc   (Assembler::zero, DONE_LABEL) ;
       masm.testl (tmpReg, 0x02) ;
       masm.jcc   (Assembler::zero, Stacked) ;

       // It's inflated
       masm.movptr(boxReg, Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2)) ;
       masm.xorptr(boxReg, r15_thread) ;
       masm.orptr (boxReg, Address (tmpReg, ObjectMonitor::recursions_offset_in_bytes()-2)) ;
       masm.jcc   (Assembler::notZero, DONE_LABEL) ;
       masm.movptr(boxReg, Address (tmpReg, ObjectMonitor::cxq_offset_in_bytes()-2)) ;
       masm.orptr (boxReg, Address (tmpReg, ObjectMonitor::EntryList_offset_in_bytes()-2)) ;
       masm.jcc   (Assembler::notZero, CheckSucc) ;
       masm.movptr(Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
       masm.jmp   (DONE_LABEL) ;

       if ((EmitSync & 65536) == 0) {
         Label LSuccess, LGoSlowPath ;
         masm.bind  (CheckSucc) ;
         masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         masm.jcc   (Assembler::zero, LGoSlowPath) ;

         // I'd much rather use lock:andl m->_owner, 0 as it's faster than the
         // the explicit ST;MEMBAR combination, but masm doesn't currently support
         // "ANDQ M,IMM".  Don't use MFENCE here.  lock:add to TOS, xchg, etc
         // are all faster when the write buffer is populated.
         masm.movptr (Address (tmpReg, ObjectMonitor::owner_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         if (os::is_MP()) {
            masm.lock () ; masm.addl (Address(rsp, 0), 0) ;
         }
         masm.cmpptr(Address (tmpReg, ObjectMonitor::succ_offset_in_bytes()-2), (int32_t)NULL_WORD) ;
         masm.jcc   (Assembler::notZero, LSuccess) ;

         masm.movptr (boxReg, (int32_t)NULL_WORD) ;                   // box is really EAX
         if (os::is_MP()) { masm.lock(); }
         masm.cmpxchgptr(r15_thread, Address(tmpReg, ObjectMonitor::owner_offset_in_bytes()-2));
         masm.jcc   (Assembler::notEqual, LSuccess) ;
         // Intentional fall-through into slow-path

         masm.bind  (LGoSlowPath) ;
         masm.orl   (boxReg, 1) ;                      // set ICC.ZF=0 to indicate failure
         masm.jmp   (DONE_LABEL) ;

         masm.bind  (LSuccess) ;
         masm.testl (boxReg, 0) ;                      // set ICC.ZF=1 to indicate success
         masm.jmp   (DONE_LABEL) ;
       }

       masm.bind  (Stacked) ;
       masm.movptr(tmpReg, Address (boxReg, 0)) ;      // re-fetch
       if (os::is_MP()) { masm.lock(); }
       masm.cmpxchgptr(tmpReg, Address(objReg, 0)); // Uses RAX which is box

       if (EmitSync & 65536) {
          masm.bind (CheckSucc) ;
       }
       masm.bind(DONE_LABEL);
       if (EmitSync & 32768) {
          masm.nop();                      // avoid branch to branch
       }
    }
  
#line 31802 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void safePoint_pollNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 11276 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    AddressLiteral addr(os::get_polling_page(), relocInfo::poll_type);
    __ testl(rax, addr);
  
#line 31819 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void safePoint_poll_farNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// cr
  {
    MacroAssembler _masm(&cbuf);

#line 11292 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    __ relocate(relocInfo::poll_type);
    __ testl(rax, Address(opnd_array(1)->as_Register(ra_,this,idx1)/* poll */, 0));
  
#line 31836 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallStaticJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2040 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->max_vector_size() > 16) {
      // Clear upper bits of YMM registers when current compiled code uses
      // wide vectors to avoid AVX <-> SSE transition penalty during call.
      MacroAssembler _masm(&cbuf);
      __ vzeroupper();
    }
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == clear_avx_size(), "correct size prediction");
  
#line 31859 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2073 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA STATIC CALL
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to
    // determine who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_DISP32);
    } else if (_optimized_virtual) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     opt_virtual_call_Relocation::spec(),
                     RELOC_DISP32);
    } else {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     static_call_Relocation::spec(),
                     RELOC_DISP32);
    }
    if (_method) {
      // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  
#line 31892 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 823 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 31910 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallStaticJavaHandleNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// rbp_mh_SP_save
  {

#line 2040 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->max_vector_size() > 16) {
      // Clear upper bits of YMM registers when current compiled code uses
      // wide vectors to avoid AVX <-> SSE transition penalty during call.
      MacroAssembler _masm(&cbuf);
      __ vzeroupper();
    }
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == clear_avx_size(), "correct size prediction");
  
#line 31933 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 808 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    debug_only(int off0 = cbuf.insts_size());
    MacroAssembler _masm(&cbuf);
    // RBP is preserved across all calls, even compiled calls.
    // Use it to preserve RSP in places where the callee might change the SP.
    __ movptr(rbp_mh_SP_save, rsp);
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == preserve_SP_size(), "correct size prediction");
  
#line 31947 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2073 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA STATIC CALL
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to
    // determine who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));

    if (!_method) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     runtime_call_Relocation::spec(),
                     RELOC_DISP32);
    } else if (_optimized_virtual) {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     opt_virtual_call_Relocation::spec(),
                     RELOC_DISP32);
    } else {
      emit_d32_reloc(cbuf,
                     (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                     static_call_Relocation::spec(),
                     RELOC_DISP32);
    }
    if (_method) {
      // Emit stub for static call
      emit_java_to_interp(cbuf);
    }
  
#line 31980 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 818 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    MacroAssembler _masm(&cbuf);
    __ movptr(rsp, rbp_mh_SP_save);
  
#line 31989 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 823 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 32007 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallDynamicJavaDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2040 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->max_vector_size() > 16) {
      // Clear upper bits of YMM registers when current compiled code uses
      // wide vectors to avoid AVX <-> SSE transition penalty during call.
      MacroAssembler _masm(&cbuf);
      __ vzeroupper();
    }
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == clear_avx_size(), "correct size prediction");
  
#line 32030 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2103 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // JAVA DYNAMIC CALL
    // !!!!!
    // Generate  "movq rax, -1", placeholder instruction to load oop-info
    // emit_call_dynamic_prologue( cbuf );
    cbuf.set_insts_mark();

    // movq rax, -1
    emit_opcode(cbuf, Assembler::REX_W);
    emit_opcode(cbuf, 0xB8 | RAX_enc);
    emit_d64_reloc(cbuf,
                   (int64_t) Universe::non_oop_word(),
                   oop_Relocation::spec_for_immediate(), RELOC_IMM64);
    address virtual_call_oop_addr = cbuf.insts_mark();
    // CALL to fixup routine.  Fixup routine uses ScopeDesc info to determine
    // who we intended to call.
    cbuf.set_insts_mark();
    emit_d8(cbuf, (0xE8 /*primary()*/));
    emit_d32_reloc(cbuf,
                   (int) (opnd_array(1)->method() - ((intptr_t) cbuf.insts_end()) - 4),
                   virtual_call_Relocation::spec(virtual_call_oop_addr),
                   RELOC_DISP32);
  
#line 32058 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 823 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    if (VerifyStackAtCalls) {
      // Check that stack depth is unchanged: find majik cookie on stack
      int framesize = ra_->reg2offset_unchecked(OptoReg::add(ra_->_matcher._old_SP, -3*VMRegImpl::slots_per_word));
      MacroAssembler _masm(&cbuf);
      Label L;
      __ cmpptr(Address(rsp, framesize), (int32_t)0xbadb100d);
      __ jccb(Assembler::equal, L);
      // Die if stack mismatch
      __ int3();
      __ bind(L);
    }
  
#line 32076 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallRuntimeDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2040 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->max_vector_size() > 16) {
      // Clear upper bits of YMM registers when current compiled code uses
      // wide vectors to avoid AVX <-> SSE transition penalty during call.
      MacroAssembler _masm(&cbuf);
      __ vzeroupper();
    }
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == clear_avx_size(), "correct size prediction");
  
#line 32099 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2052 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // No relocation needed
    MacroAssembler _masm(&cbuf);
    __ mov64(r10, (int64_t) opnd_array(1)->method());
    __ call(r10);
  
#line 32110 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallLeafDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2040 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    debug_only(int off0 = cbuf.insts_size());
    if (ra_->C->max_vector_size() > 16) {
      // Clear upper bits of YMM registers when current compiled code uses
      // wide vectors to avoid AVX <-> SSE transition penalty during call.
      MacroAssembler _masm(&cbuf);
      __ vzeroupper();
    }
    debug_only(int off1 = cbuf.insts_size());
    assert(off1 - off0 == clear_avx_size(), "correct size prediction");
  
#line 32133 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2052 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // No relocation needed
    MacroAssembler _masm(&cbuf);
    __ mov64(r10, (int64_t) opnd_array(1)->method());
    __ call(r10);
  
#line 32144 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CallLeafNoFPDirectNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  {

#line 2052 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // No relocation needed
    MacroAssembler _masm(&cbuf);
    __ mov64(r10, (int64_t) opnd_array(1)->method());
    __ call(r10);
  
#line 32162 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void RetNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xC3 /*primary()*/));
  
#line 32177 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void TailCalljmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// method_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 32195 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 32203 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ & 7);
  
#line 32212 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void tailjmpIndNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// ex_oop
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// 
  {

#line 1769 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0x5a));
  
#line 32228 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    if (opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ >= 8) {
      emit_opcode(cbuf, Assembler::REX_B);
    }
  
#line 32238 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 1751 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    emit_opcode(cbuf, (0xFF /*primary()*/));
  
#line 32246 "../generated/adfiles/ad_x86_64.cpp"
  }
  {

#line 2009 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // INC, DEC, IDIV, IMOD, JMP indirect, ...
    emit_rm(cbuf, 0x3, (0x4 /*secondary()*/), opnd_array(1)->reg(ra_,this,idx1)/* jump_target */ & 7);
  
#line 32255 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void CreateExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint CreateExceptionNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void RethrowExceptionNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {

#line 2888 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    cbuf.set_insts_mark();
    emit_opcode(cbuf, 0xE9); // jmp entry
    emit_d32_reloc(cbuf,
                   (int) (OptoRuntime::rethrow_stub() - cbuf.insts_end() - 4),
                   runtime_call_Relocation::spec(),
                   RELOC_DISP32);
  
#line 32288 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void tlsLoadPNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// 
  // User did not define which encode class to use.
}

uint tlsLoadPNode::size(PhaseRegAlloc *ra_) const {
  assert(VerifyOops || MachNode::size(ra_) <= 0, "bad fixed size");
  return (VerifyOops ? MachNode::size(ra_) : 0);
}

void ShouldNotReachHereNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 5;
  unsigned idx1 = 5; 	// 
  {
    MacroAssembler _masm(&cbuf);

#line 886 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ int3();
  
#line 32317 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 900 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32334 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 912 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32351 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 912 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32368 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 923 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32385 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_immNode::eval_constant(Compile* C) {
  {

#line 924 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32394 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 935 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32410 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 947 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32427 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 947 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32444 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 959 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32461 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addF_reg_immNode::eval_constant(Compile* C) {
  {

#line 960 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32470 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 971 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32486 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 983 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32503 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 983 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32520 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 994 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32537 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_immNode::eval_constant(Compile* C) {
  {

#line 995 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32546 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void addD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1006 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32562 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1018 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32579 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 1018 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32596 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1030 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vaddsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32613 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void addD_reg_immNode::eval_constant(Compile* C) {
  {

#line 1031 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32622 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1042 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32638 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1054 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32655 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1065 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32672 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_immNode::eval_constant(Compile* C) {
  {

#line 1066 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32681 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1077 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32697 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1089 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32714 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1101 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32731 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subF_reg_immNode::eval_constant(Compile* C) {
  {

#line 1102 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32740 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1113 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32756 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1125 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32773 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1136 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32790 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_immNode::eval_constant(Compile* C) {
  {

#line 1137 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32799 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void subD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1148 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32815 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1160 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32832 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1172 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vsubsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32849 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void subD_reg_immNode::eval_constant(Compile* C) {
  {

#line 1173 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32858 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1184 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 32874 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1196 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32891 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1196 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulss(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32908 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1207 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 32925 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_immNode::eval_constant(Compile* C) {
  {

#line 1208 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 32934 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1219 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 32950 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1231 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 32967 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 1231 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 32984 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1243 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33001 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulF_reg_immNode::eval_constant(Compile* C) {
  {

#line 1244 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33010 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1255 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33026 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1267 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33043 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1267 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulsd(opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33060 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1278 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33077 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_immNode::eval_constant(Compile* C) {
  {

#line 1279 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33086 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void mulD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1290 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33102 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1302 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33119 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_reg_mem_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src2
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src1
  {
    MacroAssembler _masm(&cbuf);

#line 1302 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src1 */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33136 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1314 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmulsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33153 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void mulD_reg_immNode::eval_constant(Compile* C) {
  {

#line 1315 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33162 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1326 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33178 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1338 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33195 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1349 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divss(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33212 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_immNode::eval_constant(Compile* C) {
  {

#line 1350 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33221 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1361 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33237 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1373 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33254 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1385 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33271 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divF_reg_immNode::eval_constant(Compile* C) {
  {

#line 1386 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33280 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1397 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33296 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1409 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33313 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1420 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divsd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33330 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_immNode::eval_constant(Compile* C) {
  {

#line 1421 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33339 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void divD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1432 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */);
  
#line 33355 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_reg_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 1444 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()));
  
#line 33372 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1456 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vdivsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33389 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void divD_reg_immNode::eval_constant(Compile* C) {
  {

#line 1457 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(2));
#line 33398 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void absF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1467 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ andps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signmask()));
  
#line 33413 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void absF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1478 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vandps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(float_signmask()), vector256);
  
#line 33431 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void absD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1492 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ andpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signmask()));
  
#line 33447 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void absD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1504 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vandpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(double_signmask()), vector256);
  
#line 33465 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1517 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ xorps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(float_signflip()));
  
#line 33481 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negF_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1528 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vxorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(float_signflip()), vector256);
  
#line 33499 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  {
    MacroAssembler _masm(&cbuf);

#line 1542 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ xorpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, ExternalAddress(double_signflip()));
  
#line 33515 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void negD_reg_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1554 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vxorpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */,
              ExternalAddress(double_signflip()), vector256);
  
#line 33533 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtF_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1568 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 33549 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtF_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1580 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33565 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtF_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1591 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtss(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33581 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtF_immNode::eval_constant(Compile* C) {
  {

#line 1592 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 33590 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void sqrtD_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1603 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */);
  
#line 33605 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtD_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1615 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33621 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtD_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1626 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ sqrtsd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33637 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void sqrtD_immNode::eval_constant(Compile* C) {
  {

#line 1627 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 33646 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void loadV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 1641 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33661 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 1653 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33677 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 1665 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33693 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void loadV32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 1677 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmovdqu(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
  
#line 33709 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeV4Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1689 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33726 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeV8Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1700 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33743 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeV16Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1711 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdqu(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33760 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void storeV32Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1722 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ vmovdqu(Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()), opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 33777 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1735 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 33795 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1749 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 33813 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1764 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33832 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl32BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1781 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklbw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33852 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1796 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33868 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4B_immNode::eval_constant(Compile* C) {
  {

#line 1797 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate4_imm(opnd_array(1)->constant(), 1));
#line 33877 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1806 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 33892 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8B_immNode::eval_constant(Compile* C) {
  {

#line 1807 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 1));
#line 33901 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl16B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1817 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33917 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16B_immNode::eval_constant(Compile* C) {
  {

#line 1818 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 1));
#line 33926 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl32B_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1830 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33943 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl32B_immNode::eval_constant(Compile* C) {
  {

#line 1831 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 1));
#line 33952 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4B_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1843 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33967 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8B_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1853 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33983 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16B_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1863 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 33999 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl32B_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1873 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    // Use vxorpd since AVX does not have vpxor for 256-bit (AVX2 will have it).
    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34017 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1887 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34034 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1899 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34051 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1912 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34069 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 1927 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshuflw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34088 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1941 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34104 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2S_immNode::eval_constant(Compile* C) {
  {

#line 1942 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate4_imm(opnd_array(1)->constant(), 2));
#line 34113 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1951 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34128 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4S_immNode::eval_constant(Compile* C) {
  {

#line 1952 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 2));
#line 34137 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1962 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34153 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8S_immNode::eval_constant(Compile* C) {
  {

#line 1963 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 2));
#line 34162 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl16S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 1975 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34179 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16S_immNode::eval_constant(Compile* C) {
  {

#line 1976 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 2));
#line 34188 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2S_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1988 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34203 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4S_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 1998 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34219 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8S_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2008 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34235 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl16S_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2018 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    // Use vxorpd since AVX does not have vpxor for 256-bit (AVX2 will have it).
    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34253 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2032 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34270 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2044 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34287 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2057 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34305 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 2070 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
  
#line 34321 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2I_immNode::eval_constant(Compile* C) {
  {

#line 2071 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 4));
#line 34330 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 2081 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34346 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4I_immNode::eval_constant(Compile* C) {
  {

#line 2082 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 4));
#line 34355 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl8I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 2094 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34372 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8I_immNode::eval_constant(Compile* C) {
  {

#line 2095 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, replicate8_imm(opnd_array(1)->constant(), 4));
#line 34381 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2108 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34397 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2120 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
  
#line 34414 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2133 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, 0x00);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34432 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2I_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2146 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34448 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4I_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2156 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34464 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8I_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2166 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    // Use vxorpd since AVX does not have vpxor for 256-bit (AVX2 will have it).
    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34482 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2181 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34499 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2194 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* src */);
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34517 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 2245 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34534 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2L_immNode::eval_constant(Compile* C) {
  {

#line 2246 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 34543 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl4L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// con
  {
    MacroAssembler _masm(&cbuf);

#line 2258 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, InternalAddress(__ code()->consts()->start() + constant_offset()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34560 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4L_immNode::eval_constant(Compile* C) {
  {

#line 2259 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"
    _constant = C->constant_table().add(this, opnd_array(1));
#line 34569 "../generated/adfiles/ad_x86_64.cpp"
  }
}
void Repl2L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2272 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34585 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2285 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, Address::make_raw(opnd_array(1)->base(ra_,this,idx1), opnd_array(1)->index(ra_,this,idx1), opnd_array(1)->scale(), opnd_array(1)->disp(ra_,this,idx1), opnd_array(1)->disp_is_oop()));
    __ punpcklqdq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
    __ vinserti128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34603 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2L_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2298 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34619 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4L_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2308 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    // Use vxorpd since AVX does not have vpxor for 256-bit (AVX2 will have it).
    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34637 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2321 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
  
#line 34653 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2331 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
  
#line 34669 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2342 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x00);
    __ vinsertf128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34686 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2F_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2354 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34702 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4F_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2364 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ xorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34718 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl8F_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2374 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vxorps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34735 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2386 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
  
#line 34751 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2397 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pshufd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, 0x44);
    __ vinsertf128h(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34768 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl2D_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2409 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ xorpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */);
  
#line 34784 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void Repl4D_zeroNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// zero
  {
    MacroAssembler _masm(&cbuf);

#line 2419 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vxorpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, vector256);
  
#line 34801 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2435 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34818 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2445 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 34836 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2456 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34853 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2466 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 34871 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2477 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34888 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2487 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 34906 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd16B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2498 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 34924 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd32B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2509 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 34942 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd32B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2520 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 34960 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2532 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 34977 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2542 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 34995 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2553 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35012 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2563 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35030 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2574 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35047 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2584 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35065 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2595 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35083 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2606 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35101 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd16S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2617 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35119 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2629 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35136 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2639 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35154 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2650 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35171 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2660 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35189 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2671 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35207 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2682 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35225 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2693 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35243 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2705 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ paddq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35260 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2715 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35278 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2726 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35296 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2737 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35314 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2748 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpaddq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35332 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2760 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35349 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2770 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35367 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2781 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35384 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2791 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35402 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2802 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35420 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2813 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35438 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd8F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2824 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vaddps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35456 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2836 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ addpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35473 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2846 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35491 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2857 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35509 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2868 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35527 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vadd4D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2879 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vaddpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35545 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2893 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35562 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2903 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35580 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2914 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35597 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2924 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35615 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2935 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubb(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35632 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2945 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35650 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub16B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2956 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35668 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub32B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 2967 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35686 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub32B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 2978 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubb(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35704 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 2990 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35721 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3000 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35739 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3011 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35756 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3021 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35774 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3032 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35791 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3042 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35809 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3053 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35827 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3064 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35845 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub16S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3075 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35863 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3087 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35880 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3097 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35898 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3108 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 35915 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3118 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35933 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3129 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35951 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3140 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 35969 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3151 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 35987 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3163 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psubq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36004 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3173 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36022 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3184 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36040 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3195 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36058 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4L_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3206 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsubq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36076 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3218 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36093 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3228 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36111 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3239 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36128 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3249 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36146 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3260 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36164 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3271 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36182 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub8F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3282 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vsubps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36200 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3294 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ subpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36217 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3304 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36235 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3315 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36253 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3326 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36271 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsub4D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3337 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vsubpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36289 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3351 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36306 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3361 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36324 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3372 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36341 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3382 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36359 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3393 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pmullw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36376 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3403 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36394 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3414 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36412 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3425 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36430 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul16S_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3436 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpmullw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36448 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3448 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pmulld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36465 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3458 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36483 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3469 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pmulld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36500 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3479 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36518 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3490 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36536 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3501 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36554 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8I_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3512 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpmulld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36572 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3524 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36589 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3534 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36607 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3545 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36624 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3555 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36642 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3566 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36660 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3577 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36678 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul8F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3588 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vmulps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36696 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3600 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ mulpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36713 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3610 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36731 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3621 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36749 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3632 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36767 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vmul4D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3643 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vmulpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36785 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv2FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3657 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36802 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv2F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3667 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36820 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv4FNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3678 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divps(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36837 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv4F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3688 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36855 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv4F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3699 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36873 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv8F_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3710 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36891 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv8F_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3721 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vdivps(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36909 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv2DNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 3733 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ divpd(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 36926 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv2D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3743 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36944 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv2D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3754 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36962 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv4D_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 3765 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 36980 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vdiv4D_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 3776 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vdivpd(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 36998 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vshiftcntNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 3791 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 37014 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vshiftcnt_0Node::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// cnt
  {
    MacroAssembler _masm(&cbuf);

#line 3791 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ movdl(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_Register(ra_,this,idx1)/* cnt */);
  
#line 37030 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3804 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37047 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3814 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37064 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3824 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37082 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3835 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37100 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3846 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37117 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3856 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37134 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3866 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37152 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3877 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37170 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3888 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37187 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3898 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37204 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3908 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37222 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3919 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37240 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3930 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37258 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll16S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3941 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsllw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37276 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3953 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pslld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37293 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3963 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pslld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37310 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3973 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37328 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3984 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37346 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 3995 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pslld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37363 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4005 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pslld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37380 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4015 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37398 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4026 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37416 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4037 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37434 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll8I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4048 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpslld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37452 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4060 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37469 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4070 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psllq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37486 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4080 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37504 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll2L_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4091 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsllq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37522 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4102 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsllq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37540 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsll4L_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4113 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsllq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37558 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4131 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37575 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4141 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37592 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4151 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37610 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4162 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37628 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4173 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37645 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4183 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37662 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4193 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37680 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4204 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37698 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4215 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37715 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4225 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37732 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4235 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37750 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4246 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37768 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4257 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37786 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl16S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4268 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrlw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37804 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4280 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37821 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4290 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37838 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4300 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37856 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4311 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37874 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4322 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37891 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4332 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrld(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 37908 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4342 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37926 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4353 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37944 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4364 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 37962 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl8I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4375 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrld(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 37980 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2LNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4387 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 37997 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2L_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4397 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrlq(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38014 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4407 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38032 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl2L_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4418 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrlq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38050 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4L_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4429 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrlq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38068 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsrl4L_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4440 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrlq(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38086 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4454 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 38103 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4464 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38120 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4474 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38138 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4485 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38156 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4496 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 38173 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4506 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38190 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4516 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38208 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4527 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38226 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8SNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4538 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 38243 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8S_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4548 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psraw(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38260 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4558 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38278 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4569 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38296 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra16S_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4580 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38314 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra16S_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4591 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsraw(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38332 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4603 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 38349 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4613 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38366 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4623 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38384 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra2I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4634 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38402 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4INode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4645 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */);
  
#line 38419 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4I_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4655 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ psrad(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, (int)opnd_array(2)->constant());
  
#line 38436 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4665 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38454 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra4I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4676 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38472 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8I_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4687 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* shift */, vector256);
  
#line 38490 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vsra8I_reg_immNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// shift
  {
    MacroAssembler _masm(&cbuf);

#line 4698 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpsrad(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, (int)opnd_array(2)->constant(), vector256);
  
#line 38508 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4714 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38525 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand4B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4724 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38543 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4735 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38560 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4745 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38578 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4756 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pand(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38595 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4766 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38613 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand16B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4777 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38631 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand32B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4788 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38649 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vand32B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4799 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpand(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38667 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4812 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38684 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor4B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4822 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38702 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4833 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38719 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4843 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38737 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4854 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ por(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38754 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4864 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38772 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor16B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4875 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38790 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor32B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4886 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38808 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vor32B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4897 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38826 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor4BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4910 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38843 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor4B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4920 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38861 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor8BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4931 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38878 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor8B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4941 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38896 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor16BNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// dst
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src
  {
    MacroAssembler _masm(&cbuf);

#line 4952 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    __ pxor(opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* dst */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src */);
  
#line 38913 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor16B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4962 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38931 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor16B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4973 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = false;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38949 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor32B_regNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 1;
  unsigned idx1 = 1; 	// src1
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// src2
  {
    MacroAssembler _masm(&cbuf);

#line 4984 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src1 */, opnd_array(2)->as_XMMRegister(ra_,this,idx2)/* src2 */, vector256);
  
#line 38967 "../generated/adfiles/ad_x86_64.cpp"
  }
}

void vxor32B_memNode::emit(CodeBuffer& cbuf, PhaseRegAlloc* ra_) const {
  cbuf.set_insts_mark();
  // Start at oper_input_base() and count operands
  unsigned idx0 = 2;
  unsigned idx1 = 2; 	// src
  unsigned idx2 = idx1 + opnd_array(1)->num_edges(); 	// mem
  {
    MacroAssembler _masm(&cbuf);

#line 4995 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86.ad"

    bool vector256 = true;
    __ vpxor(opnd_array(0)->as_XMMRegister(ra_,this)/* dst */, opnd_array(1)->as_XMMRegister(ra_,this,idx1)/* src */, Address::make_raw(opnd_array(2)->base(ra_,this,idx2), opnd_array(2)->index(ra_,this,idx2), opnd_array(2)->scale(), opnd_array(2)->disp(ra_,this,idx2), opnd_array(2)->disp_is_oop()), vector256);
  
#line 38985 "../generated/adfiles/ad_x86_64.cpp"
  }
}

const MachOper* loadBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUB2L_immI8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUSNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUS2L_immI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadINode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2BNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2UBNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2SNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2USNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_255Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immI_65535Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadI2L_immU31Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadUI2LNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadLNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadRangeNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadNKlassNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadFNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadD_partialNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadDNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchrT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchwNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocNTANode::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT0Node::memory_operand() const { return _opnds[1]; }
const MachOper* prefetchAllocT2Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeCNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmP0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmPNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmN0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmNNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmINode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmL0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmLNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmC0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmI16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmB0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmBNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCM0_regNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeImmCM0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeFNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeF0Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeF_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeDNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeD0Node::memory_operand() const { return _opnds[1]; }
const MachOper* popCountI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* popCountL_memNode::memory_operand() const { return _opnds[1]; }
const TypePtr *membar_acquireNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquireNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_acquire_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_acquire_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_releaseNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_releaseNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_release_lockNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_release_lockNode::memory_operand() const { return (MachOper*)-1; }
const TypePtr *membar_storestoreNode::adr_type() const { return TypePtr::BOTTOM; }
const MachOper* membar_storestoreNode::memory_operand() const { return (MachOper*)-1; }
const MachOper* cmovI_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovI_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memUNode::memory_operand() const { return _opnds[4]; }
const MachOper* cmovL_memUCFNode::memory_operand() const { return _opnds[4]; }
const MachOper* addI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* incL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* decL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadPLockedNode::memory_operand() const { return _opnds[1]; }
const MachOper* storePConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeIConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* storeLConditionalNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapPNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapLNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapINode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddI_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddINode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddL_no_resNode::memory_operand() const { return _opnds[1]; }
const MachOper* xaddLNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgINode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgLNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgPNode::memory_operand() const { return _opnds[1]; }
const MachOper* xchgNNode::memory_operand() const { return _opnds[1]; }
const MachOper* compareAndSwapNNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* subL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* negI_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* negL_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulI_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulL_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrI_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* salL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* sarL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_1Node::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* shrL_mem_CLNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorI_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorI_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* andL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* andL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* orL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* orL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* xorL_rReg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rRegNode::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_rReg_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* xorL_mem_immNode::memory_operand() const { return _opnds[1]; }
const MachOper* cmpF_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_cc_memCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_cc_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_cc_memCFNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* cmpD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* convF2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convD2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2F_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convL2D_reg_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* convI2L_reg_mem_zexNode::memory_operand() const { return _opnds[1]; }
const MachOper* compI_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testI_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compU_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compP_mem_rRegNode::memory_operand() const { return _opnds[2]; }
const MachOper* testP_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* testP_mem_reg0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compN_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* compN_mem_immNode::memory_operand() const { return _opnds[2]; }
const MachOper* testN_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* testN_mem_reg0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* testL_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* compUL_rReg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* addD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* addD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* subF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* subD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulF_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* mulD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* mulD_reg_mem_0Node::memory_operand() const { return _opnds[1]; }
const MachOper* divF_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divF_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* divD_reg_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* sqrtF_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* sqrtD_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* loadV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV16Node::memory_operand() const { return _opnds[1]; }
const MachOper* loadV32Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV4Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV8Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV16Node::memory_operand() const { return _opnds[1]; }
const MachOper* storeV32Node::memory_operand() const { return _opnds[1]; }
const MachOper* Repl2I_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* Repl4I_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* Repl8I_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* Repl2L_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* Repl4L_memNode::memory_operand() const { return _opnds[1]; }
const MachOper* vadd16B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd32B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd8S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd16S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd4I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd8I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd2L_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd4L_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd4F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd8F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd2D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vadd4D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub16B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub32B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub8S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub16S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub4I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub8I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub2L_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub4L_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub4F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub8F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub2D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vsub4D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul8S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul16S_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul4I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul8I_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul4F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul8F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul2D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vmul4D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv4F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv8F_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv2D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vdiv4D_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand16B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vand32B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor16B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vor32B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vxor16B_memNode::memory_operand() const { return _opnds[2]; }
const MachOper* vxor32B_memNode::memory_operand() const { return _opnds[2]; }


const bool Matcher::has_match_rule(int opcode) {
  assert(_last_machine_leaf < opcode && opcode < _last_opcode, "opcode in range");
  return _hasMatchRule[opcode];
}

const bool Matcher::_hasMatchRule[_last_opcode] = {
    false,  // Node
    false,  // Set
    false,  // RegN
    false,  // RegI
    false,  // RegP
    false,  // RegF
    false,  // RegD
    false,  // RegL
    false,  // RegFlags
    false,  // VecS
    false,  // VecD
    false,  // VecX
    false,  // VecY
    false,  // _last_machine_leaf
    true ,  // AbsD
    true ,  // AbsF
    false,  // AbsI
    true ,  // AddD
    true ,  // AddF
    true ,  // AddI
    true ,  // AddL
    true ,  // AddP
    false,  // Allocate
    false,  // AllocateArray
    true ,  // AndI
    true ,  // AndL
    true ,  // AryEq
    false,  // AtanD
    true ,  // Binary
    true ,  // Bool
    false,  // BoxLock
    true ,  // ReverseBytesI
    true ,  // ReverseBytesL
    true ,  // ReverseBytesUS
    true ,  // ReverseBytesS
    false,  // CProj
    true ,  // CallDynamicJava
    false,  // CallJava
    true ,  // CallLeaf
    true ,  // CallLeafNoFP
    true ,  // CallRuntime
    true ,  // CallStaticJava
    true ,  // CastII
    true ,  // CastX2P
    true ,  // CastP2X
    true ,  // CastPP
    false,  // Catch
    false,  // CatchProj
    true ,  // CheckCastPP
    true ,  // ClearArray
    false,  // ConstraintCast
    true ,  // CMoveD
    true ,  // CMoveF
    true ,  // CMoveI
    true ,  // CMoveL
    true ,  // CMoveP
    true ,  // CMoveN
    true ,  // CmpN
    true ,  // CmpD
    true ,  // CmpD3
    true ,  // CmpF
    true ,  // CmpF3
    true ,  // CmpI
    true ,  // CmpL
    true ,  // CmpL3
    true ,  // CmpLTMask
    true ,  // CmpP
    true ,  // CmpU
    true ,  // CmpUL
    true ,  // CompareAndSwapI
    true ,  // CompareAndSwapL
    true ,  // CompareAndSwapP
    true ,  // CompareAndSwapN
    true ,  // GetAndAddI
    true ,  // GetAndAddL
    true ,  // GetAndSetI
    true ,  // GetAndSetL
    true ,  // GetAndSetP
    true ,  // GetAndSetN
    false,  // Con
    true ,  // ConN
    true ,  // ConD
    true ,  // ConF
    true ,  // ConI
    true ,  // ConL
    true ,  // ConP
    true ,  // Conv2B
    true ,  // ConvD2F
    true ,  // ConvD2I
    true ,  // ConvD2L
    true ,  // ConvF2D
    true ,  // ConvF2I
    true ,  // ConvF2L
    true ,  // ConvI2D
    true ,  // ConvI2F
    true ,  // ConvI2L
    true ,  // ConvL2D
    true ,  // ConvL2F
    true ,  // ConvL2I
    true ,  // CosD
    false,  // CountedLoop
    true ,  // CountedLoopEnd
    true ,  // CountLeadingZerosI
    true ,  // CountLeadingZerosL
    true ,  // CountTrailingZerosI
    true ,  // CountTrailingZerosL
    true ,  // CreateEx
    true ,  // DecodeN
    true ,  // DivD
    true ,  // DivF
    true ,  // DivI
    true ,  // DivL
    false,  // DivMod
    true ,  // DivModI
    true ,  // DivModL
    true ,  // EncodeP
    true ,  // ExpD
    true ,  // FastLock
    true ,  // FastUnlock
    true ,  // Goto
    true ,  // Halt
    true ,  // If
    false,  // IfFalse
    false,  // IfTrue
    false,  // Initialize
    false,  // JProj
    true ,  // Jump
    false,  // JumpProj
    true ,  // LShiftI
    true ,  // LShiftL
    true ,  // LoadB
    true ,  // LoadUB
    true ,  // LoadUS
    true ,  // LoadD
    false,  // LoadD_unaligned
    true ,  // LoadF
    true ,  // LoadI
    true ,  // LoadKlass
    true ,  // LoadNKlass
    true ,  // LoadL
    false,  // LoadL_unaligned
    true ,  // LoadPLocked
    true ,  // LoadP
    true ,  // LoadN
    true ,  // LoadRange
    true ,  // LoadS
    false,  // Lock
    true ,  // LogD
    true ,  // Log10D
    false,  // Loop
    false,  // LoopLimit
    false,  // Mach
    false,  // MachProj
    true ,  // MaxI
    true ,  // MemBarAcquire
    true ,  // MemBarAcquireLock
    false,  // MemBarCPUOrder
    true ,  // MemBarRelease
    true ,  // MemBarReleaseLock
    true ,  // MemBarVolatile
    true ,  // MemBarStoreStore
    false,  // MergeMem
    true ,  // MinI
    false,  // ModD
    false,  // ModF
    true ,  // ModI
    true ,  // ModL
    true ,  // MoveI2F
    true ,  // MoveF2I
    true ,  // MoveL2D
    true ,  // MoveD2L
    true ,  // MulD
    true ,  // MulF
    true ,  // MulHiL
    true ,  // MulI
    true ,  // MulL
    false,  // Multi
    true ,  // NegD
    true ,  // NegF
    false,  // NeverBranch
    false,  // Opaque1
    false,  // Opaque2
    true ,  // OrI
    true ,  // OrL
    false,  // PCTable
    false,  // Parm
    true ,  // PartialSubtypeCheck
    false,  // Phi
    true ,  // PopCountI
    true ,  // PopCountL
    true ,  // PowD
    true ,  // PrefetchAllocation
    true ,  // PrefetchRead
    true ,  // PrefetchWrite
    false,  // Proj
    true ,  // RShiftI
    true ,  // RShiftL
    false,  // Region
    true ,  // Rethrow
    true ,  // Return
    false,  // Root
    true ,  // RoundDouble
    true ,  // RoundFloat
    true ,  // SafePoint
    false,  // SafePointScalarObject
    false,  // SCMemProj
    true ,  // SinD
    true ,  // SqrtD
    false,  // Start
    false,  // StartOSR
    true ,  // StoreB
    true ,  // StoreC
    true ,  // StoreCM
    true ,  // StorePConditional
    true ,  // StoreIConditional
    true ,  // StoreLConditional
    true ,  // StoreD
    true ,  // StoreF
    true ,  // StoreI
    true ,  // StoreL
    true ,  // StoreP
    true ,  // StoreN
    true ,  // StrComp
    true ,  // StrEquals
    true ,  // StrIndexOf
    true ,  // SubD
    true ,  // SubF
    true ,  // SubI
    true ,  // SubL
    true ,  // TailCall
    true ,  // TailJump
    true ,  // TanD
    true ,  // ThreadLocal
    false,  // Unlock
    true ,  // URShiftI
    true ,  // URShiftL
    true ,  // XorI
    true ,  // XorL
    false,  // Vector
    true ,  // AddVB
    true ,  // AddVS
    true ,  // AddVI
    true ,  // AddVL
    true ,  // AddVF
    true ,  // AddVD
    true ,  // SubVB
    true ,  // SubVS
    true ,  // SubVI
    true ,  // SubVL
    true ,  // SubVF
    true ,  // SubVD
    true ,  // MulVS
    true ,  // MulVI
    true ,  // MulVF
    true ,  // MulVD
    true ,  // DivVF
    true ,  // DivVD
    true ,  // LShiftCntV
    true ,  // RShiftCntV
    false,  // LShiftVB
    true ,  // LShiftVS
    true ,  // LShiftVI
    true ,  // LShiftVL
    false,  // RShiftVB
    true ,  // RShiftVS
    true ,  // RShiftVI
    false,  // RShiftVL
    false,  // URShiftVB
    true ,  // URShiftVS
    true ,  // URShiftVI
    true ,  // URShiftVL
    true ,  // AndV
    true ,  // OrV
    true ,  // XorV
    true ,  // LoadVector
    true ,  // StoreVector
    false,  // Pack
    false,  // PackB
    false,  // PackS
    false,  // PackI
    false,  // PackL
    false,  // PackF
    false,  // PackD
    false,  // Pack2L
    false,  // Pack2D
    true ,  // ReplicateB
    true ,  // ReplicateS
    true ,  // ReplicateI
    true ,  // ReplicateL
    true ,  // ReplicateF
    true ,  // ReplicateD
    false,  // Extract
    false,  // ExtractB
    false,  // ExtractUB
    false,  // ExtractC
    false,  // ExtractS
    false,  // ExtractI
    false,  // ExtractL
    false,  // ExtractF
    false   // ExtractD
};


bool Matcher::stack_direction() const { return false; }

int Compile::sync_stack_slots() const { return 2; }

uint Matcher::stack_alignment_in_bytes() { return StackAlignmentInBytes; }

OptoReg::Name Matcher::return_addr() const { return OptoReg::stack2reg(- 2 +
              round_to((Compile::current()->in_preserve_stack_slots() +
                        Compile::current()->fixed_slots()),
                       stack_alignment_in_slots())); }

uint Compile::in_preserve_stack_slots() { return 4 + 2 * VerifyStackAtCalls; }

uint Compile::out_preserve_stack_slots() { return SharedRuntime::out_preserve_stack_slots(); }

uint Compile::varargs_C_out_slots_killed() const { return frame::arg_reg_save_area_bytes/BytesPerInt; }

void Matcher::calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length, bool is_outgoing) {

#line 3016 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // No difference between ingoing/outgoing just pass false
    SharedRuntime::java_calling_convention(sig_bt, regs, length, false);
  
#line 39611 "../generated/adfiles/ad_x86_64.cpp"

}

void Matcher::c_calling_convention(BasicType *sig_bt, VMRegPair *regs, uint length) {

#line 3022 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    // This is obviously always outgoing
    (void) SharedRuntime::c_calling_convention(sig_bt, regs, /*regs2=*/NULL, length);
  
#line 39622 "../generated/adfiles/ad_x86_64.cpp"

}

OptoRegPair Matcher::return_value(int ideal_reg, bool is_outgoing) {

#line 3029 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0,
      0,
      RAX_num,  // Op_RegN
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegN
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0b_num,    // Op_RegD
      RAX_H_num     // Op_RegL
    };
    // Excluded flags and vector registers.
    assert(ARRAY_SIZE(hi) == _last_machine_leaf - 5, "missing type");
    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 39657 "../generated/adfiles/ad_x86_64.cpp"

}

OptoRegPair Matcher::c_return_value(int ideal_reg, bool is_outgoing) {

#line 3029 "/builddir/build/BUILD/java-1.7.0-openjdk/openjdk/hotspot/src/cpu/x86/vm/x86_64.ad"

    assert(ideal_reg >= Op_RegI && ideal_reg <= Op_RegL,
           "only return normal values");

    static const int lo[Op_RegL + 1] = {
      0,
      0,
      RAX_num,  // Op_RegN
      RAX_num,  // Op_RegI
      RAX_num,  // Op_RegP
      XMM0_num, // Op_RegF
      XMM0_num, // Op_RegD
      RAX_num   // Op_RegL
    };
    static const int hi[Op_RegL + 1] = {
      0,
      0,
      OptoReg::Bad, // Op_RegN
      OptoReg::Bad, // Op_RegI
      RAX_H_num,    // Op_RegP
      OptoReg::Bad, // Op_RegF
      XMM0b_num,    // Op_RegD
      RAX_H_num     // Op_RegL
    };
    // Excluded flags and vector registers.
    assert(ARRAY_SIZE(hi) == _last_machine_leaf - 5, "missing type");
    return OptoRegPair(hi[ideal_reg], lo[ideal_reg]);
  
#line 39692 "../generated/adfiles/ad_x86_64.cpp"

}

OptoReg::Name Matcher::inline_cache_reg() { return OptoReg::Name(RAX_num); }

int Matcher::inline_cache_reg_encode() { return _regEncode[inline_cache_reg()]; }

OptoReg::Name Matcher::interpreter_method_oop_reg() { return OptoReg::Name(RBX_num); }

int Matcher::interpreter_method_oop_reg_encode() { return _regEncode[interpreter_method_oop_reg()]; }

OptoReg::Name Matcher::interpreter_frame_pointer_reg() { return OptoReg::Name(RBP_num); }

OptoReg::Name Matcher::c_frame_pointer() const { return OptoReg::Name(RSP_num); }

// Number of callee-save + always-save registers
int  Matcher::number_of_saved_registers() {
  return 0;
};

// Check consistency of C++ compilation with ADLC options:
// Check adlc -DLINUX=1
#ifndef LINUX
#  error "LINUX must be defined"
#endif // LINUX
// Check adlc -D_GNU_SOURCE=1
#ifndef _GNU_SOURCE
#  error "_GNU_SOURCE must be defined"
#endif // _GNU_SOURCE
// Check adlc -DAMD64=1
#ifndef AMD64
#  error "AMD64 must be defined"
#endif // AMD64
// Check adlc -D_LP64=1
#ifndef _LP64
#  error "_LP64 must be defined"
#endif // _LP64
